#!/bin/sh

if [ -n "$1" ]; then
    DUAL=$1
else
    DUAL=1
fi

if [ -n "$2" ]; then
    STOP_ON_ERR=$2
else
    STOP_ON_ERR=1
fi

if [ -n "$3" ]; then
    DURATION=$3
else
    DURATION=0
fi


if [ $DUAL -eq 1 ] && [ "$CONTROLLER" == "A" ]; then
    echo fwdCmd "memstress.sh $DUAL"
fi

if [ $STOP_ON_ERR -eq 0 ]; then
    CONTINUE="--c"
else
    CONTINUE=""
fi

TESTRANGE=`lemur mem show testrange | grep 0x`

STARTADDR=$(echo $TESTRANGE | awk '{print $1}')
ENDADDR=$(echo $TESTRANGE | awk '{print $3}')
LOG_FILE="/tmp/log/memstress.log"
RESULT_FILE="/tmp/log/memstress_result"

rm /tmp/stop_flag -f

mkdir -p /tmp/log
if [ -f $LOG_FILE ]; then
    rm -rf $LOG_FILE
fi
touch $LOG_FILE
if [ -f $RESULT_FILE ]; then
    rm -rf $RESULT_FILE
fi
touch $RESULT_FILE

START_TIME=$(date +%s)
START_MESSAGE=$(date +"[%m/%d %T]")" Memory Stress Start"
echo $START_MESSAGE > $LOG_FILE

memstress $STARTADDR $ENDADDR $CONTINUE --s 1>> $LOG_FILE 2> $RESULT_FILE &
while [ 1 ]; do
    if [ -s $RESULT_FILE ]; then
        if [ $STOP_ON_ERR -eq 1 ]; then
            touch /tmp/stop_flag
            if [ $DUAL -eq 1 ]; then
                echo fwdCmd "touch /tmp/stop_flag"
            fi
            break
        fi
    fi

    if [ -f /tmp/stop_flag ]; then
        break                                 
    fi      

    if [ $DURATION -gt 0 ]; then
        CURRENT_TIME=$(date +%s)
        TIME_DIFF=$(( $CURRENT_TIME-$START_TIME ))

        if [ $TIME_DIFF -ge $DURATION ]; then
            break;
        fi
    fi
done

pid=`ps -all | grep memstress | grep "${STARTADDR}" | awk '{print $1}'`
if [ "${pid}" != "" ]; then
    kill ${pid} >/dev/null 2>&1
fi

if [ -s $RESULT_FILE ]; then
    echo $(date +"[%m/%d %T]") "Memory Stress End" >> $RESULT_FILE
    echo $(date +"[%m/%d %T]") "Memory Stress Result: FAIL" >> $RESULT_FILE
else
    echo $(date +"[%m/%d %T]") "Memory Stress End" > $RESULT_FILE
    echo $(date +"[%m/%d %T]") "Memory Stress Result: PASS" >> $RESULT_FILE
fi

sed -i "1i\\
${START_MESSAGE}" $RESULT_FILE
