#include "BBUCommand.hpp"

BBUCommand::BBUCommand(COMMAND_TYPE CommandType, string HelpContent) : CommandBaseClass(CommandType, HelpContent)
{
    //Base on command type to add different operations
    switch (CommandType)
    {
        case CMD_SHOW:
            addOperation(new BBUOperation(OPERATION_MONITOR, "Display BBU Information"));
            addOperation(new BBUOperation(OPERATION_CHARGE, "Show charge status"));
            break;
        case CMD_UTILITY:
            addOperation(new BBUOperation(OPERATION_READ, "Read BBU device data"));
            addOperation(new BBUOperation(OPERATION_WRITE, "Write data to BBU device"));
            break;
        case CMD_CHARGE:
            addOperation(new BBUOperation(OPERATION_ON, "Set the BBU to charge"));
            addOperation(new BBUOperation(OPERATION_OFF, "Set the BBU to discharge"));
            break;
        //Add NEW command and operation here
        default:
            break;
    }
}



BBUCommand::~BBUCommand()
{

}



int
BBUCommand::List(StringParser command_line)
{
    BBUDAPI       *pBBUDAPI = NULL;
    list<string>  bbu_list;
    string        bbu_device;
    ErrorMsg      *error_code = NULL;


    pBBUDAPI = new BBUDAPI("BBUDAPI");

    //Get BBU device list by API
    error_code = pBBUDAPI->Get_BBU_List(&bbu_list);
    if (error_code != NO_ERROR_MSG)
    {
        printError(error_code);
    }
    else
    {
        cout << "Names" << endl;
        while (!bbu_list.empty())
        {
            bbu_device = bbu_list.front();
            bbu_list.pop_front();

            cout << bbu_device << endl;
        }
    }

    return 0;
}



int
BBUCommand::Show(StringParser command_line)
{
   // cout << "I2C Show command execution" << endl;
    string parser_operation_name;
	  int i, err = 0;
	  string operation_name;

    command_line.getOperation(&parser_operation_name);

    //Check help or not operation than list supported operation
    if (parser_operation_name == HELP || parser_operation_name == "")
    {
        listOperation(command_line);
        return -1;
    }

    //Search for operation
    for (i = 0; i< m_operation_count; i++)
    {
        m_operation[i]->getName(&operation_name);
        if (operation_name == parser_operation_name)
        {
            err = m_operation[i]->executeOperation(command_line);
            break;
        }
    }

    if (i == m_operation_count)
    {
        cout << parser_operation_name << " operation not found" << endl;
        return -1;
    }
    else
    {
        return err;
    }

    return 0;
}



int
BBUCommand::Utility(StringParser command_line)
{
    string parser_operation_name;
	  int i, err = 0;
	  string operation_name;
	  

    command_line.getOperation(&parser_operation_name);
    
    //Check help or not operation than list supported operation
    if (parser_operation_name == HELP || parser_operation_name == "")
    {
        listOperation(command_line);
        return -1;
    }

    //Search for operation
    for (i = 0; i< m_operation_count; i++)
    {
    	  m_operation[i]->getName(&operation_name);
        if (operation_name == parser_operation_name)
        {
            err = m_operation[i]->executeOperation(command_line);
            break;
        }
    }

    if (i == m_operation_count)
    {
        cout << parser_operation_name << " operation not found" << endl;
        return -1;
    }
    else
    {
        return err;
    }
}



int
BBUCommand::Charge(StringParser command_line)
{
    string parser_operation_name;
	  int i, err = 0;
	  string operation_name;

    command_line.getOperation(&parser_operation_name);

    //Check help or not operation than list supported operation
    if (parser_operation_name == HELP || parser_operation_name == "")
    {
        listOperation(command_line);
        return -1;
    }

    //Search for operation
    for (i = 0; i< m_operation_count; i++)
    {
        m_operation[i]->getName(&operation_name);
        if (operation_name == parser_operation_name)
        {
            err = m_operation[i]->executeOperation(command_line);
            break;
        }
    }

    if (i == m_operation_count)
    {
        cout << parser_operation_name << " operation not found" << endl;
        return -1;
    }
    else
    {
        return err;
    }
}



int
BBUCommand::executeCommand(StringParser command_line)
{
	  int err = 0;
	  COMMAND_TYPE command_type = NO_COMMAND;

    command_line.getCommandType(&command_type);

    switch (command_type)
    {
        case CMD_LIST:
            err = List(command_line);
            break;
        case CMD_SHOW:
            err = Show(command_line);
            break;
        case CMD_UTILITY:
            err = Utility(command_line);
            break;
        case CMD_CHARGE:
            err = Charge(command_line);
            break;
        default:
            cout << "Not supported command" << endl;
            err = -1;
            break;
    }

    return err;
}

