#include "BBUOperation.hpp"



BBUOperation::BBUOperation(OPERATION_TYPE oprationType, string helpContent) : OperationBaseClass(oprationType, helpContent)
{

}



BBUOperation::~BBUOperation()
{
  
}



int
BBUOperation::executeOperation(StringParser command_line)
{
	  string parameter_1;
	  int    err = 0;


    command_line.getParameter(&parameter_1, 0);

    //If the first parameter is "help" or space, show help message         
    if (parameter_1 == HELP || parameter_1 == "")
    {
        Help_parameter(command_line);
        err = -1;
        return err;
    }

    //Execute operation
    switch (getOperationType())
    {
        case OPERATION_MONITOR:
            err = monitor(command_line);
            break;
        case OPERATION_READ:
            err = read(command_line);
            break;
        case OPERATION_WRITE:
            err = write(command_line);
            break;
        case OPERATION_ON:
            err = on(command_line);
            break;
        case OPERATION_OFF:
            err = off(command_line);
            break;
        case OPERATION_CHARGE:
            err = showCharge(command_line);
            break;
        default:
            cout << "Unsupport operation" << endl;
            err = -1; 
            break;
    }

    return err;
}



int
BBUOperation::Help_parameter(StringParser command_line)
{
    string module, command, operation;

    command_line.getModule(&module);
    command_line.getCommand(&command);
    getName(&operation);

    cout << "Usage:	lemur " << module << " " << command << " " << operation << " ";
    //Show parameter help message depend on operation type
    switch (getOperationType())
    {
        case OPERATION_READ:
            cout << "P1 P2 P3" << endl;
            cout << "read BBU device data" << endl;
            cout << "Syntax: read P1 P2 P3" << endl;
            cout << "P1: Register Address" << endl;
            cout << "P2: Length" << endl;
            cout << "P3: Device name" << endl;
            break;
        case OPERATION_WRITE:
            cout << "P1 P2 P3" << endl;
            cout << "Write BBU device data" << endl;
            cout << "Syntax: write P1 P2 P3" << endl;
            cout << "P1: reg. address" << endl;
            cout << "P2: write data" << endl;
            cout << "P3: Device name" << endl;
            break;
        case OPERATION_MONITOR:
            cout << "P1" << endl;
            cout << "Show BBU information" << endl;
            cout << "Syntax: monitor P1" << endl;
            cout << "P1: BBU device name" << endl;
            break;
        case OPERATION_ON:
            cout << "P1" << endl;
            cout << "Set the BBU to charge" << endl;
            cout << "Syntax: on P1" << endl;
            cout << "P1: BBU device name" << endl;
            break;
        case OPERATION_OFF:
            cout << "P1" << endl;
            cout << "Set the BBU to discharge" << endl;
            cout << "Syntax: off P1" << endl;
            cout << "P1: BBU device name" << endl;
            break;
        case OPERATION_CHARGE:
            cout << "P1" << endl;
            cout << "Show BBU charge status" << endl;
            cout << " Syntax: charge P1" << endl;
            cout << " P1: BBU device name" << endl;
            break;
       default:
            cout << "\nNot define parameter help message" << endl;
            break;
    }
    
    return 0;
}



int
BBUOperation::monitor(StringParser command_line)
{
    int      err = 0;
    BBUDAPI  *pBBUDAPI = NULL;
    ErrorMsg *error_code;
    string   WellKnownName;


    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get I2C device name
        command_line.getParameter(&WellKnownName, 0);
        //cout << "Name: " <<  WellKnownName << endl;

        //Creat BBU DAPI object 
        pBBUDAPI = new BBUDAPI("I2CDAPI");

        //Get BBU information by API
        error_code = pBBUDAPI->getBBUInformation(WellKnownName);

        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            err = -1;
        }

        delete pBBUDAPI;
    }

    return err;
}



int
BBUOperation::write(StringParser command_line)
{
    int      err = 0;
    int      StartAddress, i;
    int      data_buffer[8192], count;
    int      numberRealWrite = 0;
    BBUDAPI  *pBBUDAPI = NULL;
    ErrorMsg *error_code;
    string   WellKnownName, temp_string, error_massage;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get BBU device name
        command_line.getParameter(&WellKnownName, 2);
        //cout << "Name: " <<  WellKnownName << endl;

        //Get Start address
        command_line.getParameter(&temp_string, 0);
        command_line.convertStringToHex(temp_string, &StartAddress);
        //cout << "Start: " << StartAddress << endl;

        //Get write data
        command_line.getParameter(&temp_string, 1);
        command_line.convertHexStringToArrary(temp_string, data_buffer, &count);


        //Creat BBU DAPI object
        pBBUDAPI = new BBUDAPI("BBUDAPI");

        //Write data to BBU device by API
        error_code = pBBUDAPI->Write(WellKnownName, StartAddress, data_buffer, count, &numberRealWrite);

        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            err = -1;
        }
        else
        {
            //Write successful and print the write data
            //Get BBU device data by API
            error_code = pBBUDAPI->Read(WellKnownName, StartAddress, StartAddress + (numberRealWrite - 1), data_buffer, &count);

            if (error_code != NO_ERROR_MSG)
            {
                printError(error_code);
                err = -1;
            }
            else
            {
                for (i=0; i<count ; i++)
                {
                    if (i == 0 || (i % 16) == 0 )
                    {
                        printf("%04x: ", StartAddress+i);
                    }

                    if (StartAddress >= 0x20 && StartAddress <= 0x23)
                    {
                        printf("%02x ", data_buffer[i]);
                    }
                    else
                    {
                        printf("%04x ", data_buffer[i]);
                    }

                    if (((i+1) % 16) == 0  && i != (count-1))
                    {
                        printf("\n");
                    }
                }
                printf("\n");
            }
        }
        delete pBBUDAPI;
    }

    return err;
}



int
BBUOperation::read(StringParser command_line)
{
    int      err = 0;
    int      StartAddress, Length, EndAddress, i;
    int      data[8192], count;
    BBUDAPI  *pBBUDAPI = NULL;
    ErrorMsg *error_code;
    string   WellKnownName, temp_string, error_massage;


    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get BBU device name
        command_line.getParameter(&WellKnownName, 2);
        //cout << "Name: " <<  WellKnownName << endl;

        //Get Start address
        command_line.getParameter(&temp_string, 0);
        command_line.convertStringToHex(temp_string, &StartAddress);
        //cout << "Start: " << StartAddress << endl;

        //Get Length address
        command_line.getParameter(&temp_string, 1);
        command_line.convertStringToHex(temp_string, &Length);
        //Calculate the end address
        EndAddress = StartAddress + (Length - 1);
        //cout << "EndAddress: " << EndAddress << endl;

        //Creat BBU DAPI object
        pBBUDAPI = new BBUDAPI("BBUDAPI");

        //Read BBU data by API
        error_code = pBBUDAPI->Read(WellKnownName, StartAddress, EndAddress, data, &count);

        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            err = -1;
        }
        else
        {
            for (i=0; i<count ; i++)
            {
                if (i == 0 || (i % 16) == 0 )
                {
                    printf("%04x: ", StartAddress+i);
                }
                if (StartAddress >= 0x20 && StartAddress <= 0x23)
                {
                    printf("%02x ", data[i]);
                }
                else
                {
                    printf("%04x ", data[i]);
                }
                if (((i+1) % 16) == 0  && i != (count-1))
                {
                    printf("\n");
                }
            }
            printf("\n");
        }
        delete pBBUDAPI;
    }

    return err;
}



int
BBUOperation::on(StringParser command_line)
{
    int      err = 0;
    BBUDAPI  *pBBUDAPI = NULL;
    ErrorMsg *error_code;
    string   WellKnownName;


    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get I2C device name
        command_line.getParameter(&WellKnownName, 0);
        //cout << "Name: " <<  WellKnownName << endl;

        //Creat BBU DAPI object
        pBBUDAPI = new BBUDAPI("I2CDAPI");

        //Get BBU information by API
        error_code = pBBUDAPI->Charge(WellKnownName);

        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            err = -1;
            return err;
        }
        cout << "Status: On" << endl;

        delete pBBUDAPI;
    }

    return err;
}



int
BBUOperation::off(StringParser command_line)
{
    int      err = 0;
    BBUDAPI  *pBBUDAPI = NULL;
    ErrorMsg *error_code;
    string   WellKnownName;


    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get I2C device name
        command_line.getParameter(&WellKnownName, 0);
        //cout << "Name: " <<  WellKnownName << endl;

        //Creat BBU DAPI object
        pBBUDAPI = new BBUDAPI("I2CDAPI");

        //Get BBU information by API
        error_code = pBBUDAPI->Discharge(WellKnownName);

        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            err = -1;
            return err;
        }
        cout << "Status: Off" << endl;

        delete pBBUDAPI;
    }

    return err;
}



int
BBUOperation::showCharge(StringParser command_line)
{
    int      err = 0, status = 0;
    BBUDAPI  *pBBUDAPI = NULL;
    ErrorMsg *error_code;
    string   WellKnownName;


    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get I2C device name
        command_line.getParameter(&WellKnownName, 0);
        //cout << "Name: " <<  WellKnownName << endl;

        //Creat BBU DAPI object
        pBBUDAPI = new BBUDAPI("I2CDAPI");

        //Get BBU charge status by API
        error_code = pBBUDAPI->GetChargeStatus(WellKnownName, &status);

        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            err = -1;
            return err;
        }

        if (status == 0) //Charge turn on
        {
            cout << "Status: On" << endl;
        }
        if (status == 1) //Charge turn off
        {
            cout << "Status: Off" << endl;
        }

        delete pBBUDAPI;
    }

    return err;
}



int
BBUOperation::Syntax_check(StringParser command_line)
{
    int    i, hex[7],parameter_number = 0;
    string parameter[7];
    int    err = 0;
    string operation_name;

    getName(&operation_name);

    command_line.getParameterNumber(&parameter_number);

    //Get parameter
    for (i=0; i<parameter_number; i++)
    {
        command_line.getParameter(&parameter[i], i);
    }

    switch (getOperationType())
    {
        case OPERATION_READ:
        case OPERATION_WRITE:

            //Check parameter number
            if (parameter_number != 3)
            {
                cout << "Parameter numbers isn't 3" << endl;
                return -1;
            }

            //Check parameter 1 is numeric
            err = command_line.isHEXString(parameter[0]);
            if (err != 0)
            {
                cout << "parameter 1 isn't numeric." << endl;
                return -1;
            }

            //Check parameter 2 is numeric
            err = command_line.isHEXString(parameter[1]);
            if (err != 0)
            {
                cout << "parameter 2 isn't numeric." << endl;
                return -1;
            }

            //For debug, show input value
            //cout << "P1: " << parameter[0] << endl;
            //cout << "P2: " << hex[0] << endl;
            //cout << "P3: " << hex[1] << endl;
            break;
        case OPERATION_MONITOR:
        case OPERATION_ON:
        case OPERATION_OFF:
        case OPERATION_CHARGE:
            //Check parameter number
            if (parameter_number != 1)
            {
                cout << "Parameter numbers isn't 1" << endl;
                return -1;
            }
            break;
       default:
            cout << "Not define syntax_check for " << operation_name << endl;
            err = -1;
            break;
    }

    return err;
}
