#ifndef NVMe_OPERATION_H
#define NVMe_OPERATION_H

#include <stdio.h>
#include <iostream>
#include <string>
#include <thread>

#include "OperationBaseClass.hpp"
#include "NVMeDAPI.hpp"
#include "StorageDAPI.hpp"
#include "NVMePrint.hpp"

enum {
    TERSE = 0x1u,   // only show a few useful fields
    HUMAN = 0x2u,   // interpret some values for humans
    VS    = 0x4u,   // print vendor specific data area
    RAW   = 0x8u,   // just dump raw bytes
};

class NVMeOperation: public OperationBaseClass
{
    public:
        NVMeOperation(OPERATION_TYPE, string);
        virtual ~NVMeOperation();

        //Decide which operation to be executed
        int executeOperation(StringParser);

        //List NVMe device node
        int executeListNVMe(StringParser);

        //Show NVMe controller characteristics
        int executeShowController(StringParser);

        //Execute Error command
        INT32 executeError(StringParser);

        //Execute Utility command
        INT32 executeUtility(StringParser);

        //Execute Utility command
        INT32 executeActivateFW(StringParser);

        //Execute Utility command
        INT32 executeDownloadFW(StringParser);

        //Execute test command
        int executeTest(StringParser);

        //Show parameter message here
        int helpParameter();

        //Check parameter syntax here
        int checkSyntax(StringParser);

    private:
        OPERATION_TYPE operationType;
        NVMeDAPI* nvmeDAPI;
        StorageDAPI* storageDAPI;
        thread* progressHandler;
        void handleTestProgress(UINT32 in_u32TestSizeMB);
        // Format integer with thousands separator
        const char *format_with_thousands_sep(char * str, int strsize, uint64_t val, const char * thousands_sep = 0);
        // Format capacity with SI prefixes
        const char *format_capacity(char * str, int strsize, uint64_t val, const char * decimal_point = 0);

};

#endif
