#include "SATACommand.hpp"

SATACommand::SATACommand(COMMAND_TYPE CommandType, string strHelpContent) : CommandBaseClass(CommandType, strHelpContent)
{
     //Base on command type to add different operations
     switch (CommandType)
     {
     case CMD_LIST:
	  addOperation(new SATAOperation(OPERATION_CONTROLLER, "Controller Name."));
//	  addOperation(new SATAOperation(OPERATION_PORT, "Port Name."));
	  addOperation(new SATAOperation(OPERATION_DRIVE, "Controller-Drive Name."));
	  break;
     case CMD_SHOW:
	  addOperation(new SATAOperation(OPERATION_ALL,      "Display All Information.            (Controller)"));
	  addOperation(new SATAOperation(OPERATION_ID,       "Display ID.                         (Controller)  "));
	  addOperation(new SATAOperation(OPERATION_BDF,      "Display Bus:Device.Function.        (Controller) "));
	  addOperation(new SATAOperation(OPERATION_PORT,     "Display Port Amount or Port Number. (Controller | Drive)"));
	  addOperation(new SATAOperation(OPERATION_NODE,     "Display Device Node Name.           (Drive)"));
	  addOperation(new SATAOperation(OPERATION_SN,       "Display Serial Number.              (Drive)"));
	  addOperation(new SATAOperation(OPERATION_MODEL,    "Display Model Number.               (Drive)"));
	  addOperation(new SATAOperation(OPERATION_FW,       "Display Firmware Version.           (Drive)"));
	  addOperation(new SATAOperation(OPERATION_CAPACITY, "Display Capacity.                   (Drive)"));
	  addOperation(new SATAOperation(OPERATION_PORT,     "Display Port Number.                (Drive)"));
//	  addOperation(new SATAOperation(OPERATION_LINK,     "Display Link Status                 (Drive)"));
	  addOperation(new SATAOperation(OPERATION_SPEED,    "Display Link Speed.                 (Drive)"));
	  addOperation(new SATAOperation(OPERATION_TEMP,     "Display Temperature.                (Drive)"));
	  break;
     case CMD_TEST:
	  addOperation(new SATAOperation(OPERATION_IDENTIFY, "SATA Identify Command Test"));
	  addOperation(new SATAOperation(OPERATION_READPFMC, "SATA Read Performance Test"));
	  addOperation(new SATAOperation(OPERATION_WRITEPFMC, "SATA Write Performance Test"));
	  addOperation(new SATAOperation(OPERATION_WRC, "SATA Read/Write Test"));
	  break;
     case CMD_ERROR:
	  addOperation(new SATAOperation(OPERATION_ALL, "Get All SATA Error statistics"));
	  addOperation(new SATAOperation(OPERATION_TRANSPORT_STATE_TRANSITION_ERR, "Get SATA Transport State Transition Error Register"));
	  addOperation(new SATAOperation(OPERATION_LINK_SEQUENCE_ERR, "Get SATA Link Sequence Error Register"));
	  addOperation(new SATAOperation(OPERATION_HANDSHAKE_ERR, "Get SATA Handshake Error Register"));
	  addOperation(new SATAOperation(OPERATION_CRC_ERR, "Get SATA CRC Error Register"));
          addOperation(new SATAOperation(OPERATION_DISPARITY_ERR, "Get SATA Disparity Error Register"));
          addOperation(new SATAOperation(OPERATION_10B_TO_8B_DECODE_ERR, "Get SATA 10b to 8b Decode Error Register"));
	  addOperation(new SATAOperation(OPERATION_PHY_INTERNAL_ERR, "Get SATA PHY Internal Error Register"));
	  addOperation(new SATAOperation(OPERATION_INTERNAL_ERR, "Get SATA Internal Error Register"));
          addOperation(new SATAOperation(OPERATION_PROTOCOL_ERR, "Get SATA Protocol Error Register"));
          addOperation(new SATAOperation(OPERATION_PERSISTENT_COMMUNICATION_OR_DATA_INTEGRITY_ERR, "Get SATA Persistent Communication or Data Integrity Error Register"));
          addOperation(new SATAOperation(OPERATION_TRANSIENT_DATA_INTEGRITY_ERR, "Get SATA Transient Data Integrity Error Register"));
          addOperation(new SATAOperation(OPERATION_RECOVER_COMMUNICATIONS_ERR, "Get SATA Recover Communications Error Register"));
          addOperation(new SATAOperation(OPERATION_RECOVER_DATA_INTREGRITY_ERR, "Get SATA Recover Data Intregrity Error Register"));
	  break;
     case CMD_UTILITY:
          addOperation(new SATAOperation(OPERATION_SMART, "Get SATA Drive SMART Information"));
	  break;
	  //Add NEW command and operation here
     default:
	  break;
     }
}

SATACommand::~SATACommand()
{

}

INT32 SATACommand::executeCommand(StringParser CommandLine)
{
     INT32 s32Err = 0;
     string strOperationName = "";

     string strParserstrOperationName = "";    
     CommandLine.getOperation(&strParserstrOperationName);

     COMMAND_TYPE eCommandType = (COMMAND_TYPE)0;
     CommandLine.getCommandType(&eCommandType);
     switch(eCommandType)
     {
     case CMD_LIST:
     case CMD_SHOW:
     case CMD_TEST:
     case CMD_ERROR:
     case CMD_UTILITY:
	  if(strParserstrOperationName=="help" || strParserstrOperationName=="")
	  {
	       listOperation(CommandLine);
	       return -1;
	  }
	  break;
     default:
	  cout << "Not supported command" << endl;
	  return -1;
	  break;
     }

     INT32 s32OperationIndex = 0;
     for(s32OperationIndex=0; s32OperationIndex<m_operation_count; s32OperationIndex++)
     {
	  m_operation[s32OperationIndex]->getName(&strOperationName);
	  //printf("m_operation[%d]=%s\n", s32OperationIndex, strOperationName.c_str());
	  if (strOperationName == strParserstrOperationName)
	  {
	       s32Err = m_operation[s32OperationIndex]->executeOperation(CommandLine);
	       break;
	  }
     }

     if(s32OperationIndex == m_operation_count)
     {
	  cout << strParserstrOperationName << " operation not found" << endl;
	  return -1;
     }

     return s32Err;
}
