/*******************************************************************************

NAME			$RCSfile: commRpt.c,v $
SUMMARY			Report mechanism for PikesPeak
VERSION			$Revision: 1.6 $
UPDATE DATE		$Date: 2010/07/06 09:25:27 $
PROGRAMMER		$Author: alan $

		Copyright 2009 LSI Corporation. All Rights Reserved.

DESCRIPTION:
    The functions used to report diagnostics status

REFERENCE:

*******************************************************************************/

/***  INCLUDES  ***/
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/proc_fs.h>
#include <asm/uaccess.h>
#include <linux/kallsyms.h>
#include <linux/module.h>

#include <commTypes.h>

#include "diag_proc.h"

//#if defined(CHASSIS_WSAS)
//#include "wemblySAS.h"
//#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,30)
static DECLARE_MUTEX(call_and_get_mutex);
#else
static DEFINE_SEMAPHORE(call_and_get_mutex);
#endif


static UINT64 g_return_last_request_value=0;
/***************/
/* proc system */
/***************/

#define CMD_BUFFER_SIZE (MAX_FUNC_NAME_SIZE+MAX_FUNC_PARM_SIZE)
void (*func_0) (void);
void (*func_1) (unsigned long);
void (*func_2) (unsigned long, unsigned long);
void (*func_3) (unsigned long, unsigned long, unsigned long);
void (*func_4) (unsigned long, unsigned long, unsigned long, unsigned long);
void (*func_5) (unsigned long, unsigned long, unsigned long, unsigned long, unsigned long);
void (*func_a) (char *);	// ALANWANG, 2010/07/01
#if defined(CHASSIS_WSAS)
void (*func_dcm) (unsigned short, char*);
#endif


int gdiagProcRetrunValue = 0;
EXPORT_SYMBOL(gdiagProcRetrunValue);

int diagProcRead(char *page, char **start, off_t off,
			  int count, int *eof, void *data)
{
	printk("Call Kernel Function:\n");
	printk("Usage:\necho \"FUNCTION [ARG1] [ARG2] [ARG3] [ARG4] [ARG5]\" > /proc/diag\n");
	printk("Use < > to quote a string\n");
	printk("Convert signed int to UINT32. e.g. -1=0xFFFFFFFF\n");
	printk("Example: echo \"printk <%%s %%d 0x%%x\\n> <Hello> 0x10 32\" > /proc/diag\n\n");

	return (0);
}

int diagProcWrite(struct file *file, const char __user *buf,
			   unsigned long length, void *data)
{
	int cmd_len;
	char *arg_ptr, *next_arg;
	char cmd_buf[CMD_BUFFER_SIZE] = "";//command buffer (user input string)
	char func_name[MAX_FUNC_NAME_SIZE] = ""; //function name buffer
	char func_parm[MAX_FUNC_PARM_SIZE] = ""; //function parameter buffer
	UINT64 func_addr; //target debug function's address
#if (defined(inventec) | defined(CHASSIS_WSAS))
    unsigned short dcmarg1 = 0;
    char *dcmarg2 = NULL;
#endif
#ifdef MEASURE_EXECUTION_TIME
	unsigned int start_jiffies;
	unsigned int exec_jiffies;
#endif


	int n = 0; //the length of function name
	int arg_idx = 0;
	unsigned long args[MAX_ARG_NUM];


	gdiagProcRetrunValue=0;

	//error if user didn't send data or send to long data
	if (!buf || (length > (CMD_BUFFER_SIZE))) { 
		return -EINVAL;
	}

	//get data from user and put it to cmd_buf
	if (copy_from_user(cmd_buf, buf, length) ) {
		return -EFAULT;
	}
	cmd_len=length-1;
	cmd_buf[cmd_len]='\0';
	//get function name
	arg_ptr=cmd_buf;
	next_arg=cmd_buf;
	arg_ptr=strsep(&next_arg,", ()");
	
	if (next_arg == NULL) {
	 	strncpy(func_name, arg_ptr, cmd_len);
	} else {
		n = next_arg - arg_ptr;
		strncpy(func_name, arg_ptr, n);
		func_name[n] = '\0';
		strncpy(func_parm, next_arg, cmd_len-n);
	}

	// get function addr
	func_addr = kallsyms_lookup_name(func_name);
	if (func_addr == 0) {
		printk("[PROC_DIAG] function <%s> not found!\n", func_name);
		return -EFAULT;
	}
#ifdef MEASURE_EXECUTION_TIME
	start_jiffies = jiffies;
#endif
#ifndef ALANWANG
	// Is it a ssFun, Added by Alan Wang, 2010/07/01
	if (strncmp (cmd_buf, "ssFun", strlen("ssFun")) == 0){
		func_a = (void*) func_addr;
//		printk ( KERN_EMERG "ARG:'%s' '%s'\n", arg_ptr, next_arg);
		(*func_a)(next_arg);
		goto result;
		//return length;
	}
#endif
	// parsing arguments
	arg_idx=0;
	arg_ptr=func_parm;
	next_arg=func_parm;

	do {
		if (arg_ptr[0] == '<') {
			next_arg=arg_ptr;
			do {
                if ((next_arg[0]=='\\') && (next_arg[1]=='n')) {
					next_arg[0]=' ';
					next_arg[1]='\n';
					next_arg++;
				}	
				next_arg++;
			} while ((*next_arg != '\0') && (*next_arg != '>'));
			if (*next_arg == '>') {
				*next_arg = '\0';
				next_arg++;
			}
			else {
				printk("[PROC_DIAG] can not find matching '>'!\n");
				return -EFAULT;
			}
		}
		else {
			arg_ptr=strsep(&next_arg,", ()");
		}
		if (arg_ptr != NULL) {
			if (arg_ptr[0]!='\0') {
				if (!strncmp("<", arg_ptr, 1)) {
					args[arg_idx] = (unsigned long) arg_ptr+1;
				}
				else if (!strncmp("0x", arg_ptr, 2)) {
					args[arg_idx] = simple_strtoul(arg_ptr, NULL, 16);
				}

#if defined(CHASSIS_WSAS)
                else if((arg_idx == 0) && (!strncmp("dcmfwd",func_name, 6))){
                    dcmarg1 = (unsigned short)simple_strtoul(arg_ptr, NULL, 10);
                    // use a symbol that won't be used as delimiter 
                    arg_ptr = strsep(&next_arg,":");
                    if(arg_ptr == NULL){
                        printk("dbg: NULL for delimiter \"\n");
                    }
                    dcmarg2 = arg_ptr;
                    arg_idx = 2;
                    break;
                }
#endif

				else {
					args[arg_idx] = simple_strtoul(arg_ptr, NULL, 10);
				}
				arg_idx++;
			}
			arg_ptr=next_arg;
		}
		else break;
	} while ((arg_ptr != NULL) && (arg_idx < MAX_ARG_NUM));

	// call the function
	switch (arg_idx) {
	case 0:
		func_0 = (void*)func_addr;
		(*func_0)();
		break;
	case 1:
		func_1 = (void*)func_addr;
		(*func_1)(args[0]);
		break;
	case 2:
#if defined(CHASSIS_WSAS)
        if(!strncmp("dcmfwd",func_name, 6)){
            func_dcm = (void*)func_addr;
            (*func_dcm)(dcmarg1, dcmarg2);
        }
        else{
            func_2 = (void*)func_addr;
            (*func_2)(args[0],args[1]);
        }
#else
		func_2 = (void*)func_addr;
		(*func_2)(args[0],args[1]);
#endif
		break;
	case 3:
		func_3 = (void*)func_addr;
		(*func_3)(args[0],args[1],args[2]);
		break;
	case 4:
		func_4 = (void*)func_addr;
		(*func_4)(args[0],args[1],args[2],args[3]);
		break;
	case 5:
		func_5 = (void*)func_addr;
		(*func_5)(args[0],args[1],args[2],args[3],args[4]);
		break;
	default:
		printk("[PROC_DIAG] function  not supported!\n");
		break;
	}

result:

#ifdef MEASURE_EXECUTION_TIME
	exec_jiffies = jiffies - start_jiffies;
	printk("Execution time: [%d] sec.\n", exec_jiffies/HZ);
#endif
	if(gdiagProcRetrunValue) return -EFAULT;
	return length;	
}

/**********************/
/* /proc/setparam-dbg */
/**********************/
int diagParamRead
 (
 char 	*buf, 
 char	**start, 
 off_t 	offset,
 int 	length,
 int 	*eof, 
 void 	*data
 )
{
	int nbytes=0;
	nbytes += sprintf(buf+nbytes, "%d", (int)g_return_last_request_value);
	up(&call_and_get_mutex);
//	init_MUTEX(&call_and_get_mutex);//This function will be called several times // Alan Wang, Unresonable!!!
	return nbytes;
}


int diagParamWrite
 (
 struct file 	*file, 
 const char 	*buf, 
 unsigned long 	length, 
 void 			*data
 )
{
	int cmd_len;
	char *arg_ptr, *next_arg; //, *cmd_ptr
	char cmd_buf[CMD_BUFFER_SIZE] = "";//command buffer (user input string)
	char func_name[MAX_FUNC_NAME_SIZE] = ""; //function name buffer
	char func_parm[MAX_FUNC_PARM_SIZE] = ""; //function parameter buffer
	UINT64 func_addr; //target debug function's address
	UINT8 i=0;
	UINT8 firstHexP[2]={0}, secondHexP[2]={0};

	int n = 0; //the length of function name

	//error if user didn't send data or send to long data
	if (!buf || (length > (CMD_BUFFER_SIZE))) { 
		return -EINVAL;
	}

	//get data from user and put it to cmd_buf
	if (copy_from_user(cmd_buf, buf, length) ) {
		return -EFAULT;
	}
	cmd_len=length-1;
	cmd_buf[cmd_len]='\0';

//	printk("[DEBUG] set parameter from user: %s\n", cmd_buf);
	//get function name
	arg_ptr=cmd_buf;
	next_arg=cmd_buf;
	arg_ptr=strsep(&next_arg,"=");
	
	if (next_arg == NULL) {
	 	strncpy(func_name, arg_ptr, cmd_len);
	} else {
		n = next_arg - arg_ptr;
		strncpy(func_name, arg_ptr, n);
		func_name[n] = '\0';
		strncpy(func_parm, next_arg, cmd_len-n);
	}

	{//Tricky, To avoid same parameter
		if((func_name[strlen(func_name)-2]=='Q') && ((func_name[strlen(func_name)-1]<='9')||func_name[strlen(func_name)-1]>='1'))
			func_name[strlen(func_name)-2] = '\0';
//			printk("The func name is %s\n",func_name);
	}

	// get function addr
	func_addr = kallsyms_lookup_name(func_name); //diff
//	printk("[DEBUG] func_name=<%s> func_addr=0x%08lx parm=%s\n", func_name, func_addr, func_parm);
	if (func_addr==0) { //diff
		printk("[PROC_DBG] function <%s> not found!\n", func_name);
		return -EFAULT;
	}

	if(func_parm[0] == '?'){ //Pattern
		if(func_parm[1] <= 'F' && func_parm[1]>='1'){
			int show_loop = simple_strtoul(&func_parm[1], NULL, 16);
			printk("%s:\n", func_name);
			while(show_loop--){
				printk("\t0X%08X\n",  *(UINT8 *)func_addr);
				func_addr += 4;
			}
		}
		else{
//			printk("%s = 0X%08X\n", func_name, (unsigned int)*(UINT32 *)(func_addr));
			down(&call_and_get_mutex);
			g_return_last_request_value = *(UINT8 *)(func_addr);
		}
	}else if(func_parm[0] == 'A'){ //Append pattern
		int offset=0;
		unsigned char offset_char[3]={0};
		strncpy(offset_char, func_parm+1, 2);
		offset=simple_strtoul(offset_char, 0, 0);
//	printk("value is %s\n", func_parm);
		for(i=5; i<strlen(func_parm);i=i+2){
			firstHexP[0] = func_parm[i];
			secondHexP[0] = func_parm[i+1];
			*(UINT8 *)(func_addr + offset + i/2 - 2) = simple_strtoul(firstHexP, NULL, 16)<<4 | simple_strtoul(secondHexP, NULL, 16);
		}
	}else if(func_parm[0] == '0' && func_parm[1]=='x'){ //Pattern
//		printk("Input pattern:%s, strlen=%d\n", func_parm, strlen(func_parm));
		//Note: Doesn't check its parameter maximum length
		for(i=2; i<strlen(func_parm);i=i+2){
			firstHexP[0] = func_parm[i];
			secondHexP[0] = func_parm[i+1];
			*(UINT8 *)(func_addr + i/2 - 1) = simple_strtoul(firstHexP, NULL, 16)<<4 | simple_strtoul(secondHexP, NULL, 16);
//			printk("i=%d: %02X\n", i, *(UINT8 *)(func_addr + i/2 - 1) ); 
		}
	}else	
		*(UINT8 *)func_addr = simple_strtoul(func_parm, NULL, 10);
	return length;	
}


