/****************************************************************************
*                                                                           *
* Copyright 2004-2011 NetApp Inc. All rights reserved. This file is         *
* confidential and a trade secret of NetApp Inc. The receipt or             *
* possession of this file does not convey any rights to reproduce or        *
* disclose its contents or to manufacture, use, or sell anything it may     *
* describe, in whole, or in part, without the specific written consent of   *
* LSI Corporation.                                                          *
*                                                                           *
****************************************************************************/
/****************************************************************************
*                                                                           *
* NAME            base-zebulon.c                                            *
* SUMMARY                                                                   *
* VERSION         %version: %                                               *
* UPDATE DATE     %date_modified: %                                         *
* PROGRAMMER      %created_by:    Merck Hung %                              *
*                                                                           *
*        Copyright 2002-2011 NetApp Inc. All Rights Reserved.               *
*                                                                           *
* DESCRIPTION:                                                              *
*                                                                           *
****************************************************************************/

#include <linux/kobject.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/fcntl.h>
#include <linux/device.h>

#include <linux/pci.h>
#include <linux/wait.h>
#include <linux/interrupt.h>
#include <linux/platform_device.h>
#include <linux/ioctl.h>
#include <linux/types.h>
#include <linux/fs.h>
#include "commTypes.h"
#include "fpgaRegLib.h"
#include "fpgaCmdLib.h"

#include "pci-zebulon.h"
#include "fpga-subfuncs.h"
#include "midplane.h"


static volatile void __iomem *mmioVirtual = 0;
static volatile void __iomem *nvsramVirtual = 0;
extern struct kobject *firmware_kobj;
static struct kobject *pFpgaKobj = NULL;


EXTERN VOID fpgaInit( VOID *mmioVirtual, VOID *nvsramVirtual );
EXTERN VOID fpgaEppInit(VOID);
EXTERN VOID fpgaSas2x36PhyEnable(BOOLEAN status);
EXTERN VOID fpgaEnableQuickSwitch(BOOLEAN status);
extern void sslock_Init( void );


// #########################
unsigned int ssMem_TotalRAMMB=0;
unsigned int ssMem_TestStartMB=0;

void sslock_mem_Init(void){
#if 0
    ssMem_TotalRAMMB = gMemDiagRegionSize >> 20;
#else
    ssMem_TotalRAMMB = 8192;
#endif

    if(ssMem_TotalRAMMB > 4096)
        ssMem_TestStartMB=4096;
    else
        ssMem_TestStartMB=2048;

    ssMem_TotalRAMMB -= ssMem_TestStartMB;
}
// #########################


volatile void __iomem *getFpgaMmioVirtual( void ) {

	return mmioVirtual;
}
EXPORT_SYMBOL(getFpgaMmioVirtual);


volatile void __iomem *getFpgaNvsramVirtual( void ) {

    return nvsramVirtual;
}
EXPORT_SYMBOL(getFpgaNvsramVirtual);


static struct pci_device_id fpgaZebulonIds[] = {

	{ PCI_DEVICE( FPGA_ZEBULON_VENID, FPGA_ZEBULON_DEVID ) },
	{ PCI_DEVICE( NETAPP_VENDOR_ID, FPGA_ZEBULON_DEVID ) },
    {}
};


static void fpgaZebulonUpdateResource( 
	unsigned long baseAddr, unsigned int irqNum,
	struct platform_device **pdev, int num ) {

	int i, j;
	struct platform_device *xpdev;

	for( i = 0 ; i < num ; i++ ) {

		xpdev = pdev[ i ];
		for( j = 0 ; j < xpdev->num_resources ; j++ ) {
			switch( j ) {

				case 0: // Base address
					(xpdev->resource[ j ]).start += baseAddr;
					(xpdev->resource[ j ]).end += baseAddr;
					break;

				case 1: // IRQ
					(xpdev->resource[ j ]).start = irqNum;
					(xpdev->resource[ j ]).end = irqNum;
					break;

				default: // More than expectation
					break;
			}
		}
	}
}


static void platform_del_devices( struct platform_device **devs, int num ) {

	for( ; num ; num-- )
		platform_device_unregister( devs[ num - 1 ] );
}

static ssize_t CtrlIdShow( 
	struct kobject *kobj,
	struct kobj_attribute *attr,
	char *buf ) {

	unsigned char id;

	id = fpgaRegisterRead( FPGA_BOARD_ID );
	return sprintf( buf, "0x%x", id & 0x1F );
}

static ssize_t CtrlRoleShow( 
	struct kobject *kobj,
	struct kobj_attribute *attr,
	char *buf ) {

#if 0
	unsigned char id;

	id = fpgaRegisterRead( FPGA_BOARD_ID );
	return sprintf( buf, "%d", (id & 0x80) ? 1 : 0 );
#else
	unsigned char role;
	
	role = fpgaCtlRoleRead();
	return sprintf( buf, "%d", role );
#endif

}

static ssize_t CtrlSlotShow( 
	struct kobject *kobj,
	struct kobj_attribute *attr,
	char *buf ) {

	unsigned char id;

	id = fpgaRegisterRead( FPGA_BOARD_ID );
	return sprintf( buf, "%d", (id & 0x80) ? 1 : 0 );

}

static ssize_t DriveInPlaceShow( 
	struct kobject *kobj,
	struct kobj_attribute *attr,
	char *buf ) {

	unsigned long status;
	unsigned char drive_in_place_0;
	unsigned char drive_in_place_1;
	unsigned char drive_in_place_2;

	drive_in_place_0 = fpgaRegisterRead( FPGA_DRIVE_INPLACE_0 );
	drive_in_place_1 = fpgaRegisterRead( FPGA_DRIVE_INPLACE_1 );
	drive_in_place_2 = fpgaRegisterRead( FPGA_DRIVE_INPLACE_2 );

	status = drive_in_place_0 | (drive_in_place_1<<8) | (drive_in_place_2<<16);

	return sprintf( buf, "0x%lx", status);
}

static ssize_t EnclosureIdShow(
    struct kobject *kobj,
    struct kobj_attribute *attr,
    char *buf ) {

    unsigned char id;

    id = fpgaRegisterRead( FPGA_SUBSYSTEM_ID ) & 0x0F;
    return sprintf( buf, "%d", (int)id );
}


//HIC IDs
#define INVALID_HOST_CARD       0x00
#define DEFAULT_NO_CARD         0x0F 	//No host card
#define ELK_PARK_ID_IB          0x04    //InfiniBand host card
#define ACADIA_QUAD             0x05
#define ACADIA_DUAL             0x02
#define ACADIA_PLUS_QUAD        0x0B
#define ACADIA_PLUS_DUAL        0x0A
#define SAMOA_QUAD              0x07
#define SAMOA_DUAL              0x01
#define SAMOA_PLUS_QUAD         0x0C
#define SAMOA_PLUS_DUAL         0x09
#define ZION                    0x06
#define ZION_PLUS               0x0D

static ssize_t HostCardIdShow(
    struct kobject *kobj,
    struct kobj_attribute *attr,
    char *buf ) {

    unsigned char id;

    id = fpgaRegisterRead( FPGA_HOST_BOARD_ID );
    return sprintf( buf, "0x%x", id );
}


typedef enum host_card_type {
	HOST_CARD_NONE = 0,
	HOST_CARD_FC,
	HOST_CARD_ISCSI,
	HOST_CARD_IB,
	HOST_CARD_SAS
} HOST_CARD_TYPE;


static ssize_t HostCardTypeShow(
    struct kobject *kobj,
    struct kobj_attribute *attr,
    char *buf ) {

    unsigned char id;
	int hicType = HOST_CARD_NONE;

	// Read Host Card ID
    id = fpgaRegisterRead( FPGA_HOST_BOARD_ID );
	switch( id ) {

	case SAMOA_PLUS_QUAD:
	case SAMOA_PLUS_DUAL:
		hicType = HOST_CARD_FC;
		break;

	case ZION:
	case ZION_PLUS:
		hicType = HOST_CARD_ISCSI;
		break;

	case ELK_PARK_ID_IB:
		hicType = HOST_CARD_IB;
		break;

	case ACADIA_QUAD:
	case ACADIA_DUAL:
	case ACADIA_PLUS_QUAD:
	case ACADIA_PLUS_DUAL:
		hicType = HOST_CARD_SAS;
		break;

	default:
		hicType = HOST_CARD_NONE;
		break;
	}

    return sprintf( buf, "%d", hicType );
}


unsigned int getNumOfDisks( unsigned char id ) {

	switch( id ) {

	case MIDPLANE_EBBETS:
		return 12;

	case MIDPLANE_CAMDEN:
		return 24;

	case MIDPLANE_WEMBLEY:
	case MIDPLANE_WEMBLEY_EVT:
		return 60;

	case MIDPLANE_TEST:
	case MIDPLANE_STANDARD:
	case MIDPLANE_UNKNOWN:
	default:
		break;
	}

	return 0;
}


static ssize_t NumOfDisksShow(
    struct kobject *kobj,
    struct kobj_attribute *attr,
    char *buf ) {

    unsigned char id;

    id = fpgaRegisterRead( FPGA_SUBSYSTEM_ID ) & 0x0F;
    return sprintf( buf, "%d", getNumOfDisks( id ) );
}


#define MPT_EXPANDER_CONFIG_BASE_ADDR               0xC0000000
#define MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET          0x44
#define MPT_EXPANDER_PHY_CONFIG_HIGH_OFFSET         0x48

#define MPT_EXPANDER_PHY_CONNECTED_TO_EXTERN_PORT_START     0
#define MPT_EXPANDER_PHY_CONNECTED_TO_EXTERN_PORT_END       3
#define MPT_EXPANDER_PHY_CONNECTED_TO_PEER_DEV_IOC_START    4
#define MPT_EXPANDER_PHY_CONNECTED_TO_PEER_DEV_IOC_END      7
#define MPT_EXPANDER_PHY_CONNECTED_TO_LOCAL_DEV_IOC_START   8
#define MPT_EXPANDER_PHY_CONNECTED_TO_LOCAL_DEV_IOC_END     11
#define MPT_EXPANDER_PHY_CONNECTED_TO_HDD_START         	12
#define MPT_EXPANDER_PHY_CONNECTED_TO_HDD_END           	35


extern UINT32 fpgaEppReadDword( UINT32 addr );
extern VOID fpgaEppWriteDword( UINT32 addr, UINT32 data );


static ssize_t ExpPhyCfgShow(
    struct kobject *kobj,
    struct kobj_attribute *attr,
    char *buf ) {

	unsigned int low, high;

	low = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET );
	high = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_HIGH_OFFSET );
	high &= 0x0F;

    return sprintf( buf, "%1.1X%8.8X", high, low );
}


static ssize_t ExpPhyCfgStore(
	struct kobject *kobj,
	struct kobj_attribute *attr,
	const char *buf,
	size_t count ) {

	unsigned int phy_num, org_value;

	phy_num = simple_strtoul( buf, NULL, 10 );
	if( phy_num > MPT_EXPANDER_PHY_CONNECTED_TO_HDD_END )
		return -EINVAL;

    if( phy_num <= 31 ) {

        org_value = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET );
        org_value |= (1 << phy_num);
        fpgaEppWriteDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET, org_value );

    }
	else {

        org_value = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_HIGH_OFFSET );
        org_value |= (1 << (phy_num - 32));
        fpgaEppWriteDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_HIGH_OFFSET, org_value );
    }

    return count;
}


static ssize_t ExpPhyEnableStore(
	struct kobject *kobj,
	struct kobj_attribute *attr,
	const char *buf,
	size_t count ) {

	unsigned int phy_num, org_value;

	phy_num = simple_strtoul( buf, NULL, 10 );
	if( phy_num > MPT_EXPANDER_PHY_CONNECTED_TO_HDD_END )
		return -EINVAL;

    if( phy_num <= 31 ) {

        org_value = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET );
        org_value |= (1 << phy_num);
        fpgaEppWriteDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET, org_value );

    }
	else {

        org_value = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_HIGH_OFFSET );
        org_value |= (1 << (phy_num - 32));
        fpgaEppWriteDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_HIGH_OFFSET, org_value );
    }

    return count;
}

static ssize_t ExpPhyDisableStore(
	struct kobject *kobj,
	struct kobj_attribute *attr,
	const char *buf,
	size_t count ) {

	unsigned int phy_num, org_value;

	phy_num = simple_strtoul( buf, NULL, 10 );
	if( phy_num > MPT_EXPANDER_PHY_CONNECTED_TO_HDD_END )
		return -EINVAL;

    if( phy_num <= 31 ) {

        org_value = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET );
        org_value &= (~(1 << phy_num));
        fpgaEppWriteDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET, org_value );

    }
	else {

        org_value = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_HIGH_OFFSET );
        org_value &= (~(1 << (phy_num - 32)));
        fpgaEppWriteDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_HIGH_OFFSET, org_value );
    }

    return count;
}

static ssize_t ExpPhyExpansionPortStore(
    struct kobject *kobj,
    struct kobj_attribute *attr,
    const char *buf,
    size_t count ) {

	unsigned int input, val;

	input = simple_strtoul( buf, NULL, 10 );
	if( input == 1 ) {

		val = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET );
		val |= 0x0000000F;
		fpgaEppWriteDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET, val );
	}
	else if( !input ) {

        val = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET );
        val &= ~0x0000000F;
        fpgaEppWriteDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET, val );
	}
	else
		return -EINVAL;

	return count;
}


static ssize_t ExpPhyRemotePortStore(
    struct kobject *kobj,
    struct kobj_attribute *attr,
    const char *buf,
    size_t count ) {

    unsigned int input, val;

    input = simple_strtoul( buf, NULL, 10 );
    if( input == 1 ) {

        val = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET );
        val |= 0x000000F0;
        fpgaEppWriteDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET, val );
    }
    else if( !input ) {

        val = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET );
        val &= ~0x000000F0;
        fpgaEppWriteDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET, val );
    }
    else
        return -EINVAL;

    return count;
}


static ssize_t ExpPhyLocalPortStore(
    struct kobject *kobj,
    struct kobj_attribute *attr,
    const char *buf,
    size_t count ) {

    unsigned int input, val;

    input = simple_strtoul( buf, NULL, 10 );
    if( input == 1 ) {

        val = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET );
        val |= 0x00000F00;
        fpgaEppWriteDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET, val );
    }
    else if( !input ) {

        val = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET );
        val &= ~0x00000F00;
        fpgaEppWriteDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET, val );
    }
    else
        return -EINVAL;

    return count;
}


static ssize_t ExpPhyHardDrivePortStore(
    struct kobject *kobj,
    struct kobj_attribute *attr,
    const char *buf,
    size_t count ) {

    unsigned int input, val;

    input = simple_strtoul( buf, NULL, 10 );
    if( input == 1 ) {

        val = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET );
        val |= 0xFFFFF000;
        fpgaEppWriteDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET, val );

		val = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_HIGH_OFFSET );
		val |= 0x0000000F;
		fpgaEppWriteDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_HIGH_OFFSET, val );
    }
    else if( !input ) {

        val = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET );
        val &= ~0xFFFFF000;
        fpgaEppWriteDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET, val );

        val = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_HIGH_OFFSET );
        val &= ~0x0000000F;
        fpgaEppWriteDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_HIGH_OFFSET, val );
    }
    else
        return -EINVAL;

    return count;
}


static ssize_t ExpPhyHardDriveMapStore(
    struct kobject *kobj,
    struct kobj_attribute *attr,
    const char *buf,
    size_t count ) {

    unsigned int input, val, high, low;

    input = simple_strtoul( buf, NULL, 16 );
	high = (input >> 20) & 0x0000000F;
	low = (input << 12) & 0xFFFFF000;

    val = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET );
    val &= 0x00000FFF;
	val |= low;
    fpgaEppWriteDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET, val );

    val = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_HIGH_OFFSET );
    val &= 0xFFFFFFF0;
    val |= high;
    fpgaEppWriteDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_HIGH_OFFSET, val );

	return count;
}


static ssize_t ExpPhySasCtlMapStore(
    struct kobject *kobj,
    struct kobj_attribute *attr,
    const char *buf,
    size_t count ) {

    unsigned int input, val, low;

    input = simple_strtoul( buf, NULL, 16 );
	low = input & 0x00000FFF;

    val = fpgaEppReadDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET );
    val &= 0xFFFFF000;
	val |= low;
    fpgaEppWriteDword( MPT_EXPANDER_CONFIG_BASE_ADDR + MPT_EXPANDER_PHY_CONFIG_LOW_OFFSET, val );

	return count;
}


static ssize_t Sas2x36PhyCfgStore(
    struct kobject *kobj,
    struct kobj_attribute *attr,
    const char *buf,
    size_t count ) {

    unsigned int val;

    // Convert value
    val = simple_strtoul( buf, NULL, 10 );
    if( val != 0 && val != 1 )
        return -EINVAL;

    // Config register bit
    if( val )
        fpgaSas2x36PhyEnable( 1 );
    else
        fpgaSas2x36PhyEnable( 0 );

    return count;
}


VOID fpgaSas2x36Reset( BOOLEAN status );
static ssize_t Sas2x36ResetStore(
    struct kobject *kobj,
    struct kobj_attribute *attr,
    const char *buf,
    size_t count ) {

	unsigned int val;

	// Convert value
	val = simple_strtoul( buf, NULL, 10 );
	if( val != 0 && val != 1 )
		return -EINVAL;

	// Config register bit
	if( val )
		fpgaSas2x36Reset( 1 );
	else
		fpgaSas2x36Reset( 0 );

    return count;
}


static void enableDCM( void ) {

    fpgaLedAssign( 0x01, 1 );
    fpgaLedAssign( 0x04, 1 );
    fpgaLedAssign( 0x07, 1 );
    fpgaLedAssign( 0x0A, 1 );
    fpgaLedAssign( 0x0D, 1 );
}


static void disableDCM( void ) {

    fpgaLedAssign( 0x01, 0 );
    fpgaLedAssign( 0x04, 0 );
    fpgaLedAssign( 0x07, 0 );
    fpgaLedAssign( 0x0A, 0 );
    fpgaLedAssign( 0x0D, 0 );
}


static ssize_t DcmEnableStore(
    struct kobject *kobj,
    struct kobj_attribute *attr,
    const char *buf,
    size_t count ) {

	unsigned int val;

	// Convert value
	val = simple_strtoul( buf, NULL, 10 );
	if( val != 0 && val != 1 )
		return -EINVAL;

	// Config register bit
	if( val )
		enableDCM();
	else
		disableDCM();

    return count;
}


static VOID fpgaShutOffPs1
 (
 BOOLEAN status
 )
{
    UINT8   temp;

    //release disable bit
    temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_2);
    if(status)
    {
        temp |= FPGA_SHUT_OFF_PS_BIT;
    }
    else
    {
        temp &= (~FPGA_SHUT_OFF_PS_BIT);
    }
    fpgaEncodedRegisterWrite();
    fpgaRegisterWrite(FPGA_PROTECTED_WRITE_2, temp);
}


static ssize_t ShutOffPsuStore(
    struct kobject *kobj,
    struct kobj_attribute *attr,
    const char *buf,
    size_t count ) {

	// Shut off PSU
	fpgaShutOffPs1( 1 );
    return count;
}


static struct kobj_attribute DriveInPlace_attribute =
	__ATTR(DriveInPlace, 0440, DriveInPlaceShow, NULL);
static struct kobj_attribute CtrlId_attribute =
	__ATTR(CtrlId, 0440, CtrlIdShow, NULL);
static struct kobj_attribute CtrlRole_attribute =
	__ATTR(CtrlRole, 0440, CtrlRoleShow, NULL);
static struct kobj_attribute CtrlSlot_attribute =
	__ATTR(CtrlSlot, 0440, CtrlSlotShow, NULL);
static struct kobj_attribute EnclosureId_attribute =
	__ATTR(EnclosureId, 0440, EnclosureIdShow, NULL);
static struct kobj_attribute HostCardType_attribute =
	__ATTR(HostCardType, 0440, HostCardTypeShow, NULL);
static struct kobj_attribute HostCardId_attribute =
    __ATTR(HostCardId, 0440, HostCardIdShow, NULL);
static struct kobj_attribute NumOfDisks_attribute =
    __ATTR(NumOfDisks, 0440, NumOfDisksShow, NULL);
static struct kobj_attribute ExpPhyCfg_attribute =
	__ATTR(ExpPhyCfg, 0660, ExpPhyCfgShow, ExpPhyCfgStore);
static struct kobj_attribute ExpPhyEnable_attribute =
	__ATTR(ExpPhyEnable, 0220, NULL, ExpPhyEnableStore);
static struct kobj_attribute ExpPhyDisable_attribute =
	__ATTR(ExpPhyDisable, 0220, NULL, ExpPhyDisableStore);
static struct kobj_attribute ExpPhyExpansionPort_attribute =
	__ATTR(ExpPhyExpansionPort, 0220, NULL, ExpPhyExpansionPortStore);
static struct kobj_attribute ExpPhyRemotePort_attribute =
    __ATTR(ExpPhyRemotePort, 0220, NULL, ExpPhyRemotePortStore);
static struct kobj_attribute ExpPhyLocalPort_attribute =
	__ATTR(ExpPhyLocalPort, 0220, NULL, ExpPhyLocalPortStore);
static struct kobj_attribute ExpPhyHardDrivePort_attribute =
    __ATTR(ExpPhyHardDrivePort, 0220, NULL, ExpPhyHardDrivePortStore);
static struct kobj_attribute ExpPhySasCtlMap_attribute =
    __ATTR(ExpPhySasCtlMap, 0220, NULL, ExpPhySasCtlMapStore);
static struct kobj_attribute ExpPhyHardDriveMap_attribute =
    __ATTR(ExpPhyHardDriveMap, 0220, NULL, ExpPhyHardDriveMapStore);
static struct kobj_attribute Sas2x36PhyCfg_attribute =
	__ATTR(Sas2x36PhyCfg, 0220, NULL, Sas2x36PhyCfgStore);
static struct kobj_attribute Sas2x36Reset_attribute =
	__ATTR(Sas2x36Reset, 0220, NULL, Sas2x36ResetStore);
static struct kobj_attribute DcmEnable_attribute =
    __ATTR(DcmEnable, 0220, NULL, DcmEnableStore);
static struct kobj_attribute ShutOffPsu_attribute =
	__ATTR(ShutOffPsu, 0220, NULL, ShutOffPsuStore);

static struct attribute *fpgaZebulonAttrs[] = {
	&DriveInPlace_attribute.attr,
	&CtrlId_attribute.attr,
	&CtrlRole_attribute.attr,
	&CtrlSlot_attribute.attr,
	&EnclosureId_attribute.attr,
	&HostCardType_attribute.attr,
	&HostCardId_attribute.attr,
	&NumOfDisks_attribute.attr,
	&ExpPhyCfg_attribute.attr,
	&ExpPhyEnable_attribute.attr,
	&ExpPhyDisable_attribute.attr,
	&ExpPhyExpansionPort_attribute.attr,
	&ExpPhyRemotePort_attribute.attr,
	&ExpPhyLocalPort_attribute.attr,
	&ExpPhyHardDrivePort_attribute.attr,
	&ExpPhySasCtlMap_attribute.attr,
	&ExpPhyHardDriveMap_attribute.attr,
	&Sas2x36PhyCfg_attribute.attr,
	&Sas2x36Reset_attribute.attr,
	&DcmEnable_attribute.attr,
	&ShutOffPsu_attribute.attr,
	NULL,
};
static struct attribute_group fpgaZebulonAttrGroup = {
	.attrs = fpgaZebulonAttrs,
};


static int fpgaZebulonProbe(
	struct pci_dev *pdev, const struct pci_device_id *id ) {

	int err, ret;
	unsigned long fpgaMemBase;
	unsigned long nvsramMemBase;

	// PCI Initialization
    err = pci_enable_device( pdev );
    if( err ) {

        dev_err( &pdev->dev, "Failed to enable Zebulon PCI device (%d)\n", err );
        goto ErrExit;
    }


	// Only initialize function 0
	if( PCI_FUNC( pdev->devfn ) != 0 )
		return 0;

	// Get the BAR1
    fpgaMemBase = pci_resource_start( pdev, 0 );
    if( !fpgaMemBase ) {

        dev_err( &pdev->dev, "Zebulon BAR1 not initialized\n" );
        err = -ENODEV;
        goto ErrExit;
    }

	// Get the BAR2
    nvsramMemBase = pci_resource_start( pdev, 1 );
    if( !nvsramMemBase ) {

        dev_err( &pdev->dev, "Zebulon BAR2 not initialized\n" );
        err = -ENODEV;
        goto ErrExit;
    }

   
	// IO remapping (FPGA Reg)
    mmioVirtual = (volatile void __iomem *)ioremap( fpgaMemBase, FPGA_ZEBULON_CFGLEN );
    if( !mmioVirtual ) {

        printk( KERN_ERR "fpgaZebulonProbe: ioremap failed\n" );
		err = -ENOMEM;
        goto ErrExit;
    }

    // IO remapping (Nvsram Reg)
    nvsramVirtual = (volatile void __iomem *)ioremap( nvsramMemBase, pci_resource_len( pdev, 1 ) );
    if( !nvsramVirtual ) {

        printk( KERN_ERR "fpgaZebulonProbe: ioremap failed\n" );
        err = -ENOMEM;
        goto ErrExit1;
    }

	pci_enable_msi(pdev);

	// Update device resources
	fpgaZebulonUpdateResource( fpgaMemBase, pdev->irq, fpgaSubdevices, ARRAY_SIZE( fpgaSubdevices ) );
	fpgaZebulonUpdateResource( nvsramMemBase, pdev->irq, fpgaNvsram, ARRAY_SIZE( fpgaNvsram ) );

	// Register Platform Devices
	platform_add_devices( fpgaSubdevices, ARRAY_SIZE( fpgaSubdevices ) );
	platform_add_devices( fpgaNvsram, ARRAY_SIZE( fpgaNvsram ) );
	
	// Initialize Legacy FPGA code
	fpgaInit( (VOID *)mmioVirtual, (VOID *)nvsramVirtual );

#if 0
	// Enable Quick Switch
	fpgaEnableQuickSwitch( 1 );
#endif

	// Initialize SSLOCK
	sslock_Init();
	sslock_mem_Init();

	// Create FPGA SYSFS entries
	pFpgaKobj = kobject_create_and_add( "fpgaZebulon", firmware_kobj );
	if( !pFpgaKobj ) {

		printk( KERN_WARNING "Cannot allocate a kobject for FPGA zebulon\n" );
		goto doExit;
	}
	ret = sysfs_create_group( pFpgaKobj, &fpgaZebulonAttrGroup );
	if( ret )
		kobject_put( pFpgaKobj );
		
	zebulon_wdt_init();

doExit:
	return 0;

ErrExit1:
	iounmap( mmioVirtual );
	pci_disable_device( pdev );

ErrExit:
	return err;
}


static void fpgaZebulonRemove( struct pci_dev *pdev ) {

	// Release sysfs & kobject
	if( pFpgaKobj ) {

		sysfs_remove_group( pFpgaKobj, &fpgaZebulonAttrGroup );
		kobject_put( pFpgaKobj );
	}
	
	zebulon_wdt_exit();

    // Only function 0
    if( !PCI_FUNC( pdev->devfn ) ) {
		// Deregister Platform Devices
		platform_del_devices( fpgaNvsram, ARRAY_SIZE( fpgaNvsram ) );
		platform_del_devices( fpgaSubdevices, ARRAY_SIZE( fpgaSubdevices ) );

		// Unmap
		iounmap( nvsramVirtual );
		iounmap( mmioVirtual );

		pci_disable_msi(pdev);
	}

	// Disable PCI devices
	pci_disable_device( pdev );
	return;
}


static struct pci_driver fpgaZebulonDriver = {

    .name       = FPGA_ZEBULON_NAME,
    .id_table   = fpgaZebulonIds,
    .probe      = fpgaZebulonProbe,
    .remove     = fpgaZebulonRemove,
};


static int __init fpgaZebulonInit( void ) {

	int ret;

	// Register PCI driver
	ret = pci_register_driver( &fpgaZebulonDriver );
	if( ret ) {

		printk( KERN_ERR "Cannot register Rocket/Zebulon FPGA Driver\n" );
		return ret;
	}

	printk( KERN_INFO "NetApp Rocket/Zebulon FPGA Driver\n" );
	return 0;
}


static void __exit fpgaZebulonExit( void ) {

	// Unregister PCI driver
	pci_unregister_driver( &fpgaZebulonDriver );
}


module_init( fpgaZebulonInit );
module_exit( fpgaZebulonExit );

MODULE_AUTHOR( "merck.hung@netapp.com" );
MODULE_DESCRIPTION( "Rocket/Zebulon FPGA PCI Driver" );
MODULE_LICENSE( "GPL" );


