/*******************************************************************************

NAME            fpgaCrossCtrlDiag.c
SUMMARY         Zebulon fpga cross controller function diagnostics
VERSION         %version: 1 %
UPDATE DATE     %date_modified: Dec 30 14:30 2009 %
PROGRAMMER      %created_by:    Jim Tu %

        Copyright 2009 Quanta Corporation. All Rights Reserved.

DESCRIPTION:
        This file has code to test cross controller function of Zebulon fpga

NOTES:

REFERENCE:

*******************************************************************************/

#include <linux/kernel.h>
#include <linux/delay.h>
 
#include <commTypes.h>
#include <commRpt.h>
#include <commMEC.h>
#include <commNvsramApi.h>
#include <commSysInfo.h>

#include "fpgaRegLib.h"
#include "fpgaMidplaneI2cLib.h"
#include "fpgaIntMaskDiag.h"
#include "fpgaResetDiag.h" 
#include "fpgaBbuPsDiag.h"

EXTERN VOID fpgaI2cMdpl1Select(VOID);

/************************************/
/**  Audio Alarm Related Function  **/
/************************************/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaAudioAlarmEnable
*
* SUMMARY:      enable/disable audio alarm
*
* SCOPE:        public
*
* DESCRIPTION:  enable/disable audio alarm
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaAudioAlarmEnable
 (
 BOOLEAN status
 )
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_MISCELLANEOUS_CONTROL_1);
    if(status)
	{
        result = fpgaRegisterWrite(FPGA_MISCELLANEOUS_CONTROL_1, temp | AUD_ALARM_EN_BIT);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_MISCELLANEOUS_CONTROL_1, temp & (~AUD_ALARM_EN_BIT));
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaAlarmCancelInt
*
* SUMMARY:      verify if the audio alarm cancel bit is asserted
*
* SCOPE:        public
*
* DESCRIPTION:  verify if the audio alarm cancel bit is asserted
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
INT32 fpgaAlarmCancelInt(VOID)
{
    if(fpgaRegisterRead(FPGA_SUBSYSTEM_STATUS_LINES_2) & ALARM_CANCEL_INT_BIT)
	{
        return TRUE;
    }
	else
	{
        return FALSE;
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaClearAlarmCancelInt
*
* SUMMARY:      clear audio alarm cancel latch
*
* SCOPE:        public
*
* DESCRIPTION:  clear audio alarm cancel latch
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaClearAlarmCancelInt(VOID)
{
    INT32 result;
    result = fpgaRegisterWrite(FPGA_CLEAR_ALARM_CANCEL_LATCH, 0x01);
}

/*****************************************/
/**  Firmware Running Related Function  **/
/*****************************************/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaClearAlarmCancelInt
*
* SUMMARY:      clear audio alarm cancel latch
*
* SCOPE:        public
*
* DESCRIPTION:  clear audio alarm cancel latch
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaFwRunningSet
 (
 BOOLEAN status
 )
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_1);
    if(status)
	{
        temp |= FW_RUNNING_BIT;
    }
	else
	{
        temp &= (~FW_RUNNING_BIT);
    }
	
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_1, temp); 
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaAltFwRunning
*
* SUMMARY:      verify if the alternate FW running is asserted
*
* SCOPE:        public
*
* DESCRIPTION:  verify if the alternate FW running is asserted
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
INT32 fpgaAltFwRunning(VOID)
{
    if(fpgaRegisterRead(FPGA_LEGACY_MISCELLANEOUS_INFORMATION_1) & ALT_FW_RUNNING_BIT)
	{
        return FALSE;
    }
	else
	{
        return TRUE;
    }
}

/****************************************/
/**  Alternate Reset Related Function  **/
/****************************************/

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaAltResetOutSet
*
* SUMMARY:      assert alternate reset in from Quanta midplane
*
* SCOPE:        private
*
* DESCRIPTION:  assert alternate reset in from Quanta midplane
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaAltResetOutSet(VOID)
{
	midplaneGpioWrite(&ALT_RESET_IN_L, 0);  //emulate alternate reset
	
	mdelay(1000);
	
	midplaneGpioWrite(&ALT_RESET_IN_L, 1);  //emulate alternate reset
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaAltResetIn
*
* SUMMARY:      return alternate reset latch status
*
* SCOPE:        private
*
* DESCRIPTION:  return alternate reset latch status
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE: 	there was a alternate reset
*				FASLE:	there was not a alternate reset
*/
INT32 fpgaAltResetIn(VOID)
{
    if(fpgaRegisterRead(FPGA_LEGACY_MISCELLANEOUS_INFORMATION_1) & ALT_RSTIN_BIT)
	{
        return FALSE;
    }
	else
	{
        return TRUE;
    }
}

/*************************************/
/**  Card_IO_Test Related Function  **/
/*************************************/

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaCardIoTestSet
*
* SUMMARY:      set card IO pin state
*
* SCOPE:        public
*
* DESCRIPTION:  set card IO pin state
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaCardIoTestSet
 (
 BOOLEAN status
 )
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_MIDPLANE_MISCELLANEOUS);
    if(status)
	{
        temp |= CARD_IO_TEST_BIT;
    }
	else
	{
        temp &= (~CARD_IO_TEST_BIT);
    }

    result = fpgaRegisterWrite(FPGA_MIDPLANE_MISCELLANEOUS, temp); 
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaCardIoTestRead
*
* SUMMARY:      read card IO pin status
*
* SCOPE:        public
*
* DESCRIPTION:  read card IO pin status
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
INT32 fpgaCardIoTestRead(VOID)
{
    if(fpgaRegisterRead(FPGA_MIDPLANE_MISCELLANEOUS) & CARD_IO_TEST_BIT)
	{
        return TRUE;
    }
	else
	{
        return FALSE;
    }
}


/*************************************/
/**  Quick Switch Related Function  **/
/*************************************/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaEnablePairing
*
* SUMMARY:      enable/diable EN_PAIRING quick switch
*
* SCOPE:        public
*
* DESCRIPTION:  enable/diable EN_PAIRING quick switch
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaEnablePairing
 (
 BOOLEAN status
 )
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_ENABLE_QUICK_SWITCH);
    if(status)
	{
        temp |= EN_PAIRING_BIT;
    }
	else
	{
        temp &= (~EN_PAIRING_BIT);
    }

    result = fpgaRegisterWrite(FPGA_ENABLE_QUICK_SWITCH, temp); 
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaEnableLsCross
*
* SUMMARY:      enable/diable LS_CROSS quick switch
*
* SCOPE:        public
*
* DESCRIPTION:  enable/diable LS_CROSS quick switch
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaEnableLsCross
(
 BOOLEAN status
)
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_ENABLE_QUICK_SWITCH);
    if(status)
	{
        temp |= EN_LS_CROSS_BIT;
    }
	else
	{
        temp &= (~EN_LS_CROSS_BIT);
    }

    result = fpgaRegisterWrite(FPGA_ENABLE_QUICK_SWITCH, temp); 
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaEnableLsInterCon
*
* SUMMARY:      enable/diable LS_INTER_CON quick switch
*
* SCOPE:        public
*
* DESCRIPTION:  enable/diable LS_INTER_CON quick switch
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaEnableLsInterCon
(
 BOOLEAN status
)
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_ENABLE_QUICK_SWITCH);
    if(status)
	{
        temp |= EN_LS_INTER_CON_BIT;
    }
	else
	{
        temp &= (~EN_LS_INTER_CON_BIT);
    }

    result = fpgaRegisterWrite(FPGA_ENABLE_QUICK_SWITCH, temp); 
}


/*********************************************/
/**  Cross Controller Diagnostics Function  **/
/*********************************************/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaAudioAlarmDiag
*
* SUMMARY:      verify audio alarm function
*
* SCOPE:        public
*
* DESCRIPTION:  verify audio alarm function
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
INT32 fpgaAudioAlarmDiag(VOID)
{
	INT32 result = TRUE;
    
	if(!midplaneTypeVerification())
	{	//for LSI midplane
		fpgaAlarmCancelMask(TRUE);    //mask the interrupt
		if(fpgaAlarmCancelInt())
		{
			commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_ALARM_CANCEL_INT_ERROR, 
							MSG_FPGA_ALARM_CANCEL_INT_ERROR, 0, 0, 0, 0);
			result = FALSE;
		}
	}
	else
	{	//for Quanta test midplane
		fpgaAudioAlarmEnable(TRUE);   //turn audio alarm and verify
		mdelay(50);
		if(!midplaneGpioRead(&AUD_ALARM_OUT))
		{
			commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_ALARM_ERROR, 
							MSG_FPGA_ALARM_ERROR, 0, 0, 0, 0);
			result = FALSE;
			printk("[FPGA MIDPLANE] Alarm Enable Fail\n");
		}
		
		fpgaAudioAlarmEnable(FALSE);
		mdelay(50);
		if(midplaneGpioRead(&AUD_ALARM_OUT))
		{
			commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_ALARM_ERROR, 
							MSG_FPGA_ALARM_ERROR, 0, 0, 0, 0);
			result = FALSE;
			printk("[FPGA MIDPLANE] Alarm Disable Fail\n");
		}
		
		midplaneGpioWrite(&ALARM_MUTE_L, 0);	//emulate the button is pushed
		mdelay(100);
		
		
		if(fpgaAlarmCancelInt()) 
		{	//the INT is asserted
			midplaneGpioWrite(&ALARM_MUTE_L, 1);	//emulate the butten is release	
			mdelay(100);
			
			fpgaClearAlarmCancelInt();
			mdelay(5);
			
			if(fpgaAlarmCancelInt())
			{  //INT is not cleared
				commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_CLEAR_ALARM_CANCEL_INT_ERROR, 
								MSG_FPGA_CLEAR_ALARM_CANCEL_INT_ERROR, 0, 0, 0, 0);
				result = FALSE;
			}
			printk("[FPGA MIDPLANE PIN] Audio alarm mute INT asserted\n");
		} 
		else 
		{	//the INT is not asserted
			commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_ALARM_CANCEL_INT_ERROR, 
							MSG_FPGA_ALARM_CANCEL_INT_ERROR, 0, 0, 0, 0);
			result = FALSE;
			printk("[FPGA MIDPLANE PIN] Audio alarm mute INT not assert asserted\n");
		}
		
		midplaneGpioWrite(&ALARM_MUTE_L, 1);
		mdelay(100);
		
		if(fpgaAlarmCancelInt()) 
		{
			commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_ALARM_CANCEL_INT_ERROR, 
							MSG_FPGA_ALARM_CANCEL_INT_ERROR, 0, 0, 0, 0);
			result = FALSE;
		}
		
		printk("[FPGA MIDPLANE PIN] Audio alarm pin test done\n");
	}
    
	fpgaAudioAlarmEnable(FALSE);
	fpgaAlarmCancelMask(FALSE);
    
	return result;
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaFwRunningDiag
*
* SUMMARY:      verify FW running / alternate FW running function
*
* SCOPE:        public
*
* DESCRIPTION:  verify FW running / alternate FW running function
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
INT32 fpgaFwRunningDiag(VOID)
{
	INT32 result = TRUE;
    
	if(!midplaneTypeVerification())
	{
		//for LSI midplane
		if(fpgaAltFwRunning())
		{
			commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_ALT_FW_RUNNING_ERROR,
							MSG_FPGA_ALT_FW_RUNNING_ERROR, 0, 0, 0, 0);
			result = FALSE;
		}
		
    	}
	else
	{	//Quanta midplane
		if( ((fpgaRegisterRead(FPGA_ALTERNATE_BOARD_ID) & 0x1F) 
				== (fpgaRegisterRead(FPGA_BOARD_ID) & 0x1F))
			|| ((fpgaRegisterRead(FPGA_ALTERNATE_BOARD_ID) & 0x1F) == 0x10) )
		{	//if PSOC is fuctional
			/***  verify FW_RUNNING  ***/
			fpgaFwRunningSet(FALSE);
			if(!midplaneGpioRead(&FW_RUNNING_L))
			{  //read alternate controller
				commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_FW_RUNNING_ERROR, 
								MSG_FPGA_FW_RUNNING_ERROR, 0, 0, 0, 0);
				result = FALSE;
			}
    
			fpgaFwRunningSet(TRUE);
			if(midplaneGpioRead(&FW_RUNNING_L))
			{  //read alternate controller
				commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_FW_RUNNING_ERROR, 
							MSG_FPGA_FW_RUNNING_ERROR, 0, 0, 0, 0);
				result = FALSE;
			}
		
			fpgaFwRunningSet(FALSE);
    
			/***  verify ALT_FW_RUNNING  ***/
			midplaneGpioWrite(&ALT_FW_RUNNING_L, 0);	//emulate ALT_FW_RUNNING from alternate controller
			mdelay(5);
		
			if(!fpgaAltFwRunning())
			{
				commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_ALT_FW_RUNNING_ERROR, 
								MSG_FPGA_ALT_FW_RUNNING_ERROR, 0, 0, 0, 0);
				result = FALSE;
			}
    
			midplaneGpioWrite(&ALT_FW_RUNNING_L, 1);	//emulate ALT_FW_RUNNING canceled from alternate controller
			mdelay(5);
		
		
			if(fpgaAltFwRunning())
			{
				commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_ALT_FW_RUNNING_ERROR, 
								MSG_FPGA_ALT_FW_RUNNING_ERROR, 0, 0, 0, 0);
				result = FALSE;
			}
		
			printk("[FPGA Cross Ctrl] Alt. FW Running Test Done\n");
		}
		else
		{
			printk("[FPGA CROSS] WARNING!! PSOC gets wrong fixture ID. Alt. FW Running Test Done Skip\n");
		}
	}
	
	return result;
}    

#if 0	//modified by Jim Tu, 2009-12-30
		//Unused in FPGA REV.08
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaAlternateNmiDiag
*
* SUMMARY:      verify the FPGA alternate NMI function
*
* SCOPE:        public
*
* DESCRIPTION:  verify the FPGA alternate NMI function
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaAlternateNmiDiag(VOID)
{
    INT32 result = TRUE;
	
    fpgaNmiOutEnable(FALSE);          //mask NMI to processor
    
	if(!midplaneTypeVerification())
	{
		//for LSI midplane
		if(midplaneTypeVerification() && fpgaAlternateNmiIn())
		{
			commReportError(MEC_FPGA, FPGA_INTERRUPT_MEC, FPGA_ALTERNATE_NMI_IN_ERROR, 
							MSG_FPGA_ALTERNATE_NMI_IN_ERROR, 0, 0, 0, 0);
			result = FALSE;
		}
	}
	else if((fpgaRegisterRead(FPGA_ALTERNATE_BOARD_ID) & 0x1F) == (fpgaRegisterRead(FPGA_BOARD_ID) & 0x1F))
	{	//if PSOC is fuctional
		//Quanta midplane
    	//verify Alt_NMI_OUT
    	fpgaSetAlternateNmiOut(TRUE);
    	if(midplaneGpioRead(&ALT_NMI_L))
		{     //Alt_NMI not sent to alternate controller
        	commReportError(MEC_FPGA, FPGA_INTERRUPT_MEC, FPGA_ALTERNATE_NMI_OUT_ERROR, 
							MSG_FPGA_ALTERNATE_NMI_OUT_ERROR, 0, 0, 0, 0);
        	result = FALSE;
    	}
    	fpgaSetAlternateNmiOut(FALSE);
    	if(!midplaneGpioRead(&ALT_NMI_L))
		{     //Alt_NMI not sent to alternate controller
        	commReportError(MEC_FPGA, FPGA_INTERRUPT_MEC, FPGA_ALTERNATE_NMI_OUT_ERROR, 
							MSG_FPGA_ALTERNATE_NMI_OUT_ERROR, 0, 0, 0, 0);
        	result = FALSE;
    	}
	
    	//verify Alt_NMI_IN
    	midplaneGpioWrite(&ALT_NMI_IN_L, 0);  //emulate Alt_NMI from alternate controller
		mdelay(5);
		
    	if(!fpgaAlternateNmiIn())
		{
        	commReportError(MEC_FPGA, FPGA_INTERRUPT_MEC, FPGA_ALTERNATE_NMI_IN_ERROR, 
							MSG_FPGA_ALTERNATE_NMI_IN_ERROR, 0, 0, 0, 0);
        	result = FALSE;
			printk("[FPGA Cross Ctrl] Alt. NMI In 1\n");
		}
		else
		{	
			////verify clear NMI latch
			midplaneGpioWrite(&ALT_NMI_IN_L, 1);	//emulate Alt_NMI canceled from alternate controller
			fpgaClearAlternateNmiIn();				//clear Alt_NMI latch
    		if(fpgaAlternateNmiIn())
			{
        		commReportError(MEC_FPGA, FPGA_INTERRUPT_MEC, FPGA_CLEAR_ALTERNATE_NMI_IN_ERROR, 
								MSG_FPGA_CLEAR_ALTERNATE_NMI_IN_ERROR, 0, 0, 0, 0);
        		result = FALSE;
    		}
    	}
		
    	midplaneGpioWrite(&ALT_NMI_IN_L, 1);//emulate Alt_NMI canceled from alternate controller
		mdelay(5);
		
		if(fpgaAlternateNmiIn())
		{
        	commReportError(MEC_FPGA, FPGA_INTERRUPT_MEC, FPGA_ALTERNATE_NMI_IN_ERROR, 
							MSG_FPGA_ALTERNATE_NMI_IN_ERROR, 0, 0, 0, 0);
        	result = FALSE;
			printk("[FPGA Cross Ctrl] Alt. NMI In 2\n");
    	}
		
		printk("[FPGA Cross Ctrl] Alt. NMI Test Done\n");
	}
	else
	{
		printk("[FPGA CROSS] WARNING!! PSOC gets wrong fixture ID. Alt. NMI Test Done Skip\n");
	}

    return result;
}
#endif

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaAlternateInplaceDiag
*
* SUMMARY:      verify the FPGA alternate inplace & RAID function
*
* SCOPE:        public
*
* DESCRIPTION:  verify the FPGA alternate inplace & RAID function
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaAlternateInplaceRaidDiag(VOID)
{
    INT32 result = TRUE;
    
	if(!midplaneTypeVerification())
	{
		//for LSI midplane
		//verify ALT_INPL_N
		if(!fpgaAlternateInplaceInt())
		{
			commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_ALT_INPL_IN_ERROR, 
							MSG_FPGA_ALT_INPL_IN_ERROR, 0, 0, 0, 0);
			result = FALSE;
		}
		
		//verify ALT_RAID_ESM
		if(!(fpgaRegisterRead(FPGA_ALTERNATE_BOARD_ID) & ALT_CTRLR_ESM_N_BIT))
		{
			commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_ALT_RAID_ESM_ERROR, 
							MSG_FPGA_ALT_RAID_ESM_ERROR, 0, 0 ,0, 0);
			result = FALSE;
		}
	}
	else
	{
		//Quanta midplane
#if 0
    	//ALT_INPL_L
		midplaneGpioWrite(&ALT_INPL_L, 1);
		if(fpgaAlternateInplaceInt())
		{
			commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_ALT_INPL_IN_ERROR, 
							MSG_FPGA_ALT_INPL_IN_ERROR, 0, 0, 0, 0);
			result = FALSE;
		}
		
		midplaneGpioWrite(&ALT_INPL_L, 0);
		if(!fpgaAlternateInplaceInt())
		{
			commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_ALT_INPL_IN_ERROR, 
							MSG_FPGA_ALT_INPL_IN_ERROR, 0, 0, 0, 0);
			result = FALSE;
		}
		else
		{
			midplaneGpioWrite(&ALT_INPL_L, 1);
			fpgaClearAlternateInplaceInt();
			if(fpgaAlternateInplaceInt())
			{
				commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_CLEAR_ALT_INPL_IN_ERROR, 
								MSG_FPGA_CLEAR_ALT_INPL_IN_ERROR, 0, 0, 0, 0);
				result = FALSE;
			}
		}
		midplaneGpioWrite(&ALT_INPL_L, 0);	//emulate alternate inplace
#endif
		//ALT_RAID_ESM_L
		midplaneGpioWrite(&ALT_RAID_ESM_L, 0);
		if(fpgaRegisterRead(FPGA_ALTERNATE_BOARD_ID) & ALT_CTRLR_ESM_N_BIT)
		{
			commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_ALT_RAID_ESM_ERROR, 
							MSG_FPGA_ALT_RAID_ESM_ERROR, 0, 0 ,0, 0);
			result = FALSE;
		}
		midplaneGpioWrite(&ALT_RAID_ESM_L, 1);
		if(!(fpgaRegisterRead(FPGA_ALTERNATE_BOARD_ID) & ALT_CTRLR_ESM_N_BIT))
		{
			commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_ALT_RAID_ESM_ERROR, 
							MSG_FPGA_ALT_RAID_ESM_ERROR, 0, 0 ,0, 0);
			result = FALSE;
		}

		printk("[FPGA Cross Ctrl] Alt. INPL & RAID Test Done\n");
	}

    return result;
}

/*****************************************/
/**  Midplane Pin Diagnostics Function  **/
/*****************************************/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaMisIdPinDiag
*
* SUMMARY:      verify midplane pins function
*
* SCOPE:        public
*
* DESCRIPTION:  verify midplane pins function
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaMisIdPinDiag(VOID)
{
	INT32 result = TRUE;
    
	if(!midplaneTypeVerification())
	{
    }
	else
	{	//for Quanta midplane
    
		/***  Miscellaneous  ***/
		//SS_ID
		midplaneGpioWrite(&SS_ID_0, 0);
		midplaneGpioWrite(&SS_ID_1, 0);
		midplaneGpioWrite(&SS_ID_2, 0);
		midplaneGpioWrite(&SS_ID_3, 0);
		if((fpgaRegisterRead(FPGA_SUBSYSTEM_ID) & 0x0F) != 0x00)
		{
			commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_SS_ID_ERROR, 
							MSG_FPGA_SS_ID_ERROR, 0, 0, 0, 0);
			result = FALSE;
		}
		
		midplaneGpioWrite(&SS_ID_0, 1);
		midplaneGpioWrite(&SS_ID_1, 1);
		midplaneGpioWrite(&SS_ID_2, 1);
		midplaneGpioWrite(&SS_ID_3, 1);
		if((fpgaRegisterRead(FPGA_SUBSYSTEM_ID) & 0x0F) != 0x0F){
			commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_SS_ID_ERROR,
							MSG_FPGA_SS_ID_ERROR, 0, 0, 0, 0);
			result = FALSE;
		}
#if 0    
		//CONT_POSN_L
		midplaneGpioWrite(&CONT_POSN_L, 1);
		if(!(fpgaRegisterRead(FPGA_BOARD_ID) & 0x80))
		{
			commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_CONT_POSN_ERROR, 
							MSG_FPGA_CONT_POSN_ERROR, 0, 0, 0, 0);
			result = FALSE;
		}
		
    	midplaneGpioWrite(&CONT_POSN_L, 0);
		if((fpgaRegisterRead(FPGA_BOARD_ID) & 0x80))
		{
			commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_CONT_POSN_ERROR, 
							MSG_FPGA_CONT_POSN_ERROR, 0, 0, 0, 0);
			result = FALSE;
		}
		
    	if(sysMidplaneType == TEST_MIDPLANE_VER_1) 
		{
			//set UUT as slot B
			midplaneGpioWrite(&CONT_POSN_L, 1);
		}
		else
		{
			//set UUT as slot A
			if(midplaneGpioRead(&CONT_POSN_L))
			{	
				printk("[WARNING] Change FIXTURE slot position to Slot A\n");
				midplaneGpioWrite(&CONT_POSN_L, 0);
			}
		}
#endif		
		printk("[FPGA MIDPLANE PIN] SYS_DEF_LP pin test done\n");
	}
	 
	return result;
}

VOID fpgaMidAltInplSet(VOID)
{
	if(midplaneTypeVerification())
	{
		midplaneGpioWrite(&ALT_INPL_L, 0);
	}
}

VOID fpgaMidAltInplRead(VOID)
{
	if(midplaneTypeVerification())
	{
		midplaneGpioRead(&ALT_INPL_L);
	}
}

VOID fpgaMidAltInplUnSet(VOID)
{
	if(midplaneTypeVerification())
	{
		midplaneGpioWrite(&ALT_INPL_L, 1);
	}
}

VOID fpgaMidSlotASet(VOID)
{	
	if(midplaneTypeVerification())
	{
		midplaneGpioWrite(&CONT_POSN_L, 0);
	}
}

VOID fpgaMidSlotBSet(VOID)
{
	if(midplaneTypeVerification())
	{
		midplaneGpioWrite(&CONT_POSN_L, 1);
	}
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaQuickSwitchDiag
*
* SUMMARY:      Quick Switch Enable Diagnostics Function
*
* SCOPE:        public
*
* DESCRIPTION:  Quick Switch Enable Diagnostics Function
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaQuickSwitchDiag(VOID)
{
	INT32 result = TRUE;

	if(midplaneTypeVerification())
	{	//Quanta midplane
		fpgaI2cMdpl1Select();
		//Disable all QuickSwitch 
		fpgaEnablePairing(FALSE);
		fpgaEnableLsInterCon(FALSE);
		fpgaEnableLsCross(FALSE);
    
		//verify EN_PAIRING
		midplaneGpioWrite(&ALT_RAID_ESM_L, 0);
		if(!fpgaRegisterRead(FPGA_ALTERNATE_BOARD_ID) & ALT_CTRLR_ESM_N_BIT)
		{
			commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_EN_PAIRING_ERROR, 
							MSG_FPGA_EN_PAIRING_ERROR, 0, 0, 0, 0);
			result = FALSE;
		}
		
		//verify EN_LS_INTER_CON
		fpgaFwRunningSet(TRUE);
    	if(!midplaneGpioRead(&FW_RUNNING_L))
		{     //ALT_FW_RUNNING not sent to alternate controller
        	commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_EN_LS_INTER_CON_ERROR, 
							MSG_FPGA_EN_LS_INTER_CON_ERROR, 0, 0, 0, 0);
        	result = FALSE;
    	}
		
		//verify EN_LS_CROSS_L
		midplaneGpioWrite(&SS_ID_0, 0);
		midplaneGpioWrite(&SS_ID_1, 0);
		midplaneGpioWrite(&SS_ID_2, 0);
		midplaneGpioWrite(&SS_ID_3, 0);
		if((fpgaRegisterRead(FPGA_SUBSYSTEM_ID) & 0x0F) != 0x0F)
		{
			commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_EN_LS_CROSS_L_ERROR, 
							MSG_FPGA_EN_LS_CROSS_L_ERROR, 0, 0, 0, 0);
			result = FALSE;
		}
		
		//Enable all QuickSwitch 
		fpgaEnablePairing(TRUE);
		fpgaEnableLsInterCon(TRUE);
		fpgaEnableLsCross(TRUE);
		
		//verify EN_PAIRING
		midplaneGpioWrite(&ALT_RAID_ESM_L, 0);
		if(fpgaRegisterRead(FPGA_ALTERNATE_BOARD_ID) & ALT_CTRLR_ESM_N_BIT)
		{
			commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_EN_PAIRING_ERROR, 
							MSG_FPGA_EN_PAIRING_ERROR, 0, 0, 0, 0);
			result = FALSE;
		}
		
		//verify EN_LS_INTER_CON
		fpgaFwRunningSet(TRUE);
    	if(midplaneGpioRead(&FW_RUNNING_L))
		{     //ALT_FW_RUNNING not sent to alternate controller
        	commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_EN_LS_INTER_CON_ERROR, 
							MSG_FPGA_EN_LS_INTER_CON_ERROR, 0, 0, 0, 0);
        	result = FALSE;
    	}
		
		midplaneGpioWrite(&SS_ID_0, 0);
		midplaneGpioWrite(&SS_ID_1, 0);
		midplaneGpioWrite(&SS_ID_2, 0);
		midplaneGpioWrite(&SS_ID_3, 0);
		
		if((fpgaRegisterRead(FPGA_SUBSYSTEM_ID) & 0x0F) != 0x00)
		{
			commReportError(MEC_FPGA, FPGA_MISCELLANEOUS_MEC, FPGA_EN_LS_CROSS_L_ERROR, 
							MSG_FPGA_EN_LS_CROSS_L_ERROR, 0, 0, 0, 0);
			result = FALSE;
		}
		
		midplaneGpioWrite(&SS_ID_0, 1);
		midplaneGpioWrite(&SS_ID_1, 1);
		midplaneGpioWrite(&SS_ID_2, 1);
		midplaneGpioWrite(&SS_ID_3, 1);
    }
	else
	{
		printk("[FPGA QUICK SWITCH] Please use manufactury test midplane. \n");
	}

	return result;
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaAltResetInDiag
*
* SUMMARY:      Alternate reset in diagnostics unction
*
* SCOPE:        public
*
* DESCRIPTION:  Alternate reset in diagnostics unction
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaAltResetInDiag
 (
 INT32	mode, 
 UINT8 	bootingC, 
 UINT8 	bootingCC
 )
{
    INT32 result = TRUE;

	if(midplaneTypeVerification())
	{	//Quanta Midplane
		/***  Verify Alterante Reset In  ***/
		if(mode == 0){      //just in fpgaAltResetInDiag
			controllerBootCountCompareSet();
    		
			printk("[FPGA Reset] Prepare to reset....\n");
			//altResetPeer();	//wait for IPC
    
			mdelay(1000);	//wait for IPC protocol
			//should be in reset

			if(!fpgaAltResetIn())
			{
				*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_3) 
					|= FPGA_RESET_RESULT_ALT_RST_IN_FAIL;
				result = FALSE;
				printk("[FPGA Reset] fpgaAltResetIn Fail, did not reset\n");
			} 
		}
		
		/***  Verify if the alterante_reset_in cause the UUT reset  ***/
    
		if(mode == 1)
		{
			if(bootingCC == bootingC)
			{
				*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_3) 
					|= FPGA_RESET_RESULT_ALT_RST_IN_FAIL;
				printk("[FPGA Reset] fpgaAltResetIn Fail, did not reset\n");
			}
        
			if(!fpgaAltResetIn())
			{
				*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_3) 
					|= FPGA_RESET_RESULT_ALT_RST_IN_FAIL;
				printk("[FPGA Reset] fpgaAltResetIn Fail\n");
			}
    
			/***  Verify Clear Alternate Reset Latch  ***/
			fpgaClearAlternateReset();    //clear latch
			if(fpgaAltResetIn())
			{
				*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_3) 
					|= FPGA_RESET_RESULT_ALT_RST_CLEAR_FAIL;
				printk("[FPGA Reset] Clear fpgaAltResetIn Fail\n");
			}
			
			printk("[FPGA Reset] fpgaAltResetIn test done \n");
		}
	}

    return result;
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaAltResetOutDiag
*
* SUMMARY:      Alternate reset out diagnostics unction
*
* SCOPE:        public
*
* DESCRIPTION:  Alternate reset out diagnostics unction
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaAltResetOutDiag(VOID)
{
    INT32 result = TRUE;
    
	if(midplaneTypeVerification())
	{	//Quanta midplane
		/***  Verify Alternate Reset Out  ***/
		fpgaAlternateControllerReset(TRUE);
    
		if(midplaneGpioRead(&ALT_RESET_L))
		{
			*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_3) 
				|= FPGA_RESET_RESULT_ALT_RST_OUT_FAIL;
			result = FALSE;
		}
    
		fpgaAlternateControllerReset(FALSE);
    
		if(!midplaneGpioRead(&ALT_RESET_L))
		{
			*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_3) 
				|= FPGA_RESET_RESULT_ALT_RST_OUT_FAIL;
			result = FALSE;
		} 
	}
    
    return result;
}

/*********************************/
/**  Main Diagnostics Function  **/
/*********************************/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaCrossCtrlDiag
*
* SUMMARY:      Cross controller pin Diagnostics Function
*
* SCOPE:        public
*
* DESCRIPTION:   Cross controller pin Diagnostics Function
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaCrossCtrlDiag(VOID)
{     
	//fpgaI2cMdpl1Select();
	//fpgaMaskIntAll(TRUE);
	if(fpgaFwRunningDiag() && fpgaAltResetOutDiag() && fpgaAlternateInplaceRaidDiag())
	{
		fpgaClearSubsystemInt();
		//fpgaMaskIntAll(FALSE);
		return TRUE;
	}
	else
	{
		fpgaClearSubsystemInt();
		//fpgaMaskIntAll(FALSE);
		return FALSE;
	}
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaMidplanePinDiag
*
* SUMMARY:      Midplane pin Diagnostics Function
*
* SCOPE:        public
*
* DESCRIPTION:  Midplane pin Diagnostics Function
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaMidPinDiag(VOID)
{   
	printk("[FPGA MIDPLANE PIN] SYS_DEF_LP pin test start\n");
	//fpgaI2cMdpl1Select();
	//fpgaMaskIntAll(TRUE);
	if(fpgaMisIdPinDiag() /*&& fpgaAudioAlarmDiag()*/)
	{
		fpgaClearSubsystemInt();
		//fpgaMaskIntAll(FALSE);
		return TRUE;
	}
	else
	{
		fpgaClearSubsystemInt();
		//fpgaMaskIntAll(FALSE);
		return FALSE;
	}
}
