/*******************************************************************************

NAME            fpgaI2cCtl.c
SUMMARY         Zebulon FPGA I2C function diagnostics
VERSION         %version: 1 %
UPDATE DATE     %date_modified: Apr 27 20:30 2009 %
PROGRAMMER      %created_by:    Jim Tu %

        Copyright 2009 Quanta Corporation. All Rights Reserved.

DESCRIPTION:
        This file has code to test I2C function of Zebulon FPGA
		Not include the I2C master controller in Zebulon
		
NOTES:

REFERENCE:

*******************************************************************************/
#include <linux/kernel.h>
#include <linux/i2c.h>
#include <linux/delay.h>
#include <asm/io.h>

#include "commTypes.h"
#include "commRpt.h"
#include "commI2cApi.h"
#include "commMEC.h"

#include "fpgaRegLib.h"
#include "fpgaI2cDiag.h"

/******************************************/
/**  I2C Bus Interface Control Function  **/
/******************************************/

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaI2cMux9548Reset
*
* SUMMARY:      reset I2C 9548 MUX
*
* SCOPE:        private
*
* DESCRIPTION:  reset I2C 9548 MUX by asserting I2C_RESET bit
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaI2cMux9548Reset(BOOLEAN status)
{
    INT32 	result;
    UINT8	temp;
	
	temp = fpgaRegisterRead(FPGA_MISCELLANEOUS_CONTROL_2);
    if(status)
	{
        result = fpgaRegisterWrite(FPGA_MISCELLANEOUS_CONTROL_2, temp | I2C_RESET_BIT);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_MISCELLANEOUS_CONTROL_2, temp & (~I2C_RESET_BIT));
    }
}


/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaI2cMdpl1Reset
*
* SUMMARY:      reset midplane I2C bus 1
*
* SCOPE:        private
*
* DESCRIPTION:  reset midplane I2C bus 1
*
* RESTRICTIONS:
*
* RETURNS:
*/
VOID fpgaI2cMdpl1Reset(BOOLEAN status)
{
    INT32   result;
    UINT8   temp;

    temp = fpgaRegisterRead(FPGA_I2C_CONTROL_2);
    if(status)
    {
        result = fpgaRegisterWrite(FPGA_I2C_CONTROL_2, temp | MDPL_BUS1_RST_BIT);
    }
    else
    {
        result = fpgaRegisterWrite(FPGA_I2C_CONTROL_2, temp & (~MDPL_BUS1_RST_BIT));
    }
}
EXPORT_SYMBOL(fpgaI2cMdpl1Reset);


/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaI2cMdpl2Reset
*
* SUMMARY:      reset midplane I2C bus 2
*
* SCOPE:        private
*
* DESCRIPTION:  reset midplane I2C bus 2
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaI2cMdpl2Reset(BOOLEAN status)
{
    INT32 	result;
    UINT8 	temp;
    
    temp = fpgaRegisterRead(FPGA_I2C_CONTROL_2);
    if(status)
	{
        result = fpgaRegisterWrite(FPGA_I2C_CONTROL_2, temp | MDPL_BUS2_RST_BIT);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_I2C_CONTROL_2, temp & (~MDPL_BUS2_RST_BIT));
    }
}
EXPORT_SYMBOL(fpgaI2cMdpl2Reset);


/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaI2cMdpl1Select
*
* SUMMARY:      select midplane I2C bus 1
*
* SCOPE:        private
*
* DESCRIPTION:  select midplane I2C bus 1
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaI2cMdpl1Select()
{
    INT32 	result;
	UINT8 	temp;
    
    temp = fpgaRegisterRead(FPGA_I2C_CONTROL_2);
    //result = fpgaRegisterWrite(FPGA_I2C_CONTROL_2, temp & 0xFC);
    result = fpgaRegisterWrite(FPGA_I2C_CONTROL_2, temp | MDPL_BUS1_SEL_BIT);
}
EXPORT_SYMBOL(fpgaI2cMdpl1Select);

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaI2cMdpl2Select
*
* SUMMARY:      select midplane I2C bus 2
*
* SCOPE:        private
*
* DESCRIPTION:  select midplane I2C bus 2
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaI2cMdpl2Select()
{
    INT32 	result;
	UINT8 	temp;
    
    temp = fpgaRegisterRead(FPGA_I2C_CONTROL_2);
    //result = fpgaRegisterWrite(FPGA_I2C_CONTROL_2, temp & 0xFC);
    result = fpgaRegisterWrite(FPGA_I2C_CONTROL_2, temp|MDPL_BUS2_SEL_BIT);
}
EXPORT_SYMBOL(fpgaI2cMdpl2Select);

VOID fpgaI2cMdplEnable()
{
    fpgaRegisterWrite(FPGA_I2C_CONTROL_2, MDPL_BUS2_SEL_BIT | MDPL_BUS2_SEL_BIT);
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaI2cMasterMuxSelect
*
* SUMMARY:      select the I2C MUX in FPGA
*
* SCOPE:        private
*
* DESCRIPTION:  select the I2C MUX in FPGA
*				0b00:	I2C MUX 9548
*				0b01:	Midplane connector bus 1
*				0b10:	Midplane connector bus 2
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaI2cMasterMusSelect
 (
 UINT8 selection
 )
{
	INT32 	result;
	UINT8	temp;
	
	temp = fpgaRegisterRead(FPGA_I2C_CONTROL_2);
	if(selection < 0x03)
	{
		temp &= 0xCF;				//clear I2C MUX selection
		temp |= (selection << 4); 	//assert I2C MUX selection
		result = fpgaRegisterWrite(FPGA_I2C_CONTROL_2, temp);
	}
	else
	{
		printk("[FPGA I2C] FPGA I2C MUX selection is wrong!! \n");
	}
}

/*******************************************/
/**  Processor I2C Line Control Function  **/
/*******************************************/

/***  Write I2C Bus  ***/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProcI2cSdaLow
*
* SUMMARY:      pull low processor I2C SDA
*
* SCOPE:        private
*
* DESCRIPTION:  pull low processor I2C SDA
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaProcI2cSdaLow(BOOLEAN status)
{
    INT32 	result;
    UINT8 	temp;
    
    temp = fpgaRegisterRead(FPGA_I2C_CONTROL_1);
    if(status){
        result = fpgaRegisterWrite(FPGA_I2C_CONTROL_1, temp & (~I2C_DATA_BIT));
    }else{
        result = fpgaRegisterWrite(FPGA_I2C_CONTROL_1, temp | I2C_DATA_BIT);
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProcI2cSclLow
*
* SUMMARY:      pull low processor I2C SCL
*
* SCOPE:        private
*
* DESCRIPTION:  pull low processor I2C SCL
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaProcI2cSclLow(BOOLEAN status)
{
    INT32 	result;
    UINT8 	temp;
    
    temp = fpgaRegisterRead(FPGA_I2C_CONTROL_1);
    if(status){
        result = fpgaRegisterWrite(FPGA_I2C_CONTROL_1, temp & (~I2C_CLK_BIT));
    }else{
        result = fpgaRegisterWrite(FPGA_I2C_CONTROL_1, temp | I2C_CLK_BIT);
    }
}

/***  Read I2C Bus  ***/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProcI2cSdaRead
*
* SUMMARY:      read processor I2C SDA status
*
* SCOPE:        private
*
* DESCRIPTION:  read processor I2C SDA status
*
* RESTRICTIONS:
* 
* RETURNS:     	TRUE:	SDA high
*				FALSE:	SDA low 
*/
INT32 fpgaProcI2cSdaRead(VOID)
{
    if(fpgaRegisterRead(FPGA_I2C_CONTROL_1) & I2C_DATA_BIT)
	{
        return TRUE;
    }
	else
	{
        return FALSE;
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProcI2cSclRead
*
* SUMMARY:      read processor I2C SCL status
*
* SCOPE:        private
*
* DESCRIPTION:  read processor I2C SCL status
*
* RESTRICTIONS:
* 
* RETURNS:     	TRUE:	SCL high
*				FALSE:	SCL low 
*/
INT32 fpgaProcI2cSclRead(VOID)
{
    if(fpgaRegisterRead(FPGA_I2C_CONTROL_1) & I2C_CLK_BIT)
	{
        return TRUE;
    }
	else
	{
        return FALSE;
    }
}


