/*******************************************************************************

NAME            fpgaIdReadDiag.c
SUMMARY         Zebulon FPGA ID read function diagnostics
VERSION         %version: 1 %
UPDATE DATE     %date_modified: Apr 27 19:20 2009 %
PROGRAMMER      %created_by:    Jim Tu %

        Copyright 2009 Quanta Corporation. All Rights Reserved.

DESCRIPTION:
        This file has code to test ID read function of Zebulon FPGA

NOTES:

REFERENCE:

*******************************************************************************/
#include <linux/kernel.h>
#include <linux/delay.h>

#include <commTypes.h>
#include <commRpt.h>
#include <commMEC.h>
#include <commSysInfo.h>

#include "fpgaRegLib.h"
#include "fpgaIdReadDiag.h"

/***  Import Global Value  ***/

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaUutPosition
*
* SUMMARY:      Return the controller position
*
* SCOPE:        public
*
* DESCRIPTION:  Return the controller position
*
* RESTRICTIONS:
* 
* RETURNS:      0: in slot 1
*				1: in slot 2
*/
INT32 fpgaUutPosition(VOID)
{
    if(fpgaRegisterRead(FPGA_BOARD_ID) & CONTROLLER_POSITION_BIT)
	{
    	return 1;	//in slot 2
    }
	else
	{
    	return 0;	//in slot 1
    }
}

/***********************************/
/***  Main Diagnostics Function  ***/
/***********************************/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaIdDiag
*
* SUMMARY:      test ID registers
*
* SCOPE:        public
*
* DESCRIPTION:  verify if the ID registers return correct content
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE: 	pass
*				FALSE:	fail
*/
INT32 fpgaIdDiag(VOID)
{
    INT32 result = TRUE;

    /***  Compare board ID  ***/
    if(
	((fpgaRegisterRead(FPGA_BOARD_ID) & 0x1F) != (PIKESPEAK_BOARD_ID & 0x1F) ) &&
	((fpgaRegisterRead(FPGA_BOARD_ID) & 0x1F) != (SOYUZ_BOARD_ID & 0x1F) )
      )
    {
        commReportError(MEC_FPGA, FPGA_ID_READ_MEC, FPGA_BOARD_ID_ERROR, 
						MSG_FPGA_BOARD_ID_ERROR, 0, 0, 0, 0);
        result = FALSE;
    }
	
	/***  Compare alternate board ID  ***/
    if(
	((fpgaRegisterRead(FPGA_ALTERNATE_BOARD_ID) & 0x1F) != 0x10) &&
	((fpgaRegisterRead(FPGA_BOARD_ID) & 0x1F) != (PIKESPEAK_BOARD_ID & 0x1F) ) &&
	((fpgaRegisterRead(FPGA_BOARD_ID) & 0x1F) != (SOYUZ_BOARD_ID & 0x1F) )
      )
    {
        commReportError(MEC_FPGA, FPGA_ID_READ_MEC, FPGA_ALTERNATE_BOARD_ID_ERROR, 
						MSG_FPGA_ALTERNATE_BOARD_ID_ERROR, 0, 0, 0, 0);
        result = FALSE;
		printk("[FPGA ID] WARNING!! Alternate Controller ID: %02X\n", fpgaRegisterRead(FPGA_ALTERNATE_BOARD_ID));
    }

	/***  Compare HIC ID  ***/
	if(fpgaRegisterRead(FPGA_MISCELLANEOUS_CONTROL_3) & DRST_MASK_BIT)
	{	//if DRST_MASK is asserted, the host card ID should be hidden
		if(sysHostCardType == HOST_CARD_NONE)			//no host card
		{	
    		if(fpgaRegisterRead(FPGA_HOST_BOARD_ID) != DEFAULT_NO_CARD)
			{
       	    	commReportError(MEC_FPGA, FPGA_ID_READ_MEC, FPGA_HOST_BOARD_ID_ERROR, 
								MSG_FPGA_HOST_BOARD_ID_ERROR, 0, 0, 0, 0);
            	result = FALSE;
        	}
		}
		printk("[FPGA ID] The DRST_MASK bit is asserted. The host card ID will be set to 0xFF.\n");
	}
	else
	{	//if DRST_MASK is not asserted, the host card ID should be correct
	
    	//Compare HIC ID
    	if(sysHostCardType == HOST_CARD_NONE)			//no host card
		{	
    		if(fpgaRegisterRead(FPGA_HOST_BOARD_ID) != DEFAULT_NO_CARD)
			{
       	    	commReportError(MEC_FPGA, FPGA_ID_READ_MEC, FPGA_HOST_BOARD_ID_ERROR, 
								MSG_FPGA_HOST_BOARD_ID_ERROR, 0, 0, 0, 0);
            	result = FALSE;
        	}
    	}
		else if(sysHostCardType == HOST_CARD_IB)		//IB host card
		{		
    		if(fpgaRegisterRead(FPGA_HOST_BOARD_ID) != ELK_PARK_ID_IB)
			{
       	    	commReportError(MEC_FPGA, FPGA_ID_READ_MEC, FPGA_HOST_BOARD_ID_ERROR, 
								MSG_FPGA_HOST_BOARD_ID_ERROR, 0, 0, 0, 0);
            	result = FALSE;
        	}
    	}
		else if(sysHostCardType == HOST_CARD_FC)		//FC host card
		{		
    		if(fpgaRegisterRead(FPGA_HOST_BOARD_ID) != MANITOU_ID_FC)
			{
       	   		commReportError(MEC_FPGA, FPGA_ID_READ_MEC, FPGA_HOST_BOARD_ID_ERROR, 
								MSG_FPGA_HOST_BOARD_ID_ERROR, 0, 0, 0, 0);
            	result = FALSE;
        	}
    	}
		else if(sysHostCardType == HOST_CARD_ISCSI)		//iSCSI host card
		{		
    		if(fpgaRegisterRead(FPGA_HOST_BOARD_ID) != GLEN_COVE_ID_ISCSI)
			{
       	    	commReportError(MEC_FPGA, FPGA_ID_READ_MEC, FPGA_HOST_BOARD_ID_ERROR, 
								MSG_FPGA_HOST_BOARD_ID_ERROR, 0, 0, 0, 0);
            	result = FALSE;
        	}
    	}
		else if(sysHostCardType == HOST_CARD_SAS)		//SAS host card
		{		
    		if(fpgaRegisterRead(FPGA_HOST_BOARD_ID) != GLACIER_ID_SAS)
			{
       	    	commReportError(MEC_FPGA, FPGA_ID_READ_MEC, FPGA_HOST_BOARD_ID_ERROR, 
								MSG_FPGA_HOST_BOARD_ID_ERROR, 0, 0, 0, 0);
            	result = FALSE;
        	}
    	}
	}

    /***  compare FPGA device ID  ***/
    if(fpgaRegisterRead(FPGA_DEVICE_ID) != FPGA_DEVICE_ID_VERSION)
	{
        commReportError(MEC_FPGA, FPGA_ID_READ_MEC, FPGA_FPGA_DEVICE_ID_ERROR, 
						MSG_FPGA_FPGA_DEVICE_ID_ERROR, 0, 0, 0, 0);
        result = FALSE;
    }
    
    return result;
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaVersionShow
*
* SUMMARY:      show FPGA revision
*
* SCOPE:        public
*
* DESCRIPTION:  show FPGA revision
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaVersionShow(VOID)
{
    printk("FPGA Revision: %02X \n", fpgaRegisterRead(FPGA_REVISION));
}


UINT8 fpgaAlternateBoardIdRead(VOID)
{
	return fpgaRegisterRead(FPGA_ALTERNATE_BOARD_ID);
}

UINT8 fpagRevisionRead(VOID)
{
	return fpgaRegisterRead(FPGA_REVISION);
}

VOID fpagHicVerify(VOID)
{
	UINT8	hicCardId;
	
	hicCardId = fpgaRegisterRead(FPGA_HOST_BOARD_ID);
	
   	//Compare HIC ID
	switch (hicCardId)
	{
		/* QE8 */
		case MANITOU_ID_FC:
			sysHostCardType = HOST_CARD_FC;
			break;
		/* ISCSI */
		case GLEN_COVE_ID_ISCSI:
			sysHostCardType = HOST_CARD_ISCSI;
			break;
		/* IB */
		case ELK_PARK_ID_IB:
			sysHostCardType = HOST_CARD_IB;
			break;
		/* SAS */
		case GLACIER_ID_SAS:
			sysHostCardType = HOST_CARD_SAS;
			break;
		default:
			sysHostCardType = HOST_CARD_NONE;
	}
}
