/*******************************************************************************

NAME            fpgaIntMaskDiag.c
SUMMARY         Zebulon fpga Interrupt/mask function diagnostics
VERSION         %version: 1 %
UPDATE DATE     %date_modified: Dec 30 14:23 2009 %
PROGRAMMER      %created_by:    Jim Tu %

        Copyright 2009 Quanta Corporation. All Rights Reserved.

DESCRIPTION:
        This file has code to test Interrupt/mask function of Zebulon fpga

NOTES:
		Modified by Jim Tu, 2009-12-30: Remove NMI to processor test

REFERENCE:

*******************************************************************************/

#include <linux/delay.h>
#include <linux/kernel.h>

#include <commTypes.h>
#include <commRpt.h>
#include <commMEC.h>

#include "fpgaRegLib.h"
#include "fpgaMidplaneI2cLib.h"
#include "fpgaIntMaskDiag.h"
#include "fpgaResetDiag.h" 

/**************************************/
/**  Interrupt Information Function  **/
/**************************************/

//alternate controller INT32errupt
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaClearAlternateInplaceInt
*
* SUMMARY:      clear alternate controller inplace INT32errupt latch
*
* SCOPE:        public
*
* DESCRIPTION:  clear alternate controller inplace INT32errupt latch
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaClearAlternateInplaceInt(VOID)
{
    INT32 	result;
    result = fpgaRegisterWrite(FPGA_CLEAR_ALTERNATE_INPLACE_INTERRUPT, 0x01);
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaAlternateInplaceInt
*
* SUMMARY:      verify if the alternate controller is attached
*
* SCOPE:        public
*
* DESCRIPTION:  verify if the alternate controller is attached 
*				by reading ALT_INPL pin
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	alternate controller is attached
*				FALSE:	alternate controller is not attached
*/
INT32 fpgaAlternateInplaceInt(VOID)
{
    if(fpgaRegisterRead(FPGA_SUBSYSTEM_STATUS_LINES_3) & ALT_INPL_BIT)
	{
        return TRUE;
    }
	else
	{
        return FALSE;
    }
}


//subsystem INT32errupt
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaSubsystemInt
*
* SUMMARY:      verify if the subsystem INT32errupt is asserted
*
* SCOPE:        public
*
* DESCRIPTION:  verify if the subsystem INT32errupt is asserted 
*				by reading SUBSYS_INT pin
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	the subsystem INT32errupt is asserted
*				FALSE:	the subsystem INT32errupt is not asserted
*/
INT32 fpgaSubsystemInt(VOID)
{
    if(fpgaRegisterRead(FPGA_INTERRUPT_SOURCE_INT_B) & SUBSYS_INT_BIT)
	{
        return TRUE;
    }
	else
	{
        return FALSE;
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaClearSubsystemInt
*
* SUMMARY:      clear subsystem INT32errupt latch
*
* SCOPE:        public
*
* DESCRIPTION:  clear subsystem INT32errupt latch
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaClearSubsystemInt(VOID)
{
    INT32 result;
    result = fpgaRegisterWrite(FPGA_CLEAR_SUBSYSTEM_INTERRUPT, 0x01);
}


//harness flex inplace INT32errupt

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaSubsystemInt
*
* SUMMARY:      verify if the harness inplace INT32errupt is asserted
*
* SCOPE:        public
*
* DESCRIPTION:  verify if the harness inplace is asserted 
*				by reading SYS_DEF_LP_7 pin
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	the harness inplace INT32errupt is asserted
*				FALSE:	the harness inplace INT32errupt is not asserted
*/
INT32 fpgaHarnessInplaceInt(VOID)
{
    if(fpgaRegisterRead(FPGA_ENCLOSURE_DEFINED_VALUE_1) & SYS_DEF_LP7_D_BIT)
	{
        return TRUE;
    }
	else
	{
        return FALSE;
    }
}

//temperature INT32errupt
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaControllerOverTempInt
*
* SUMMARY:      verify if the controller over temperature INT32errupt is asserted
*
* SCOPE:        public
*
* DESCRIPTION:  verify if the controller over temperature is asserted 
*				by reading TMP_INT pin
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	the controller over temperature INT32errupt is asserted
*				FALSE:	the controller over temperature INT32errupt is not asserted
*/
INT32 fpgaControllerOverTempInt(VOID)
{
    if(fpgaRegisterRead(FPGA_SUBSYSTEM_STATUS_LINES_2) & TMP_INT_BIT)
	{
        return TRUE;
    }
	else
	{
        return FALSE;
    }
}
EXPORT_SYMBOL(fpgaControllerOverTempInt);

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaControllerOverTempInt
*
* SUMMARY:      verify if the CPU over temperature INT32errupt is asserted
*
* SCOPE:        public
*
* DESCRIPTION:  verify if the CPU over temperature is asserted 
*				by reading CPU_TMP_INT pin
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	the CPU over temperature INT32errupt is asserted
*				FALSE:	the CPU over temperature INT32errupt is not asserted
*/
INT32 fpgaThermostateInt(VOID)
{
    if(fpgaRegisterRead(FPGA_SUBSYSTEM_STATUS_LINES_2) & CPU_TMP_INT_BIT)
	{
        return TRUE;
    }
	else
	{
        return FALSE;
    }
}


/********************************/
/**  NMI Information Function  **/
/********************************/
#if 0
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaDirectedReset
*
* SUMMARY:      verify if the direct reset INT32errupt is asserted
*
* SCOPE:        public
*
* DESCRIPTION:  verify if the direct reset INT32errupt is asserted 
*				by reading DIR_RST pin
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	the direct reset INT32errupt is asserted
*				FALSE:	the direct reset INT32errupt is not asserted
*/
INT32 fpgaDirectedReset(VOID)
{
    if(fpgaRegisterRead(FPGA_HIGH_PRIORITY_INTERRUPT_SOURCE) & DIR_RST_BIT)
	{
        return TRUE;
    }
	else
	{
        return FALSE;
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaClearAlternateNmiIn
*
* SUMMARY:      clear alternate NMI latch
*
* SCOPE:        public
*
* DESCRIPTION:  clear alternate NMI latch
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaClearAlternateNmiIn(VOID)
{
    INT32 result;
    result = fpgaRegisterWrite(FPGA_CLEAR_ALTERNATE_NMI_IN_LATCH, 0x01);
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaAlternateNmiIn
*
* SUMMARY:      verify if the alternate NMI is asserted
*
* SCOPE:        public
*
* DESCRIPTION:  verify if the alternate NMI is asserted 
*				by reading ALT_NMI_IN pin
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	the alternate NMI is asserted
*				FALSE:	the alternate NMI is not asserted
*/
INT32 fpgaAlternateNmiIn(VOID)
{
    if(fpgaRegisterRead(FPGA_HIGH_PRIORITY_INTERRUPT_SOURCE) & ALT_NMI_IN_BIT)
	{
        return TRUE;
    }
	else
	{
        return FALSE;
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaSetAlternateNmiOut
*
* SUMMARY:      set alternate NMI to alternate controller
*
* SCOPE:        public
*
* DESCRIPTION:  set alternate NMI to alternate controller
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaSetAlternateNmiOut
 (
 BOOLEAN status
 )
{    
    INT32 result;
    UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_1);
	
    if(status)
	{
        temp |= ALT_NMI_BIT;
    }
	else
	{
        temp &= (~ALT_NMI_BIT);    
    }
	
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_1, temp);  
}
#endif

/*******************************/
/**  Interrupt Mask Function  **/
/*******************************/
/***  Subsystem Mask 1  ***/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaPsAcGoodIntMask
*
* SUMMARY:      enable/disable AC good INT32errupt mask
*
* SCOPE:        public
*
* DESCRIPTION:  enable/disable AC good temperature INT32errupt mask
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaPsAcGoodIntMask
(
 BOOLEAN 	status
)
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_SUBSYSTEM_INTERRUPT_MASK_1);
	
    if(status)
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_1, temp | PS_AC_GOOD_INT_MASK_BIT);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_1, temp & (~PS_AC_GOOD_INT_MASK_BIT));
    }
}

/***  Subsystem Mask 2  ***/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaPs0FaultMask
*
* SUMMARY:      enable/disable power supply fault INT32errupt mask
*
* SCOPE:        public
*
* DESCRIPTION:  enable/disable power supply fault INT32errupt mask
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaPs0FaultMask
 (
 BOOLEAN 	status
 )
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_SUBSYSTEM_INTERRUPT_MASK_2);
	
    if(status)
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_2, temp | PS0_FAULT_MASK_BIT);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_2, temp & (~PS0_FAULT_MASK_BIT));
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaPs1FaultMask
*
* SUMMARY:      enable/disable power supply fault INT32errupt mask
*
* SCOPE:        public
*
* DESCRIPTION:  enable/disable power supply fault INT32errupt mask
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaPs1FaultMask
 (
 BOOLEAN 	status
 )
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_SUBSYSTEM_INTERRUPT_MASK_2);
	
    if(status)
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_2, temp | PS1_FAULT_MASK_BIT);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_2, temp & (~PS1_FAULT_MASK_BIT));
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaThermostateIntMask
*
* SUMMARY:      enable/disable CPU temperature INT32errupt mask
*
* SCOPE:        public
*
* DESCRIPTION:  enable/disable CPU temperature INT32errupt mask
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaThermostateIntMask
 (
 BOOLEAN 	status
 )
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_SUBSYSTEM_INTERRUPT_MASK_2);
	
    if(status)
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_2, temp | CPU_TMP_MASK_BIT);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_2, temp & (~CPU_TMP_MASK_BIT));
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaTempIntMask
*
* SUMMARY:      enable/disable controller temperature INT32errupt mask
*
* SCOPE:        public
*
* DESCRIPTION:  enable/disable controller temperature INT32errupt mask
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaTempIntMask
 (
 BOOLEAN 	status
 )
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_SUBSYSTEM_INTERRUPT_MASK_2);
	
    if(status)
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_2, temp | TMP_MASK_BIT);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_2, temp & (~TMP_MASK_BIT));
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaAlarmCancelMask
*
* SUMMARY:      enable/disable alarm cancel INT32errupt mask
*
* SCOPE:        public
*
* DESCRIPTION:  enable/disable alarm cancel INT32errupt mask
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaAlarmCancelMask
 (
 BOOLEAN 	status
 )
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_SUBSYSTEM_INTERRUPT_MASK_2);
	
    if(status)
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_2, temp | ALARM_CANCEL_MASK_BIT);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_2, temp & (~ALARM_CANCEL_MASK_BIT));
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaPs0OverTempMask
*
* SUMMARY:      enable/disable power supply 0 over temperature INT32errupt mask
*
* SCOPE:        public
*
* DESCRIPTION:  enable/disable power supply 0 over temperature INT32errupt mask
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaPs0OverTempMask
 (
 BOOLEAN 	status
 )
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_SUBSYSTEM_INTERRUPT_MASK_2);
	
    if(status)
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_2, temp | PS0_OVER_TEMP_MASK_BIT);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_2, temp & (~PS0_OVER_TEMP_MASK_BIT));
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaPs1OverTempMask
*
* SUMMARY:      enable/disable power supply 1 over temperature INT32errupt mask
*
* SCOPE:        public
*
* DESCRIPTION:  enable/disable power supply 1 over temperature INT32errupt mask
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaPs1OverTempMask
 (
 BOOLEAN 	status
 )
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_SUBSYSTEM_INTERRUPT_MASK_2);
	
    if(status)
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_2, temp | PS1_OVER_TEMP_MASK_BIT);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_2, temp & (~PS1_OVER_TEMP_MASK_BIT));
    }
}

/***  Subsystem Mask 3  ***/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaPs0InplMask
*
* SUMMARY:      enable/disable power supply 0 inplace INT32errupt mask
*
* SCOPE:        public
*
* DESCRIPTION:  enable/disable power supply 0 inplace INT32errupt mask
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaPs0InplMask
 (
 BOOLEAN 	status
 )
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_SUBSYSTEM_INTERRUPT_MASK_3);
	
    if(status)
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_3, temp | PS0_INPL_MASK_BIT);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_3, temp & (~PS0_INPL_MASK_BIT));
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaPs1InplMask
*
* SUMMARY:      enable/disable power supply 1 inplace INT32errupt mask
*
* SCOPE:        public
*
* DESCRIPTION:  enable/disable power supply 1 inplace INT32errupt mask
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaPs1InplMask
 (
 BOOLEAN 	status
 )
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_SUBSYSTEM_INTERRUPT_MASK_3);
	
    if(status)
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_3, temp | PS1_INPL_MASK_BIT);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_3, temp & (~PS1_INPL_MASK_BIT));
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaBbuInplMask
*
* SUMMARY:      enable/disable BBU inplace INT32errupt mask
*
* SCOPE:        public
*
* DESCRIPTION:  enable/disable BBU inplace INT32errupt mask
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaBbuInplMask
 (
 BOOLEAN 	status
 )
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_SUBSYSTEM_INTERRUPT_MASK_3);
	
    if(status)
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_3, temp | BBU_INPL_MASK_BIT);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_3, temp & (~BBU_INPL_MASK_BIT));
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaAltInplMask
*
* SUMMARY:      enable/disable alternate controller inplace INT32errupt mask
*
* SCOPE:        public
*
* DESCRIPTION:  enable/disable alternate controller inplace INT32errupt mask
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaAltInplMask
 (
 BOOLEAN 	status
 )
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_SUBSYSTEM_INTERRUPT_MASK_3);
	
    if(status)
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_3, temp | ALT_INPL_MASK_BIT);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_3, temp & (~ALT_INPL_MASK_BIT));
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaBbuDrawingIntMask
*
* SUMMARY:      enable/disable BBU drawing INT32errupt mask
*
* SCOPE:        public
*
* DESCRIPTION:  enable/disable BBU drawing INT32errupt mask
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaBbuDrawingIntMask
 (
 BOOLEAN 	status
 )
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_SUBSYSTEM_INTERRUPT_MASK_3);
	
    if(status)
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_3, temp | HOTSWAT_PGOOD_MASK_BIT);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_3, temp & (~HOTSWAT_PGOOD_MASK_BIT));
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaBbuEnableMask
*
* SUMMARY:      enable/disable BBU enable INT32errupt mask
*
* SCOPE:        public
*
* DESCRIPTION:  enable/disable BBU enable INT32errupt mask
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaBbuEnableMask
 (
 BOOLEAN 	status
 )
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_SUBSYSTEM_INTERRUPT_MASK_3);
	
    if(status)
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_3, temp | BBU_ENABLE_MASK_BIT);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_3, temp & (~BBU_ENABLE_MASK_BIT));
    }
}

/***  Interrupt Mask B  ***/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaSubsystemIntMask
*
* SUMMARY:      enable/disable subsystem INT32errupt mask
*
* SCOPE:        public
*
* DESCRIPTION:  enable/disable subsystem INT32errupt mask
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaSubsystemIntMask
 (
 BOOLEAN 	status
 )
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_INTERRUPT_MASK_B);
	
    if(status)
	{
        result = fpgaRegisterWrite(FPGA_INTERRUPT_MASK_B, temp | SUBSYS_MASK_BIT);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_INTERRUPT_MASK_B, temp & (~SUBSYS_MASK_BIT));
    }
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaMaskIntAll
*
* SUMMARY:      enable/disable all INT32errupt mask
*
* SCOPE:        public
*
* DESCRIPTION:  enable/disable all INT32errupt mask
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaMaskIntAll
 (
 BOOLEAN 	status
 )
{
    INT32 	result;
    if(status)
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_1, 0xFF);
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_2, 0xFF);
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_3, 0xFF);
        result = fpgaRegisterWrite(FPGA_INTERRUPT_MASK_B, 0xFF);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_1, 0x00);
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_2, 0x00);
        result = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_3, 0x00);
        result = fpgaRegisterWrite(FPGA_INTERRUPT_MASK_B, 0x00);
	}
}

/*************************/
/**  NMI Mask Function  **/
/*************************/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaDirectedResetMaskSet
*
* SUMMARY:      enable/disable direct reset mask
*
* SCOPE:        public
*
* DESCRIPTION:  enable/disable direct reset mask
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaDirectedResetMaskSet
 (
 BOOLEAN 	status
 )
{
    INT32 	result;
	UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_MISCELLANEOUS_CONTROL_3);
	
    if(status)
	{
        result = fpgaRegisterWrite(FPGA_MISCELLANEOUS_CONTROL_3, temp | DRST_MASK_BIT);
    }
	else
	{
        result = fpgaRegisterWrite(FPGA_MISCELLANEOUS_CONTROL_3, temp & (~DRST_MASK_BIT));
    }
}



/*************************************/
/**  Interrupt Out Contol Function  **/
/*************************************/
#if 0	//Modified by Jim Tu, 2009-12-30
		//Unused in FPGA ERV.08
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaNmiOutEnable
*
* SUMMARY:      enable/disable sending NMI to processor
*
* SCOPE:        public
*
* DESCRIPTION:  enable/disable sending NMI to processor
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaNmiOutEnable
 (
 BOOLEAN 	status
 )
{    
    INT32 	result;
    UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_1);
	
    if(status)
	{
        temp |= NMI_OUT_EN_BIT;
    }
	else
	{
        temp &= (~NMI_OUT_EN_BIT);
    }
	
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_1, temp); 
}
#endif

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaTestModeEnable
*
* SUMMARY:      enable/disable INT32errupt test mode
*
* SCOPE:        public
*
* DESCRIPTION:  enable/disable INT32errupt test mode
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaTestModeEnable
 (
 BOOLEAN 	status
 )
{
    INT32 	result;
    UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_1);
	
    if(status)
	{
        temp |= TEST_MODE_EN_BIT;
    }
	else
	{
        temp &= (~TEST_MODE_EN_BIT);
    }
	
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_1, temp); 
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaTestModeInterruptMaskDiag
*
* SUMMARY:      verify the FPGA INT32errupt mask function
*
* SCOPE:        public
*
* DESCRIPTION:  verify the FPGA INT32errupt mask function
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaTestModeInterruptMaskDiag(VOID)
{
    INT32 result = TRUE;
    INT32 temp, i;
    
    fpgaSubsystemIntMask(FALSE);  //allow INT to Subsystem INT
    fpgaTestModeEnable(TRUE);     //enable fault injection
    
    /***  Verify Subsystem INT  ***/
    temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_STATUS_LINES_1, 0x00);
    temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_STATUS_LINES_2, 0x00);
    temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_STATUS_LINES_3, 0x00);
	
	//subsystem status lines 1
    for(i = 0; i < 6; i++)
	{
		//assert interrupt
        temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_STATUS_LINES_1, 0x01 << i);
		
		//verify if the subsystem interrupt has been asserted
        if(!fpgaSubsystemInt())
		{
            commReportError(MEC_FPGA, FPGA_INTERRUPT_MEC, FPGA_SUBSYSTEM_INT_ERROR, 
							MSG_FPGA_SUBSYSTEM_INT_ERROR, 0, 0, 0, 0);
            result = FALSE;
			printk("[FPGA INT&MASK] Int REG 1 bit: %d error \n", i);
        }
		
		if(i == 1)
		{
            i = 4;      //jump to bit 4
        }
		fpgaClearSubsystemInt();
		mdelay(5);
    }
    
	//subsystem status lines 2
    for(i = 0; i < 8; i++)
	{
		//assert interrupt
        temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_STATUS_LINES_2, 0x01 << i);
		
		//verify if the subsystem interrupt has been asserted
        if(!fpgaSubsystemInt())
		{
            commReportError(MEC_FPGA, FPGA_INTERRUPT_MEC, FPGA_SUBSYSTEM_INT_ERROR, 
							MSG_FPGA_SUBSYSTEM_INT_ERROR, 0, 0, 0, 0);
            result = FALSE;
			printk("[FPGA INT&MASK] Int REG 2 bit: %d error \n", i);
        }
            
		if(i == 3)
		{
            i++;      //jump to bit 4
        }
		fpgaClearSubsystemInt();
		mdelay(5);
    }
    
	//subsystem status lines 3
    for(i = 0; i < 6; i++)
	{
		//assert interrupt
        temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_STATUS_LINES_3, 0x01 << i);
		
		//verify if the subsystem interrupt has been asserted
        if(!fpgaSubsystemInt())
		{
            commReportError(MEC_FPGA, FPGA_INTERRUPT_MEC, FPGA_SUBSYSTEM_INT_ERROR, 
							MSG_FPGA_SUBSYSTEM_INT_ERROR, 0, 0, 0, 0);
            result = FALSE;
			printk("[FPGA INT&MASK] Int REG 3 bit: %d error \n", i);
        }
		
		if(i == 3)
		{
            i++;      //jump to bit 4
        }
		fpgaClearSubsystemInt();
		mdelay(5);
    }
    
    //clear subsystem INT
    temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_STATUS_LINES_1, 0x00);
    temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_STATUS_LINES_2, 0x00);
    temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_STATUS_LINES_3, 0x00);

    fpgaClearSubsystemInt();
    
    //verify clear subsystem function
    if(fpgaSubsystemInt())
	{
        commReportError(MEC_FPGA, FPGA_INTERRUPT_MEC, FPGA_CLEAR_SUBSYSTEM_INT_ERROR, 
                        MSG_FPGA_CLEAR_SUBSYSTEM_INT_ERROR, 0, 0, 0, 0);
        result = FALSE;
    }
    printk("[FPGA INT&MASK] Interrupt Test Done \n");
		
    /***  Verify INT Mask  ***/
    //set all Mask 
    temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_1, 0xFF);
    temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_2, 0xFF);
    temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_3, 0xFF);
    
	//subsystem mask 1
    for(i = 0; i < 6; i++)
	{
		//assert interrupt
        temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_STATUS_LINES_1, 0x01 << i);
		
		//verify if the subsystem interrupt has not been asserted
        if(fpgaSubsystemInt())
		{
            commReportError(MEC_FPGA, FPGA_INTERRUPT_MEC, (FPGA_INT_MASK_ERROR_BASE + i), 
							MSG_FPGA_INT_MASK_ERROR_BASE, 1, i, 0, DG_FPGA_INT_FORMAT);
            result = FALSE;
			printk("[FPGA INT&MASK] Mask REG 1 bit: %d error \n", i);
        }
		
		if(i == 1)
		{
            
			i = 4;      //jump to bit 4
        }
		fpgaClearSubsystemInt();
		mdelay(5);
    }
    
	//subsystem mask 2
    for(i = 0; i < 8; i++)
	{
        //assert interrupt
        temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_STATUS_LINES_2, 0x01 << i);
		
		//verify if the subsystem interrupt has not been asserted
        if(fpgaSubsystemInt())
		{
            commReportError(MEC_FPGA, FPGA_INTERRUPT_MEC, (FPGA_INT_MASK_ERROR_BASE + 8 + i), 
							MSG_FPGA_INT_MASK_ERROR_BASE, 2, i, 0, DG_FPGA_INT_FORMAT);
            result = FALSE;
			printk("[FPGA INT&MASK] Mask REG 2 bit: %d error \n", i);
        }
	        
        if(i == 3){
            i++;      //jump to bit 4
        }
		fpgaClearSubsystemInt();
		mdelay(5);
    }
    
	//subsystem mask 3
    for(i = 0; i < 6; i++)
	{
        //assert interrupt
        temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_STATUS_LINES_3, 0x01 << i);
		
		//verify if the subsystem interrupt has not been asserted
        if(fpgaSubsystemInt())
		{
            commReportError(MEC_FPGA, FPGA_INTERRUPT_MEC, (FPGA_INT_MASK_ERROR_BASE + 16 + i), 
							MSG_FPGA_INT_MASK_ERROR_BASE, 3, i, 0, DG_FPGA_INT_FORMAT);
            result = FALSE;
			printk("[FPGA INT&MASK] Mask REG 3 bit: %d error \n", i);
        }
		
		if(i == 3){
            i++;      //jump to bit 4
        }
		fpgaClearSubsystemInt();
		mdelay(5);
    }
    
    //clear subsystem INT
    temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_STATUS_LINES_1, 0x00);
    temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_STATUS_LINES_2, 0x00);
    temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_STATUS_LINES_3, 0x00);
    
    //release Mask 
    //clear subsystem INT
    temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_1, 0x00);
    temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_2, 0x00);
    temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_INTERRUPT_MASK_3, 0x00);
    
    fpgaClearSubsystemInt();
    
	printk("[FPGA INT&MASK] Interrupt Mask Test Done \n");

#if 0	
    /***  Verify INT Mask  ***/
    fpgaSubsystemIntMask(TRUE);
    
    temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_STATUS_LINES_2, PS1_OVER_TEMP_BIT);    //assert INT
    
    if(fpgaSubsystemInt())
	{
        commReportError(MEC_FPGA, FPGA_INTERRUPT_MEC, FPGA_SUBSYSTEM_INT_MASK_ERROR, 
						MSG_FPGA_SUBSYSTEM_INT_MASK_ERROR, 0, 0, 0, 0);
        result = FALSE;
    }
	
	//clear subsystem INT
    temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_STATUS_LINES_1, 0x00);
    temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_STATUS_LINES_2, 0x00);
    temp = fpgaRegisterWrite(FPGA_SUBSYSTEM_STATUS_LINES_3, 0x00);
	
	fpgaClearSubsystemInt();
#endif
    
    fpgaSubsystemIntMask(FALSE);  
    fpgaTestModeEnable(FALSE);    //disable fault injection
	
    return result;
}

#if 0	//Modified by Jim Tu, 2009-12-30
		//Unused in FPGA REV.08
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaDirectedResetMaskDiag
*
* SUMMARY:      verify the FPGA direct reset function
*
* SCOPE:        public
*
* DESCRIPTION:  verify the FPGA direct reset function
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaDirectedResetMaskDiag(VOID)
{
    INT32 result = TRUE;
    INT32 temp;
	
    //disable NMI to processor
    fpgaNmiOutEnable(FALSE);
	
    fpgaDirectedResetMaskSet(TRUE);      //set directed reset mask
    temp = fpgaRegisterWrite(FPGA_SET_DIRECTED_RESET_LATCH, DIR_RST_SET_BIT);  //set all directed reset
	
    //if NMI is sent to processors, it means the NMI enable switch is out of function
    if(fpgaProcessorNmi())
	{                 
        commReportError(MEC_FPGA, FPGA_INTERRUPT_MEC, FPGA_NMI_ERROR, 
						MSG_FPGA_NMI_ERROR, 0, 0, 0, 0);
        result = FALSE;
    }
    
    temp = fpgaRegisterWrite(FPGA_SET_DIRECTED_RESET_LATCH, 0x00);  //release    
    fpgaClearDirectedReset();
	
    //enable NMI to processor
    fpgaNmiOutEnable(TRUE);
    
    fpgaDirectedResetMaskSet(TRUE);      //set directed reset mask
    temp = fpgaRegisterWrite(FPGA_SET_DIRECTED_RESET_LATCH, DIR_RST_SET_BIT);  //set all directed reset

    if(fpgaRegisterRead(FPGA_HIGH_PRIORITY_INTERRUPT_SOURCE) != DIR_RST_BIT)
	{ 
		commReportError(MEC_FPGA, FPGA_INTERRUPT_MEC, FPGA_DIRECTED_RESET_MASK_ERROR, 
						MSG_FPGA_DIRECTED_RESET_MASK_ERROR, 0, 0, 0, 0);
        result = FALSE;
    }

    //if NMI is not sent to processors, it means the NMI is out of function
    if(!fpgaProcessorNmi())
	{
        commReportError(MEC_FPGA, FPGA_INTERRUPT_MEC, FPGA_NMI_ERROR, 
						MSG_FPGA_NMI_ERROR, 0, 0, 0, 0);
        result = FALSE;
    }
    
    temp = fpgaRegisterWrite(FPGA_SET_DIRECTED_RESET_LATCH, 0x00);  //release    
	fpgaClearDirectedReset();
	fpgaDirectedResetMaskSet(FALSE);		//clear direct reset mask    
    
    //disable NMI to processor
    fpgaNmiOutEnable(FALSE);
	
    return result;
}
#endif

/*********************************/
/**  Main Diagnostics Function  **/
/*********************************/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaAlternateNmiDiag
*
* SUMMARY:      verify the FPGA interrupt and mask function
*
* SCOPE:        public
*
* DESCRIPTION:  verify the FPGA interrupt and mask function
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaInterruptMaskDiag(VOID)
{   
	return fpgaTestModeInterruptMaskDiag();
}

VOID fpgaIntDiagLoop(UINT32 time)
{
	int i;
	for(i = 0; i < time; i++)
	{
		fpgaInterruptMaskDiag();
	}
}

/*********************************/
/**  Main Diagnostics Function  **/
/*********************************/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaPushBtnDiag
*
* SUMMARY:      verify the push button function
*
* SCOPE:        public
*
* DESCRIPTION:  verify the push button function
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaPushBtnDiag(VOID)
{
	INT32	result = TRUE;
	
	//verfiy the interrupt is inserted or not
	if((fpgaRegisterRead(FPGA_SUBSYSTEM_STATUS_LINES_1) & PUSH_BTN_LT_BIT) != PUSH_BTN_LT_BIT)
	{ 
		commReportError(MEC_FPGA, FPGA_INTERRUPT_MEC, FPGA_PUSH_BTN_ERROR, 
						MSG_FPGA_PUSH_BTN_ERROR, 0, 0, 0, 0);
        result = FALSE;
    }

	//clear interrupt latch
	fpgaRegisterWrite(FPGA_CLEAN_PUSH_BUTTON_LATCH, 1);
	
	if((fpgaRegisterRead(FPGA_SUBSYSTEM_STATUS_LINES_1) & PUSH_BTN_LT_BIT) == PUSH_BTN_LT_BIT)
	{ 
		commReportError(MEC_FPGA, FPGA_INTERRUPT_MEC, FPGA_CLEAR_PUSH_BTN_ERROR, 
						MSG_FPGA_CLEAR_PUSH_BTN_ERROR, 0, 0, 0, 0);
        result = FALSE;
    }
				
	return result;
}
