/*******************************************************************************

NAME            fpgaMidplaneI2cLib.c
SUMMARY         Quanta test midplane I2C function library
VERSION         %version: 1 %
UPDATE DATE     %date_modified: Apr 30 16:00 2009 %
PROGRAMMER      %created_by:    Jim Tu %

        Copyright 2009 Quanta Corporation. All Rights Reserved.

DESCRIPTION:
        This file includes all Quanta test midplane I2C functions

NOTES:

REFERENCE:

*******************************************************************************/

#include <linux/kernel.h>
#include <linux/i2c.h>
#include <linux/delay.h>

#include <commTypes.h>
#include <commRpt.h>
#include <commI2cApi.h>
#include <commMEC.h>
#include <commSysInfo.h>

#include "fpgaRegLib.h"
#include "fpgaI2cDiag.h"
#include "fpgaMidplaneI2cLib.h"

#if defined(CHASSIS_WSAS)
#include "wemblySAS.h"
#endif

EXTERN VOID fpgaEnableQuickSwitch(BOOLEAN status);
EXTERN VOID fpgaMidAltInplSet(VOID);
EXTERN VOID fpgaMidSlotASet(VOID);
EXTERN VOID fpgaMidSlotBSet(VOID);
EXTERN VOID fpgaPsocReset(BOOLEAN status);
EXTERN VOID fpgaPwrEngagement(BOOLEAN vccPs, BOOLEAN vccBbu); 

/*******************************/
/**  I2C GPIO Access Address  **/
/*******************************/

/***  BBU & PS Diag  ***/
I2C_GPIO_M_PIN  	PWR_INPL_0_L;
I2C_GPIO_M_PIN  	PWR_INPL_1_L;
I2C_GPIO_M_PIN  	PS0_FAULT_L;
I2C_GPIO_M_PIN  	PS1_FAULT_L;
I2C_GPIO_M_PIN  	PS0_CONTROL_L_BP1;
I2C_GPIO_M_PIN  	VCC_BBU_ENABLE;
I2C_GPIO_M_PIN  	VCC_PS_ENABLE;
I2C_GPIO_M_PIN  	VCC_BBU_SWITCH_ENABLE;
I2C_GPIO_M_PIN  	VCC_PS_ALT_ENABLE;
I2C_GPIO_M_PIN  	AC_GOOD;

/***  Emulate BBU module  ***/
I2C_GPIO_M_PIN  	BBU_INPLACE_L;
I2C_GPIO_M_PIN  	BBU_ENABLE;
I2C_GPIO_M_PIN  	BBU_DISCHARGE;
I2C_GPIO_M_PIN  	VCC12_INPUT;

/***  LED Diag  ***/
I2C_GPIO_M_PIN  	RDY_RMV_LED_IN;  
I2C_GPIO_M_PIN  	ALT_RTR_LED_L;
I2C_GPIO_M_PIN  	PS0_RTR_LED_L;
I2C_GPIO_M_PIN  	PS1_RTR_LED_L;
I2C_GPIO_M_PIN  	BOX_ID_LED_L;	//Box Identify LED
I2C_GPIO_M_PIN  	SUMMARY_FLT_LED_L;	//Summary Fault LED
I2C_GPIO_M_PIN  	BOX_OVER_TEMP_LED_L;	//Box Over-Temp LED
I2C_GPIO_M_PIN  	SYS_DEF_HP_4_N;	//reserved pin

/***  Cross Controller  ***/
I2C_GPIO_M_PIN  	INPL_L;   
I2C_GPIO_M_PIN  	ALT_INPL_L;
I2C_GPIO_M_PIN  	FW_RUNNING_L;
I2C_GPIO_M_PIN  	ALT_FW_RUNNING_L;
I2C_GPIO_M_PIN  	ALT_RESET_L;  
I2C_GPIO_M_PIN  	ALT_RESET_IN_L;    
I2C_GPIO_M_PIN  	ALT_NMI_L;  
I2C_GPIO_M_PIN  	ALT_NMI_IN_L;
I2C_GPIO_M_PIN  	RAID_ESM_L;     
I2C_GPIO_M_PIN  	ALT_RAID_ESM_L; 

/***  Midplane Connection Diag  ***/
I2C_GPIO_M_PIN  	CONT_POSN_L;    
I2C_GPIO_M_PIN  	CARD_IO_TEST;
I2C_GPIO_M_PIN  	MP_I2C1_RST_L;      
I2C_GPIO_M_PIN  	MP_I2C2_RST_L;             
I2C_GPIO_M_PIN		AUD_ALARM_OUT;	//audio alarm
I2C_GPIO_M_PIN		SS_ID_0;		//sybsystem ID 1
I2C_GPIO_M_PIN		SS_ID_1;		//subsystem ID 2
I2C_GPIO_M_PIN		SS_ID_2;		//subsystem ID 3
I2C_GPIO_M_PIN		SS_ID_3;		//subsystem ID 4
I2C_GPIO_M_PIN		ALARM_MUTE_L;	//alarm cancel
I2C_GPIO_M_PIN		HARNESS_INPL_L;	//harness inplace

/***  Drive Diag  ***/
I2C_GPIO_M_PIN 		D_INPL_L[24];
I2C_GPIO_M_PIN 		D_FAULT_LED_L[24];
I2C_GPIO_M_PIN 		D_REPL_LED_L[24];

/********************************/
/**  I2C GPIO Access Function  **/
/********************************/

/*******************************************************************************
* PROCEDURE
*
* NAME:         midplaneI2c1HotSwapBufferReadyCheck
*
* SUMMARY:      verify if I2C hot swap buffer 1 is ready
*
* SCOPE:        private
*
* DESCRIPTION:  verify if I2C hot swap buffer 1 is ready
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	ready
*				FALSE:	not ready
*/
INT32 midplaneI2c1HotSwapBufferReadyCheck(VOID)
{
	if((fpgaRegisterRead(FPGA_I2C_STATUS) & 0x03) != I2C1_READY_BIT)	//if hot swap buffer isn't ready
    {
        //printk("[MIDPLANE] Retriggering hot swap buffer. \n");
		
		//fpgaI2cAllReset();						//reset bus to trigger hot swap buffer
		mdelay(20);
		
		if((fpgaRegisterRead(FPGA_I2C_STATUS) & 0x03) != I2C1_READY_BIT)
		{	//if hot swap buffer is still not ready
			//commReportError(MEC_FPGA, FPGA_I2C_MEC, FPGA_I2C_HOT_SWAP_1_ERROR, 
			//				MSG_FPGA_I2C_HOT_SWAP_1_ERROR, 0, 0, 0, 0);
			//printk("[MIDPLANE I2C] Hot Swap Buffer 1 is not ready!! \n");
			return FALSE;
		}
    }
	
	return TRUE;
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         midplaneI2c2HotSwapBufferReadyCheck
*
* SUMMARY:      verify if I2C hot swap buffer 2 is ready
*
* SCOPE:        private
*
* DESCRIPTION:  verify if I2C hot swap buffer 2 is ready
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	ready
*				FALSE:	not ready
*/
INT32 midplaneI2c2HotSwapBufferReadyCheck(VOID)
{
	if((fpgaRegisterRead(FPGA_I2C_STATUS) & 0x03) != I2C2_READY_BIT)	//if hot swap buffer isn't ready
    {
        //printk("[MIDPLANE] Retriggering hot swap buffer. \n");
		
		//fpgaI2cAllReset();	//reset bus to trigger hot swap buffer
		mdelay(20);
		
		if((fpgaRegisterRead(FPGA_I2C_STATUS) & 0x03) != I2C2_READY_BIT)
		{	//if hot swap buffer is still not ready
			//commReportError(MEC_FPGA, FPGA_I2C_MEC, FPGA_I2C_HOT_SWAP_2_ERROR, 
			//				MSG_FPGA_I2C_HOT_SWAP_2_ERROR, 0, 0, 0, 0);
			//printk("[MIDPLANE I2C] Hot Swap Buffer 2 is not ready!! \n");
			return FALSE;
		}
    }
	
	return TRUE;
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         midplane9545Channel
*
* SUMMARY:      set 9545 channel
*
* SCOPE:        private
*
* DESCRIPTION:  set 9545 channel
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
INT32 midplane9545Channel
 (
 struct i2c_adapter *adap,
 UINT8 				channel
 )
{
    struct i2c_msg 	msg;
    UINT8 				bufOut, bufIn;
    INT32 			ret;
	INT32			result = TRUE;
		        
    msg.addr = (I2C_GPIO_9545_ADDRESS >> 1);
    msg.flags = 0; //write
    msg.len = 1;
    
    /* set 9545 switch in midplane */
    switch(channel)
	{	
		case I2C_GPIO_9545_CHANNEL_0:
			bufOut = I2C_GPIO_9545_CHANNEL_0;
			break;
	
		case I2C_GPIO_9545_CHANNEL_1:
			bufOut = I2C_GPIO_9545_CHANNEL_1;
			break;
	
		case I2C_GPIO_9545_CHANNEL_2:
			bufOut = I2C_GPIO_9545_CHANNEL_2;
			break;
			
		case I2C_GPIO_9545_CHANNEL_3:
			bufOut = I2C_GPIO_9545_CHANNEL_3;
			break;
		
		default:
	    	printk("[MIDPLANE] Wrong 9545 channel! 0x%02X \n", bufOut);
	    	break;
    }
    msg.buf = &bufOut;
    
    /* send to switch */
	if ((ret = i2c_transfer(adap, &msg, 1)) != 1)
	{
		//commReportError(MEC_FPGA, MIDPLANE_LED_MEC, MIDPLANE_9545_ERROR,
		//				MSG_MIDPLANE_9545_ERROR, 0, 0, 0, 0);
		printk("[MIDPLANE] Write 9545 channel error! ret = %d, 0x%02X \n", ret, bufOut);
		result = FALSE;
    }
    
    /* read from switch */
    msg.flags = I2C_M_RD;
    msg.buf = &bufIn;
    
	if ((ret = i2c_transfer(adap, &msg, 1)) != 1)
	{		
		//commReportError(MEC_FPGA, MIDPLANE_MISCELLANEOUS_MEC, MIDPLANE_9545_ERROR,
		//				MSG_MIDPLANE_9545_ERROR, 0, 0, 0, 0);
		printk("[MIDPLANE] Read 9545 channel error! ret = %d, 0x%02X \n", ret, bufIn);
		result = FALSE;
	}
    
	if(bufOut != (bufIn & 0x0F))
	{
		//commReportError(MEC_FPGA, MIDPLANE_MISCELLANEOUS_MEC, MIDPLANE_9545_ERROR,
		//				MSG_MIDPLANE_9545_ERROR, 0, 0, 0, 0);
    	printk("[MIDPLANE] Set 9545 channel failed! 0x%02X \n", (bufIn & 0x0F));
		result = FALSE;
	}
	
	return result;
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         midplaneGpioWrite
*
* SUMMARY:      set 9555 pin high/low	
*
* SCOPE:        public
*
* DESCRIPTION:  set 9555 pin high/low
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
INT32 midplaneGpioWrite
 (
 I2C_GPIO_M_PIN *address,
 BOOLEAN 		status
 )
{
    struct i2c_msg 		msgOut;
    struct i2c_msg 		msgIn[2];
    struct i2c_adapter 	*adap;
	UINT8  				offset;
    UINT8  				bufIn[2];
    UINT8  				bufOut[3];
    INT32  				ret;
    
	if(address->I2C_9555_direction)
	{
    	if((address != &BBU_ENABLE) && (address != &BBU_DISCHARGE))
		{
			printk("[MIDPLANE] This pin is input and should not be written! \n");
    	}
	}
    
	fpgaI2cMdpl1Select();
	
    /* set I2C adapter */
    adap = zebulonAdapter[ZEBULON_I2C_BUS_1];
	
	if(midplaneI2c1HotSwapBufferReadyCheck() == FALSE)
	{	//if hot swap buffer isn't ready		
		if(midplane9545Channel(adap, address->I2C_9545_channel) == FALSE)
		{
			printk("[MIDPLANE] Hot Swap Buffer 1 is not ready. Break before 9555 write.\n");
			return FALSE;							//break out
		}
		else
		{
			printk("[MIDPLANE] Hot Swap Buffer 1 is not ready. I2C is still functional.\n");
		}
	}

    /* set 9545 switch in midplane */
    midplane9545Channel(adap, address->I2C_9545_channel);
	
    /* read 9555 register value */
    /* command write */
	offset = address->I2C_9555_port;
	
    msgIn[0].addr = address->I2C_9555_address;
    msgIn[0].flags = 0;
    msgIn[0].len = 1;
    msgIn[0].buf = &offset;

    /* read content */
    msgIn[1].addr = address->I2C_9555_address;
    msgIn[1].flags = I2C_M_RD;
    msgIn[1].len = 2;
    msgIn[1].buf = bufIn;

    if((ret = i2c_transfer(adap, msgIn, 2)) != 2)
	{
		printk("-------------------------------------------------------------------------\n");
		printk("[MIDPLANE] Channel: 0x%02X, Address: 0x%02X, port: 0x%02X, pin: 0x%02X. \n",
						address->I2C_9545_channel, address->I2C_9555_address,
						address->I2C_9555_port, address->I2C_9555_pin);
		
		//commReportError(MEC_FPGA, MIDPLANE_MISCELLANEOUS_MEC, MIDPLANE_9555_ERROR,
		//				MSG_MIDPLANE_9555_ERROR, 0, 0, 0, 0);
		printk("[MIDPLANE] Write 9555 command error! ret = %d \n", ret);
				
		return FALSE;
   	}
    
    /* set 9555 register value */
    bufOut[1] = bufIn[0];
    bufOut[2] = bufIn[1];
    
    if(status){
		bufOut[1] |= address->I2C_9555_pin;
    }else{
		bufOut[1] &= ~(address->I2C_9555_pin);
    }
    
    /* set shift out message */
    msgOut.addr = address->I2C_9555_address;
    msgOut.flags = 0; //write
    msgOut.len = 3;
    bufOut[0] = address->I2C_9555_port;
    msgOut.buf = bufOut;

    if((ret = i2c_transfer(adap, &msgOut, 1)) != 1)
	{
		//commReportError(MEC_FPGA, MIDPLANE_MISCELLANEOUS_MEC, MIDPLANE_9555_ERROR,
		//				MSG_MIDPLANE_9555_ERROR, 0, 0, 0, 0);
		if((address != &BBU_ENABLE) && (address != &BBU_DISCHARGE))
		{
			printk("[MIDPLANE] Write 9555 register error! ret = %d \n", ret);
    
		}
	}

    return TRUE;
}

/*******************************************************************************
* PROCEDURE
* NAME:         midplaneGpioRead
*
* SUMMARY:      read 9555 pin status
*
* SCOPE:        public
*
* DESCRIPTION:  read 9555 pin status
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
INT32 midplaneGpioRead
 (
 I2C_GPIO_M_PIN 	*address
 )
{
    struct i2c_msg 		msg[2];
    struct i2c_adapter 	*adap;
    UINT8  				buf[2];
    INT32 				ret;

	fpgaI2cMdpl1Select();
	
    /* set I2C adapter */
    adap = zebulonAdapter[ZEBULON_I2C_BUS_1];
	
	if(midplaneI2c1HotSwapBufferReadyCheck() == FALSE)
	{	//if hot swap buffer isn't ready		
		if(midplane9545Channel(adap, address->I2C_9545_channel) == FALSE)
		{
			printk("[MIDPLANE] Hot Swap Buffer 1 is not ready. Break before 9555 read.\n");
			return FALSE;							//break out
		}
		else
		{
			printk("[MIDPLANE] Hot Swap Buffer 1 is not ready. I2C is still functional.\n");
		}
	}
	
    /* set 9545 switch in midplane */
    midplane9545Channel(adap, address->I2C_9545_channel);

    /* command write */
    msg[0].addr = address->I2C_9555_address;
    msg[0].flags = 0;
    msg[0].len = 1;
    buf[0] = address->I2C_9555_port;
    msg[0].buf = buf;

    /* read content */
    msg[1].addr = address->I2C_9555_address;
    msg[1].flags = I2C_M_RD;
    msg[1].len = 2;
    msg[1].buf = buf;

    if((ret = i2c_transfer(adap, msg, 2)) != 2)
	{
		printk("-------------------------------------------------------------------------\n");
		printk("[MIDPLANE] Channel: 0x%02X, Address: 0x%02X, port: 0x%02X, pin: 0x%02X. \n",
						address->I2C_9545_channel, address->I2C_9555_address,
						address->I2C_9555_port, address->I2C_9555_pin);
		
		//commReportError(MEC_FPGA, MIDPLANE_MISCELLANEOUS_MEC, MIDPLANE_9555_ERROR,
		//				MSG_MIDPLANE_9555_ERROR, 0, 0, 0, 0);
		printk("[MIDPLANE] Write 9555 command error! ret = %d \n", ret);
				
		return FALSE;
    }	

    if(buf[0] & address->I2C_9555_pin)
	{
    	return TRUE;
    }
	else
	{
    	return FALSE;
    }
}

/********************************/
/**  I2C GPIO Define Function  **/
/********************************/
/*******************************************************************************
* PROCEDURE
*
* NAME:         midplaneI2cGpioConfig
*
* SUMMARY:      config 9555 register
*
* SCOPE:        public
*
* DESCRIPTION:  config 9555 register
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
INT32 midplaneI2cGpioConfig
(
 UINT8 address, 
 UINT8 channel, 
 UINT8 config0, 
 UINT8 config1
)
{   
    struct i2c_msg 		msgOut;
    struct i2c_adapter 	*adap;
    UINT8  				bufOut[3];
    INT32  				ret;

    /* set I2C adapter */
    adap = zebulonAdapter[ZEBULON_I2C_BUS_1];
	if( !adap )
		return FALSE;

    /* set 9545 switch in midplane */
    mid9545ChSet(channel);
    
    bufOut[0] = I2C_9555_CONF_PORT_0;
    bufOut[1] = config0;
    bufOut[2] = config1;
    
    /* set shift out message */
    msgOut.addr = address;
    msgOut.flags = 0; //write
    msgOut.len = 3;
    msgOut.buf = bufOut;

	if((ret = i2c_transfer(adap, &msgOut, 1)) != 1)
	{
		//commReportError(MEC_FPGA, MIDPLANE_MISCELLANEOUS_MEC, MIDPLANE_9555_ERROR,
		//				MSG_MIDPLANE_9555_ERROR, 0, 0, 0, 0);
		printk("[MIDPLANE] Configure 9555 register error! ret = %d \n", ret);
	}
    
	return ret;
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         midplaneI2cGpioSetDefault
*
* SUMMARY:      config 9555 output register to default value
*
* SCOPE:        public
*
* DESCRIPTION:  config 9555 register
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID midplaneI2cGpioSetDefault(VOID)
{
	mid9545ChSet(I2C_GPIO_9545_CHANNEL_0);
	
	mid9555Write(I2C_GPIO_9555_1_8, I2C_9555_OUTPUT_PORT_0, 0xFF, 0xFF);
	mid9555Write(I2C_GPIO_9555_2_8, I2C_9555_OUTPUT_PORT_0, 0xFF, 0xFF);
	mid9555Write(I2C_GPIO_9555_3_8, I2C_9555_OUTPUT_PORT_0, 0xFF, 0xFF);
	
	mid9545ChSet(I2C_GPIO_9545_CHANNEL_1);
	
	mid9555Write(I2C_GPIO_9555_4_8, I2C_9555_OUTPUT_PORT_0, 0xFF, 0xFF);
	mid9555Write(I2C_GPIO_9555_5_8, I2C_9555_OUTPUT_PORT_0, 0xFF, 0xFF);
	
	mid9545ChSet(I2C_GPIO_9545_CHANNEL_2);
	
	mid9555Write(I2C_GPIO_9555_6_8, I2C_9555_OUTPUT_PORT_0, 0xFF, 0xFF);
	mid9555Write(I2C_GPIO_9555_7_8, I2C_9555_OUTPUT_PORT_0, 0xFF, 0xFF);
	mid9555Write(I2C_GPIO_9555_8_8, I2C_9555_OUTPUT_PORT_0, 0xFF, 0xFF);
	
	printk("[MIDPLANE] Midplane GPIO defaults are set to high.\n");
}

VOID midplaneI2cGpioValueShow(VOID)
{
	mid9545ChSet(I2C_GPIO_9545_CHANNEL_0);
	
	mid9555Read(I2C_GPIO_9555_1_8, I2C_9555_OUTPUT_PORT_0);
	mid9555Read(I2C_GPIO_9555_1_8, I2C_9555_OUTPUT_PORT_1);
	mid9555Read(I2C_GPIO_9555_2_8, I2C_9555_OUTPUT_PORT_0);
	mid9555Read(I2C_GPIO_9555_2_8, I2C_9555_OUTPUT_PORT_1);
	mid9555Read(I2C_GPIO_9555_3_8, I2C_9555_OUTPUT_PORT_0);
	mid9555Read(I2C_GPIO_9555_3_8, I2C_9555_OUTPUT_PORT_1);
	
	mid9545ChSet(I2C_GPIO_9545_CHANNEL_1);
	
	mid9555Read(I2C_GPIO_9555_4_8, I2C_9555_OUTPUT_PORT_0);
	mid9555Read(I2C_GPIO_9555_4_8, I2C_9555_OUTPUT_PORT_1);
	mid9555Read(I2C_GPIO_9555_5_8, I2C_9555_OUTPUT_PORT_0);
	mid9555Read(I2C_GPIO_9555_5_8, I2C_9555_OUTPUT_PORT_1);
	
	mid9545ChSet(I2C_GPIO_9545_CHANNEL_2);
	
	mid9555Read(I2C_GPIO_9555_6_8, I2C_9555_OUTPUT_PORT_0);
	mid9555Read(I2C_GPIO_9555_6_8, I2C_9555_OUTPUT_PORT_1);
	mid9555Read(I2C_GPIO_9555_7_8, I2C_9555_OUTPUT_PORT_0);
	mid9555Read(I2C_GPIO_9555_7_8, I2C_9555_OUTPUT_PORT_1);
	mid9555Read(I2C_GPIO_9555_8_8, I2C_9555_OUTPUT_PORT_0);
	mid9555Read(I2C_GPIO_9555_8_8, I2C_9555_OUTPUT_PORT_1);
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         midplaneI2cGpioPinDefine
*
* SUMMARY:      set 9555 pins power up configuration
*
* SCOPE:        public
*
* DESCRIPTION:  set 9555 pins power up configuration
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID midplaneI2cGpioPinDefine(VOID)
{
    if(midplaneTypeVerification())
	{	//Quanta midplane
        //printk("[MIDPLANE] MATTERHORN test midplane config setting... \n");
		
		/****************************/
		/**  Define I/O Direction  **/
		/****************************/
		
		midplaneI2cGpioConfig(I2C_GPIO_9555_1, I2C_GPIO_9545_CHANNEL_0, 
								I2C_9555_1_PORT0_CONF_M, I2C_9555_1_PORT1_CONF_M);
		//printk("[MIDPLANE] 9555_1 is set... \n");
		
		midplaneI2cGpioConfig(I2C_GPIO_9555_2, I2C_GPIO_9545_CHANNEL_0, 
								I2C_9555_2_PORT0_CONF_M, I2C_9555_2_PORT1_CONF_M);
		//printk("[MIDPLANE] 9555_2 is set... \n");
						
		midplaneI2cGpioConfig(I2C_GPIO_9555_3, I2C_GPIO_9545_CHANNEL_0, 
								I2C_9555_3_PORT0_CONF_M, I2C_9555_3_PORT1_CONF_M);
		//printk("[MIDPLANE] 9555_3 is set... \n");
				
		midplaneI2cGpioConfig(I2C_GPIO_9555_4, I2C_GPIO_9545_CHANNEL_1, 
								I2C_9555_4_PORT0_CONF_M, I2C_9555_4_PORT1_CONF_M);
		//printk("[MIDPLANE] 9555_4 is set... \n");
				
		midplaneI2cGpioConfig(I2C_GPIO_9555_5, I2C_GPIO_9545_CHANNEL_1, 
								I2C_9555_5_PORT0_CONF_M, I2C_9555_5_PORT1_CONF_M);
		//printk("[MIDPLANE] 9555_5 is set... \n");
				
		midplaneI2cGpioConfig(I2C_GPIO_9555_6, I2C_GPIO_9545_CHANNEL_2, 
								I2C_9555_6_PORT0_CONF_M, I2C_9555_6_PORT1_CONF_M);
		//printk("[MIDPLANE] 9555_6 is set... \n");
				
		midplaneI2cGpioConfig(I2C_GPIO_9555_7, I2C_GPIO_9545_CHANNEL_2, 
								I2C_9555_7_PORT0_CONF_M, I2C_9555_7_PORT1_CONF_M);
		//printk("[MIDPLANE] 9555_7 is set... \n");
				
		midplaneI2cGpioConfig(I2C_GPIO_9555_8, I2C_GPIO_9545_CHANNEL_2, 
								I2C_9555_8_PORT0_CONF_M, I2C_9555_8_PORT1_CONF_M); 
		//printk("[MIDPLANE] 9555_8 is set... \n");  
		
	
        /*****************************************/
        /**  Power Supply & BBU Infomation Pin  **/
        /*****************************************/
        /** PWR_INPL_0_L **/
        PWR_INPL_0_L.I2C_9545_channel =        I2C_GPIO_9545_CHANNEL_2;
        PWR_INPL_0_L.I2C_9555_address =        I2C_GPIO_9555_7;
        PWR_INPL_0_L.I2C_9555_port =           I2C_9555_OUTPUT_PORT_1;
        PWR_INPL_0_L.I2C_9555_pin =            I2C_9555_P3_BIT;
        PWR_INPL_0_L.I2C_9555_direction =      I2C_9555_D_OUT;
        
        /** PWR_INPL_1_L **/
        PWR_INPL_1_L.I2C_9545_channel =        I2C_GPIO_9545_CHANNEL_2;
        PWR_INPL_1_L.I2C_9555_address =        I2C_GPIO_9555_7;
        PWR_INPL_1_L.I2C_9555_port =           I2C_9555_OUTPUT_PORT_1;
        PWR_INPL_1_L.I2C_9555_pin =            I2C_9555_P4_BIT;
        PWR_INPL_1_L.I2C_9555_direction =      I2C_9555_D_OUT;
        
        /** PS0_FAULT_L **/
        PS0_FAULT_L.I2C_9545_channel =         I2C_GPIO_9545_CHANNEL_2;
        PS0_FAULT_L.I2C_9555_address =         I2C_GPIO_9555_6;
        PS0_FAULT_L.I2C_9555_port =            I2C_9555_OUTPUT_PORT_1;
        PS0_FAULT_L.I2C_9555_pin =             I2C_9555_P2_BIT;
        PS0_FAULT_L.I2C_9555_direction =       I2C_9555_D_OUT;
        
        /** PS1_FAULT_L **/
        PS1_FAULT_L.I2C_9545_channel =         I2C_GPIO_9545_CHANNEL_2;
        PS1_FAULT_L.I2C_9555_address =         I2C_GPIO_9555_6;
        PS1_FAULT_L.I2C_9555_port =            I2C_9555_OUTPUT_PORT_1;
        PS1_FAULT_L.I2C_9555_pin =             I2C_9555_P3_BIT;
        PS1_FAULT_L.I2C_9555_direction =       I2C_9555_D_OUT;
        
        /** VCC_BBU_ENABLE **/
        VCC_BBU_ENABLE.I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_2;
        VCC_BBU_ENABLE.I2C_9555_address =      I2C_GPIO_9555_7;
        VCC_BBU_ENABLE.I2C_9555_port =         I2C_9555_OUTPUT_PORT_0;
        VCC_BBU_ENABLE.I2C_9555_pin =          I2C_9555_P0_BIT;
        VCC_BBU_ENABLE.I2C_9555_direction =    I2C_9555_D_OUT;
		
		/** VCC_BBU_SWITCH_ENABLE **/
        VCC_BBU_SWITCH_ENABLE.I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_2;
        VCC_BBU_SWITCH_ENABLE.I2C_9555_address =      I2C_GPIO_9555_8;
        VCC_BBU_SWITCH_ENABLE.I2C_9555_port =         I2C_9555_INPUT_PORT_0;
        VCC_BBU_SWITCH_ENABLE.I2C_9555_pin =          I2C_9555_P7_BIT;
        VCC_BBU_SWITCH_ENABLE.I2C_9555_direction =    I2C_9555_D_IN;
        
        /** VCC_PS_ENABLE **/
        VCC_PS_ENABLE.I2C_9545_channel =       I2C_GPIO_9545_CHANNEL_2;
        VCC_PS_ENABLE.I2C_9555_address =       I2C_GPIO_9555_8;
        VCC_PS_ENABLE.I2C_9555_port =          I2C_9555_OUTPUT_PORT_1;
        VCC_PS_ENABLE.I2C_9555_pin =           I2C_9555_P7_BIT;
        VCC_PS_ENABLE.I2C_9555_direction =     I2C_9555_D_OUT;
        
		/** VCC_PS_ALT_ENABLE **/
        VCC_PS_ALT_ENABLE.I2C_9545_channel =   I2C_GPIO_9545_CHANNEL_2;
        VCC_PS_ALT_ENABLE.I2C_9555_address =   I2C_GPIO_9555_8;
        VCC_PS_ALT_ENABLE.I2C_9555_port =      I2C_9555_OUTPUT_PORT_1;
        VCC_PS_ALT_ENABLE.I2C_9555_pin =       I2C_9555_P6_BIT;
        VCC_PS_ALT_ENABLE.I2C_9555_direction = I2C_9555_D_OUT;
		
        /** BBU_INPLACE_L **/
        BBU_INPLACE_L.I2C_9545_channel =       I2C_GPIO_9545_CHANNEL_2;
        BBU_INPLACE_L.I2C_9555_address =       I2C_GPIO_9555_8;
        BBU_INPLACE_L.I2C_9555_port =          I2C_9555_OUTPUT_PORT_1;
        BBU_INPLACE_L.I2C_9555_pin =           I2C_9555_P4_BIT;
        BBU_INPLACE_L.I2C_9555_direction =     I2C_9555_D_OUT;
		
		/** BBU_ENABLE **/
        BBU_ENABLE.I2C_9545_channel =			I2C_GPIO_9545_CHANNEL_2;
        BBU_ENABLE.I2C_9555_address =			I2C_GPIO_9555_8;
        BBU_ENABLE.I2C_9555_port =				I2C_9555_INPUT_PORT_1;
        BBU_ENABLE.I2C_9555_pin =				I2C_9555_P2_BIT;
        BBU_ENABLE.I2C_9555_direction =			I2C_9555_D_IN;
		
		/** BBU_DISCHARGE **/
        BBU_DISCHARGE.I2C_9545_channel =		I2C_GPIO_9545_CHANNEL_2;
        BBU_DISCHARGE.I2C_9555_address =		I2C_GPIO_9555_8;
        BBU_DISCHARGE.I2C_9555_port =			I2C_9555_INPUT_PORT_1;
        BBU_DISCHARGE.I2C_9555_pin =			I2C_9555_P3_BIT;
        BBU_DISCHARGE.I2C_9555_direction =		I2C_9555_D_IN;
        
        /** VCC12_INPUT **/
        VCC12_INPUT.I2C_9545_channel =         I2C_GPIO_9545_CHANNEL_2;
        VCC12_INPUT.I2C_9555_address =         I2C_GPIO_9555_6;
        VCC12_INPUT.I2C_9555_port =            I2C_9555_INPUT_PORT_1;
        VCC12_INPUT.I2C_9555_pin =             I2C_9555_P7_BIT;
        VCC12_INPUT.I2C_9555_direction =       I2C_9555_D_IN;
		
		/** PS0_CONTROL_L_BP1 **/
        PS0_CONTROL_L_BP1.I2C_9545_channel =	I2C_GPIO_9545_CHANNEL_2;
        PS0_CONTROL_L_BP1.I2C_9555_address =	I2C_GPIO_9555_8;
        PS0_CONTROL_L_BP1.I2C_9555_port =		I2C_9555_INPUT_PORT_0;
        PS0_CONTROL_L_BP1.I2C_9555_pin =		I2C_9555_P0_BIT;
        PS0_CONTROL_L_BP1.I2C_9555_direction =	I2C_9555_D_IN;
		
		/** AC_GOOD **/
        AC_GOOD.I2C_9545_channel =				I2C_GPIO_9545_CHANNEL_2;
        AC_GOOD.I2C_9555_address =				I2C_GPIO_9555_8;
        AC_GOOD.I2C_9555_port =					I2C_9555_OUTPUT_PORT_0;
        AC_GOOD.I2C_9555_pin =					I2C_9555_P5_BIT;
        AC_GOOD.I2C_9555_direction =			I2C_9555_D_OUT;
        
        /************************************/
        /**  Subsystem LED Infomation Pin  **/
        /************************************/
        /** SUM_FLT_LED_L **/
        SUMMARY_FLT_LED_L.I2C_9545_channel =    I2C_GPIO_9545_CHANNEL_2;
        SUMMARY_FLT_LED_L.I2C_9555_address =    I2C_GPIO_9555_6;
        SUMMARY_FLT_LED_L.I2C_9555_port =       I2C_9555_INPUT_PORT_0;
        SUMMARY_FLT_LED_L.I2C_9555_pin =        I2C_9555_P1_BIT;
        SUMMARY_FLT_LED_L.I2C_9555_direction =  I2C_9555_D_IN;
        
        /** IDENTIFY_LED_L **/
        BOX_ID_LED_L.I2C_9545_channel =         I2C_GPIO_9545_CHANNEL_2;
        BOX_ID_LED_L.I2C_9555_address =         I2C_GPIO_9555_6;
        BOX_ID_LED_L.I2C_9555_port =            I2C_9555_INPUT_PORT_0;
        BOX_ID_LED_L.I2C_9555_pin =             I2C_9555_P0_BIT;
        BOX_ID_LED_L.I2C_9555_direction =       I2C_9555_D_IN;
		
		/** BOX_OVER_TEMP_LED_L **/
        BOX_OVER_TEMP_LED_L.I2C_9545_channel =		I2C_GPIO_9545_CHANNEL_2;
        BOX_OVER_TEMP_LED_L.I2C_9555_address = 		I2C_GPIO_9555_6;
        BOX_OVER_TEMP_LED_L.I2C_9555_port =			I2C_9555_INPUT_PORT_0;
        BOX_OVER_TEMP_LED_L.I2C_9555_pin =			I2C_9555_P2_BIT;
        BOX_OVER_TEMP_LED_L.I2C_9555_direction =	    I2C_9555_D_IN;
		
		/** PS0_RTR_LED_L **/
        PS0_RTR_LED_L.I2C_9545_channel =		I2C_GPIO_9545_CHANNEL_2;
        PS0_RTR_LED_L.I2C_9555_address = 		I2C_GPIO_9555_6;
        PS0_RTR_LED_L.I2C_9555_port =			I2C_9555_INPUT_PORT_1;
        PS0_RTR_LED_L.I2C_9555_pin =			I2C_9555_P0_BIT;
        PS0_RTR_LED_L.I2C_9555_direction =		I2C_9555_D_IN;
		
		/** PS1_RTR_LED_L **/
        PS1_RTR_LED_L.I2C_9545_channel =		I2C_GPIO_9545_CHANNEL_2;
        PS1_RTR_LED_L.I2C_9555_address = 		I2C_GPIO_9555_6;
        PS1_RTR_LED_L.I2C_9555_port =			I2C_9555_INPUT_PORT_1;
        PS1_RTR_LED_L.I2C_9555_pin =			I2C_9555_P1_BIT;
        PS1_RTR_LED_L.I2C_9555_direction =		I2C_9555_D_IN;
        
        /********************************/
        /**  Subsystem Infomation Pin  **/
        /********************************/
        /** CONT_POSN_L **/
        CONT_POSN_L.I2C_9545_channel =          I2C_GPIO_9545_CHANNEL_2;
        CONT_POSN_L.I2C_9555_address =          I2C_GPIO_9555_7;
        CONT_POSN_L.I2C_9555_port =             I2C_9555_OUTPUT_PORT_1;
        CONT_POSN_L.I2C_9555_pin =              I2C_9555_P5_BIT;
        CONT_POSN_L.I2C_9555_direction =        I2C_9555_D_OUT;
        
        /** HARNESS_INPL_L **/
        HARNESS_INPL_L.I2C_9545_channel =       I2C_GPIO_9545_CHANNEL_2;
        HARNESS_INPL_L.I2C_9555_address =       I2C_GPIO_9555_7;
        HARNESS_INPL_L.I2C_9555_port =          I2C_9555_OUTPUT_PORT_0;
        HARNESS_INPL_L.I2C_9555_pin =           I2C_9555_P6_BIT;
        HARNESS_INPL_L.I2C_9555_direction =     I2C_9555_D_OUT;
        
        /** ALARM_MUTE_L **/
        ALARM_MUTE_L.I2C_9545_channel =      	I2C_GPIO_9545_CHANNEL_2;
        ALARM_MUTE_L.I2C_9555_address =      	I2C_GPIO_9555_7;
        ALARM_MUTE_L.I2C_9555_port =         	I2C_9555_OUTPUT_PORT_0;
        ALARM_MUTE_L.I2C_9555_pin =         	I2C_9555_P5_BIT;
        ALARM_MUTE_L.I2C_9555_direction =    	I2C_9555_D_OUT;
        
        /** AUD_ALARM_OUT **/
        AUD_ALARM_OUT.I2C_9545_channel =        I2C_GPIO_9545_CHANNEL_2;
        AUD_ALARM_OUT.I2C_9555_address =        I2C_GPIO_9555_6;
        AUD_ALARM_OUT.I2C_9555_port =           I2C_9555_INPUT_PORT_0;
        AUD_ALARM_OUT.I2C_9555_pin =            I2C_9555_P3_BIT;
        AUD_ALARM_OUT.I2C_9555_direction =      I2C_9555_D_IN;
        
        /** SS_ID **/
        SS_ID_3.I2C_9545_channel =        		I2C_GPIO_9545_CHANNEL_2;
        SS_ID_3.I2C_9555_address =        		I2C_GPIO_9555_7;
        SS_ID_3.I2C_9555_port =          		I2C_9555_OUTPUT_PORT_0;
        SS_ID_3.I2C_9555_pin =            		I2C_9555_P4_BIT;
        SS_ID_3.I2C_9555_direction =      		I2C_9555_D_OUT;
        SS_ID_2.I2C_9545_channel =        		I2C_GPIO_9545_CHANNEL_2;
        SS_ID_2.I2C_9555_address =        		I2C_GPIO_9555_7;
        SS_ID_2.I2C_9555_port =           		I2C_9555_OUTPUT_PORT_0;
        SS_ID_2.I2C_9555_pin =            		I2C_9555_P3_BIT;
        SS_ID_2.I2C_9555_direction =      		I2C_9555_D_OUT;
        SS_ID_1.I2C_9545_channel =        		I2C_GPIO_9545_CHANNEL_2;
        SS_ID_1.I2C_9555_address =        		I2C_GPIO_9555_7;
        SS_ID_1.I2C_9555_port =           		I2C_9555_OUTPUT_PORT_0;
        SS_ID_1.I2C_9555_pin =            		I2C_9555_P2_BIT;
        SS_ID_1.I2C_9555_direction =      		I2C_9555_D_OUT;
        SS_ID_0.I2C_9545_channel =        		I2C_GPIO_9545_CHANNEL_2;
        SS_ID_0.I2C_9555_address =        		I2C_GPIO_9555_7;
        SS_ID_0.I2C_9555_port =           		I2C_9555_OUTPUT_PORT_0;
        SS_ID_0.I2C_9555_pin =            		I2C_9555_P1_BIT;
        SS_ID_0.I2C_9555_direction =      		I2C_9555_D_OUT;
		
		/** MP_I2C1_RST_L **/
        MP_I2C1_RST_L.I2C_9545_channel =       I2C_GPIO_9545_CHANNEL_2;
        MP_I2C1_RST_L.I2C_9555_address =       I2C_GPIO_9555_6;
        MP_I2C1_RST_L.I2C_9555_port =          I2C_9555_INPUT_PORT_1;
        MP_I2C1_RST_L.I2C_9555_pin =           I2C_9555_P5_BIT;
        MP_I2C1_RST_L.I2C_9555_direction =     I2C_9555_D_IN;
		
		/** MP_I2C2_RST_L **/
        MP_I2C2_RST_L.I2C_9545_channel =       I2C_GPIO_9545_CHANNEL_2;
        MP_I2C2_RST_L.I2C_9555_address =       I2C_GPIO_9555_6;
        MP_I2C2_RST_L.I2C_9555_port =          I2C_9555_INPUT_PORT_0;
        MP_I2C2_RST_L.I2C_9555_pin =           I2C_9555_P6_BIT;
        MP_I2C2_RST_L.I2C_9555_direction =     I2C_9555_D_IN;
		
		/** CARD_IO_TEST **/
        CARD_IO_TEST.I2C_9545_channel =        I2C_GPIO_9545_CHANNEL_2;
        CARD_IO_TEST.I2C_9555_address =        I2C_GPIO_9555_7;
        CARD_IO_TEST.I2C_9555_port =           I2C_9555_INPUT_PORT_1;
        CARD_IO_TEST.I2C_9555_pin =            I2C_9555_P6_BIT;
        CARD_IO_TEST.I2C_9555_direction =      I2C_9555_D_OUT;
		
		/** SYS_DEF_HP_4_N **/
        SYS_DEF_HP_4_N.I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_2;
        SYS_DEF_HP_4_N.I2C_9555_address =      I2C_GPIO_9555_8;
        SYS_DEF_HP_4_N.I2C_9555_port =         I2C_9555_INPUT_PORT_0;
        SYS_DEF_HP_4_N.I2C_9555_pin =          I2C_9555_P1_BIT;
        SYS_DEF_HP_4_N.I2C_9555_direction =    I2C_9555_D_IN;
        
        /**********************************/
        /**  Inter-Board Infomation Pin  **/
        /**********************************/
        /** ALT_NMI_IN_L **/
        ALT_NMI_IN_L.I2C_9545_channel =        I2C_GPIO_9545_CHANNEL_2;
        ALT_NMI_IN_L.I2C_9555_address =        I2C_GPIO_9555_7;
        ALT_NMI_IN_L.I2C_9555_port =           I2C_9555_OUTPUT_PORT_1;
        ALT_NMI_IN_L.I2C_9555_pin =            I2C_9555_P1_BIT;
        ALT_NMI_IN_L.I2C_9555_direction =      I2C_9555_D_OUT;
        
        /** ALT_NMI_L **/
        ALT_NMI_L.I2C_9545_channel =           I2C_GPIO_9545_CHANNEL_2;
        ALT_NMI_L.I2C_9555_address =           I2C_GPIO_9555_6;
        ALT_NMI_L.I2C_9555_port =              I2C_9555_INPUT_PORT_0;
        ALT_NMI_L.I2C_9555_pin =               I2C_9555_P6_BIT;
        ALT_NMI_L.I2C_9555_direction =         I2C_9555_D_IN;
    
        /** RDY_RMV_LED_IN **/
        RDY_RMV_LED_IN.I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_2;
        RDY_RMV_LED_IN.I2C_9555_address =      I2C_GPIO_9555_7;
        RDY_RMV_LED_IN.I2C_9555_port =         I2C_9555_OUTPUT_PORT_1;
        RDY_RMV_LED_IN.I2C_9555_pin =          I2C_9555_P7_BIT;
        RDY_RMV_LED_IN.I2C_9555_direction =    I2C_9555_D_OUT;
		
        /** ALT_RTR_LED_L **/
        ALT_RTR_LED_L.I2C_9545_channel =       I2C_GPIO_9545_CHANNEL_2;
        ALT_RTR_LED_L.I2C_9555_address =       I2C_GPIO_9555_6;
        ALT_RTR_LED_L.I2C_9555_port =          I2C_9555_INPUT_PORT_1;
        ALT_RTR_LED_L.I2C_9555_pin =           I2C_9555_P4_BIT;
        ALT_RTR_LED_L.I2C_9555_direction =     I2C_9555_D_IN;
		
        /** ALT_FW_RUNNING_L **/
        ALT_FW_RUNNING_L.I2C_9545_channel =    I2C_GPIO_9545_CHANNEL_2;
        ALT_FW_RUNNING_L.I2C_9555_address =    I2C_GPIO_9555_7;
        ALT_FW_RUNNING_L.I2C_9555_port =       I2C_9555_OUTPUT_PORT_0;
        ALT_FW_RUNNING_L.I2C_9555_pin =        I2C_9555_P7_BIT;
        ALT_FW_RUNNING_L.I2C_9555_direction =  I2C_9555_D_OUT;
        
        /** FW_RUNNING_L **/
        FW_RUNNING_L.I2C_9545_channel =        I2C_GPIO_9545_CHANNEL_2;
        FW_RUNNING_L.I2C_9555_address =        I2C_GPIO_9555_6;
        FW_RUNNING_L.I2C_9555_port =           I2C_9555_INPUT_PORT_0;
        FW_RUNNING_L.I2C_9555_pin =            I2C_9555_P4_BIT;
        FW_RUNNING_L.I2C_9555_direction =      I2C_9555_D_IN;
        
        /** ALT_INPL_L **/
        ALT_INPL_L.I2C_9545_channel =          I2C_GPIO_9545_CHANNEL_2;
        ALT_INPL_L.I2C_9555_address =          I2C_GPIO_9555_7;
        ALT_INPL_L.I2C_9555_port =             I2C_9555_OUTPUT_PORT_1;
        ALT_INPL_L.I2C_9555_pin =              I2C_9555_P0_BIT;
        ALT_INPL_L.I2C_9555_direction =        I2C_9555_D_OUT;
        
        /** CTRLR_INPL_L **/
        INPL_L.I2C_9545_channel =              I2C_GPIO_9545_CHANNEL_2;
        INPL_L.I2C_9555_address =              I2C_GPIO_9555_6;
        INPL_L.I2C_9555_port =                 I2C_9555_INPUT_PORT_0;
        INPL_L.I2C_9555_pin =                  I2C_9555_P5_BIT;
        INPL_L.I2C_9555_direction =            I2C_9555_D_IN;

        /** ALT_RAID_ESM_L **/
        ALT_RAID_ESM_L.I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_2;
        ALT_RAID_ESM_L.I2C_9555_address =      I2C_GPIO_9555_8;
        ALT_RAID_ESM_L.I2C_9555_port =         I2C_9555_OUTPUT_PORT_0;
        ALT_RAID_ESM_L.I2C_9555_pin =          I2C_9555_P3_BIT;
        ALT_RAID_ESM_L.I2C_9555_direction =    I2C_9555_D_OUT;
        
        /** RAID_ESM_L **/
        RAID_ESM_L.I2C_9545_channel =          I2C_GPIO_9545_CHANNEL_2;
        RAID_ESM_L.I2C_9555_address =          I2C_GPIO_9555_8;
        RAID_ESM_L.I2C_9555_port =             I2C_9555_INPUT_PORT_0;
        RAID_ESM_L.I2C_9555_pin =              I2C_9555_P2_BIT;
        RAID_ESM_L.I2C_9555_direction =        I2C_9555_D_IN;
            
        /** ALT_RESET_IN_L **/
        ALT_RESET_IN_L.I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_2;
        ALT_RESET_IN_L.I2C_9555_address =      I2C_GPIO_9555_7;
        ALT_RESET_IN_L.I2C_9555_port =         I2C_9555_OUTPUT_PORT_1;
        ALT_RESET_IN_L.I2C_9555_pin =          I2C_9555_P2_BIT;
        ALT_RESET_IN_L.I2C_9555_direction =    I2C_9555_D_OUT;
        
        /** ALT_RESET_L **/
        ALT_RESET_L.I2C_9545_channel =         I2C_GPIO_9545_CHANNEL_2;
        ALT_RESET_L.I2C_9555_address =         I2C_GPIO_9555_6;
        ALT_RESET_L.I2C_9555_port =            I2C_9555_INPUT_PORT_0;
        ALT_RESET_L.I2C_9555_pin =             I2C_9555_P7_BIT;
        ALT_RESET_L.I2C_9555_direction =       I2C_9555_D_IN;    
        
        
        /****************************/
        /**  Drive Infomation Pin  **/
        /****************************/
        /** Drive Fault LED **/
        D_FAULT_LED_L[0].I2C_9545_channel =       I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[0].I2C_9555_address =       I2C_GPIO_9555_1;
        D_FAULT_LED_L[0].I2C_9555_port =          I2C_9555_INPUT_PORT_0;
        D_FAULT_LED_L[0].I2C_9555_pin =           I2C_9555_P0_BIT;
        D_FAULT_LED_L[0].I2C_9555_direction =     I2C_9555_D_IN;
        
        D_FAULT_LED_L[1].I2C_9545_channel =       I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[1].I2C_9555_address =       I2C_GPIO_9555_1;
        D_FAULT_LED_L[1].I2C_9555_port =          I2C_9555_INPUT_PORT_0;
        D_FAULT_LED_L[1].I2C_9555_pin =           I2C_9555_P1_BIT;
        D_FAULT_LED_L[1].I2C_9555_direction =     I2C_9555_D_IN;
        
        D_FAULT_LED_L[2].I2C_9545_channel =       I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[2].I2C_9555_address =       I2C_GPIO_9555_1;
        D_FAULT_LED_L[2].I2C_9555_port =          I2C_9555_INPUT_PORT_0;
        D_FAULT_LED_L[2].I2C_9555_pin =           I2C_9555_P2_BIT;
        D_FAULT_LED_L[2].I2C_9555_direction =     I2C_9555_D_IN;
        
        D_FAULT_LED_L[3].I2C_9545_channel =       I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[3].I2C_9555_address =       I2C_GPIO_9555_1;
        D_FAULT_LED_L[3].I2C_9555_port =          I2C_9555_INPUT_PORT_0;
        D_FAULT_LED_L[3].I2C_9555_pin =           I2C_9555_P3_BIT;
        D_FAULT_LED_L[3].I2C_9555_direction =     I2C_9555_D_IN;
        
        D_FAULT_LED_L[4].I2C_9545_channel =       I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[4].I2C_9555_address =       I2C_GPIO_9555_1;
        D_FAULT_LED_L[4].I2C_9555_port =          I2C_9555_INPUT_PORT_0;
        D_FAULT_LED_L[4].I2C_9555_pin =           I2C_9555_P4_BIT;
        D_FAULT_LED_L[4].I2C_9555_direction =     I2C_9555_D_IN;
        
        D_FAULT_LED_L[5].I2C_9545_channel =       I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[5].I2C_9555_address =       I2C_GPIO_9555_1;
        D_FAULT_LED_L[5].I2C_9555_port =          I2C_9555_INPUT_PORT_0;
        D_FAULT_LED_L[5].I2C_9555_pin =           I2C_9555_P5_BIT;
        D_FAULT_LED_L[5].I2C_9555_direction =     I2C_9555_D_IN;
        
        D_FAULT_LED_L[6].I2C_9545_channel =       I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[6].I2C_9555_address =       I2C_GPIO_9555_1;
        D_FAULT_LED_L[6].I2C_9555_port =          I2C_9555_INPUT_PORT_0;
        D_FAULT_LED_L[6].I2C_9555_pin =           I2C_9555_P6_BIT;
        D_FAULT_LED_L[6].I2C_9555_direction =     I2C_9555_D_IN;
        
        D_FAULT_LED_L[7].I2C_9545_channel =       I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[7].I2C_9555_address =       I2C_GPIO_9555_1;
        D_FAULT_LED_L[7].I2C_9555_port =          I2C_9555_INPUT_PORT_0;
        D_FAULT_LED_L[7].I2C_9555_pin =           I2C_9555_P7_BIT;
        D_FAULT_LED_L[7].I2C_9555_direction =     I2C_9555_D_IN;
        
        D_FAULT_LED_L[8].I2C_9545_channel =       I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[8].I2C_9555_address =       I2C_GPIO_9555_1;
        D_FAULT_LED_L[8].I2C_9555_port =          I2C_9555_INPUT_PORT_1;
        D_FAULT_LED_L[8].I2C_9555_pin =           I2C_9555_P0_BIT;
        D_FAULT_LED_L[8].I2C_9555_direction =     I2C_9555_D_IN;
        
        D_FAULT_LED_L[9].I2C_9545_channel =       I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[9].I2C_9555_address =       I2C_GPIO_9555_1;
        D_FAULT_LED_L[9].I2C_9555_port =          I2C_9555_INPUT_PORT_1;
        D_FAULT_LED_L[9].I2C_9555_pin =           I2C_9555_P1_BIT;
        D_FAULT_LED_L[9].I2C_9555_direction =     I2C_9555_D_IN;
        
        D_FAULT_LED_L[10].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[10].I2C_9555_address =      I2C_GPIO_9555_1;
        D_FAULT_LED_L[10].I2C_9555_port =         I2C_9555_INPUT_PORT_1;
        D_FAULT_LED_L[10].I2C_9555_pin =          I2C_9555_P2_BIT;
        D_FAULT_LED_L[10].I2C_9555_direction =    I2C_9555_D_IN;
        
        D_FAULT_LED_L[11].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[11].I2C_9555_address =      I2C_GPIO_9555_1;
        D_FAULT_LED_L[11].I2C_9555_port =         I2C_9555_INPUT_PORT_1;
        D_FAULT_LED_L[11].I2C_9555_pin =          I2C_9555_P3_BIT;
        D_FAULT_LED_L[11].I2C_9555_direction =    I2C_9555_D_IN;
        
        D_FAULT_LED_L[12].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[12].I2C_9555_address =      I2C_GPIO_9555_1;
        D_FAULT_LED_L[12].I2C_9555_port =         I2C_9555_INPUT_PORT_1;
        D_FAULT_LED_L[12].I2C_9555_pin =          I2C_9555_P4_BIT;
        D_FAULT_LED_L[12].I2C_9555_direction =    I2C_9555_D_IN;
        
        D_FAULT_LED_L[13].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[13].I2C_9555_address =      I2C_GPIO_9555_1;
        D_FAULT_LED_L[13].I2C_9555_port =         I2C_9555_INPUT_PORT_1;
        D_FAULT_LED_L[13].I2C_9555_pin =          I2C_9555_P5_BIT;
        D_FAULT_LED_L[13].I2C_9555_direction =    I2C_9555_D_IN;
        
        D_FAULT_LED_L[14].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[14].I2C_9555_address =      I2C_GPIO_9555_1;
        D_FAULT_LED_L[14].I2C_9555_port =         I2C_9555_INPUT_PORT_1;
        D_FAULT_LED_L[14].I2C_9555_pin =          I2C_9555_P6_BIT;
        D_FAULT_LED_L[14].I2C_9555_direction =    I2C_9555_D_IN;

        D_FAULT_LED_L[15].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[15].I2C_9555_address =      I2C_GPIO_9555_1;
        D_FAULT_LED_L[15].I2C_9555_port =         I2C_9555_INPUT_PORT_1;
        D_FAULT_LED_L[15].I2C_9555_pin =          I2C_9555_P7_BIT;
        D_FAULT_LED_L[15].I2C_9555_direction =    I2C_9555_D_IN;
		
		D_FAULT_LED_L[16].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[16].I2C_9555_address =      I2C_GPIO_9555_3;
        D_FAULT_LED_L[16].I2C_9555_port =         I2C_9555_INPUT_PORT_0;
        D_FAULT_LED_L[16].I2C_9555_pin =          I2C_9555_P0_BIT;
        D_FAULT_LED_L[16].I2C_9555_direction =    I2C_9555_D_IN;
        
        D_FAULT_LED_L[17].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[17].I2C_9555_address =      I2C_GPIO_9555_3;
        D_FAULT_LED_L[17].I2C_9555_port =         I2C_9555_INPUT_PORT_0;
        D_FAULT_LED_L[17].I2C_9555_pin =          I2C_9555_P1_BIT;
        D_FAULT_LED_L[17].I2C_9555_direction =    I2C_9555_D_IN;
        
        D_FAULT_LED_L[18].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[18].I2C_9555_address =      I2C_GPIO_9555_3;
        D_FAULT_LED_L[18].I2C_9555_port =         I2C_9555_INPUT_PORT_0;
        D_FAULT_LED_L[18].I2C_9555_pin =          I2C_9555_P2_BIT;
        D_FAULT_LED_L[18].I2C_9555_direction =    I2C_9555_D_IN;
        
        D_FAULT_LED_L[19].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[19].I2C_9555_address =      I2C_GPIO_9555_3;
        D_FAULT_LED_L[19].I2C_9555_port =         I2C_9555_INPUT_PORT_0;
        D_FAULT_LED_L[19].I2C_9555_pin =          I2C_9555_P3_BIT;
        D_FAULT_LED_L[19].I2C_9555_direction =    I2C_9555_D_IN;
        
        D_FAULT_LED_L[20].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[20].I2C_9555_address =      I2C_GPIO_9555_3;
        D_FAULT_LED_L[20].I2C_9555_port =         I2C_9555_INPUT_PORT_0;
        D_FAULT_LED_L[20].I2C_9555_pin =          I2C_9555_P4_BIT;
        D_FAULT_LED_L[20].I2C_9555_direction =    I2C_9555_D_IN;
        
        D_FAULT_LED_L[21].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[21].I2C_9555_address =      I2C_GPIO_9555_3;
        D_FAULT_LED_L[21].I2C_9555_port =         I2C_9555_INPUT_PORT_0;
        D_FAULT_LED_L[21].I2C_9555_pin =          I2C_9555_P5_BIT;
        D_FAULT_LED_L[21].I2C_9555_direction =    I2C_9555_D_IN;
        
        D_FAULT_LED_L[22].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[22].I2C_9555_address =      I2C_GPIO_9555_3;
        D_FAULT_LED_L[22].I2C_9555_port =         I2C_9555_INPUT_PORT_0;
        D_FAULT_LED_L[22].I2C_9555_pin =          I2C_9555_P6_BIT;
        D_FAULT_LED_L[22].I2C_9555_direction =    I2C_9555_D_IN;

        D_FAULT_LED_L[23].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_FAULT_LED_L[23].I2C_9555_address =      I2C_GPIO_9555_3;
        D_FAULT_LED_L[23].I2C_9555_port =         I2C_9555_INPUT_PORT_0;
        D_FAULT_LED_L[23].I2C_9555_pin =          I2C_9555_P7_BIT;
        D_FAULT_LED_L[23].I2C_9555_direction =    I2C_9555_D_IN;
 
        /** Drive Replace LED **/
        D_REPL_LED_L[0].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[0].I2C_9555_address =      I2C_GPIO_9555_2;
        D_REPL_LED_L[0].I2C_9555_port =         I2C_9555_INPUT_PORT_0;
        D_REPL_LED_L[0].I2C_9555_pin =          I2C_9555_P0_BIT;
        D_REPL_LED_L[0].I2C_9555_direction =    I2C_9555_D_IN;
        
        D_REPL_LED_L[1].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[1].I2C_9555_address =      I2C_GPIO_9555_2;
        D_REPL_LED_L[1].I2C_9555_port =         I2C_9555_INPUT_PORT_0;
        D_REPL_LED_L[1].I2C_9555_pin =          I2C_9555_P1_BIT;
        D_REPL_LED_L[1].I2C_9555_direction =    I2C_9555_D_IN;
        
        D_REPL_LED_L[2].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[2].I2C_9555_address =      I2C_GPIO_9555_2;
        D_REPL_LED_L[2].I2C_9555_port =         I2C_9555_INPUT_PORT_0;
        D_REPL_LED_L[2].I2C_9555_pin =          I2C_9555_P2_BIT;
        D_REPL_LED_L[2].I2C_9555_direction =    I2C_9555_D_IN;
        
        D_REPL_LED_L[3].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[3].I2C_9555_address =      I2C_GPIO_9555_2;
        D_REPL_LED_L[3].I2C_9555_port =         I2C_9555_INPUT_PORT_0;
        D_REPL_LED_L[3].I2C_9555_pin =          I2C_9555_P3_BIT;
        D_REPL_LED_L[3].I2C_9555_direction =    I2C_9555_D_IN;
        
        D_REPL_LED_L[4].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[4].I2C_9555_address =      I2C_GPIO_9555_2;
        D_REPL_LED_L[4].I2C_9555_port =         I2C_9555_INPUT_PORT_0;
        D_REPL_LED_L[4].I2C_9555_pin =          I2C_9555_P4_BIT;
        D_REPL_LED_L[4].I2C_9555_direction =    I2C_9555_D_IN;
        
        D_REPL_LED_L[5].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[5].I2C_9555_address =      I2C_GPIO_9555_2;
        D_REPL_LED_L[5].I2C_9555_port =         I2C_9555_INPUT_PORT_0;
        D_REPL_LED_L[5].I2C_9555_pin =          I2C_9555_P5_BIT;
        D_REPL_LED_L[5].I2C_9555_direction =    I2C_9555_D_IN;
        
        D_REPL_LED_L[6].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[6].I2C_9555_address =      I2C_GPIO_9555_2;
        D_REPL_LED_L[6].I2C_9555_port =         I2C_9555_INPUT_PORT_0;
        D_REPL_LED_L[6].I2C_9555_pin =          I2C_9555_P6_BIT;
        D_REPL_LED_L[6].I2C_9555_direction =    I2C_9555_D_IN;
        
        D_REPL_LED_L[7].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[7].I2C_9555_address =      I2C_GPIO_9555_2;
        D_REPL_LED_L[7].I2C_9555_port =         I2C_9555_INPUT_PORT_0;
        D_REPL_LED_L[7].I2C_9555_pin =          I2C_9555_P7_BIT;
        D_REPL_LED_L[7].I2C_9555_direction =    I2C_9555_D_IN;
        
        D_REPL_LED_L[8].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[8].I2C_9555_address =      I2C_GPIO_9555_2;
        D_REPL_LED_L[8].I2C_9555_port =         I2C_9555_INPUT_PORT_1;
        D_REPL_LED_L[8].I2C_9555_pin =          I2C_9555_P0_BIT;
        D_REPL_LED_L[8].I2C_9555_direction =    I2C_9555_D_IN;
        
        D_REPL_LED_L[9].I2C_9545_channel =      I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[9].I2C_9555_address =      I2C_GPIO_9555_2;
        D_REPL_LED_L[9].I2C_9555_port =         I2C_9555_INPUT_PORT_1;
        D_REPL_LED_L[9].I2C_9555_pin =          I2C_9555_P1_BIT;
        D_REPL_LED_L[9].I2C_9555_direction =    I2C_9555_D_IN;
        
        D_REPL_LED_L[10].I2C_9545_channel =     I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[10].I2C_9555_address =     I2C_GPIO_9555_2;
        D_REPL_LED_L[10].I2C_9555_port =        I2C_9555_INPUT_PORT_1;
        D_REPL_LED_L[10].I2C_9555_pin =         I2C_9555_P2_BIT;
        D_REPL_LED_L[10].I2C_9555_direction =   I2C_9555_D_IN;
        
        D_REPL_LED_L[11].I2C_9545_channel =     I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[11].I2C_9555_address =     I2C_GPIO_9555_2;
        D_REPL_LED_L[11].I2C_9555_port =        I2C_9555_INPUT_PORT_1;
        D_REPL_LED_L[11].I2C_9555_pin =         I2C_9555_P3_BIT;
        D_REPL_LED_L[11].I2C_9555_direction =   I2C_9555_D_IN;
        
        D_REPL_LED_L[12].I2C_9545_channel =     I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[12].I2C_9555_address =     I2C_GPIO_9555_2;
        D_REPL_LED_L[12].I2C_9555_port =        I2C_9555_INPUT_PORT_1;
        D_REPL_LED_L[12].I2C_9555_pin =         I2C_9555_P4_BIT;
        D_REPL_LED_L[12].I2C_9555_direction =   I2C_9555_D_IN;
        
        D_REPL_LED_L[13].I2C_9545_channel =     I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[13].I2C_9555_address =     I2C_GPIO_9555_2;
        D_REPL_LED_L[13].I2C_9555_port =        I2C_9555_INPUT_PORT_1;
        D_REPL_LED_L[13].I2C_9555_pin =         I2C_9555_P5_BIT;
        D_REPL_LED_L[13].I2C_9555_direction =   I2C_9555_D_IN;
        
        D_REPL_LED_L[14].I2C_9545_channel =     I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[14].I2C_9555_address =     I2C_GPIO_9555_2;
        D_REPL_LED_L[14].I2C_9555_port =        I2C_9555_INPUT_PORT_1;
        D_REPL_LED_L[14].I2C_9555_pin =         I2C_9555_P6_BIT;
        D_REPL_LED_L[14].I2C_9555_direction =   I2C_9555_D_IN;

        D_REPL_LED_L[15].I2C_9545_channel =     I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[15].I2C_9555_address =     I2C_GPIO_9555_2;
        D_REPL_LED_L[15].I2C_9555_port =        I2C_9555_INPUT_PORT_1;
        D_REPL_LED_L[15].I2C_9555_pin =         I2C_9555_P7_BIT;
        D_REPL_LED_L[15].I2C_9555_direction =   I2C_9555_D_IN;

        D_REPL_LED_L[16].I2C_9545_channel =     I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[16].I2C_9555_address =     I2C_GPIO_9555_3;
        D_REPL_LED_L[16].I2C_9555_port =        I2C_9555_INPUT_PORT_1;
        D_REPL_LED_L[16].I2C_9555_pin =         I2C_9555_P0_BIT;
        D_REPL_LED_L[16].I2C_9555_direction =   I2C_9555_D_IN;
        
        D_REPL_LED_L[17].I2C_9545_channel =     I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[17].I2C_9555_address =     I2C_GPIO_9555_3;
        D_REPL_LED_L[17].I2C_9555_port =        I2C_9555_INPUT_PORT_1;
        D_REPL_LED_L[17].I2C_9555_pin =         I2C_9555_P1_BIT;
        D_REPL_LED_L[17].I2C_9555_direction =   I2C_9555_D_IN;
        
        D_REPL_LED_L[18].I2C_9545_channel =     I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[18].I2C_9555_address =     I2C_GPIO_9555_3;
        D_REPL_LED_L[18].I2C_9555_port =        I2C_9555_INPUT_PORT_1;
        D_REPL_LED_L[18].I2C_9555_pin =         I2C_9555_P2_BIT;
        D_REPL_LED_L[18].I2C_9555_direction =   I2C_9555_D_IN;
        
        D_REPL_LED_L[19].I2C_9545_channel =     I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[19].I2C_9555_address =     I2C_GPIO_9555_3;
        D_REPL_LED_L[19].I2C_9555_port =        I2C_9555_INPUT_PORT_1;
        D_REPL_LED_L[19].I2C_9555_pin =         I2C_9555_P3_BIT;
        D_REPL_LED_L[19].I2C_9555_direction =   I2C_9555_D_IN;
        
        D_REPL_LED_L[20].I2C_9545_channel =     I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[20].I2C_9555_address =     I2C_GPIO_9555_3;
        D_REPL_LED_L[20].I2C_9555_port =        I2C_9555_INPUT_PORT_1;
        D_REPL_LED_L[20].I2C_9555_pin =         I2C_9555_P4_BIT;
        D_REPL_LED_L[20].I2C_9555_direction =   I2C_9555_D_IN;
        
        D_REPL_LED_L[21].I2C_9545_channel =     I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[21].I2C_9555_address =     I2C_GPIO_9555_3;
        D_REPL_LED_L[21].I2C_9555_port =        I2C_9555_INPUT_PORT_1;
        D_REPL_LED_L[21].I2C_9555_pin =         I2C_9555_P5_BIT;
        D_REPL_LED_L[21].I2C_9555_direction =   I2C_9555_D_IN;
        
        D_REPL_LED_L[22].I2C_9545_channel =     I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[22].I2C_9555_address =     I2C_GPIO_9555_3;
        D_REPL_LED_L[22].I2C_9555_port =        I2C_9555_INPUT_PORT_1;
        D_REPL_LED_L[22].I2C_9555_pin =         I2C_9555_P6_BIT;
        D_REPL_LED_L[22].I2C_9555_direction =   I2C_9555_D_IN;

        D_REPL_LED_L[23].I2C_9545_channel =     I2C_GPIO_9545_CHANNEL_0;
        D_REPL_LED_L[23].I2C_9555_address =     I2C_GPIO_9555_3;
        D_REPL_LED_L[23].I2C_9555_port =        I2C_9555_INPUT_PORT_1;
        D_REPL_LED_L[23].I2C_9555_pin =         I2C_9555_P7_BIT;
        D_REPL_LED_L[23].I2C_9555_direction =   I2C_9555_D_IN;
     
        /** Drive Inplace **/
        D_INPL_L[0].I2C_9545_channel =          I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[0].I2C_9555_address =          I2C_GPIO_9555_4;
        D_INPL_L[0].I2C_9555_port =             I2C_9555_OUTPUT_PORT_0;
        D_INPL_L[0].I2C_9555_pin =              I2C_9555_P0_BIT;
        D_INPL_L[0].I2C_9555_direction =        I2C_9555_D_OUT;

        D_INPL_L[1].I2C_9545_channel =          I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[1].I2C_9555_address =          I2C_GPIO_9555_4;
        D_INPL_L[1].I2C_9555_port =             I2C_9555_OUTPUT_PORT_0;
        D_INPL_L[1].I2C_9555_pin =              I2C_9555_P1_BIT;
        D_INPL_L[1].I2C_9555_direction =        I2C_9555_D_OUT;
        
        D_INPL_L[2].I2C_9545_channel =          I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[2].I2C_9555_address =          I2C_GPIO_9555_4;
        D_INPL_L[2].I2C_9555_port =             I2C_9555_OUTPUT_PORT_0;
        D_INPL_L[2].I2C_9555_pin =              I2C_9555_P2_BIT;
        D_INPL_L[2].I2C_9555_direction =        I2C_9555_D_OUT;
        
        D_INPL_L[3].I2C_9545_channel =          I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[3].I2C_9555_address =          I2C_GPIO_9555_4;
        D_INPL_L[3].I2C_9555_port =             I2C_9555_OUTPUT_PORT_0;
        D_INPL_L[3].I2C_9555_pin =              I2C_9555_P3_BIT;
        D_INPL_L[3].I2C_9555_direction =        I2C_9555_D_OUT;
	
        D_INPL_L[4].I2C_9545_channel =          I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[4].I2C_9555_address =          I2C_GPIO_9555_4;
        D_INPL_L[4].I2C_9555_port =             I2C_9555_OUTPUT_PORT_0;
        D_INPL_L[4].I2C_9555_pin =              I2C_9555_P4_BIT;
        D_INPL_L[4].I2C_9555_direction =        I2C_9555_D_OUT;
        
        D_INPL_L[5].I2C_9545_channel =          I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[5].I2C_9555_address =          I2C_GPIO_9555_4;
        D_INPL_L[5].I2C_9555_port =             I2C_9555_OUTPUT_PORT_0;
        D_INPL_L[5].I2C_9555_pin =              I2C_9555_P5_BIT;
        D_INPL_L[5].I2C_9555_direction =        I2C_9555_D_OUT;
        
        D_INPL_L[6].I2C_9545_channel =          I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[6].I2C_9555_address =          I2C_GPIO_9555_4;
        D_INPL_L[6].I2C_9555_port =             I2C_9555_OUTPUT_PORT_0;
        D_INPL_L[6].I2C_9555_pin =              I2C_9555_P6_BIT;
        D_INPL_L[6].I2C_9555_direction =        I2C_9555_D_OUT;
            
        D_INPL_L[7].I2C_9545_channel =          I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[7].I2C_9555_address =          I2C_GPIO_9555_4;
        D_INPL_L[7].I2C_9555_port =             I2C_9555_OUTPUT_PORT_0;
        D_INPL_L[7].I2C_9555_pin =              I2C_9555_P7_BIT;
        D_INPL_L[7].I2C_9555_direction =        I2C_9555_D_OUT;
        
        D_INPL_L[8].I2C_9545_channel =          I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[8].I2C_9555_address =          I2C_GPIO_9555_4;
        D_INPL_L[8].I2C_9555_port =             I2C_9555_OUTPUT_PORT_1;
        D_INPL_L[8].I2C_9555_pin =              I2C_9555_P0_BIT;
        D_INPL_L[8].I2C_9555_direction =        I2C_9555_D_OUT;
        
        D_INPL_L[9].I2C_9545_channel =          I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[9].I2C_9555_address =          I2C_GPIO_9555_4;
        D_INPL_L[9].I2C_9555_port =             I2C_9555_OUTPUT_PORT_1;
        D_INPL_L[9].I2C_9555_pin =              I2C_9555_P1_BIT;
        D_INPL_L[9].I2C_9555_direction =        I2C_9555_D_OUT;
        
        D_INPL_L[10].I2C_9545_channel =         I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[10].I2C_9555_address =         I2C_GPIO_9555_4;
        D_INPL_L[10].I2C_9555_port =            I2C_9555_OUTPUT_PORT_1;
        D_INPL_L[10].I2C_9555_pin =             I2C_9555_P2_BIT;
        D_INPL_L[10].I2C_9555_direction =       I2C_9555_D_OUT;
        
        D_INPL_L[11].I2C_9545_channel =         I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[11].I2C_9555_address =         I2C_GPIO_9555_4;
        D_INPL_L[11].I2C_9555_port =            I2C_9555_OUTPUT_PORT_1;
        D_INPL_L[11].I2C_9555_pin =             I2C_9555_P3_BIT;
        D_INPL_L[11].I2C_9555_direction =       I2C_9555_D_OUT;
        
        D_INPL_L[12].I2C_9545_channel =         I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[12].I2C_9555_address =         I2C_GPIO_9555_4;
        D_INPL_L[12].I2C_9555_port =            I2C_9555_OUTPUT_PORT_1;
        D_INPL_L[12].I2C_9555_pin =             I2C_9555_P4_BIT;
        D_INPL_L[12].I2C_9555_direction =       I2C_9555_D_OUT;
        
        D_INPL_L[13].I2C_9545_channel =         I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[13].I2C_9555_address =         I2C_GPIO_9555_4;
        D_INPL_L[13].I2C_9555_port =            I2C_9555_OUTPUT_PORT_1;
        D_INPL_L[13].I2C_9555_pin =             I2C_9555_P5_BIT;
        D_INPL_L[13].I2C_9555_direction =       I2C_9555_D_OUT;
        
        D_INPL_L[14].I2C_9545_channel =         I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[14].I2C_9555_address =         I2C_GPIO_9555_4;
        D_INPL_L[14].I2C_9555_port =            I2C_9555_OUTPUT_PORT_1;
        D_INPL_L[14].I2C_9555_pin =             I2C_9555_P6_BIT;
        D_INPL_L[14].I2C_9555_direction =       I2C_9555_D_OUT;

        D_INPL_L[15].I2C_9545_channel =         I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[15].I2C_9555_address =         I2C_GPIO_9555_4;
        D_INPL_L[15].I2C_9555_port =            I2C_9555_OUTPUT_PORT_1;
        D_INPL_L[15].I2C_9555_pin =             I2C_9555_P7_BIT;
        D_INPL_L[15].I2C_9555_direction =       I2C_9555_D_OUT;  

        D_INPL_L[16].I2C_9545_channel =         I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[16].I2C_9555_address =         I2C_GPIO_9555_5;
        D_INPL_L[16].I2C_9555_port =            I2C_9555_OUTPUT_PORT_0;
        D_INPL_L[16].I2C_9555_pin =             I2C_9555_P0_BIT;
        D_INPL_L[16].I2C_9555_direction =       I2C_9555_D_OUT;
        
        D_INPL_L[17].I2C_9545_channel =         I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[17].I2C_9555_address =         I2C_GPIO_9555_5;
        D_INPL_L[17].I2C_9555_port =            I2C_9555_OUTPUT_PORT_0;
        D_INPL_L[17].I2C_9555_pin =             I2C_9555_P1_BIT;
        D_INPL_L[17].I2C_9555_direction =       I2C_9555_D_OUT;
        
        D_INPL_L[18].I2C_9545_channel =         I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[18].I2C_9555_address =         I2C_GPIO_9555_5;
        D_INPL_L[18].I2C_9555_port =            I2C_9555_OUTPUT_PORT_0;
        D_INPL_L[18].I2C_9555_pin =             I2C_9555_P2_BIT;
        D_INPL_L[18].I2C_9555_direction =       I2C_9555_D_OUT;
        
        D_INPL_L[19].I2C_9545_channel =         I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[19].I2C_9555_address =         I2C_GPIO_9555_5;
        D_INPL_L[19].I2C_9555_port =            I2C_9555_OUTPUT_PORT_0;
        D_INPL_L[19].I2C_9555_pin =             I2C_9555_P3_BIT;
        D_INPL_L[19].I2C_9555_direction =       I2C_9555_D_OUT;
        
        D_INPL_L[20].I2C_9545_channel =         I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[20].I2C_9555_address =         I2C_GPIO_9555_5;
        D_INPL_L[20].I2C_9555_port =            I2C_9555_OUTPUT_PORT_0;
        D_INPL_L[20].I2C_9555_pin =             I2C_9555_P4_BIT;
        D_INPL_L[20].I2C_9555_direction =       I2C_9555_D_OUT;
        
        D_INPL_L[21].I2C_9545_channel =         I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[21].I2C_9555_address =         I2C_GPIO_9555_5;
        D_INPL_L[21].I2C_9555_port =            I2C_9555_OUTPUT_PORT_0;
        D_INPL_L[21].I2C_9555_pin =             I2C_9555_P5_BIT;
        D_INPL_L[21].I2C_9555_direction =       I2C_9555_D_OUT;
        
        D_INPL_L[22].I2C_9545_channel =         I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[22].I2C_9555_address =         I2C_GPIO_9555_5;
        D_INPL_L[22].I2C_9555_port =            I2C_9555_OUTPUT_PORT_0;
        D_INPL_L[22].I2C_9555_pin =             I2C_9555_P6_BIT;
        D_INPL_L[22].I2C_9555_direction =       I2C_9555_D_OUT;

        D_INPL_L[23].I2C_9545_channel =         I2C_GPIO_9545_CHANNEL_1;
        D_INPL_L[23].I2C_9555_address =         I2C_GPIO_9555_5;
        D_INPL_L[23].I2C_9555_port =            I2C_9555_OUTPUT_PORT_0;
        D_INPL_L[23].I2C_9555_pin =             I2C_9555_P7_BIT;
        D_INPL_L[23].I2C_9555_direction =       I2C_9555_D_OUT;  
		
		printk("[MIDPLANE] PikesPeak/Soyuz test midplane config is set.\n");
		
		/**********************************/
		/**  Rewrite I/O default value  **/
		/*********************************/
		
		midplaneI2cGpioSetDefault();
		
		midplaneI2cGpioValueShow();
	}
}


/****************************************/
/* BBU Current Monitor INA209 Functions */
/****************************************/

/*******************************************************************************
* PROCEDURE
*
* NAME:         ina209CurrentMonitor
*
* SUMMARY:      read INA 209 current reading
*
* SCOPE:        public
*
* DESCRIPTION:  read INA 209 current reading
*
* RESTRICTIONS:
* 
* RETURNS:      current reading
*/
UINT16 ina209CurrentMonitor
 (
 UINT8 address
 )
{
	struct i2c_client *client = NULL;
    struct i2c_msg 		msg;
    UINT8  				bufOut, bufIn[2];
    INT32  				ret;

	// Get INA device
    //20120418 Luap : Masked (TAHOE_X86_PLATFORM)	
	//client = getI2cDevClient( INA209_NAME, (address >> 1), MUX_I2C_BUS_5 );
    if( !client ) {

		printk( "No such INA209 device on I2C address: 0x%2.2X\n", address );
		return -ENODEV;
    }

	// set calibration
	ina209Write( address, MID_INA209_CALIBRA_P, 0xFF, 0xFF );
	udelay(50);

    bufOut = MID_INA209_CURRENT_P;	//set poINT32er to curret register
    
    /* command write */
    msg.addr = (address >> 1);
    msg.flags = 0;
    msg.len = 1;
    msg.buf = &bufOut;

	ret = i2c_transfer( client->adapter, &msg, 1 );
	if( ret != 1 )
	{
		//commReportError(MEC_FPGA, MIDPLANE_MISCELLANEOUS_MEC, MIDPLANE_INA209_ERROR,
		//				MSG_MIDPLANE_INA209_ERROR, 0, 0, 0, 0);
		printk("[MIDPLANE] Read INA209 register pointer error! ret = %d \n", ret);
    }
    udelay(50);

    /* read content */
    msg.flags = I2C_M_RD;
    msg.buf = bufIn;
    
	ret = i2c_transfer( client->adapter, &msg, 1 );
    if( ret != 1 )
	{
		//commReportError(MEC_FPGA, MIDPLANE_MISCELLANEOUS_MEC, MIDPLANE_INA209_ERROR,
		//				MSG_MIDPLANE_INA209_ERROR, 0, 0, 0, 0);
		printk("[MIDPLANE] Read INA209 register error! ret = %d \n", ret);
    }
    
	//printk("Current: 0x%04X \n", (((UINT16)bufIn[0] << 8) | (UINT16)bufIn[1]));
	
    return (((UINT16)bufIn[0] << 8) | (UINT16)bufIn[1]);
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         ina209Write
*
* SUMMARY:      write INA 209 register
*
* SCOPE:        public
*
* DESCRIPTION:  write INA 209 register
*
* RESTRICTIONS:
* 
* RETURNS:      current reading
*/
VOID ina209Write
 (
 UINT8 address, 
 UINT8 regPointer, 
 UINT8 data1, 
 UINT8 data2
 )
{
	struct i2c_client *client = NULL;
    struct i2c_msg 		msg;
    UINT8  				bufOut[3];
    INT32  				ret;

	// Get I2c client
	//20120418 Luap : Masked (TAHOE_X86_PLATFORM)
	//client = getI2cDevClient( INA209_NAME, (address >> 1), MUX_I2C_BUS_5 );
	if( !client )
		return;

    bufOut[0] = regPointer;
	bufOut[1] = data1;
	bufOut[2] = data2;
    
    /* command write */
    msg.addr = (address >> 1);
    msg.flags = 0;
    msg.len = 3;
    msg.buf = bufOut;

	ret = i2c_transfer( client->adapter, &msg, 1 );
    if( ret != 1 )
	{
		//commReportError(MEC_FPGA, MIDPLANE_MISCELLANEOUS_MEC, MIDPLANE_INA209_ERROR,
		//				MSG_MIDPLANE_INA209_ERROR, 0, 0, 0, 0);
		printk("[MIDPLANE] Write INA209 register error! ret = %d \n", ret);
    }
}


/*******************************************************************************
* PROCEDURE
*
* NAME:         ina209Read
*
* SUMMARY:      read INA 209 register value
*
* SCOPE:        public
*
* DESCRIPTION:  read INA 209 register value
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
UINT16 ina209Read
 (
 UINT8 address,
 UINT8 regPointer
 )
{
	struct i2c_client *client = NULL;
    struct i2c_msg 		msg;
    UINT8  				bufOut, bufIn[2];
    INT32  				ret;

    // Get I2c client
	//20120418 Luap : Masked (TAHOE_X86_PLATFORM)
    //client = getI2cDevClient( INA209_NAME, (address >> 1), MUX_I2C_BUS_5 );
    if( !client )
        return -ENODEV;
	
    bufOut = regPointer;
    
    /* command write */
    msg.addr = (address >> 1);
    msg.flags = 0;
    msg.len = 1;
    msg.buf = &bufOut;

	ret = i2c_transfer( client->adapter, &msg, 1 );
	if( ret != 1 )
	{
		//commReportError(MEC_FPGA, MIDPLANE_MISCELLANEOUS_MEC, MIDPLANE_INA209_ERROR,
		//				MSG_MIDPLANE_INA209_ERROR, 0, 0, 0, 0);
		printk("[MIDPLANE] Read INA209 register error! ret = %d \n", ret);
    }

    /* read content */
    msg.flags = I2C_M_RD;
	msg.len = 2;
    msg.buf = bufIn;
    
	ret = i2c_transfer( client->adapter, &msg, 1 );
	if( ret != 1 )
	{
		//commReportError(MEC_FPGA, MIDPLANE_MISCELLANEOUS_MEC, MIDPLANE_INA209_ERROR,
		//				MSG_MIDPLANE_INA209_ERROR, 0, 0, 0, 0);
		printk("[MIDPLANE] Read INA209 register error! ret = %d \n", ret);
    }
    
	//printk("0x%04X \n", (((UINT16)bufIn[0] << 8) | (UINT16)bufIn[1]));
	
    return (((UINT16)bufIn[0] << 8) | (UINT16)bufIn[1]);
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         testMidplaneTypeVerify
*
* SUMMARY:      verify midplane type
*
* SCOPE:        public
*
* DESCRIPTION:  verify midplane type
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID testMidplaneTypeVerify(VOID)
{

	//enable quick switch
	fpgaEnableQuickSwitch(TRUE);

	if(((fpgaRegisterRead(FPGA_SUBSYSTEM_ID) & 0x0F) == SBB2_MIDPLANE_ID_1) ||
		((fpgaRegisterRead(FPGA_SUBSYSTEM_ID) & 0x0F) == SBB2_MIDPLANE_ID_2))
	{
		sysMidplaneType = LSI_MIDPLANE_VER_0;
		//define as LSI midplane
	}
#if defined(CHASSIS_WSAS)
    else if ((fpgaRegisterRead(FPGA_SUBSYSTEM_ID) & 0x0F) == DVT_WEMBLY_MIDPLANE_ID
        || ((fpgaRegisterRead(FPGA_SUBSYSTEM_ID) & 0x0F) == EVT_WEMBLY_MIDPLANE_ID)){
        // set as LSI midplane
        sysMidplaneType = LSI_MIDPLANE_VER_0;
    }
#endif
	else if((fpgaRegisterRead(FPGA_SUBSYSTEM_ID) & 0x0F) == QUANTA_TEST_MIDPLANE_ID)
	{
		sysMidplaneType = TEST_MIDPLANE_WO_BBU_CABLE;

		//config GPIO define
		midplaneI2cGpioPinDefine();

		//emulate BBU and alternate controller inplace
		midplaneGpioWrite(&ALT_INPL_L, 0);
		midplaneGpioWrite(&BBU_INPLACE_L, 0);
		midplaneGpioWrite(&BBU_ENABLE, 0);
		midplaneGpioWrite(&BBU_DISCHARGE, 0);

		//define as Quanta test midplane with/without BBU cable attached
    	if((fpgaRegisterRead(FPGA_SUBSYSTEM_STATUS_LINES_3) & BBU_INPL_BIT) == BBU_INPL_BIT) 
		{
			/* BBU_INPLACE does exist */
			sysMidplaneType = TEST_MIDPLANE_VER_1;
		}
#if 0
		else	//Modify for TestMidplane Changes
		{
			midplaneGpioWrite(&BBU_INPLACE_L, 1);
		}
#endif
		fpgaPwrEngagement(TRUE, TRUE);
	}
	else
	{
		/* If the ID is wrong, treat it as LSI midplane	*/ 
		
		sysMidplaneType = LSI_MIDPLANE_VER_0;
		
		printk("================================================n");
		printk("Wrong midplane ID!!! Please check the midplane.\n");
		printk("Midplane ID read from FPGA: 0x%02X.\n", (fpgaRegisterRead(FPGA_SUBSYSTEM_ID) & 0x0F));
		printk("                                \n");
		printk("#######   ####    #####  ##		\n");
		printk("##      ##    ##   ##    ##		\n");
		printk("######  ########   ##    ##		\n");
		printk("##      ##    ##   ##    ##		\n");
		printk("##      ##    ##  #####  #######\n");
		printk("                                \n");
		printk("===============================================\n");
	}
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         midplaneTypeVerification
*
* SUMMARY:      Return the midplane type
*
* SCOPE:        public
*
* DESCRIPTION:  Return the midplane type by verifing subsystem ID 
				and the BBU I2C bus
*
* RESTRICTIONS:
* 
* RETURNS:      0: LSI midplane
*				1: Quanta midplane
*/
INT32 midplaneTypeVerification(VOID)
{
	if(sysMidplaneType == LSI_MIDPLANE_VER_0)
	{	//LSI midplane
		return 0;
	}
	else
	{	//Quanta test midplane
		return 1;
	}
}

/****************************************************/
/* Quanta Test Midplane PCA9545 & PCA9555 Functions */
/****************************************************/

/*******************************************************************************
* PROCEDURE
*
* NAME:         mid9545ChSet
*
* SUMMARY:      set 9545 channel
*
* SCOPE:        public
*
* DESCRIPTION:  set 9545 channel
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID mid9545ChSet
 (
 UINT8 channel
 )
{
    struct i2c_msg 		msg;
    struct i2c_adapter 	*adap;
    UINT8  				bufOut, bufIn;
    INT32  				ret;
	
	fpgaI2cMdpl1Select();
	
    /* set I2C adapter */
    adap = zebulonAdapter[ZEBULON_I2C_BUS_1];
	
	if(midplaneI2c1HotSwapBufferReadyCheck() == FALSE)
	{	//if hot swap buffer isn't ready
    	printk("[MIDPLANE] Hot swap chip 1 error!\n");
	}
	
    bufOut = channel;
    
    /* command write */
    msg.addr = (I2C_GPIO_9545_ADDRESS >> 1);
    msg.flags = 0;
    msg.len = 1;
    
	msg.buf = &bufOut;

    if((ret = i2c_transfer(adap, &msg, 1)) != 1)
	{
		//commReportError(MEC_FPGA, MIDPLANE_MISCELLANEOUS_MEC, MIDPLANE_9545_ERROR,
		//				MSG_MIDPLANE_9545_ERROR, 0, 0, 0, 0);
		printk("[MIDPLANE] Read 9545 channel error! ret = %d, 0x%02X \n", ret, bufIn);
    }
    
    //mdelay(5);
    /* read content */
    msg.flags = I2C_M_RD;
    msg.buf = &bufIn;
    
    if((ret = i2c_transfer(adap, &msg, 1)) != 1)
	{
		//commReportError(MEC_FPGA, MIDPLANE_MISCELLANEOUS_MEC, MIDPLANE_9545_ERROR,
		//				MSG_MIDPLANE_9545_ERROR, 0, 0, 0, 0);
		printk("[MIDPLANE] Read 9545 channel error! ret = %d, 0x%02X \n", ret, bufIn);
    }
    
    //printk("[MIDPLANE] Midplane channel is set to 0x%02X\n", bufIn);
}


/*******************************************************************************
* PROCEDURE
*
* NAME:         mid9555Read
*
* SUMMARY:      read 9555 register
*
* SCOPE:        public
*
* DESCRIPTION:  read 9555 register
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID mid9555Read
 (
 UINT8 address, 
 UINT8 command
 )
{
    struct i2c_msg 		msg[2];
    struct i2c_adapter 	*adap;
    UINT8  				bufOut, bufIn[2];
    INT32  				ret;
	
    /* set I2C adapter */
    adap = zebulonAdapter[ZEBULON_I2C_BUS_1];
	
	if(midplaneI2c1HotSwapBufferReadyCheck() == FALSE)
	{	//if hot swap buffer isn't ready
    	printk("[MIDPLANE] Hot swap chip 1 error!\n");
	}
	
    bufOut = command;
	
    /* command write */
    msg[0].addr = (address >> 1);
    msg[0].flags = 0;
    msg[0].len = 1;
    msg[0].buf = &bufOut;
    
    /* read content */
	msg[1].addr = (address >> 1);
    msg[1].len = 2;
    msg[1].flags = I2C_M_RD;
    msg[1].buf = bufIn;
    
    if((ret = i2c_transfer(adap, msg, 2)) != 2)
	{
		printk("[MIDPLANE] Read 9555 register error! ret = %d \n", ret);
    }
    
    printk("PCA9555 register: 0x%02X \n", bufIn[0]);
}


/*******************************************************************************
* PROCEDURE
*
* NAME:         mid9555Write
*
* SUMMARY:      write 9555 register
*
* SCOPE:        public
*
* DESCRIPTION:  write 9555 register
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID mid9555Write
 (
 UINT8 address, 
 UINT8 command, 
 UINT8 data0, 
 UINT8 data1
 )
{
    struct i2c_msg 		msg;
    struct i2c_adapter 	*adap;
    UINT8  				bufOut[3];
    INT32  				ret;

	/* set I2C adapter */
    adap = zebulonAdapter[ZEBULON_I2C_BUS_1];
	
	if(midplaneI2c1HotSwapBufferReadyCheck() == FALSE)
	{	//if hot swap buffer isn't ready
    	printk("[MIDPLANE] Hot swap chip 1 error!\n");
	}
	
    bufOut[0] = command;
	bufOut[1] = data0;
	bufOut[2] = data1;
	
    /* command write */
    msg.addr = (address >> 1);
    msg.flags = 0;
    msg.len = 3;
    msg.buf = bufOut;

    if((ret = i2c_transfer(adap, &msg, 1)) != 1)
	{
		printk("[MIDPLANE] Write 9555 register error! ret = %d \n", ret);
    }
}


