/*******************************************************************************

NAME            fpgaResetDiag.c
SUMMARY         Zebulon FPGA reset function diagnostics
VERSION         %version: 1 %
UPDATE DATE     %date_modified: May 3 18:00 2009 %
PROGRAMMER      %created_by:    Jim Tu %

        Copyright 2009 Quanta Corporation. All Rights Reserved.

DESCRIPTION:
        This file has code to test reset function of Zebulon FPGA

NOTES:

REFERENCE:

*******************************************************************************/

#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/i2c.h>
#include <linux/pci.h>
#include <linux/pci_regs.h>
#include <linux/interrupt.h>

#include <commTypes.h>
#include <commRpt.h>
#include <commMEC.h>
#include <commNvsramApi.h>

#include "fpgaRegLib.h"
#include "fpgaMidplaneI2cLib.h"
#include "fpgaIntMaskDiag.h"
#include "fpgaResetDiag.h" 
#include "fpgaI2cDiag.h"

/***  Import Function  ***/
//EXTERN BOOLEAN 	qe8IsFourFuncDevice(VOID);

EXTERN INT32 	fpgaAltResetInDiag(INT32 mode, UINT8 bootingC, UINT8 bootingCC);
EXTERN INT32 	fpgaAltResetOutDiag(VOID);
EXTERN VOID		fpgaNvsramUnlockAll(VOID);

EXTERN UINT8    ctrlBootCountGet(VOID);
EXTERN UINT8    ctrlBootCountCompareGet(VOID);
EXTERN VOID		ctrlBootCountCompareSet(VOID);

/******************************/
/**  Reset Control Function  **/
/******************************/

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaAltResetIn
*
* SUMMARY:      clear alternate reset latch
*
* SCOPE:        private
*
* DESCRIPTION:  clear alternate reset latch
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaClearAlternateReset(VOID)
{  
    INT32 result;
    result = fpgaRegisterWrite(FPGA_CLEAR_ALTERNATE_RESET_IN_LATCH, 0x01);
}

#if 0
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaClearDirectedReset
*
* SUMMARY:      clear direct reset latch
*
* SCOPE:        private
*
* DESCRIPTION:  clear direct reset latch
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaClearDirectedReset(VOID)
{  
    INT32 result;
    result = fpgaRegisterWrite(FPGA_CLEAR_DIRECTED_RESET_LATCH, 0x01);
}
#endif

/***  Protected Write 1  ***/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaAlternateControllerReset
*
* SUMMARY:      reset alternate controller
*
* SCOPE:        private
*
* DESCRIPTION:  reset alternate controller
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaAlternateControllerReset
(
 BOOLEAN status
)
{   
    INT32 	result;
    UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_1);
    if(status)
	{
        temp |= ALT_RST_OUT_BIT;
    }
	else
	{
        temp &= (~ALT_RST_OUT_BIT);
    }
	
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_1, temp);  
}
EXPORT_SYMBOL(fpgaAlternateControllerReset);

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaBoardReset
*
* SUMMARY:      reset controller board
*
* SCOPE:        private
*
* DESCRIPTION:  reset controller board
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaBoardReset
(
 BOOLEAN status
)
{   
    INT32 	result;
    UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_1);
    if(status)
	{
        temp |= BOARD_RESET_BIT;
    }
	else
	{
        temp &= (~BOARD_RESET_BIT);
    }
	
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_1, temp);
}
EXPORT_SYMBOL(fpgaBoardReset);


/***  Protected Write 2  ***/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaPsocReset
*
* SUMMARY:      reset PSOC
*
* SCOPE:        private
*
* DESCRIPTION:  reset PSOC
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaPsocReset
(
 BOOLEAN status
)
{   
    INT32 	result;
    UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_2);
    if(status)
	{
        temp |= PSOC_RESET_BIT;
    }
	else
	{
        temp &= (~PSOC_RESET_BIT);
    }
	
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_2, temp);
}
EXPORT_SYMBOL(fpgaPsocReset);

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaQe8Reset
*
* SUMMARY:      reset QE8
*
* SCOPE:        private
*
* DESCRIPTION:  reset QE8
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaQe8Reset
(
 BOOLEAN status
)
{   
    INT32 	result;
    UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_2);
    if(status)
	{
        temp |= RST_PCIE_QE8_L_BIT;
    }
	else
	{
        temp &= (~RST_PCIE_QE8_L_BIT);
    }
	
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_2, temp);
}

EXPORT_SYMBOL(fpgaQe8Reset);
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaQe84to1
*
* SUMMARY:      change QE8 from 4-function PCI to 1-function
*
* SCOPE:        private
*
* DESCRIPTION:  change QE8 from 4-function PCI to 1-function
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaQe84to1(VOID)
{   
    INT32 	result;
    UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_2);
	
    temp |= QE8_CFG12_1FNCT_BIT;
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_2, temp);
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaQe81to4
*
* SUMMARY:      change QE8 from 1-function PCI to 4-function
*
* SCOPE:        private
*
* DESCRIPTION:  change QE8 from 1-function PCI to 4-function
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaQe81to4(VOID)
{   
    INT32 	result;
    UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_2);
	
    temp &= (~QE8_CFG12_1FNCT_BIT);
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_2, temp);
}

/***  Protected Write 3  ***/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaEthernetReset
*
* SUMMARY:      reset Ethernet chip
*
* SCOPE:        private
*
* DESCRIPTION:  reset Ethernet chip
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaEthernetReset
(
 BOOLEAN status
)
{   
    INT32 	result;
    UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_3);
    if(status)
	{
        temp |= ETHERNET_RST_BIT;
    }
	else
	{
        temp &= (~ETHERNET_RST_BIT);
    }
	
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_3, temp);
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaCpldReset
*
* SUMMARY:      reset CPLD
*
* SCOPE:        private
*
* DESCRIPTION:  reset CPLD
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaCpldReset
(
 BOOLEAN status
)
{   
    INT32 	result;
    UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_3);
    if(status)
	{
        temp |= FPGA_RST_CPLD_BIT;
    }
	else
	{
        temp &= (~FPGA_RST_CPLD_BIT);
    }
	
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_3, temp);
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaPcieSwitchReset
*
* SUMMARY:      reset PCIE switch
*
* SCOPE:        private
*
* DESCRIPTION:  reset PCIE switch
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaPcieSwitchReset
(
 BOOLEAN status
)
{   
    INT32 	result;
    UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_3);
    if(status)
	{
        temp |= PCIE_SWT_RST_BIT;
    }
	else
	{
        temp &= (~PCIE_SWT_RST_BIT);
    }
	
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_3, temp);
}


/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaHostCardReset
*
* SUMMARY:      reset HIC
*
* SCOPE:        private
*
* DESCRIPTION:  reset HIC
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaHostCardReset
(
 BOOLEAN status
)
{   
    INT32 	result;
    UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_3);
    if(status)
	{
        temp |= HOST_CARD_RST_BIT;
    }
	else
	{
        temp &= (~HOST_CARD_RST_BIT);
    }
	
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_3, temp);
}

EXPORT_SYMBOL(fpgaHostCardReset);

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaSas2IocReset
*
* SUMMARY:      reset Falcon SAS2 IOC
*
* SCOPE:        private
*
* DESCRIPTION:  reset Falcon SAS2 IOC
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaSas2IocReset
(
 BOOLEAN status
)
{   
    INT32 	result;
    UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_3);
    if(status)
	{
        temp |= RST_FALCON_BIT;
    }
	else
	{
        temp &= (~RST_FALCON_BIT);
    }
	
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_3, temp);
}

/***  Protected Write 4 -for Ibex Beak-  ***/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaAdrEn
*
* SUMMARY:      enable/disable ADR to Ibex Peak
*
* SCOPE:        private
*
* DESCRIPTION:  enable/disable ADR to Ibex Peak
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaAdrEn
(
 BOOLEAN status
)
{   
    INT32 	result;
    UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_4);
    if(status)
	{
        temp |= ADR_EN_BIT;
    }
	else
	{
        temp &= (~ADR_EN_BIT);
    }
	
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_4, temp);
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaIchPwrBtn
*
* SUMMARY:      enable/disable ICH power button to Ibex Peak
*
* SCOPE:        private
*
* DESCRIPTION:  enable/disable ICH power button to Ibex Peak
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
#if defined(CTRLER_PIKESPEAK)
VOID fpgaIchPwrBtn
(
 BOOLEAN status
)
{   
    INT32 	result;
    UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_4);
    if(status)
	{
        temp |= ICH_PWR_BTN_BIT;
    }
	else
	{
        temp &= (~ICH_PWR_BTN_BIT);
    }
	
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_4, temp);
}
#endif

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaSlpS3
*
* SUMMARY:      enable/disable sleep S3 to Ibex Peak
*
* SCOPE:        private
*
* DESCRIPTION:  enable/disable sleep S3 to Ibex Peak
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
#if defined(CTRLER_PIKESPEAK)
VOID fpgaSlpS3
(
 BOOLEAN status
)
{   
    INT32 	result;
    UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_4);
    if(status)
	{
        temp |= SLP_S3_BIT;
    }
	else
	{
        temp &= (~SLP_S3_BIT);
    }
	
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_4, temp);
}
#endif

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaClkForceLow
*
* SUMMARY:      enable/disable clock low to DDR3
*
* SCOPE:        private
*
* DESCRIPTION:  enable/disable clock low to DDR3
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaClkForceLow
(
 BOOLEAN status
)
{   
    INT32 	result;
    UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_4);
    if(status)
	{
        temp |= CKE_FORCE_LOW_BIT;
    }
	else
	{
        temp &= (~CKE_FORCE_LOW_BIT);
    }
	
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_4, temp);
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaIsataReset
*
* SUMMARY:      reset iSATA device
*
* SCOPE:        private
*
* DESCRIPTION:  reset iSATA device
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaIsataReset
(
 BOOLEAN status
)
{   
    INT32 	result;
    UINT8 	temp;
    
	temp = fpgaRegisterRead(FPGA_PROTECTED_WRITE_4);
    if(status)
	{
        temp |= RST_ISATA_BIT;
    }
	else
	{
        temp &= (~RST_ISATA_BIT);
    }
	
	fpgaEncodedRegisterWrite();
    result = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_4, temp);
}

/***************************/
/**  Reset Test Function  **/
/***************************/

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaSetDirectedReset
*
* SUMMARY:      assert/de-assert direct reset
*
* SCOPE:        private
*
* DESCRIPTION:  assert/de-assert direct reset
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpgaSetDirectedReset
(
 BOOLEAN status
)
{
    INT32 result;
	if(status)
	{
		result = fpgaRegisterWrite(FPGA_SET_DIRECTED_RESET_LATCH, DIR_RST_SET_BIT);
	}
	else
	{
		result = fpgaRegisterWrite(FPGA_SET_DIRECTED_RESET_LATCH, 0);
	}
}

/**********************************/
/**  Reset Diagnostics Function  **/
/**********************************/
#if 0
EXTERN INT32 fpgaDirectedResetMaskDiag(VOID);
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaDirectedResetDiag
*
* SUMMARY:      direct reset diagnostics function
*
* SCOPE:        public
*
* DESCRIPTION:  direct reset diagnostics function
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaDirectedResetDiag(VOID)
{   
	INT32 result = TRUE;
    
	fpgaNmiOutEnable(FALSE);
    
	/***  Verify "Directed Reset"  ***/
	fpgaSetDirectedReset(TRUE);  //assert directed reset
    
	if(!fpgaDirectedReset()){
		*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_2) 
							|= FPGA_RESET_RESULT_DIRECT_FAIL;
	}
    
	/***  Verify "Clear Directed Reset"  ***/
	fpgaSetDirectedReset(FALSE);  //release directed reset
	fpgaClearDirectedReset();
    
	if(fpgaDirectedReset())
	{
		*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_2)
							|= FPGA_RESET_RESULT_DIRECT_CLEAR_FAIL;
	}
    
    return result;
}
#endif
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaComponentResetDiag
*
* SUMMARY:      component reset diagnostics function
*
* SCOPE:        public
*
* DESCRIPTION:  component reset diagnostics function
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaComponentResetDiag(VOID)
{
    INT32 			result = TRUE;
	UINT32 			value;
	UINT32			orig;
	struct pci_dev *pdev = NULL;
    

	//
    // PSOC reset diag, need two controller to diag.
	//
	orig = fpgaRegisterRead( FPGA_ALTERNATE_BOARD_ID );

	// Hold PSOC in reset state
    fpgaPsocReset( TRUE );
    mdelay( 10 );

	// Read again
	value = fpgaRegisterRead( FPGA_ALTERNATE_BOARD_ID );
    if( value != 0x10 )
	{          
        *(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_2) 
			|= FPGA_RESET_RESULT_PSOC_FAIL;
		result = FALSE;
    }

	// Release the PSOC
    fpgaPsocReset( FALSE );
    mdelay( 1000 );	//Wait PSOC for initialization

	// Read again
	value = fpgaRegisterRead( FPGA_ALTERNATE_BOARD_ID );
	if( value != orig ) {

        *(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_2) 
			|= FPGA_RESET_RESULT_PSOC_FAIL;
		result = FALSE;
		printk( "[FPGA Reset] PSOC Reset Fail. 0x%08X\n", value );
	}
	else {

		printk( "[FPGA Reset] PSOC Reset Pass.\n" );
	}
	

	//
    // Ethernet 82576 reset diag
	//
	pdev = pci_get_device( 0x8086, 0x10C9, NULL );
	if( !pdev ) {

		printk( "Cannot found NIC 82576 devices\n" );
		goto SkipNext1;
	}

	// Disable 82576 IRQ
	disable_irq( pdev->irq );

	// Read BAR1
	pci_read_config_dword( pdev, FPGA_RESET_BAR1_OFFSET, &orig );
	printk( "[FPGA Reset] NIC 82576 BAR1 0x%08X \n", orig );

	// Hold 82576 in reset state
	fpgaEthernetReset( TRUE );
	mdelay( 200 );

	// Read BAR1 again
	pci_read_config_dword( pdev, FPGA_RESET_BAR1_OFFSET, &value );
	if( value != 0xFFFFFFFF )
	{
   		*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_1) 
			|= FPGA_RESET_RESULT_ETHERNET_FAIL;
		result = FALSE;
		printk( "[FPGA Reset] Ethernet Reset Fail. 0x%08X\n", value );
   	} 
	else 
	{
		printk( "[FPGA Reset] Ethernet Reset Pass.\n" );
	}

	// Release 82576
    fpgaEthernetReset( FALSE );
	pci_dev_put( pdev );
	pdev = NULL;


#if 0
	mdelay( 1000 );

	// Wait for device become ready
	for( i = 30 ; i ; i-- ) {

		pci_read_config_dword( pdev, FPGA_RESET_BAR1_OFFSET, &value );
		printk( "new value = 0x%8.8X\n", value );
		if( value == orig )
			break;

		mdelay( 1000 );
	}
	if( !i ) {

		printk( "[FPGA Reset] Ethernet NIC 82576 wasn't able to being recovered\n" );
		printk( "[FPGA Reset] Disable NIC 82576 IRQ in order to avoid system crash\n" );
		result = FALSE;
	}
	else {

		// Reenable IRQ
		enable_irq( pdev->irq );
		printk( "[FPGA Reset] Ethernet NIC 82576 done. \n" );
	}
#endif
	

SkipNext1:


#if 0
    //I2C MUX reset diag
    if( I2cSwitchTest() != 0 )
	{                       //normal return should be 0
        *(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_1) 
			|= FPGA_RESET_RESULT_I2C_MUX_FAIL;
		printk( "[FPGA Reset] I2C MUX Reset Fail. \n" );
		result = FALSE;
    } 
	else 
	{
		printk( "[FPGA Reset] I2C MUX Reset Pass. \n" );
	}
#endif
    

    // CPLD reset diag
    // TBD
	

	//
    // LSI Mustang reset duag
	//
	pdev = pci_get_device( 0x1000, 0x0087, NULL );
	pdev = pci_get_device( 0x1000, 0x0087, pdev );
	pdev = pci_get_device( 0x1000, 0x0087, pdev );
	if( !pdev ) {

		printk( "Cannot found SAS2308 devices\n" );
		goto SkipNext2;
	}

	// Disable Mustang IRQ
	disable_irq( pdev->irq );

	// Read the original BAR1
	pci_read_config_dword( pdev, FPGA_RESET_BAR1_OFFSET, &orig );
	printk( "[FPGA Reset] SAS2308 BAR1 0x%08X \n", orig );

	// Hold the mustang in reset state
	fpgaSas2IocReset( TRUE );
	mdelay( 200 );

	// Read the BAR1 again
	pci_read_config_dword( pdev, FPGA_RESET_BAR1_OFFSET, &value );
	if( value != 0xFFFFFFFF )
	{
   		*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_2) 
			|= FPGA_RESET_RESULT_SAS2IOC_FAIL;
		result = FALSE;
		printk( "[FPGA Reset] SAS2308 Reset Fail. 0x%08X \n", value );
   	} 
	else 
	{
		printk( "[FPGA Reset] SAS2308 Reset Pass.\n" );
	}

	// Release the mustang
    fpgaSas2IocReset( FALSE );
	pci_dev_put( pdev );
    pdev = NULL;


#if 0
	// Wait for device become ready
	for( i = 100 ; i ; i-- ) {

		pci_read_config_dword( pdev, FPGA_RESET_BAR1_OFFSET, &value );
		if( value == orig )
			break;

		mdelay( 200 );
	}
	if( !i ) {

		printk( "[FPGA Reset] SAS2308 wasn't able to being recovered\n" );
		printk( "[FPGA Reset] Disable SAS2308 IRQ in order to avoid system crash\n" );
		result = FALSE;
	}
	else {

		// Reenable IRQ
		enable_irq( pdev->irq );
		printk( "[FPGA Reset] SAS2308 done. \n" );
	}
#endif


SkipNext2:


    //SASx36 Reset
    //TBD
	

	//iSATA Reset 
	//TBD, need to access disk
	

	if(midplaneTypeVerification())
	{
		//
		// Quanta Midplane
		//


		//
    	// Midplane I2C1 reset
		//
    	fpgaI2cMdpl1Reset(TRUE);
    	if(!midplaneGpioRead(&MP_I2C1_RST_L))
		{
    	    *(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_1) 
								|= FPGA_RESET_RESULT_MID_I2C_1_FAIL;
    	}
    	fpgaI2cMdpl1Reset(FALSE);
    	if(midplaneGpioRead(&MP_I2C1_RST_L))
		{
    	    *(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_1) 
								|= FPGA_RESET_RESULT_MID_I2C_1_FAIL;
    	}
        

		//
	    // Midplane I2C2 reset
		//
    	fpgaI2cMdpl2Reset(TRUE);
    	if(!midplaneGpioRead(&MP_I2C2_RST_L))
		{
        	*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_1) 
								|= FPGA_RESET_RESULT_MID_I2C_2_FAIL;
 	   }
    	fpgaI2cMdpl2Reset(FALSE);
   		if(midplaneGpioRead(&MP_I2C2_RST_L))
		{
        	*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_1) 
								|= FPGA_RESET_RESULT_MID_I2C_2_FAIL;
    	}
	}
    printk( "[FPGA Reset] Mid I2C done.\n" );
	

    return result;
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaQe8LowPowerDiag
*
* SUMMARY:      QE8 lower power mode diagnostics function
*
* SCOPE:        public
*
* DESCRIPTION:  QE8 lower power mode diagnostics function
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
#if 0
INT32 fpgaQe8LowPowerDiag(VOID)
{
    INT32 result = TRUE;

    //QE8 4-function device to 1-function device
	if(!qe8IsFourFuncDevice())
	{          
        commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_QE8_4TO1_ERROR, 
                        MSG_FPGA_QE8_4TO1_ERROR, 0, 0, 0, 0);
        result = FALSE;
    }
	
    fpgaQe84to1();		//hold QE8 in 1-function device
    mdelay(20);
        
    if(qe8IsFourFuncDevice())
	{          
        commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_QE8_4TO1_ERROR, 
                        MSG_FPGA_QE8_4TO1_ERROR, 0, 0, 0, 0);
        result = FALSE;
    }
	
    fpgaQe81to4();		//return to 4-function device

    return result;
}
#endif

/*********************************/
/**  Main Diagnostics Function  **/
/*********************************/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaResetDiag
*
* SUMMARY:      FPGA reset diagnostics function
*
* SCOPE:        public
*
* DESCRIPTION:  FPGA reset diagnostics function
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaResetDiag(VOID)
{
    INT32 	result = TRUE;
    //the following values are stored in NOR Flash
    VOID* 	diagSeqRecord;
    UINT8 	bootingCount;
    UINT8 	bootingCountComparison;
    
    fpgaMaskIntAll(TRUE);
    
    fpgaNvsramUnlockAll();
    
    //load bootingCount and bootingCount_compare
    bootingCount = ctrlBootCountGet();
    bootingCountComparison = ctrlBootCountCompareGet();
    
    diagSeqRecord = (gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_DIAG_SEQ_RECORD_OFFSET);
    
    //verify if this the first time in reset test
    if( (*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_ENCODED_UINT8_0_LOCATION) != FPGA_RESET_ENCODED_PATTERN_0) |    
		(*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_ENCODED_UINT8_1_LOCATION) != FPGA_RESET_ENCODED_PATTERN_1) |    
		(*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_ENCODED_UINT8_2_LOCATION) != FPGA_RESET_ENCODED_PATTERN_2) ) 
	{
		//set checkpoINT32
		*(UINT8*)diagSeqRecord = FPGA_RESET_SEQ_NORMAL;
		*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_1) = FPGA_RESET_RESULT_GOOD;
		*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_2) = FPGA_RESET_RESULT_GOOD;
	
		//set syndrome
		*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_ENCODED_UINT8_0_LOCATION) = FPGA_RESET_ENCODED_PATTERN_0;    
		*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_ENCODED_UINT8_1_LOCATION) = FPGA_RESET_ENCODED_PATTERN_1;    
		*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_ENCODED_UINT8_2_LOCATION) = FPGA_RESET_ENCODED_PATTERN_2;	
    }
    

    /* RESET_SEQ_NORMAL */
    if(*(UINT8*)diagSeqRecord == FPGA_RESET_SEQ_NORMAL)
	{
		*(UINT8*)diagSeqRecord = FPGA_RESET_SEQ_ALT_IN;
		
		/*
		if(!fpgaDirectedResetMaskDiag()) 
		{
			*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_2) |= FPGA_RESET_RESULT_DRST_MASK_FAIL;
			printk("[FPGA Reset] Directer reset mask fail. \n");
		} 
		else 
		{
			printk("[FPGA Reset] Directer reset mask test done. \n");
		}
		*/
		
		printk("[FPGA Reset] alt-reset test start\n");
		
		//verift alternate reset out
		fpgaAltResetOutDiag();
		
		//start verify alternate reset in
        result = fpgaAltResetInDiag(0, bootingCount, bootingCountComparison);
    }
    
	/* RESET_SEQ_ALT_IN */
    if(*(UINT8*)diagSeqRecord == FPGA_RESET_SEQ_ALT_IN)
	{
		*(UINT8*)diagSeqRecord = FPGA_RESET_SEQ_COMPONENT;
        result = fpgaAltResetInDiag(1, bootingCount, bootingCountComparison);
    }
    
    /* RESET_SEQ_COMPONENT */
    if(*(UINT8*)diagSeqRecord == FPGA_RESET_SEQ_COMPONENT)
	{
        *(UINT8*)diagSeqRecord = FPGA_RESET_SEQ_FINISH;
    	printk("[FPGA Reset] Component reset test start.\n");
		//fpgaDirectedResetDiag();
		fpgaComponentResetDiag();

		printk("[FPGA Reset] Component reset test done....Prepare to Reset.\n");
    
		//set PCI reset
		ctrlBootCountCompareSet();
	
		fpgaBoardReset(TRUE);        //hold board in reset
        mdelay(5);
        fpgaBoardReset(FALSE);
	
		//should be reset
	
		if(ctrlBootCountGet() == ctrlBootCountCompareGet())
		{
			commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_BOARD_RESET_ERROR, 
							MSG_FPGA_BOARD_RESET_ERROR, 0, 0, 0, 0);
			result = FALSE;
			*(UINT8*)diagSeqRecord = FPGA_RESET_SEQ_NORMAL;	
		}
    }
    
	/* RESET_SEQ_FINISH */
    if(*(UINT8*)diagSeqRecord == FPGA_RESET_SEQ_FINISH)
	{
    	*(UINT8*)diagSeqRecord = FPGA_RESET_SEQ_NORMAL;
		printk("[FPGA Reset] Component reset test done.\n");
	 
    	if(ctrlBootCountGet() == ctrlBootCountCompareGet())
		{
			*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_1) |= FPGA_RESET_RESULT_BOARD_FAIL;
			printk("[FPGA Reset] CtrlBootCountGet: 0x%02X\n", ctrlBootCountGet());
			printk("[FPGA Reset] CtrlBootCountCompareGet: 0x%02X\n", ctrlBootCountCompareGet());
		}
    }
    
	/********************************/
    /*	Print Error, Result Byte 1  */
	/********************************/
	if(*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_1) & FPGA_RESET_RESULT_MID_I2C_1_FAIL)
	{
		commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_I2C_MDPL1_RESET_ERROR, MSG_FPGA_I2C_MDPL1_RESET_ERROR, 0, 0, 0, 0);
		result = FALSE;
    }
	
	if(*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_1) & FPGA_RESET_RESULT_MID_I2C_2_FAIL)
	{
		commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_I2C_MDPL2_RESET_ERROR, MSG_FPGA_I2C_MDPL2_RESET_ERROR, 0, 0, 0, 0);
		result = FALSE;
    }
	
	if(*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_1) & FPGA_RESET_RESULT_BOARD_FAIL)
	{
		commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_BOARD_RESET_ERROR, MSG_FPGA_BOARD_RESET_ERROR, 0, 0, 0, 0);
		result = FALSE;
    }
    	
	if(*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_1) & FPGA_RESET_RESULT_PSOC_FAIL)
	{
		commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_PSOC_RESET_ERROR, MSG_FPGA_PSOC_RESET_ERROR, 0, 0, 0, 0);
		result = FALSE;
    }
	
	if(*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_1) & FPGA_RESET_RESULT_ETHERNET_FAIL)
	{
		commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_ETHERNET_RESET_ERROR, MSG_FPGA_ETHERNET_RESET_ERROR, 0, 0, 0, 0);
		result = FALSE;
    }
	
	if(*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_1) & FPGA_RESET_RESULT_I2C_MUX_FAIL)
	{
		commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_I2C_MUX_RESET_ERROR, MSG_FPGA_I2C_MUX_RESET_ERROR, 0, 0, 0, 0);
		result = FALSE;
    }
		
	if(*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_1) & FPGA_RESET_RESULT_CPLD_FAIL)
	{
		commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_CPLD_RESET_ERROR, MSG_FPGA_CPLD_RESET_ERROR, 0, 0, 0, 0);
		result = FALSE;
    }
	
	/********************************/
    /*	Print Error, Result Byte 2  */
	/********************************/
	if(*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_2) & FPGA_RESET_RESULT_DIRECT_FAIL)
	{
		commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_DIRECTED_RESET_ERROR, MSG_FPGA_DIRECTED_RESET_ERROR, 0, 0, 0, 0);
		result = FALSE;
    }
	
	if(*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_2) & FPGA_RESET_RESULT_DIRECT_CLEAR_FAIL)
	{
		commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_DIRECTED_RESET_CLEAR_ERROR, MSG_FPGA_DIRECTED_RESET_CLEAR_ERROR, 0, 0, 0, 0);
		result = FALSE;
    }
	
	if(*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_2) & FPGA_RESET_RESULT_SAS2IOC_FAIL)
	{
		commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_SAS2IOC_RESET_ERROR, MSG_FPGA_SAS2IOC_RESET_ERROR, 0, 0, 0, 0);
		result = FALSE;
    }
	
	if(*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_2) & FPGA_RESET_RESULT_SAS2X36_FAIL)
	{
		commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_SAS2X36_RESET_ERROR, MSG_FPGA_SAS2X36_RESET_ERROR, 0, 0, 0, 0);
		result = FALSE;
    }
	
	if(*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_2) & FPGA_RESET_RESULT_PCIE_SWITCH_FAIL)
	{
		commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_PCIE_SWITCH_RESET_ERROR, MSG_FPGA_PCIE_SWITCH_RESET_ERROR, 0, 0, 0, 0);
		result = FALSE;
    }
	
	if(*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_2) & FPGA_RESET_RESULT_QE8_FAIL)
	{
		commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_QE8_RESET_ERROR, MSG_FPGA_QE8_RESET_ERROR, 0, 0, 0, 0);
		result = FALSE;
    }
	
	if(*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_2) & FPGA_RESET_RESULT_ISATA_FAIL)
	{
		commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_ISATA_RESET_ERROR, MSG_FPGA_ISATA_RESET_ERROR, 0, 0, 0, 0);
		result = FALSE;
    }
	
	if(*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_2) & FPGA_RESET_RESULT_DRST_MASK_FAIL)
	{
		commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_DIRECTED_RESET_MASK_ERROR, MSG_FPGA_DIRECTED_RESET_MASK_ERROR, 0, 0, 0, 0);
		result = FALSE;
	}
	
	/********************************/
    /*	Print Error, Result Byte 3  */
	/********************************/
	if(*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_3) & FPGA_RESET_RESULT_ALT_RST_OUT_FAIL)
	{
		commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_ALT_RST_OUT_ERROR, MSG_FPGA_ALT_RST_OUT_ERROR, 0, 0, 0, 0);
		result = FALSE;
    }
	
	if(*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_3) & FPGA_RESET_RESULT_ALT_RST_IN_FAIL)
	{
		commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_ALT_RST_IN_ERROR, MSG_FPGA_ALT_RST_IN_ERROR, 0, 0, 0, 0);
		result = FALSE;
    }
	
	if(*(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_RESULT_OFFSET_3) & FPGA_RESET_RESULT_ALT_RST_CLEAR_FAIL)
	{
		commReportError(MEC_FPGA, FPGA_RESET_MEC, FPGA_ALT_RST_CLEAR_ERROR, MSG_FPGA_ALT_RST_CLEAR_ERROR, 0, 0, 0, 0);
		result = FALSE;
    }
		
	fpgaMaskIntAll(FALSE);
    
    //reset syndrome
    *(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_ENCODED_UINT8_0_LOCATION) = 0;    
    *(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_ENCODED_UINT8_1_LOCATION) = 0;    
    *(volatile UINT8*)(gFpgaBar2VirtualP + NVSRAM_ADDRESS_BASE + FPGA_RESET_ENCODED_UINT8_2_LOCATION) = 0;
	
    return result;
}


