/*******************************************************************************

NAME            fpgaUartDiag.c
SUMMARY         Zebulon FPGA UART function diagnostics
VERSION         %version: 1 %
UPDATE DATE     %date_modified: May 5 14:20 2009 %
PROGRAMMER      %created_by:    Jim Tu %

        Copyright 2009 Quanta Corporation. All Rights Reserved.

DESCRIPTION:
        This file has code to test UART function of Zebulon FPGA

NOTES:

REFERENCE:

*******************************************************************************/

#include <linux/kernel.h>
#include <linux/delay.h>
 
#include <diag_types.h>
#include <diag_api.h>
#include <mec.h>

#include "fpgaRegLib.h"
#include "fpgaUartDiag.h"

/*************************/
/**  UART MUX Function  **/
/*************************/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpagUartSelect
*
* SUMMARY:      switch UART MUX source
*
* SCOPE:        private
*
* DESCRIPTION:  switch UART MUX source by writing the UART control register 
*				in FPGA
*
*				CPU_UART_16C550C 	0x00
*				BOBCAT_UART 		0x01
*				BOBCAT_SER_DBG	 	0x02
*				FALCON_UART 		0x03
*				HIC_UART 			0x04
*
* RESTRICTIONS:
* 
* RETURNS:      
*/
VOID fpagUartSelect
 (
 UINT8 uartSourceSelect
 )
{
	if(uartSourceSelect <= FPGA_HIC_UART)
	{
		fpgaRegisterWrite(FPGA_UART_CONTROL, uartSourceSelect);
	}
	else
	{
		printk("[UART] UART selection is wrong. \n");
	}
}


/****************************/
/**  Diagnostics Function  **/
/****************************/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaBobcatUartDiag
*
* SUMMARY:      MUX selection from Bobcat UART diagnostics
*
* SCOPE:        private
*
* DESCRIPTION:  verify if the UART from Bobcat SAS2x36 expender to FPGA MUX 
*				is functional
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaBobcatUartDiag()
{
	INT32	result = TRUE;
	
	//diagnostics function is TBD
	
	return result;
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaBobcatSerialDbgDiag
*
* SUMMARY:      MUX selection from Bobcat serial debug port diagnostics
*
* SCOPE:        private
*
* DESCRIPTION:  verify if the serial debug port from Bobcat SAS2x36 expender
* 				to FPGA MUX is functional
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaBobcatSerialDbgDiag()
{
	INT32	result = TRUE;
	
	//diagnostics function is TBD
	
	return result;
}


/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaFalconUartDiag
*
* SUMMARY:      MUX selection from Falcon UART diagnostics
*
* SCOPE:        private
*
* DESCRIPTION:  verify if the UART from Falcon SAS2008 IOC to FPGA MUX 
*				is functional
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaFalconUartDiag()
{
	INT32	result = TRUE;
	
	//diagnostics function is TBD
	
	return result;
}

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaHostCardUartDiag
*
* SUMMARY:      MUX selection from HIC UART diagnostics
*
* SCOPE:        private
*
* DESCRIPTION:  verify if the UART from HIC to FPGA MUX is functional
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaHostCardUartDiag()
{
	INT32	result = TRUE;
	
	//diagnostics function is TBD
	
	return result;
}

/*************************/
/**  UART MUX Function  **/
/*************************/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaUartDiag
*
* SUMMARY:      UART MUX selection diagnostics
*
* SCOPE:        private
*
* DESCRIPTION:  verify if the FPGA UART MUX is functional
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaUartDiag()
{
	INT32	result = FALSE;
	
	if(fpgaBobcatUartDiag() & fpgaBobcatSerialDbgDiag()
		& fpgaFalconUartDiag() & fpgaHostCardUartDiag())
	{
		result = TRUE;
	}
	
	return result;
}