/*******************************************************************************

NAME            commNvsramApi.c
SUMMARY         NVSRAM temp. information function
VERSION         %version: 1 %
UPDATE DATE     %date_modified: Jul 6 19:20 2009 %
PROGRAMMER      %created_by:    Jim Tu %

        Copyright 2009 Quanta Corporation. All Rights Reserved.

DESCRIPTION:
        This file has code to access NVSRAM content for Pikes Peak RAID
		controller

NOTES:

REFERENCE:

*******************************************************************************/

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/string.h>
#include <linux/vmalloc.h>
#include <asm/uaccess.h>
#include <asm/io.h>

#include <commTypes.h>
#include <commRpt.h>

#include "nvsramBootCtlDiag.h"


/***  Global Value  ***/
UINT32  gDiagIndexCount;
INT32   gCtrlFirstBootVerify;		//tag if it is the first boot of the main board
INT32	gDiagResultLogNvsram;
INT32	gBootCount=0;
INT32	gBootCountCompare=0;
INT32   gManualDrive=1;				//tag if it's running manual selected diagnostics
		//0, manual; 1, with FC; 2, with ISCSI; 3, with IB; 4, without HIC
INT32	gMBoardStressRestart=0;		//tag if it's board stress restart

EXTERN void* gFpgaBar2VirtualP;

EXTERN void fpgaNvsramUnlockAll(void);
EXTERN void fpgaNvsramLockAll(void);

/***********************************/
/**  Boot Count Related Function  **/
/***********************************/

/***********************************************************
	Name: controllerBootCountGet
	Synopsis: 
	Description: get boot count value

	Input Variables :  
	Output Variables: 
	Return Value: boot count value

	Globals:

***********************************************************/
UINT8 controllerBootCountGet(void)
{
    gBootCount = *(volatile UINT8*)BOOT_COUNT_LOCATION;
	return gBootCount;
}

UINT8 ctrlBootCountGet(void)
{
	return controllerBootCountGet();
}

/***********************************************************
	Name: controllerBootCountIncrease
	Synopsis: 
	Description: boot count value increase 1

	Input Variables :  
	Output Variables: 
	Return Value:

	Globals:

***********************************************************/
void controllerBootCountIncrease(void)
{
	UINT8 bootCountTemp;
	int i = 0;

    fpgaNvsramUnlockAll();
	bootCountTemp = *(volatile UINT8*)BOOT_COUNT_LOCATION;
    *(volatile UINT8*)BOOT_COUNT_LOCATION = bootCountTemp + 1;
	
	while(*(volatile UINT8*)BOOT_COUNT_LOCATION != (bootCountTemp + 1))
	{
		*(volatile UINT8*)BOOT_COUNT_LOCATION = bootCountTemp + 1;
		i++;
		
		if (i == 5){
			printk("Boot Count Increase Error!!!\n");
			printk("                                \n");
			printk("#######   ####    #####  ##		\n");
			printk("##      ##    ##   ##    ##		\n");
			printk("######  ########   ##    ##		\n");
			printk("##      ##    ##   ##    ##		\n");
			printk("##      ##    ##  #####  #######\n");
			printk("                                \n");
			break;
		}
	}
}

/***********************************************************
	Name: ControllerBootCountReset
	Synopsis: 
	Description: reset boot count value to 0

	Input Variables :  
	Output Variables: 
	Return Value:

	Globals:

***********************************************************/
void controllerBootCountReset(void)
{
	int i = 0;

    fpgaNvsramUnlockAll();
    *(volatile UINT8*)BOOT_COUNT_LOCATION = 0;
    gBootCount = *(volatile UINT8*)BOOT_COUNT_LOCATION;
	
	while(*(volatile UINT8*)BOOT_COUNT_LOCATION != 0)
	{
		*(volatile UINT8*)BOOT_COUNT_LOCATION = 0;
		gBootCount = *(volatile UINT8*)BOOT_COUNT_LOCATION;
		i++;
		
		if (i == 5){
			printk("Boot Count Reset Error!!!\n");
			printk("                                \n");
			printk("#######   ####    #####  ##		\n");
			printk("##      ##    ##   ##    ##		\n");
			printk("######  ########   ##    ##		\n");
			printk("##      ##    ##   ##    ##		\n");
			printk("##      ##    ##  #####  #######\n");
			printk("                                \n");
			break;
		}
	}
}

/***********************************************************
	Name: ControllerBootCountCompareGet
	Synopsis: 
	Description: get boot count compare value

	Input Variables :  
	Output Variables: 
	Return Value: boot count compare value

	Globals:

***********************************************************/
UINT8 controllerBootCountCompareGet(void)
{
    gBootCountCompare = *(volatile UINT8*)BOOT_COUNT_COMPARE_LOCATION;
	return gBootCountCompare;
}
UINT8 ctrlBootCountCompareGet(void)
{
	return controllerBootCountCompareGet();
}
/***********************************************************
	Name: ControllerBootCountCompareSet
	Synopsis: 
	Description: set boot count compare value as boot count value

	Input Variables :  
	Output Variables: 
	Return Value:

	Globals:

***********************************************************/
void controllerBootCountCompareSet(void)
{
	int i = 0;

    fpgaNvsramUnlockAll();
    *(volatile UINT8*)BOOT_COUNT_COMPARE_LOCATION = *(volatile UINT8*)BOOT_COUNT_LOCATION;
	
	while(*(volatile UINT8*)BOOT_COUNT_COMPARE_LOCATION != *(volatile UINT8*)BOOT_COUNT_LOCATION)
	{
		*(volatile UINT8*)BOOT_COUNT_COMPARE_LOCATION = *(volatile UINT8*)BOOT_COUNT_LOCATION;
		i++;
		
		if (i == 5){
			printk("Boot Count Compare Set Error!!!\n");
			printk("                                \n");
			printk("#######   ####    #####  ##		\n");
			printk("##      ##    ##   ##    ##		\n");
			printk("######  ########   ##    ##		\n");
			printk("##      ##    ##   ##    ##		\n");
			printk("##      ##    ##  #####  #######\n");
			printk("                                \n");
			break;
		}
	}
}
void ctrlBootCountCompareSet(void)
{
	controllerBootCountCompareSet();
}

       
/******************************************/
/**  Boot Verification Related Function  **/
/******************************************/

/***********************************************************
	Name: CtrlFirstBootVerify
	Synopsis: 
	Description: verify the controller has been rebooted by diag program

	Input Variables :  
	Output Variables: 
	Return Value: 

	Globals: gCtrlFirstBootVerify
	           TRUE: Not reboot by diag program
	           FALSE: reboot by diag program 

***********************************************************/       
void ctrlFirstBootVerify(void)
{
    if( (*(volatile UINT8*)BOOT_COUNT_ENCODED_BYTE_0_LOCATION == BOOT_COUNT_ENCODED_PATTERN_0) &&    
        (*(volatile UINT8*)BOOT_COUNT_ENCODED_BYTE_1_LOCATION == BOOT_COUNT_ENCODED_PATTERN_1) &&    
        (*(volatile UINT8*)BOOT_COUNT_ENCODED_BYTE_2_LOCATION == BOOT_COUNT_ENCODED_PATTERN_2) )
	{
        //if encoded bytes match the encoded pattern
        
        gCtrlFirstBootVerify = FALSE;    
        
    }else{   
        gCtrlFirstBootVerify = TRUE;   
    }   
}   

/***********************************************************
	Name: ControllerFirstBootStateSet
	Synopsis: 
	Description: set encoded patterns to encoded bytes

	Input Variables :  
	Output Variables: 
	Return Value: 

	Globals:

***********************************************************/
void controllerFirstBootStateSet( void ) {

    fpgaNvsramUnlockAll();
    *(volatile UINT8*)BOOT_COUNT_ENCODED_BYTE_0_LOCATION = BOOT_COUNT_ENCODED_PATTERN_0;    
    *(volatile UINT8*)BOOT_COUNT_ENCODED_BYTE_1_LOCATION = BOOT_COUNT_ENCODED_PATTERN_1;    
    *(volatile UINT8*)BOOT_COUNT_ENCODED_BYTE_2_LOCATION = BOOT_COUNT_ENCODED_PATTERN_2;    
}

/***********************************************************
	Name: ControllerFirstBootStateClear
	Synopsis: 
	Description: reset encoded bytes to 0

	Input Variables :  
	Output Variables: 
	Return Value: 

	Globals:

***********************************************************/
void controllerFirstBootStateClear( void ) {   

    fpgaNvsramUnlockAll();
    *(volatile UINT8*)BOOT_COUNT_ENCODED_BYTE_0_LOCATION = 0;
    *(volatile UINT8*)BOOT_COUNT_ENCODED_BYTE_1_LOCATION = 0;
    *(volatile UINT8*)BOOT_COUNT_ENCODED_BYTE_2_LOCATION = 0;
}


/************************************/
/**  Diag. Index Related Function  **/
/************************************/

/***********************************************************
	Name: ControllerDiagIndexGet
	Synopsis: 
	Description: 

	Input Variables :  
	Output Variables: 
	Return Value: 

	Globals: gDiagIndexCount

***********************************************************/
void controllerDiagIndexGet(void)
{
    gDiagIndexCount = *(volatile UINT8*)DIAG_INDEX_LOCATION;
}

/***********************************************************
	Name: ControllerDiagIndexSet
	Synopsis: 
	Description: set diag. index

	Input Variables : d_index
	Output Variables: 
	Return Value: 

	Globals:

***********************************************************/
void controllerDiagIndexSet(UINT8 d_index)
{
    fpgaNvsramUnlockAll();
    *(volatile UINT8*)DIAG_INDEX_LOCATION = d_index;
    gDiagIndexCount = *(volatile UINT8*)DIAG_INDEX_LOCATION;
}

/***********************************************************
	Name: ControllerDiagIndexReset
	Synopsis: 
	Description: reset diag. index to 0

	Input Variables :
	Output Variables: 
	Return Value: 

	Globals:

***********************************************************/
void controllerDiagIndexReset(void)
{
    fpgaNvsramUnlockAll();
    *(volatile UINT8*)DIAG_INDEX_LOCATION = 0; 
    gDiagIndexCount = *(volatile UINT8*)DIAG_INDEX_LOCATION;
}

/***********************************************************
	Name: ControllerDiagManualSet
	Synopsis: 
	Description: set manual drive status

	Input Variables : manual_status
	Output Variables: 
	Return Value: 

	Globals:

***********************************************************/
void controllerDiagManualSet(INT32 manual_status)
{
    fpgaNvsramUnlockAll();
//Puu    *(volatile UINT8*)DIAG_MANUAL_DRIVE_LOCATION = (manual_status & 0x00000001); 
    //*(volatile UINT8*)DIAG_MANUAL_DRIVE_LOCATION = (manual_status & 0x0000000F); 
    *(volatile UINT8*)DIAG_MANUAL_DRIVE_LOCATION = (manual_status & 0x0000FFFF); 
    gManualDrive = *(volatile UINT8*)DIAG_MANUAL_DRIVE_LOCATION; 
}

/***********************************************************
	Name: ControllerDiagManualGet
	Synopsis: 
	Description: get manual drive status

	Input Variables : 
	Output Variables: 
	Return Value: 

	Globals: gManualDrive

***********************************************************/
void controllerDiagManualGet(void)
{
    gManualDrive = *(volatile UINT8*)DIAG_MANUAL_DRIVE_LOCATION;
}

/***********************************************************
	Name: ResetStatusClean
	Synopsis: 
	Description: clean reset diag status

	Input Variables : 
	Output Variables: 
	Return Value: 

	Globals: 

***********************************************************/
void resetStatusClean(void)
{
    UINT32 address;	

    fpgaNvsramUnlockAll();
    
    for(address = 0x06; address < 0x10; address++)
	{
    	*(volatile UINT8*)(gFpgaBar2VirtualP + address) = 0;
    }
    
    for(address = 0x58; address < 0x60; address++)
	{
    	*(volatile UINT8*)(gFpgaBar2VirtualP + address) = 0;
    }
}

void diagTableClean(void)
{
	UINT32 address;	

    fpgaNvsramUnlockAll();
    
    for(address = 0x54; address < 0x58; address++)
	{
    	*(volatile UINT8*)(gFpgaBar2VirtualP + address) = 0;
    }
    
    for(address = 0x60; address < 0x2000; address++)
	{
    	*(volatile UINT8*)(gFpgaBar2VirtualP + address) = 0;
    }
}

/*******************************************/
/*******************************************/
/**  Diag. Result Table Related Function  **/
/*******************************************/
/*******************************************/
/***********************************************************
	Name: DiagResultToNVSRAM
	Synopsis: 
	Description: Record diag. result to MVSRAM

	Input Variables : 
	Output Variables: 
	Return Value: 

	Globals: 

***********************************************************/
void diagResultToNvsram(UINT32 itemNumber, INT32 errorState)
{
    UINT8  byteNumber;
    UINT8  temp;
    UINT8  setMask = 0xFF;
    
//    printk("[Diag Result Log] Item number: %d,", item_number);
    
    itemNumber--;
    
    byteNumber = itemNumber / 8;
    itemNumber = itemNumber % 8;
    
    if(byteNumber < (DIAG_MANUAL_DRIVE_OFFSET - DIAG_RESULT_TABLE_BASE_OFFSET)){
    	fpgaNvsramUnlockAll();
    
    	temp = *(volatile UINT8*)(DIAG_RESULT_TABLE_LOCATION + byteNumber);
    	
	//set mask
	switch(itemNumber)
	{
	    case 0:
		    setMask = 0xFE;
		    break;
		case 1:
		    setMask = 0xFD;
		    break;
		case 2:
		    setMask = 0xFB;
		    break;
		case 3:
		    setMask = 0xF7;
		    break;
		case 4:
		    setMask = 0xEF;
		    break;
		case 5:
		    setMask = 0xDF;
		    break;
		case 6:
		    setMask = 0xBF;
		    break;
		case 7:
		    setMask = 0x7F;
		    break;
	}
	
	if(errorState){
	    //still in 1
    	}else{
            temp &= setMask;
    	}
    
    	*(volatile UINT8*)(DIAG_RESULT_TABLE_LOCATION + byteNumber) = temp;
    
    	temp = 0;
    	temp = *(volatile UINT8*)(DIAG_RESULT_TABLE_LOCATION + byteNumber);
 //   	printk(" stored in the bit %d of the byte %d. The Byte content: 0x%02X.\n", item_number, byte_number, temp);
	
    }else{
    	printk(" Too much item \n");
    }
}

/***********************************************************
	Name: DiagResultFromNVSRAM
	Synopsis: 
	Description: read diag. result recorded in NVSRAM

	Input Variables : 
	Output Variables: 
	Return Value: 

	Globals: 

***********************************************************/
void diagResultFromNvsram(UINT32 itemNumber)
{
    UINT32 byteNumber;
    UINT8  temp;
    UINT8  setMask = 0x01;
    
    //printk("[Diag Result Log] Item number: %d,", item_number);
    
    itemNumber--;
    
    byteNumber = itemNumber / 8;
    itemNumber = itemNumber % 8;
    
    if(byteNumber < (DIAG_MANUAL_DRIVE_OFFSET - DIAG_RESULT_TABLE_BASE_OFFSET)){
    	temp = *(volatile UINT8*)(DIAG_RESULT_TABLE_LOCATION + byteNumber);
    
    	if(temp & (setMask << itemNumber)){
            gDiagResultLogNvsram = 1;
	    //printk("1 \n");
    	}else{
            gDiagResultLogNvsram = 0;
	    //printk("0 \n");
    	}
	
    }else{
    	printk("[Diag Result Log] Too much item \n");
    }
}

/***********************************************************
	Name: DiagResultInNVSRAMClean
	Synopsis: 
	Description: clean diag. diag recorded in NVSRAM
	
	Input Variables : 
	Output Variables: 
	Return Value: 

	Globals: 

***********************************************************/
void diagResultInNvsramClean(void)
{
    INT32 i;
    
    for(i = 0; i < (DIAG_MANUAL_DRIVE_OFFSET - DIAG_RESULT_TABLE_BASE_OFFSET); i++)
	{
    	*(volatile UINT8*)(DIAG_RESULT_TABLE_LOCATION + i) = 0xFF;
    }
    printk("[Diag Result Log] Clean is done.\n");
}

/**************************************/
/**************************************/
/**  NVSRAM Backup Related Function  **/
/**************************************/
/**************************************/
/***********************************************************
	Name: NVSRAMtoFlashBackup
	Synopsis: 
	Description: copy whole NVSRAM content to Flash

	Input Variables :
	Output Variables: 
	Return Value: 

	Globals:

***********************************************************/
void NvsramtoFlashBackup(void)
{
    //copy whole NVSRAM content to Flash
    //ImageProgram(DIAG_NVSRAM_BACKUP_BASE, NVSRAM_TOTAL_SIZE, g_PBI_NVSRAM_VIRTUAL_BASE);
    printk("[NVSRAM Backup] Backup progress is done.\n");
}

/***********************************************************
	Name: NVSRAMfromFlashRestore
	Synopsis: 
	Description: copy whole NVSRAM content from Flash

	Input Variables :
	Output Variables: 
	Return Value: 

	Globals:

***********************************************************/
void NvsramfromFlashRestore(void)
{
#if 0
    voidP flash_base;
    INT32 i, j=0;
    
    //remap first 16MB flash
    flash_base = (void*)ioremap(0xFA000000, 0x00300000);
    
    //move content from flash to NVSRAM
    for(i=0; i < NVSRAM_TOTAL_SIZE; i++){
		if(i==0x1FFF0)	//Don't touch RTC area
			break;
    	*(volatile UINT8*)(g_PBI_NVSRAM_VIRTUAL_BASE + i) = 
	*(volatile UINT8*)(flash_base + DIAG_NVSRAM_BACKUP_BASE + i);
	j++;
	
	if(j == 0x00001000){
	    DiagPostProgress("[NVSRAM Backup] Restore progress", i, NVSRAM_TOTAL_SIZE);
	    j = 0;
	}
    }
    
    iounmap(flash_base);
#endif
    printk("[NVSRAM Backup] Restore progress is done.\n");	
}

/***********************************************************
	Name: NVSRAMBackupVerify
	Synopsis: 
	Description: verify the copy in flash with NVSRAM content

	Input Variables :
	Output Variables: 
	Return Value: 

	Globals:

***********************************************************/
void NvsramBackupVerify(void)
{
#if 0
    voidP flash_base;
    INT32 i, j=0;
    
    //remap first 16MB flash
    flash_base = (void*)ioremap(0xFA000000, 0x00300000);
    
    //move content from flash to NVSRAM
    for(i=0; i < NVSRAM_TOTAL_SIZE; i++)
	{
		if(i==0x1FFF0)	//Don't touch RTC area
			break;
    	
		if(*(volatile UINT8*)(g_PBI_NVSRAM_VIRTUAL_BASE + i) != 
	    	*(volatile UINT8*)(flash_base + DIAG_NVSRAM_BACKUP_BASE + i))
		{
	    	printk("[NVSRAM Backup] Content inconsistent in byte 0x%08X \n", i);
		}
		j++;
	
		if(j == 0x00001000)
		{
	    	commPostProgress("[NVSRAM Backup] Verification progress", i, NVSRAM_TOTAL_SIZE);
	    	j = 0;
		}
    }
    
    iounmap(flash_base);
#endif
	printk("[NVSRAM Backup] Verification progress is done.\n");	
}

/***********************************************************
	Name: NVSRAMPrint
	Synopsis: 
	Description: print NVSRAM content to screen

	Input Variables : base, size
	Output Variables: 
	Return Value: 

	Globals:

***********************************************************/
void NvsramPrint(UINT32 base, UINT32 size)
{
	void* 	initialBase;
	UINT32 	i, j;
	
	initialBase = (void*)(gFpgaBar2VirtualP + base);
	
	if ((base + size) > 0x80000)
	{
		printk("[NVSRAM Print] Over the NVSRAM range.\n");
	}
	else
	{
	
		printk("\n[NVSRAM Address]    [Content]\n");	
		printk("-------------------------------------------------------------------");
	
		j = 0;
		for (i = 0; i < size; i++)
		{
			if (j == 0){	
				printk("\n 0x%08X      -  ", i);	//print address
			}
		
			printk("%02X ", *(volatile UINT8*)(initialBase + i));
			j++;
		
			if (j == 16){	//alreay print 16 bytes, change line
				j = 0;
			}
		}

		printk("\n-------------------------------------------------------------------");	
 		printk("\n[NVSRAM Print] Done.\n");	
	}
}

/*******************************************/
/*******************************************/
/**  Board Stress Reset Related Function  **/
/*******************************************/
/*******************************************/
/***********************************************************
	Name: CtrlBoardStressVerify
	Synopsis: 
	Description: verify if the controller needs to run board stress test again

	Input Variables :  
	Output Variables: 
	Return Value: 

	Globals: gMBoardStressRestart
	           TRUE: Need to enter board stress automatically
	           FALSE: No need to enter board stress automatically

***********************************************************/       
void ctrlBoardStressVerify(void)
{
    if( (*(volatile UINT8*)BOOT_COUNT_ENCODED_BS_BYTE_0_LOCATION == BOOT_COUNT_ENCODED_BS_PATTERN_0) &&    
        (*(volatile UINT8*)BOOT_COUNT_ENCODED_BS_BYTE_1_LOCATION == BOOT_COUNT_ENCODED_BS_PATTERN_1) )
	{
        //if encoded bytes match the encoded pattern
        
        gMBoardStressRestart = *(volatile UINT16*)BOOT_COUNT_ENCODED_BS_STATUS_LOCATION;    
        
    }
	else
	{   
        gMBoardStressRestart = FALSE;   
    }   
}   

/***********************************************************
	Name: CtrlBoardStressSet
	Synopsis: 
	Description: set encoded patterns to encoded bytes

	Input Variables :  rotate
	Output Variables: 
	Return Value: 

	Globals:

***********************************************************/
void CtrlBoardStressSet(UINT16 rotateCount)
{
    fpgaNvsramUnlockAll();
	
	//Set Syndrome
    *(volatile UINT8*)BOOT_COUNT_ENCODED_BS_BYTE_0_LOCATION = BOOT_COUNT_ENCODED_BS_PATTERN_0;    
    *(volatile UINT8*)BOOT_COUNT_ENCODED_BS_BYTE_1_LOCATION = BOOT_COUNT_ENCODED_BS_PATTERN_1;
	
	//Set Status
	*(volatile UINT16*)BOOT_COUNT_ENCODED_BS_STATUS_LOCATION = rotateCount;   
}   

/***********************************************************
	Name: CtrlBoardStressClear
	Synopsis: 
	Description: reset encoded bytes to 0

	Input Variables :  
	Output Variables: 
	Return Value: 

	Globals:

***********************************************************/
void ctrlBoardStressClear(void)
{   
    fpgaNvsramUnlockAll();
    
	//Clear Syndrome
	*(volatile UINT8*)BOOT_COUNT_ENCODED_BS_BYTE_0_LOCATION = 0;
    *(volatile UINT8*)BOOT_COUNT_ENCODED_BS_BYTE_1_LOCATION = 0;
	
	//Clear Status
    *(volatile UINT16*)BOOT_COUNT_ENCODED_BS_BYTE_1_LOCATION = 0;
}   


