/****************************************************************************
*                                                                           *
* Copyright 2004-2011 NetApp Inc. All rights reserved. This file is         *
* confidential and a trade secret of NetApp Inc. The receipt or             *
* possession of this file does not convey any rights to reproduce or        *
* disclose its contents or to manufacture, use, or sell anything it may     *
* describe, in whole, or in part, without the specific written consent of   *
* LSI Corporation.                                                          *
*                                                                           *
****************************************************************************/
/****************************************************************************
*                                                                           *
* NAME            i2c-zebulon.c                                             *
* SUMMARY                                                                   *
* VERSION         %version: %                                               *
* UPDATE DATE     %date_modified: %                                         *
* PROGRAMMER      %created_by:    Merck Hung %                              *
*                                                                           *
*        Copyright 2002-2011 NetApp Inc. All Rights Reserved.               *
*                                                                           *
* DESCRIPTION:                                                              *
*                                                                           *
****************************************************************************/
/*******************************************************************************

NAME			$RCSfile: i2c-zebulon.c,v $
SUMMARY			LSI Zebulon FPGA I2C Bus Driver
VERSION			$Revision: 1.3 $
UPDATE DATE		$Date: 2010/03/10 02:39:47 $
PROGRAMMER		$Author: small $

		Copyright 2009 LSI Corporation. All Rights Reserved.

DESCRIPTION:
	This driver is porting from LSI ApI2c master driver.

REFERENCE:
*******************************************************************************/
#include <linux/pci.h>
#include <linux/module.h>
#include <linux/i2c.h>
#include <linux/wait.h>
#include <linux/interrupt.h>
#include <linux/platform_device.h>
#include <linux/delay.h>

#include "commTypes.h"
#include "fpgaI2cDiag.h"
#include "fpgaRegLib.h"
#include "pci-zebulon.h"
#include "fpga-address.h"
#include "i2c-zebulon.h"
#include "i2c-address.h"


struct i2c_adapter *zebulonAdapter[ ZEBULON_I2C_BUS_NUM ];
EXPORT_SYMBOL(zebulonAdapter);

static I2C_BUS_MASTER_INFO lsiBusMasterInfo[ ZEBULON_I2C_BUS_NUM ];
static int bus_speed = ZEBULON_I2C_CLK_100MHZ;

static volatile void __iomem *mmioVirtual = 0;
static volatile void __iomem *baseVirtual[ ZEBULON_I2C_BUS_NUM ] = { 0, 0, 0, 0 };


struct i2c_adapter *i2cZebulonGetAdapter( int num ) {

	if( (num < 0) || (num >= ZEBULON_I2C_BUS_NUM) )
		return NULL;

	return zebulonAdapter[ num ];
}
EXPORT_SYMBOL(i2cZebulonGetAdapter);


void i2cLsiPrint(int bus)
{
    LSI_I2C *printBase;
	I2C_BUS_MASTER_INFO *BusMasterInfo;
	
	if (bus < 0 || bus >= ZEBULON_I2C_BUS_NUM) {
		printk("Invalid bus index(%d)\n", bus);
		return;
	}

	BusMasterInfo = &lsiBusMasterInfo[bus];

    printBase = (LSI_I2C *) BusMasterInfo->baseAddr;

    printk("\n\nLSI I2C(%d) REGISTERS:\n", bus);
    printk(" base = 0x%p\n", printBase);
    printk(" IGC = 0x%x\n", printBase->IGC);
    printk(" IIS = 0x%x\n", printBase->IIS);
    printk(" IIE = 0x%x\n", printBase->IIE);
    printk(" IWTC = 0x%x\n", printBase->IWTC);
    printk(" IIT = 0x%x\n", printBase->IIT);
    printk(" IILM = 0x%x\n", printBase->IILM);
    printk(" IILS = 0x%x\n", printBase->IILS);
    printk(" ITCDC = 0x%x\n", printBase->ITCDC);
    printk(" IM = 0x%x\n", printBase->IM);
    printk(" ISR = 0x%x\n\n", printBase->ISR);
    printk(" IMC = 0x%x\n", printBase->IMC);
    printk(" IMRTL = 0x%x\n", printBase->IMRTL);
    printk(" IMTTL = 0x%x\n", printBase->IMTTL);
    printk(" IMAR1 = 0x%x\n", printBase->IMAR1);
    printk(" IMAR2 = 0x%x\n", printBase->IMAR2);
    printk(" IMD = 0x%x\n", printBase->IMD);
    printk(" IMTFS = 0x%x\n", printBase->IMTFS);
    printk(" IMRFS = 0x%x\n", printBase->IMRFS);
    printk(" IMIE = 0x%x\n", printBase->IMIE);
    printk(" IMIS = 0x%x\n", printBase->IMIS);
    printk(" IMTB = 0x%x\n", printBase->IMTB);
    printk(" IMRB = 0x%x\n\n", printBase->IMRB);
    printk(" ISHP = 0x%x\n", printBase->ISHP);
    printk(" ISLP = 0x%x\n", printBase->ISLP);
    printk(" ISFC = 0x%x\n", printBase->ISFC);
    printk(" ISST = 0x%x\n", printBase->ISST);
    printk(" ISHT = 0x%x\n\n", printBase->ISHT);
}
EXPORT_SYMBOL(i2cLsiPrint);


void i2cLsiResetMaster(LSI_I2C *base)
{
	if (base == NULL)
		return;

	
    /*******************************************/
    /* Clear IGC first to disable the I2C Unit */
    /* then reset the i2c controller.          */
    /*******************************************/
    base->IGC = 0;
	SOFT_RESET();

    /* Set up the bus speed. We can operate at either 100 or 400 KBit */
    if ( bus_speed == ZEBULON_I2C_CLK_100MHZ) {
        base->ISHP = SCL_HIGH_PERIOD_100;
        base->ISLP = SCL_LOW_PERIOD_100;
        base->ISFC = SPIKE_FILTER_CONTROL_100;
        base->ISST = SDA_SETUP_TIME_100;
        base->ISHT = SDA_HOLD_TIME_100;
    } else {
        base->ISHP = SCL_HIGH_PERIOD_400;
        base->ISLP = SCL_LOW_PERIOD_400;
        base->ISFC = SPIKE_FILTER_CONTROL_400;
        base->ISST = SDA_SETUP_TIME_400;
        base->ISHT = SDA_HOLD_TIME_400;
    }

    base->IGC = ENABLE_MASTER;
	// FIXME: Wait for too long
	I2C_LSI_BUS_RESET_DELAY();

    /************************************************************************/
    /* NAK_RECEIVED_DURING_TX_DATA_PHASE and NAK_RECEIVED_DURING_ADDR_PHASE */
    /* are also used, but the Master sends a stop after detecting a NAK     */
    /* which results in a TRANSFER_COMPLETED_STOP_SENT interrupt.  When     */
    /* this interrupt is received, the status of the NAK interrupts are     */
    /* checked and the conditions are handled if needed.                    */
    /************************************************************************/
    base->IMIE = TRANSFER_COMPLETED_STOP_SENT | ARBITRATION_LOST 
		| TX_FIFO_LOW_THRESHOLD_REACHED | RX_FIFO_HIGH_THRESHOLD_REACHED;

    base->IIE = MASTER_INTERRUPT_ENABLE;
}


static irqreturn_t i2cLsiInterrupt(int irq, void *unused)
{
    UINT16 isr_value;
    UINT16 temp;
    int i;
    int extraData=0;
    int istat=0;
    LSI_I2C *intBase=0;
    I2C_BUS_MASTER_INFO  *intData=0;

    if( !(fpgaRegisterRead(FPGA_INTERRUPT_SOURCE_INT_B) & I2C_INT_BIT) )
        return IRQ_NONE;

    /******************************************************/
    /* Find out which master caused the interrupt and set */
    /* the base address accordingly.                      */
    /******************************************************/
	if( lsiBusMasterInfo[ 0 ].baseAddr && 
		((LSI_I2C *)lsiBusMasterInfo[ 0 ].baseAddr)->IIS & 0x0001 ) {

		istat = 1;
		intData = &(lsiBusMasterInfo[ 0 ]);
		intBase = (LSI_I2C*)intData->baseAddr;
	}
	else if( lsiBusMasterInfo[ 1 ].baseAddr &&
		((LSI_I2C *)lsiBusMasterInfo[ 1 ].baseAddr)->IIS & 0x0001 ) {

        istat = 1;
        intData = &(lsiBusMasterInfo[ 1 ]);
		intBase = (LSI_I2C*)intData->baseAddr;
	}
	if( lsiBusMasterInfo[ 2 ].baseAddr && 
		((LSI_I2C *)lsiBusMasterInfo[ 2 ].baseAddr)->IIS & 0x0001 ) {

        istat = 1;
        intData = &(lsiBusMasterInfo[ 2 ]);
		intBase = (LSI_I2C*)intData->baseAddr;
	}
	if( lsiBusMasterInfo[ 3 ].baseAddr && 
		((LSI_I2C *)lsiBusMasterInfo[ 3 ].baseAddr)->IIS & 0x0001 ) {

        istat = 1;
        intData = &(lsiBusMasterInfo[ 3 ]);
		intBase = (LSI_I2C*)intData->baseAddr;
	}

	if (!istat) {

		writeb(readb(mmioVirtual + 0x71) & 0x1, mmioVirtual + 0x10);
		goto doNothing;
	}

    while (istat)
    {
        /*************************************************************/
        /* Individual Interrupt Status is cleared when IMIS is read. */
        /*************************************************************/
        isr_value = intBase->IMIS; 
        if ( CHECK_BIT(ARBITRATION_LOST, isr_value) )
        {
            /****************************************************************/
            /* If we are in the middle of a transfer, flush the RX and TX   */
            /* FIFOs, and release the wait sem to allow the driver to retry */
            /* the transfer.                                                */
            /****************************************************************/
            if ( intData->i2cTransferInProgress )
            {
                /*************************************************/
                /* If we received data, it should be thrown away */
                /*************************************************/
                while (intBase->IMRFS > 0)
                    temp = intBase->IMD;
    
                intData->i2cStatus=ERROR;

				intData->i2cWaitDone = TRUE;
				wake_up(&intData->i2cWaitQueue);
            }
        }

        if ( CHECK_BIT(RX_FIFO_HIGH_THRESHOLD_REACHED, isr_value) )
        {
            BYTE* rcvBuf = intData->i2cReceiveBuffer;

            /* Empty the RX FIFO to make room for incoming bytes. */
            while (intBase->IMRFS != 0)
            {
                rcvBuf[intData->i2cDataIndex] = intBase->IMD;
                (intData->i2cDataIndex)++;
            }
        }

        if ( CHECK_BIT(TX_FIFO_LOW_THRESHOLD_REACHED, isr_value) )
        {
            /* Refill TX FIFO. */
            BYTE* sendBuf = intData->i2cSendBuffer;

            while ( (intBase->IMTFS < 7) && (intData->i2cDataIndex < intData->i2cSendCount) )
            {
                intBase->IMD = sendBuf[intData->i2cDataIndex];
                (intData->i2cDataIndex)++;
            }
        }

        if ( CHECK_BIT(TRANSFER_COMPLETED_STOP_SENT, isr_value) )
        {
            if ( CHECK_BIT(NAK_RECEIVED_DURING_ADDR_PHASE, isr_value) )
            {
                /****************************************************************/
                /* If we are in the middle of a transfer, release the wait sem  */
                /* to allow the driver to retry the transfer.                   */
                /****************************************************************/
                if ( intData->i2cTransferInProgress )
                {
                    intData->deviceUnresponsive = TRUE;
                    intData->i2cStatus = ERROR;

					intData->i2cWaitDone = TRUE;
					wake_up(&intData->i2cWaitQueue);
                }
            }

            if ( CHECK_BIT(NAK_RECEIVED_DURING_TX_DATA_PHASE, isr_value) )
            {
                /****************************************************************/
                /* If we are in the middle of a transfer, flush the RX and TX   */
                /* FIFOs, and release the wait sem to allow the driver to retry */
                /* the transfer.                                                */
                /****************************************************************/
                if ( intData->i2cTransferInProgress )
                {
                    /*************************************************/
                    /* If we received data, it should be thrown away */
                    /*************************************************/
                    while (intBase->IMRFS > 0)
                        temp = intBase->IMD;

                    intData->i2cStatus=ERROR;

					intData->i2cWaitDone = TRUE;
					wake_up(&intData->i2cWaitQueue);
                }
            }

            if (intData->i2cStatus == OK)
            {
                /***************************************************/
                /* Check to see if we did transmit or receive data */
                /***************************************************/
                if (!(((intData->i2cSendCount||intData->i2cRegAddr) && (!intBase->IMTB)) 
                   || (intData->i2cReceiveCount && (!intBase->IMRB))))
                {
                    /***********************************************/
                    /* Transfer Successful, read data.             */
                    /***********************************************/
                    i = intData->i2cDataIndex + intBase->IMRFS;

                    if ((i > intData->i2cReceiveCount) && (intData->i2cReceiveCount != 0))
                    {
                        /**************************************/
                        /* More data is in the receive FIFO   */
                        /* than was requested.  Flush the     */
                        /* FIFO and return an ERROR.          */
                        /**************************************/
                        extraData = intData->i2cReceiveCount - i; 
                        while (extraData > 0)
                        {
                            temp = intBase->IMD;
                            extraData--;
                        }
                        intData->i2cStatus=ERROR;
                    }
                    else
                    {
                        while ((int)intData->i2cDataIndex < i)
                        {
                            intData->i2cReceiveBuffer[intData->i2cDataIndex] = intBase->IMD;
                            (intData->i2cDataIndex)++;
                        }
                        intData->i2cBytesSent += intBase->IMTB - intData->i2cRegAddr;
                    }
                }
                /***************************************************/
                /* If we did not transmit or receive any data like */
                /* we expect, then set error status. It will happen*/
                /* if we receive spurious interrupt from master    */
                /***************************************************/
                else
                {
                    intData->i2cStatus = ERROR;
                }

				intData->i2cWaitDone = TRUE;
				wake_up(&intData->i2cWaitQueue);
            }
        }

        intBase->IIS = 0x0001; 
        istat = 0;
        /****************************************************/
        /* Check if the other master simultaneously caused  */ 
        /* the interrupt.                                   */
        /****************************************************/
		if( ((LSI_I2C *)lsiBusMasterInfo[ 0 ].baseAddr)->IIS & 0x0001 ) {

			istat = 1;
			intData = &(lsiBusMasterInfo[ 0 ]);
			intBase = (LSI_I2C*)intData->baseAddr;
		}
		else if( ((LSI_I2C *)lsiBusMasterInfo[ 1 ].baseAddr)->IIS & 0x0001 ) {

        	istat = 1;
        	intData = &(lsiBusMasterInfo[ 1 ]);
			intBase = (LSI_I2C*)intData->baseAddr;
		}
		else if( ((LSI_I2C *)lsiBusMasterInfo[ 2 ].baseAddr)->IIS & 0x0001 ) {

        	istat = 1;
        	intData = &(lsiBusMasterInfo[ 2 ]);
			intBase = (LSI_I2C*)intData->baseAddr;
		}
		else if( ((LSI_I2C *)lsiBusMasterInfo[ 3 ].baseAddr)->IIS & 0x0001 ) {

        	istat = 1;
        	intData = &(lsiBusMasterInfo[ 3 ]);
			intBase = (LSI_I2C*)intData->baseAddr;
		}

		/* Clear Interrupt */
		writeb( readb( mmioVirtual + 0x71 ) & 0x2, mmioVirtual + 0x16 );
		writeb( readb( mmioVirtual + 0x71 ) & 0x1, mmioVirtual + 0x10 );
    }

doNothing:
	return (IRQ_HANDLED);
}


static void i2cLsiTransferPacket( ULONG command, I2C_BUS_MASTER_INFO *busMasterInfo ) {

    //ULONG i2cLsiWaitTime;
    int clockCount;
    LSI_I2C *transferBase;
	ULONG timeLeft;

	//printk("WensonTest01: function:%s  line:0x%x\n",__FUNCTION__,__LINE__);

    /* 
     * In System Ticks.  In a 60 Hz system, this produces
     * a wait time of about 166 ms, 2-3 times the expected
     * completion time of the max 64 B transfer.
     */
    //i2cLsiWaitTime = 1000; /* set to 1000ms */

    transferBase = (LSI_I2C *)busMasterInfo->baseAddr;

    busMasterInfo->i2cTransferInProgress = TRUE;
    busMasterInfo->deviceUnresponsive = FALSE;

    /*****************************************************/
    /* Do not check for the bus to be free, this is done */
    /* in hardware.                                      */ 
    /*****************************************************/

    busMasterInfo->i2cStatus = OK;

    /**************************/
    /* Start the i2c transfer */
    /**************************/
    transferBase->IMC = command | ISSUE_COMMAND;

    busMasterInfo->i2cWaitDone = FALSE;

    timeLeft = wait_event_interruptible_timeout( busMasterInfo->i2cWaitQueue,
			busMasterInfo->i2cWaitDone, msecs_to_jiffies( 200 ) );

	if (!timeLeft)
    {
        /*********************************************************/
        /* If the master did not respond with a no ack, it is    */
        /* stuck waiting for the bus. This should never happen.  */
        /* If it does, the transaction will never end.  If the   */
        /* data line is held low, the slave may be holding it    */
        /* low.  Try to manually advance the clock to free the   */
        /* slave.  Reset the master and start the entire         */
        /* transaction again.                                    */
        /*********************************************************/
        if ( (transferBase->IM & SDA_STATUS) == 0 )
        {
            clockCount = 0;
            while ( (transferBase->IM & SDA_STATUS) == 0 )
            {
                /************************************************/
                /* A slave device is holding the data line low. */
                /* Try toggling the clock line to get the slave */
                /* to release the data line.  If it doesn't     */
                /* work after 20 clocks, give up.               */
                /************************************************/
                if (clockCount > 20)
                    break;

                transferBase->IM |= SCL_CONTROL;
                delayMicroseconds(5);
                transferBase->IM &= ~SCL_CONTROL;
                delayMicroseconds(5);
                clockCount++;
            }
            if ( (transferBase->IM & SDA_STATUS) == 0 )
            {
                DQ_WRITE_LSIL(DRV_BUS_HUNG, busMasterInfo->busNumber);
            }
        }

		//printk("WensonTest02: function:%s  line:0x%x\n",__FUNCTION__,__LINE__);
		i2cLsiResetMaster(transferBase);

        /**************************************************************/
        /* In case you got an interrupt before the reset, start over. */
        /**************************************************************/
        busMasterInfo->i2cBytesSent=0;
        busMasterInfo->i2cDataIndex=0;
        busMasterInfo->i2cStatus = ERROR;
    }
    busMasterInfo->i2cTransferInProgress = FALSE;
	//printk("WensonTest03: function:%s  line:0x%x\n",__FUNCTION__,__LINE__);
	//printk("WensonTest:ISR:0x%x INT_EN:0x%x INT_EN_T:0x%x\n",transferBase->IMIS,transferBase->IMIE,transferBase->IIE);

    I2C_BUS_COMPLETE_DELAY();
}


static int i2cLsiSendPacket(I2C_PACKET *packet)
{
    ULONG i = 0;
    LSI_I2C *sendBase;
    I2C_BUS_MASTER_INFO  *busMasterInfo;
    ULONG i2cRetryCount;
    ULONG originalDataIndex;

	busMasterInfo = &lsiBusMasterInfo[ packet->busNumber ];
    sendBase = (LSI_I2C *)busMasterInfo->baseAddr;
	//printk("sendBase Addr:0x%x\n",sendBase);

    /***********************************************************/
    /* We shift left by one because the transfer direction is  */
    /* the zeroth bit in the register and the address is bits  */
    /* 1-7.  The address should not be shifted when defined.   */
    /***********************************************************/
    busMasterInfo->i2cTarget = (packet->i2cAddr << 1);

    if ( packet->flags & I2C_READ )
    {
        /*****************************************************/
        /* Setup to read the data immediately after the      */
        /* the target address is sent on the bus.            */
        /*****************************************************/
        busMasterInfo->i2cReceiveBuffer = packet->dataPtr;
        busMasterInfo->i2cReceiveCount = packet->numBytes;

#if defined(CHASSIS_WSAS)
        //busMasterInfo->i2cSendCount = 0;
        busMasterInfo->i2cSendCount = packet->snumBytes;

    	if(busMasterInfo->i2cSendCount)
        	memcpy(busMasterInfo->i2cSendBuffer,packet->sdataPtr, packet->snumBytes);

        busMasterInfo->i2cTarget |= 0x01;
#else // if defined(CHASSIS_WSAS)
        busMasterInfo->i2cSendCount = 0;
        busMasterInfo->i2cTarget |= 0x01;
#endif // if defined(CHASSIS_WSAS)

        /*****************************************************/
        /* If the target device requires a register address, */
        /* then the register address and a repeated start is */
        /* written on the i2c bus before the read is done.   */
        /*****************************************************/
        if ( !(packet->flags & I2C_NO_REG_ADDR) )
        {
            busMasterInfo->i2cRegAddr = (packet->flags & I2C_DUAL_BYTE_ADDR) ? 2 : 1;
            busMasterInfo->regAddr = packet->regAddr;
            busMasterInfo->i2cCommandType = SEQUENCE_MODE_TRANSFER;
        }
        else
        {
            busMasterInfo->i2cRegAddr = 0;
            busMasterInfo->i2cCommandType = AUTOMATIC_MODE_TRANSFER;
        }

        /*************************************************************/
        /* workaround for Alder bridge board MUX addr=0xF0, need to  */
        /* set IMAR2 with regAddr instead of using regular data sent.*/
        /*************************************************************/
        if (packet->flags & I2C_10BIT)
        {
            busMasterInfo->i2cRegAddr = 0;
            busMasterInfo->i2cCommandType = AUTOMATIC_MODE_TRANSFER;
        }

    }
    else
    {
        /*****************************************************/
        /* Byte count is equal to the number of data bytes.  */
        /* The I2C address and the register addresses are    */
        /* handles seperately to simplify the 8 byte         */
        /* transfer limitation.                              */
        /*****************************************************/
        busMasterInfo->i2cSendCount = packet->numBytes;
        busMasterInfo->i2cReceiveCount = 0;
        busMasterInfo->i2cCommandType = AUTOMATIC_MODE_TRANSFER;
        memcpy(busMasterInfo->i2cSendBuffer,packet->dataPtr, packet->numBytes);

        if ( !(packet->flags & I2C_NO_REG_ADDR) )
        {
            busMasterInfo->i2cRegAddr = (packet->flags & I2C_DUAL_BYTE_ADDR) ? 2 : 1;
            busMasterInfo->regAddr = packet->regAddr;
        }
        else
            busMasterInfo->i2cRegAddr = 0;


        /*************************************************************/
        /* workaround for Alder bridge board MUX addr=0xF0, need to  */
        /* set IMAR2 with regAddr instead of using regular data sent.*/
        /*************************************************************/
        if (packet->flags & I2C_10BIT)
        {
            busMasterInfo->regAddr = packet->regAddr;
        }

    }

    busMasterInfo->i2cBytesSent=0;
    busMasterInfo->i2cDataIndex=0;

    for ( ; ; )
    {
        /*************************************************************/
        /* Keep retrying the transfer until it is successful or too  */
        /* many retries are used.                                    */
        /*************************************************************/
        i2cRetryCount = 0;
        originalDataIndex = busMasterInfo->i2cDataIndex;
        while ( i2cRetryCount <= MAX_LSI_I2C_RETRIES )
        {
            /*********************************************************/
            /* The transmit FIFO is not cleared on every failure. If */
            /* there is data in the FIFO clear it. To prevent the TX */
            /* FIFO empty interrupt clear the TX length reg first.   */
            /*********************************************************/
            if (sendBase->IMTFS)
            {
				//printk("WensonTest:There are 0x%x Bytes Data in FIFO\n",(sendBase->IMTFS)&0x0f);
                sendBase->IMTTL = 0;
                sendBase->IMC = FLUSH_MASTER_TX_FIFO | ISSUE_COMMAND;
            }

            i2cRetryCount++;
            busMasterInfo->i2cDataIndex = originalDataIndex;

            /*************************************************************/
            /* Load TX FIFO.  Do not exceed 8 bytes per transfer for bus */
            /* fairness in Mutimaster situations.  The register address  */
            /* is included in the 8 byte limit.  8 bytes is also used    */
            /* since that is the transmit and receive buffer sizes on    */
            /* the master.                                               */
            /*************************************************************/

			//printk("sendCount: 0x%x\n",busMasterInfo->i2cSendCount);
			///printk("BytesSent: 0x%x\n",busMasterInfo->i2cBytesSent);
			//printk("i2cRegAdd: 0x%x\n",busMasterInfo->i2cRegAddr);

            for (i=0; i<(busMasterInfo->i2cSendCount - busMasterInfo->i2cBytesSent + busMasterInfo->i2cRegAddr); i++)
            {
                if ( i < busMasterInfo->i2cRegAddr )
                {
                    /***************************************************/
                    /* This assumes that the only time the driver      */
                    /* accesses more than 8 bytes in one SendPacket    */
                    /* call when using a register address is if the    */
                    /* register address is the offset that the read    */
                    /* or write occurs at.  The register address must  */
                    /* be incremented to the next transfer offset.     */
                    /***************************************************/
                    if ((busMasterInfo->i2cRegAddr == 2) && (i == 0))
                        sendBase->IMD = ((busMasterInfo->regAddr + busMasterInfo->i2cDataIndex) & 0xFF00) >> 8;
                    else
                        sendBase->IMD = ((busMasterInfo->regAddr + busMasterInfo->i2cDataIndex) & 0x00FF);
                }
                else
                {
                    sendBase->IMD = busMasterInfo->i2cSendBuffer[busMasterInfo->i2cBytesSent+i-busMasterInfo->i2cRegAddr];
                    busMasterInfo->i2cDataIndex++;
                    if (i==7)
                    {
                        i++;
                        break;
                    }
                }
            }

#if defined(CHASSIS_WSAS)
        if ( ( packet->flags & I2C_READ )&&(busMasterInfo->i2cSendCount > 0) )
            busMasterInfo->i2cDataIndex = 0;
#endif

            /* Limit transfers to 64 bytes for fairness on multi-master busses. */
            if ( (busMasterInfo->i2cSendCount + busMasterInfo->i2cRegAddr - busMasterInfo->i2cBytesSent) > 64)
            {
                sendBase->IMTTL = 64;
            }
            else
            {
                sendBase->IMTTL = busMasterInfo->i2cSendCount + busMasterInfo->i2cRegAddr - busMasterInfo->i2cBytesSent;
            }

            if ( (busMasterInfo->i2cReceiveCount - busMasterInfo->i2cDataIndex) > 64)
            {
                sendBase->IMRTL = 64;
            }
            else
            {
                sendBase->IMRTL = busMasterInfo->i2cReceiveCount - busMasterInfo->i2cDataIndex;
            }

            // workaround for Alder bridge board mux addr:0xF0; need to put the reg. addr to IMAR2
            // since I2C core would send it out for 10bit device.
            if(packet->flags & I2C_10BIT)
            {
                sendBase->IMAR2 = busMasterInfo->regAddr;
            }

            sendBase->IMAR1 = busMasterInfo->i2cTarget;
            i2cLsiTransferPacket(busMasterInfo->i2cCommandType, busMasterInfo);
            if ( busMasterInfo->i2cStatus == OK )
                break;
        }

        if (busMasterInfo->i2cStatus != OK)
        {
			return (busMasterInfo->i2cStatus);
        }
        else
        {
            /* break out of loop if we're done transferring data */
            if ( (packet->flags & I2C_READ ) && (busMasterInfo->i2cDataIndex == packet->numBytes) )
                break;
            else if ((!(packet->flags & I2C_READ)) && (busMasterInfo->i2cBytesSent==packet->numBytes))
                break;

            /************************************************************/
            /* There is more data to transfer, but only 8 bytes are     */
            /* allowed at a time.                                       */
            /************************************************************/
        }
    } /* while data to transfer and previous transfer successful */

    return (OK);
}


static int i2c_zebulon_translate_packet(I2C_PACKET *packet,
		struct i2c_msg *msgs, int num)
{
	I2C_PACKET_FLAGS op = 0;

	if (num == 1) {

		op |= (msgs[0].flags & I2C_M_RD) ? I2C_READ : I2C_WRITE;
		op |= I2C_NO_REG_ADDR;

		if (msgs[0].flags & I2C_M_TEN)  // flag set for Alder special addr. 0xF0
		{
			op |= I2C_10BIT;
			packet->regAddr = msgs[0].buf[0];
			packet->dataPtr = (msgs[0].buf) + 1;  // write Data start ptr.
			packet->numBytes = msgs[0].len - 1;   // write len -1 since the cmd/reg is moved to IMAR2
		}
		else
		{
			packet->dataPtr = msgs[0].buf;
			packet->numBytes = msgs[0].len;
		}

	} else if (num == 2 &&
			!(msgs[0].flags & I2C_M_RD) &&
			msgs[1].flags & I2C_M_RD) {

		op |= I2C_READ;
		op |= (msgs[0].len == 2) ? I2C_DUAL_BYTE_ADDR : 0;
		op |= (msgs[0].flags & I2C_M_TEN) ? I2C_10BIT : 0; // flag set for Alder special addr. 0xF0

#if defined(CHASSIS_WSAS)
        //Start modify BY Diran
        if(msgs[0].len > 2)
        {
            packet->sdataPtr = &(msgs[0].buf[2]);
            packet->snumBytes = msgs[0].buf[1] - 1;
            msgs[1].len = msgs[0].buf[(msgs[0].len - 1)];

        } else {
            packet->snumBytes = 0;
        }
        //End modify By Diran
#endif

		packet->dataPtr = msgs[1].buf;
		packet->numBytes = msgs[1].len;

		if (op & I2C_DUAL_BYTE_ADDR)
			packet->regAddr = (msgs[0].buf[0] << 8) + msgs[0].buf[1];
		else
			packet->regAddr = msgs[0].buf[0];
		
	} else {
		printk("[ZEBULON I2C] unsupported transaction sequence\n");
		return (-EINVAL);
	}

	packet->i2cAddr = msgs[0].addr;
	packet->flags = op;

	return 0;
}


static DEFINE_MUTEX( i2c_lock );
static int i2c_zebulon_master_xfer( struct i2c_adapter *adap,
		struct i2c_msg *msgs, int num )
{
	I2C_BUS_MASTER_INFO *busMasterInfo = (I2C_BUS_MASTER_INFO*) adap->algo_data;
	I2C_PACKET packet;
	int err;
	char quick_buf;

#if 0
    printk("i2c_zebulon_master_xfer, num = %d\n", num);
    printk("i2c_zebulon_master_xfer, msgs[0].addr = 0x%x\n", msgs[0].addr);
    printk("i2c_zebulon_master_xfer, msgs[0].flags = 0x%x\n", msgs[0].flags);
    printk("i2c_zebulon_master_xfer, msgs[0].len = 0x%x\n", msgs[0].len);

    printk("i2c_zebulon_master_xfer, msgs[1].addr = 0x%x\n", msgs[1].addr);
    printk("i2c_zebulon_master_xfer, msgs[1].flags = 0x%x\n", msgs[1].flags);
    printk("i2c_zebulon_master_xfer, msgs[1].len = 0x%x\n", msgs[1].len);

    for(err = 0;err < (msgs[0].len); err++)
        printk("i2c_zebulon_master_xfer, msgs[0].buf[%d] = 0x%x\n", err, msgs[0].buf[err]);
#endif

	// Handle I2C Quick access (I2C Scan)
	if( num == 1 && !msgs[ 0 ].len ) {

		quick_buf = 0xFF;
		msgs[ 0 ].buf = &quick_buf;
		msgs[ 0 ].len = 1;
	}

	// Clear memory
	memset( &packet, 0, sizeof( I2C_PACKET ) );

	// Translate Linux i2c_msg to LSI I2C_PACKET
	err = i2c_zebulon_translate_packet( &packet, msgs, num );
	if( err )
		return err;

	// Determine which adapter
	packet.busNumber = busMasterInfo->busNumber;

	// Lock
	mutex_lock( &i2c_lock );


#if 0
    // Find out which bus we're going to use
    if( adap == zebulonAdapter[ 1 ] )
        fpgaI2cMdpl1Select();
    else if( adap == zebulonAdapter[ 2 ] )
        fpgaI2cMdpl2Select();
#else
    // Find out which bus we're going to use
    if( adap == zebulonAdapter[ 0 ] ) {
        fpgaI2cMdpl1Select();
	}
    else if( adap == zebulonAdapter[ 1 ] ){
        fpgaI2cMdpl2Select();
	}
#endif

	// Send packet
	err = i2cLsiSendPacket( &packet );

	// Unlock
	mutex_unlock( &i2c_lock );

#if 0
    DBGPRINT("2 i2c_zebulon_master_xfer, num = %d\n", num);
    DBGPRINT("2 i2c_zebulon_master_xfer, err = %d\n", err);
#endif

	if( err )
		return -EREMOTEIO;

	return num;
}


static uint32_t i2c_zebulon_func(struct i2c_adapter *adap)
{
	return I2C_FUNC_I2C | I2C_FUNC_SMBUS_EMUL;
}


static const struct i2c_algorithm i2c_zebulon_algo = {
	.master_xfer	= i2c_zebulon_master_xfer,
	.functionality	= i2c_zebulon_func,
};


static int i2cZebulonProbe( struct platform_device *pdev ) {

	int i, err = 0;
	struct resource *mem, *irq;
	unsigned long membase;
	struct i2c_adapter *adapter;

	// MMIO
    mem = platform_get_resource( pdev, IORESOURCE_MEM, 0 );
    if( !mem ) {

        dev_err( &pdev->dev, "no mem resource?\n" );
        return -ENODEV;
    }

	// IRQ
    irq = platform_get_resource( pdev, IORESOURCE_IRQ, 0 );
    if( !irq ) {

        dev_err( &pdev->dev, "no irq resource?\n" );
        return -ENODEV;
    }

    // Find out the resources
    membase = mem->start;

    // IO remapping
    mmioVirtual = getFpgaMmioVirtual();
    if( !mmioVirtual ) {

		dev_err( &pdev->dev, "ioremap failed\n" );
        goto exit;
    }

	// Register interrupt routine
	err = request_irq( irq->start, i2cLsiInterrupt, IRQF_SHARED, "lsi_i2c", pdev );
	if (err) {

		dev_err( &pdev->dev, "failed to request FPGA i2c master IRQ\n" );
		goto exit;
	}

	/* Register each i2c master */
	membase += ZEBULON_I2C_MASTER_OFFSET;
	for( i = 0 ; i < ZEBULON_I2C_BUS_NUM ; i++ ) {

		// Map each I2C Master
		baseVirtual[ i ] = ioremap( membase, ZEBULON_I2C_MASTER_MAP_SIZE );
		if( !baseVirtual[ i ] ) {
			printk( KERN_ERR "Zebulon base address ioremap failed\n" );
			err = -ENOMEM;
			goto exit;
		}

		/* Reset bus master */
		i2cLsiResetMaster( (LSI_I2C *)baseVirtual[ i ] );

		/* Initialize bus master info */
		memset( &lsiBusMasterInfo[ i ], 0, sizeof( I2C_BUS_MASTER_INFO ) );
		lsiBusMasterInfo[ i ].busNumber = i;
		lsiBusMasterInfo[ i ].baseAddr = (void *)baseVirtual[ i ];

		/* initialize wait queue */
		init_waitqueue_head( &(lsiBusMasterInfo[ i ].i2cWaitQueue) );

		/* Kernel specific data */
		adapter = kzalloc( sizeof( struct i2c_adapter ), GFP_KERNEL );
		if (!adapter) {

			printk( KERN_ERR "Out of memory for Zebulon adapter\n" );
			err = -ENOMEM;
			goto exit;
		}

		/* Prepare adapter data */
		adapter->owner = THIS_MODULE;
		adapter->class = I2C_CLASS_HWMON | I2C_CLASS_SPD;
		adapter->dev.parent = &pdev->dev;
		adapter->timeout = 100;	
		adapter->retries = MAX_LSI_I2C_RETRIES;
		adapter->algo = &i2c_zebulon_algo;
		adapter->algo_data = (void *)&lsiBusMasterInfo[ i ];

		snprintf( adapter->name, 
					sizeof( zebulonAdapter[ i ]->name ), 
					"%s%d", FPGA_ZEBULON_I2C_NAME, i );

		// Register i2c adapter
		err = i2c_add_adapter( adapter );
		if (err) {

			printk( KERN_ERR "Failed to add SMBus adapter\n" );
			goto exit;
		}
		printk( KERN_INFO "Zebulon I2C Bus [%s] initialized\n", adapter->name );

		/* exported adapter handle */
		zebulonAdapter[ i ] = adapter;


		/* add by wenson */
		//i2cLsiPrint(i);

		/* next i2c master */
		membase += ZEBULON_I2C_MASTER_MAP_SIZE;
	}

	//printk("FGPA INT MASK REG Before:0x%x\n",readb( mmioVirtual + FPGA_INT_MASK_REG ) );

	/* enable irq */
	writeb( 
		readb( mmioVirtual + FPGA_INT_MASK_REG ) & ~(FPGA_INT_MASK_SUBSYS | FPGA_INT_MASK_I2C), 
			mmioVirtual + FPGA_INT_MASK_REG );
	
	//printk("FGPA INT MASK REG After:0x%x\n",readb( mmioVirtual + FPGA_INT_MASK_REG ) );

	return 0;

exit:
	return err;
}


static int i2cZebulonRemove( struct platform_device *pdev ) {

    int i;
	struct resource *irq;

	// Disable irq
	writeb( 
		readb( mmioVirtual + FPGA_INT_MASK_REG ) | (FPGA_INT_MASK_SUBSYS | FPGA_INT_MASK_I2C), 
			mmioVirtual + FPGA_INT_MASK_REG );

    // Stop interrupt routine
    irq = platform_get_resource( pdev, IORESOURCE_IRQ, 0 );
    if( irq )
		free_irq( irq->start, pdev );

	// Unregister i2c adapters
    for( i = 0 ; i < ZEBULON_I2C_BUS_NUM ; i++ ) {
        if( zebulonAdapter[ i ] ) {

			// Deattach & free the resource
            i2c_del_adapter( zebulonAdapter[ i ] );
			kfree( zebulonAdapter[ i ] );
        }
    }

    return 0;
}


static struct platform_driver i2cZebulonDriver = {
    .probe      = i2cZebulonProbe,
	.remove		= i2cZebulonRemove ,
    .driver     = {
        .name   = FPGA_ZEBULON_I2C_NAME,
    },
};


static int __init i2cZebulonInit( void ) {

    return platform_driver_register( &i2cZebulonDriver );
}


static void __exit i2cZebulonExit( void ) {

    platform_driver_unregister( &i2cZebulonDriver );
}


int i2cLsiSetBusSpeed( int bus, int speed ) {

    if( (bus < 0) || (bus >= ZEBULON_I2C_BUS_NUM) ) {

        printk( KERN_ERR "Invalid bus index(%d)\n", bus );
        return -EINVAL;
    }

    if( (speed != ZEBULON_I2C_CLK_100MHZ) && (speed != ZEBULON_I2C_CLK_400MHZ) ) {

        printk( KERN_ERR "Invalid bus speed(%d)\n", speed );
        return -EINVAL;
    }

    bus_speed = speed;
    i2cLsiResetMaster( lsiBusMasterInfo[ bus ].baseAddr );
    return 0;
}
EXPORT_SYMBOL(i2cLsiSetBusSpeed);


module_init( i2cZebulonInit );
module_exit( i2cZebulonExit );


MODULE_AUTHOR( "merck.hung@netapp.com" );
MODULE_DESCRIPTION( "Rocket/Zebulon FPGA PCI Driver" );
MODULE_LICENSE( "GPL" );

