#ifndef _ZEBULON_KERNEL_MODULE_H_
#define _ZEBULON_KERNEL_MODULE_H_

#include <linux/ioctl.h>
#include <linux/types.h>

//#define TPPC476_FIXUP_MAJOR    252
#define ZEBULON_WDT_DEV_NAME        "zebulon-wdt"
/* Define our own private IOCTL */
#define ZEBULON_WDT_MODULE_IOC_MAGIC                'Q'
#define IOCTL_ZEBULON_WDT_REQUEST_FREE_IRQ        _IO(ZEBULON_WDT_MODULE_IOC_MAGIC, 1)
#define IOCTL_ZEBULON_WDT_TEST_INTERRUPT          _IO(ZEBULON_WDT_MODULE_IOC_MAGIC, 2) //for test

#endif  /* _PROCESSOR_KERNEL_MODULE_H_ */

#define FPGA_ZEBULON_VENID			PCI_VENDOR_ID_LSI_LOGIC
#define FPGA_ZEBULON_DEVID			0x7110
#define FPGA_ZEBULON_CFGLEN			256
#define FPGA_ZEBULON_NAME			"fpga_zebulon"

#define FPGA_ZEBULON_I2C_NAME		"i2c_zebulon"
#define FPGA_ZEBULON_I2C_OFF		0x00000000
#define FPGA_ZEBULON_I2C_SIZE		256

#define FPGA_ZEBULON_EXP_NAME		"exp_zebulon"
#define FPGA_ZEBULON_EXP_OFF    	0x00000000
#define FPGA_ZEBULON_EXP_SIZE   	256

#define FPGA_ZEBULON_PSOC_NAME		"psoc_zebulon"
#define FPGA_ZEBULON_PSOC_OFF   	0x00000000
#define FPGA_ZEBULON_PSOC_SIZE  	256

#define FPGA_ZEBULON_NVSRAM_NAME	"nvsram_zebulon"
#define FPGA_ZEBULON_NVSRAM_OFF		0x00000000
#define FPGA_ZEBULON_NVSRAM_SIZE	0x00000200

#define FPGA_ZEBULON_UARTMUX_NAME	"uartmux_zebulon"
#define FPGA_ZEBULON_UARTMUX_OFF	0x00000000
#define FPGA_ZEBULON_UARTMUX_SIZE	256

#define FPGA_ZEBULON_RESET_NAME		"reset_zebulon"
#define FPGA_ZEBULON_RESET_OFF		0x00000000
#define FPGA_ZEBULON_RESET_SIZE		256

#define FPGA_ZEBULON_LED_NAME		"led_zebulon"
#define FPGA_ZEBULON_LED_OFF		0x00000000
#define FPGA_ZEBULON_LED_SIZE		7

#define FPGA_ZEBULON_EPP_NAME		"epp_zebulon"
#define FPGA_ZEBULON_EPP_OFF		0x00000000
#define FPGA_ZEBULON_EPP_SIZE		4

#define FPGA_ZEBULON_ID_NAME		"id_zebulon"
#define FPGA_ZEBULON_ID_OFF			0x00000000
#define FPGA_ZEBULON_ID_SIZE		256

#define FPGA_ZEBULON_CWT_NAME		"cwt_zebulon"
#define FPGA_ZEBULON_CWT_OFF		0x00000000
#define FPGA_ZEBULON_CWT_SIZE		4

#define FPGA_ZEBULON_BBU_NAME		"bbu_zebulon"
#define FPGA_ZEBULON_BBU_OFF		0x00000000
#define FPGA_ZEBULON_BBU_SIZE		256

#define FPGA_ZEBULON_UART_NAME      "uart_zebulon"
#define FPGA_ZEBULON_UART_OFF       0x00000000
#define FPGA_ZEBULON_UART_SIZE      8

#define ENTRY_MAX_NAME				80


#define FPGA_INT_MASK_REG			0x91
#define FPGA_INT_MASK_SUBSYS		0x01
#define FPGA_INT_MASK_I2C			0x02
#define FPGA_INT_MASK_WDT			0x04
#define FPGA_INT_MASK_ALT_MSG		0x08
#define FPGA_INT_MASK_COB_UART      0x10


typedef struct _entryAttr {

	char			name[ ENTRY_MAX_NAME ];
	void			*entryReadFunc;
	void			*entryWriteFunc;
	void			*entryWriteParser;
	unsigned int	regOffset;
	unsigned int	regMask;

} entryAttr_t;


#ifdef ZEBULON_DEBUG
#define DBGPRINT( msg, args... )    printk( KERN_INFO "%s: " msg, __func__, ##args );
#else
#define DBGPRINT( msg, args... )
#endif


#define FPGA_PROTECTED_WRITE_1                      0x00000031
#define FPGA_SUBSYSTEM_STATUS_LINES_3               0x00000066
#define FPGA_ENABLE_BATTERIES                       0x00000018
#define FPGA_UART_CONTROL                           0x0000001C
#define FPGA_BOARD_ID                               0x00000001
#define FPGA_LEGACY_MISCELLANEOUS_INFORMATION_1     0x00000008
#define FPGA_SUBSYSTEM_ID                           0x0000000E
#define FPGA_HOST_BOARD_ID                          0x00000048
#define FPGA_ALTERNATE_BOARD_ID                     0x00000049
#define FPGA_SUBSYSTEM_STATUS_LINES_1               0x00000064
#define FPGA_SUBSYSTEM_STATUS_LINES_2               0x00000065
#define FPGA_SUBSYSTEM_STATUS_LINES_3               0x00000066
#define SBB2_MIDPLANE_24        0x02    //SBB 2.0 CAMDEN midplane ID (24 drives)
#define SBB2_MIDPLANE_12        0x04    //SBB 2.0 EBBETS  midplane ID (12 drives)
#define FPGA_SEVEN_SEGMENT_DISPLAY_1                0x00000002
#define FPGA_SEVEN_SEGMENT_DISPLAY_2                0x00000003
#define FPGA_SEVEN_SEGMENT_BLINK_CONTROL            0x00000098


// Prototypes
/*
UINT8 fpgaRegisterRead( UINT32 fpgaOffset );
INT32 fpgaRegisterWrite( UINT32 fpgaOffset, UINT8 data );
UINT8 fpgaNvsramByteRead( UINT32 nvsramOffset );
INT32 fpgaNvsramByteWrite( UINT32 nvsramOffset, UINT8 data );
*/

volatile void __iomem *getFpgaMmioVirtual( void );
volatile void __iomem *getFpgaNvsramVirtual( void );

void zebulon_wdt_init(void);
void zebulon_wdt_exit(void);

