#ifndef PCIDAPI_H
#define PCIDAPI_H 

#include <thread>

#include "BaseClassDAPI.hpp"
#include "MemoryDAPI.hpp"
#include "PCIDDAPI.hpp"


class PCIDAPI : public BaseClassDAPI 
{
    public:
        PCIDAPI(STRING);
        ~PCIDAPI();

        void    DisplayPCIeSwitchEepromList();


        UINT32  GetDataByBDF(PFA,
                             DATA_TYPE);

        float GetReadableSpeed(PFA,
                               DATA_TYPE);

        string GetReadableStringByID(UINT32     in_u32LogicalID,
                                     DATA_TYPE  in_eDataType);

        string getReadableStringByBDF(PFA,
                                      DATA_TYPE);

        ERRORCODE checkPresenceOfDevice(PFA);

        ErrorMsg* getErrorStatistics(PFA,
                                     PCI_ERROR_TYPE,
                                     list<PCI_ERROR>*);

        void getRetrainSupportedDeviceList(vector<RetrainInfo>*);

        ErrorMsg* clearErrors(PFA);

        ErrorMsg*  readConfig(PFA,
                              UINT32  in_u32StartOffset,
                              UINT32  in_u32EndOffset,
                              UINT8*  in_u8pBuffer);

        void ConfigWriteDword(PFA,
                              UINT32   in_u32Offset,
                              UINT32   in_u32WriteData);

        void ConfigWriteWord(PFA,
                             UINT32     in_u32Offset,
                             UINT16     in_u16WriteData);

        void ConfigWriteByte(PFA,
                             UINT32     in_u32Offset,
                             UINT8      in_u8WriteData);

        void writeConfig(PFA        in_bdf,
                         UINT32     in_u32StartOffset,
                         UINT8*     in_u8pBuffer,
                         UINT32     in_u32BufferSize);
        

        ErrorMsg*  getEepromSize(string   in_strEepromName,
                                 UINT32*  out_u32EepromSize);

        ErrorMsg*   readEeprom(string      in_strEepromName,
                               UINT32      in_u32Address,
                               UINT32      in_u32ReadSize,
                               UINT32*     out_u32pData);

        ErrorMsg*   writeEeprom(string     in_strEepromName,
                                UINT32     in_u32Address,
                                UINT32     in_u32BufferSize,
                                UINT32*    in_u32pWriteBuffer);

        ErrorMsg*   testEeprom(string      in_strEepromName);

        ErrorMsg*   saveEeprom(string  in_strEepromName,
                               STRING  in_filePath);

        ErrorMsg*   programEeprom(string  in_strEepromName,
                                  string  in_filePath);

        ErrorMsg*   testRetrain(PFA);

        ErrorMsg*   convertStringToClassCode(string    in_ClassString,
                                             UINT32*   out_u32ClassCode);

        ErrorMsg*   getDevices(vector<PCIInformantion>*    out_pPciList);

        bool isPCIe(PFA);
        
        bool isNtbDevice(RetrainInfo in_retrainInfo);

        void progressHandler(float  in_u32DWordCount);

        ErrorMsg* scanHiddenDevices(UINT16   in_u16Domain,
                                    vector<PCIInformantion>*);

        void    getHiddenDevices(vector<PCIInformantion>   in_pciList,
                                 vector<PCIInformantion>   in_pciListIO,
                                 vector<PCIInformantion>*  out_pPciList);

        ErrorMsg*  showSpecificErrorStatistics(PFA               in_bdf,
                                               PCI_ERROR_TYPE    in_pciError,
                                               bool              in_showSummary);

        ErrorMsg*   displayHBAs();
        ErrorMsg*   showEepromImageVersion(string   in_strEepromName);

        void    showPciDefinedTypes();

        void JustTest();

    private:
        MemoryDAPI  *m_memoryDAPI;
        PCIDDAPI    *m_pciDriverAPI;
        ERRORCODE    m_eErrorCode;
        ErrorMsg    *m_pErrorMsg;

};

#endif
