#ifndef StorageDAPI_HPP
#define StorageDAPI_HPP

#include <iostream>
#include <iomanip>
#include <sys/time.h>

#include "BaseClassDAPI.hpp"
#include "StorageDDAPIOperation.hpp"
#include "NVMeDAPI.hpp"
#include "SCSIFactory.hpp"

#define HEXDUMP_COLS 32

/// @class      StorageDAPI StorageDAPI.hpp "StorageDAPI.hpp"
/// @brief      Provide diagnostic and utility APIs for Storage module
/// @param      m_pStorageDDAPI is a pointer to StorageDDAPI
class StorageDAPI : public BaseClassDAPI
{
public:
     string m_strOutputResultString;

     StorageDAPI(STRING in_szClassName);
     virtual ~StorageDAPI(void);

     ErrorMsg *IOCTL(UINT32 in_u32LogicalID, StorageIOCTLOperation in_eOperation);
     ErrorMsg *generateStorageList(/*StorageInformationXMLData *inout_pXMLData*/);
     ErrorMsg *getLogicalIdByName(string in_strWellKnownName, UINT32 *out_u32LogicalID);
     ErrorMsg *getDataByID(UINT32 in_u32LogicalID, StorageDataElement in_eStorageDataElement, string *out_strpReturnData);
     ErrorMsg *getDataByName(string in_strWellKnownName, StorageDataElement in_eStorageDataElement, string *out_strpReturnData);
     ErrorMsg *getSMARTInfoByID(UINT32 in_u32LogicalID);//, string *out_strpReturnData);

     
     ErrorMsg *readPerformanceTest(UINT32 in_u32LogicalID, UINT32 in_u32StartSector, UINT32 in_u32TestSizeMB, BOOL in_blsMultiModeOutputFormat = false);
     ErrorMsg *writePerformanceTest(UINT32 in_u32LogicalID, UINT32 in_u32StartSector, UINT32 in_u32TestSizeMB, BOOL in_blsMultiModeOutputFormat = false);
     ErrorMsg *readwriteTest(UINT32 in_u32LogicalID, UINT32 in_u32StartSector, UINT32 in_u32TestSizeMB, BOOL in_blsMultiModeOutputFormat = false);

     ErrorMsg *writeZeroByID(UINT32 in_u32LogicalID, UINT32 in_u32StartSector, BOOL in_blsMultiModeOutputFormat = false);
     ErrorMsg *getPGListByID(UINT32 in_u32LogicalID, UINT32 in_u32RequestPList, UINT32 in_u32RequestGList, UINT32 in_u32Format, BOOL in_blsMultiModeOutputFormat = false);

protected:

private:
     StorageDDAPIOperation *m_pStorageDDAPIOperation;
     SCSIFactory *m_pSCSIFactory;
     SCSISenseAnalyzer *m_pSCSISenseAnalyzer;
     NVMeDAPI *m_pNVMeDAPI;
     UINT32 m_u32StorageSectorCountPerRDWRAccess;

     ErrorMsg *setSectorCountPerRDWRAccess(UINT32 in_u32LogicalID);
     ERRORCODE dumpHexDataToString(string *out_strpDataBuffer, UINT32 *in_u32pDataBuffer, UINT32 in_u32DataLengthInBytes, string in_strDumpStyle, BOOL in_blsDumpAddressHead);
     long double int128_to_double(__u8 *data);
};

#endif
