#include "USBDAPI.hpp"

USBDAPI::USBDAPI(STRING className) : BaseClassDAPI(className)
{
    m_pDataBuffer = NULL;
    m_pFileHandle = -1;
    m_errorCode =  getErrorMsg();
    if (m_errorCode != NO_ERROR_MSG)
    {
        addErrorMsg(ERROR_USB_DAPI_CONSTRUCTOR, NULL, m_errorCode);
    }

    if ((m_USBDiver = new USBDDAPI("USB_drv")) == NULL)
    {
        addErrorMsg( (ERRORCODE)(ERROR_USB_DAPI_CONSTRUCTOR | ERROR_DIAGNOSTIC_API_CONSTRUCTOR1_ERROR), "Create USB_DDAPI fail");
    }

    if((m_pMemoryDAPI = new MemoryDAPI("MemoryDAPI")) == NULL)
    {
        addErrorMsg( (ERRORCODE)(ERROR_USB_DAPI_CONSTRUCTOR | ERROR_DIAGNOSTIC_API_CONSTRUCTOR2_ERROR), "Create MEMORY_DDAPI fail");
    }

   
}



USBDAPI::~USBDAPI()
{
    if (m_USBDiver != NULL)
        delete m_USBDiver;
    if (m_pMemoryDAPI != NULL)
        delete m_pMemoryDAPI;
}


ErrorMsg * USBDAPI::displayUSBDevicesList(list<string> *pUSBList)
{

    //m_USBDiver = new USBDDAPI("USB_drv");
    m_errorCode =  m_USBDiver->getErrorMsg();
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_USB_DAPI_DISPLAY_USB_PORT_LIST, NULL, m_errorCode);
    }

    m_errorCode = m_USBDiver->getUSBDevicesList(pUSBList);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_USB_DAPI_DISPLAY_USB_PORT_LIST, NULL, m_errorCode);
    }

    return NO_ERROR_MSG;

}



ErrorMsg*   USBDAPI::getLogicalIdByName(STRING WellKnownName, UINT32 *LogicalID)
{
    if ((m_errorCode = m_USBDiver->getLogicalIdByName(WellKnownName, LogicalID)) != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERRIR_USB_DAPI_GET_LOGICAL_ID_BY_NAME , NULL, m_errorCode);
    }

    return NO_ERROR_MSG;
}





ErrorMsg* USBDAPI::getDataByID(UINT32 LogicalID,
                               USB_DATA_ELEMENT USBDataElement,
                               list<string> *returnDataInfo)
{
    USB_DATA_ELEMENT eCurrentType;
    ErrorMsg* pErrorMsg = NULL;

    switch(USBDataElement)
    {
        case USB_DATA_ALL_INFO:
            for(UINT32 index=1; index < MAX_DATA_ELEMENT_NUMBER; index++)
            {
                eCurrentType = USB_DATA_ELEMENT(index);
                pErrorMsg = m_USBDiver->getDataByID(LogicalID, eCurrentType, returnDataInfo);
                if(pErrorMsg != NO_ERROR_MSG)
                {
                    return returnErrorMsg(ERRIR_USB_DAPI_GET_DATA_ID_BY_ID, NULL, pErrorMsg);
                }
            }
            returnDataInfo->reverse();
            break;
        case USB_DATA_NODE_INFO:
        case USB_DATA_SN_INFO:
        case USB_DATA_VENDOR_INFO:
        case USB_DATA_CAPACITY_INFO:
        case USB_DATA_LINK_INFO:
        case USB_DATA_SPEED_INFO:
        case USB_DATA_PHYSICAL_BLOCK_SIZE_INFO:
        case USB_DATA_LOGICAL_BLOCK_SIZE_INFO:
        case USB_DATA_SG_NODE_INFO:
            pErrorMsg = m_USBDiver->getDataByID(LogicalID, USBDataElement, returnDataInfo);
            if(pErrorMsg != NO_ERROR_MSG)
            {
                return returnErrorMsg(ERRIR_USB_DAPI_GET_DATA_ID_BY_ID, NULL, pErrorMsg);
            }
            break;
        default:
            return returnErrorMsg((ERRORCODE)(ERRIR_USB_DAPI_GET_DATA_ID_BY_ID | ERROR_USB_DAPI_DATA_ELEMENT_NOT_FOUND), NULL, pErrorMsg);
            break;
    }


    return NO_ERROR_MSG;
}




ErrorMsg*   USBDAPI::readwriteTest(UINT32   LogicalID,
                                   list<string> *returnDataInfo,
                                   UINT32   StartSector,
                                   UINT32   TestSize,
                                   UINT32   BlockSize)
{
    ErrorMsg* pErrorMsg = NULL;
    //printf("%s\n", __func__);
    //printf("LogicalID=%d, StartSector=%d, TestSize=%d BlockSize=%d\n", LogicalID, StartSector, TestSize, BlockSize);
    //UINT64 intStartSector = StartSector *USB_BLOCK_SIZE;
    UINT64 intTestSize = TestSize * MEGA_BYTES_SIZE;





    pErrorMsg = readwriteTestSetup(LogicalID, StartSector, intTestSize, BlockSize);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST, NULL, pErrorMsg);
    }


    pErrorMsg = readwriteTestExecute(LogicalID, returnDataInfo, StartSector, intTestSize, BlockSize);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST, NULL, pErrorMsg);
    }

    pErrorMsg = readwriteTestCleanup(LogicalID);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST, NULL, pErrorMsg);
    }
    return NO_ERROR_MSG;

}





ErrorMsg*   USBDAPI::readwriteTestSetup(UINT32   LogicalID,
                                        UINT32   LogicalSector,
                                        UINT32   TestSize,
                                        UINT32   BlockSize)
{
    //printf("%s\n", __func__);

    initProgress(TestSize);


    UINT64 u64DeviceCapacity = 0;

    ErrorMsg*       pErrorMsg = NULL;
    ErrorMsg*       tempErrorMsg = NULL;
    list<string>    strTempList;
    strTempList.clear();

    // Check Test Size valid
    pErrorMsg = m_USBDiver->getDataByID(LogicalID, USB_DATA_CAPACITY_INFO, &strTempList);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP , NULL, pErrorMsg);
    }
    //printf("strTempList=%s\n", strTempList.front().c_str());
    u64DeviceCapacity = atoll(strTempList.front().c_str()) * MEGA_BYTES_SIZE;
    //printf("u64DeviceCapacity=%ld\n", u64DeviceCapacity);
    if(u64DeviceCapacity==0 || (u64DeviceCapacity - LogicalSector*USB_BLOCK_SIZE) < TestSize )
    {
        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP|ERROR_OUT_OF_RANGE), NULL, NULL);
    }
    strTempList.clear();

    // Get Device Node
    pErrorMsg = m_USBDiver->getDataByID(LogicalID, USB_DATA_NODE_INFO, &strTempList);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP , NULL, pErrorMsg);
    }
    //Open Device node
    //printf("Device Node=%s\n", strTempList.front().c_str());
    pErrorMsg = openDeviceFile(strTempList.front().c_str());
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP, NULL, pErrorMsg);
    }
    strTempList.clear();

/*
    pErrorMsg = SetFileDescription(fileno(m_pFileHandle));
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_SETUP, NULL, pErrorMsg);
    }
*/
    pErrorMsg = m_pMemoryDAPI->getMemoryBuffer(BlockSize, (SYSTEM_ADDRESS_T*)&m_pDataBuffer);
    if(pErrorMsg != NULL)
    {
        tempErrorMsg = closeDeviceFile(m_pFileHandle);
        if(tempErrorMsg != NULL)
        {
            return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP, NULL, tempErrorMsg);
        }
        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP | ERROR_USB_DAPI_ALLOCATE_BACKUP_BUFFER_FAIL) , NULL, pErrorMsg);
    }
    memset(m_pDataBuffer, 0, BlockSize);

    return NO_ERROR_MSG;
}






ErrorMsg*   USBDAPI::readwriteTestExecute(UINT32   LogicalID,
                                          list<string> *returnDataInfo,
                                          UINT32   LogicalSector,
                                          UINT32   TestSize,
                                          UINT32   BlockSize)
{
    //printf("%s\n", __func__);
    ErrorMsg*   pErrorMsg = NULL;
    ErrorMsg*   tempErrorMsg = NULL;


    timeval StartTime;
    timeval EndTime;
    timeval spendTime;
    double  Speed = 0;
    //UINT32  error = 0;
    //UINT32  u32patternNum;

    //UINT64 TestSize = in_pPerformanceInformation->TestSize;
    //UINT64 LogicalSector = in_pPerformanceInformation->StartLogicalSector;
    UINT32  intTempTestSize;
    intTempTestSize = TestSize;

    CHAR tempResultString[256];
    memset(tempResultString, 0, sizeof(tempResultString));
    //UINT64 TestSize = inout_pPerformanceInformation->TestSize;
    //UINT64 LogicalSector = inout_pPerformanceInformation->StartLogicalSector;
    //FILE *generatePattern;
    //FILE *backupData;
    //generatePattern = fopen("generatePattern.txt","w");
    //backupData = fopen("backupData.txt","w");

    UINT32  *pPatternBuffer = NULL;
    //UINT32  *pCompareBuffer = NULL;
    //u32patternNum = sizeof(u32writePattern) / sizeof(UINT32);

    //allocate pattern buffer
    pErrorMsg = m_pMemoryDAPI->getMemoryBuffer(BlockSize, (SYSTEM_ADDRESS_T*)&pPatternBuffer);
    if(pErrorMsg != NULL)
    {
        tempErrorMsg = closeDeviceFile(m_pFileHandle);
        if(tempErrorMsg != NULL)
        {
            pErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)m_pDataBuffer);
            if(pErrorMsg != NULL)
            {
                return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, pErrorMsg);
            }
            return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE, NULL, pErrorMsg);
        }

        pErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)m_pDataBuffer);
        if(pErrorMsg != NULL)
        {
            return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, pErrorMsg);
        }

        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_USB_DAPI_ALLOCATE_PATTERN_BUFFER_FAIL), NULL, pErrorMsg);
    }

    srand(time(NULL));

    gettimeofday(&StartTime, NULL);

    //generate random pattern



    while(intTempTestSize > 0)
    {

        for(UINT32 i = 0 ; i < BlockSize / sizeof(UINT32) ; i++)
        {
            *(pPatternBuffer + i) = rand() % UINT_MAX;
            //fprintf(generatePattern,"%c%c%c%c",*(pPatternBuffer+i), *(pPatternBuffer+i)>>8, *(pPatternBuffer+i)>>16, *(pPatternBuffer+i)>>24);
        }

        //write pattern to disk
        //printf("pwrite\n");
        pErrorMsg = m_USBDiver->PWRITE(m_pFileHandle,
                                       pPatternBuffer,
                                       LogicalSector,
                                       BlockSize);
        if(pErrorMsg != NULL)
        {
            tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)m_pDataBuffer);
            if(tempErrorMsg != NULL)
            {
                tempErrorMsg = closeDeviceFile(m_pFileHandle);
                if(tempErrorMsg != NULL)
                {
                    tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
                    if(tempErrorMsg != NULL)
                    {
                        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
                    }
                    return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP, NULL, tempErrorMsg);
                }

                tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
                if(tempErrorMsg != NULL)
                {
                    return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
                }
                return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
            }

            tempErrorMsg = closeDeviceFile(m_pFileHandle);
            if(tempErrorMsg != NULL)
            {
                tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
                if(tempErrorMsg != NULL)
                {
                    return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
                }
                return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP, NULL, tempErrorMsg);
            }

            tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
            if(tempErrorMsg != NULL)
            {
                return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
            }
            //m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pCompareBuffer);
            return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_USB_DAPI_WRITE_DATA_TO_STORAGE_FAIL), NULL, pErrorMsg);
        }

        //read patter from disk
        //printf("pread\n");
        pErrorMsg = m_USBDiver->PREAD(m_pFileHandle,
                                      m_pDataBuffer,
                                      LogicalSector,
                                      BlockSize);
        if(pErrorMsg != NULL)
        {
            tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)m_pDataBuffer);
            if(tempErrorMsg != NULL)
            {
                tempErrorMsg = closeDeviceFile(m_pFileHandle);
                if(tempErrorMsg != NULL)
                {
                    tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
                    if(tempErrorMsg != NULL)
                    {
                        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
                    }
                    return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP, NULL, tempErrorMsg);
                }

                tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
                if(tempErrorMsg != NULL)
                {
                    return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
                }
                return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
            }

            tempErrorMsg = closeDeviceFile(m_pFileHandle);
            if(tempErrorMsg != NULL)
            {
                tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
                if(tempErrorMsg != NULL)
                {
                    return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
                }
                return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP, NULL, tempErrorMsg);
            }

            tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
            if(tempErrorMsg != NULL)
            {
                return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
            }
            //m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pCompareBuffer);
            return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_USB_DAPI_WRITE_DATA_TO_STORAGE_FAIL), NULL, pErrorMsg);
        }

        //compare pattern
        //printf("compare\n");
        //int compareResult = memcmp(pPatternBuffer, m_pDataBuffer, BlockSize);
        int compareResult=0;
        if(compareResult != 0)
        {
            tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)m_pDataBuffer);
            if(tempErrorMsg != NULL)
            {
                tempErrorMsg = closeDeviceFile(m_pFileHandle);
                if(tempErrorMsg != NULL)
                {
                    tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
                    if(tempErrorMsg != NULL)
                    {
                        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
                    }
                    return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP, NULL, tempErrorMsg);
                }

                tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
                if(tempErrorMsg != NULL)
                {
                    return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
                }
                return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
            }

            tempErrorMsg = closeDeviceFile(m_pFileHandle);
            if(tempErrorMsg != NULL)
            {
                tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
                if(tempErrorMsg != NULL)
                {
                    return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
                }
                return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP, NULL, tempErrorMsg);
            }

            tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
            if(tempErrorMsg != NULL)
            {
                return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
            }
            return returnErrorMsg((ERRORCODE)(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_USB_DAPI_TEST_PATTERN_COMPARE_FAIL), NULL, NULL);
        }

        LogicalSector = LogicalSector + BlockSize/USB_BLOCK_SIZE;
        intTempTestSize = intTempTestSize - BlockSize;

        //printf("intTempTestSize=%d\n", intTempTestSize);
        //printf("LogicalSector=%d\n", LogicalSector);

        updateProgress(BlockSize);

    }
    gettimeofday(&EndTime, NULL);

    spendTime.tv_sec = EndTime.tv_sec - StartTime.tv_sec;
    spendTime.tv_usec = EndTime.tv_usec - StartTime.tv_usec;

    if(spendTime.tv_usec < 0)
    {
        --spendTime.tv_sec;
        spendTime.tv_usec += 1000000;
    }

    double dDataSize = (double)((double)TestSize/(MEGA_BYTES_SIZE));
    double dCostTime = (double)(spendTime.tv_sec)+ ((double)(spendTime.tv_usec)/1000000);

    if(dCostTime == 0)
    {
        m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T) m_pDataBuffer);
        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_EXECUTE | ERROR_USB_DAPI_DIVIDED_ZERO_FAIL), NULL, NULL);
    }
    else
    {
        Speed = (double)dDataSize/dCostTime;
    }
    sprintf(tempResultString, "Spend time: %lf s\n", dCostTime);
    if(tempResultString[strlen(tempResultString) - 1] == '\n')
        tempResultString[strlen(tempResultString) - 1] = '\0';
    returnDataInfo->push_back(tempResultString);
    memset(tempResultString, 0, sizeof(tempResultString));
    sprintf(tempResultString,"Speed: %5.2f M/s\n", Speed);
    if(tempResultString[strlen(tempResultString) - 1] == '\n')
        tempResultString[strlen(tempResultString) - 1] = '\0';
    returnDataInfo->push_back(tempResultString);

    tempErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)pPatternBuffer);
    if(tempErrorMsg != NULL)
    {
        tempErrorMsg = closeDeviceFile(m_pFileHandle);
        if(tempErrorMsg != NULL)
        {
            return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_SETUP, NULL, tempErrorMsg);
        }
        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_WRITE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, tempErrorMsg);
    }

    //fclose(generatePattern);
    //fclose(backupData);
    return returnErrorMsg(NO_ERROR, NULL, NULL);

}






ErrorMsg*   USBDAPI::readwriteTestCleanup(UINT32 LogicalID)
{
    //printf("%s\n", __func__);

    ErrorMsg*       tempErrorMsg = NULL;
    //printf("readPerformanceTestClean\n");

    //UINT32  Storage_LogicalID = -1;
    //CHAR    wellKnownName[(USB_DEVICE_NAME_LENGTH + 1)];
    // display debug if requested

    ErrorMsg* pErrorMsg = NULL;
    pErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)m_pDataBuffer);
    if(pErrorMsg != NULL)
    {
        tempErrorMsg = closeDeviceFile(m_pFileHandle);
        if(tempErrorMsg != NULL)
        {
            return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_CLEAN, NULL, tempErrorMsg);
        }
        return returnErrorMsg(ERROR_USB_DAPI_READ_WRITE_TEST_CLEAN, NULL, pErrorMsg);
    }

    pErrorMsg = closeDeviceFile(m_pFileHandle);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST, NULL, pErrorMsg);
    }

    return returnErrorMsg(NO_ERROR, NULL, NULL);
}





ErrorMsg*   USBDAPI::readPerformanceTest(UINT32 LogicalID,
                                         list<string> *returnDataInfo,
                                         UINT32 StartSector,
                                         UINT32 TestSize,
                                         UINT32 BlockSize)
{
    ErrorMsg* pErrorMsg = NULL;
    //printf("%s\n", __func__);
    //printf("LogicalID=%d, StartSector=%d, TestSize=%d BlockSize=%d\n", LogicalID, StartSector, TestSize, BlockSize);
    //UINT64 intStartSector = StartSector *USB_BLOCK_SIZE;
    UINT64 intTestSize = TestSize * MEGA_BYTES_SIZE;





    pErrorMsg = readPerformanceTestSetup(LogicalID, StartSector, intTestSize, BlockSize);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST, NULL, pErrorMsg);
    }


    pErrorMsg = readPerformanceTestExecute(LogicalID, returnDataInfo, StartSector, intTestSize, BlockSize);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST, NULL, pErrorMsg);
    }
    pErrorMsg = readPerformanceTestCleanup(LogicalID);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST, NULL, pErrorMsg);
    }
    return NO_ERROR_MSG;
}






ErrorMsg*   USBDAPI::readPerformanceTestSetup(UINT32   LogicalID,
                                              UINT32   LogicalSector,
                                              UINT32   TestSize,
                                              UINT32   BlockSize)
{
    //printf("%s\n", __func__);

    initProgress(TestSize);

    UINT64 u64DeviceCapacity = 0;

    ErrorMsg*       pErrorMsg = NULL;
    ErrorMsg*       tempErrorMsg = NULL;
    list<string>    strTempList;
    strTempList.clear();

    // Check Test Size valid
    pErrorMsg = m_USBDiver->getDataByID(LogicalID, USB_DATA_CAPACITY_INFO, &strTempList);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_SETUP , NULL, pErrorMsg);
    }
    //printf("strTempList=%s\n", strTempList.front().c_str());
    u64DeviceCapacity = atoll(strTempList.front().c_str()) * MEGA_BYTES_SIZE;
    //printf("u64DeviceCapacity=%ld\n", u64DeviceCapacity);
    if(u64DeviceCapacity==0 || (u64DeviceCapacity - LogicalSector*USB_BLOCK_SIZE) < TestSize )
    {
        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_SETUP|ERROR_OUT_OF_RANGE), NULL, NULL);
    }
    strTempList.clear();

    // Get Device Node
    pErrorMsg = m_USBDiver->getDataByID(LogicalID, USB_DATA_NODE_INFO, &strTempList);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_SETUP , NULL, pErrorMsg);
    }
    //Open Device node
    //printf("Device Node=%s\n", strTempList.front().c_str());
    pErrorMsg = openDeviceFile(strTempList.front().c_str());
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_SETUP, NULL, pErrorMsg);
    }
    strTempList.clear();
/*
    pErrorMsg = SetFileDescription(fileno(m_pFileHandle));
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_SETUP, NULL, pErrorMsg);
    }
*/
    pErrorMsg = m_pMemoryDAPI->getMemoryBuffer(BlockSize, (SYSTEM_ADDRESS_T*)&m_pDataBuffer);
    if(pErrorMsg != NULL)
    {
        tempErrorMsg = closeDeviceFile(m_pFileHandle);
        if(tempErrorMsg != NULL)
        {
            return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_SETUP, NULL, tempErrorMsg);
        }
        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_SETUP | ERROR_USB_DAPI_ALLOCATE_BACKUP_BUFFER_FAIL) , NULL, pErrorMsg);
    }
    memset(m_pDataBuffer, 0, BlockSize);

    return returnErrorMsg(NO_ERROR, NULL, NULL);
}





ErrorMsg*   USBDAPI::readPerformanceTestExecute(UINT32   LogicalID,
                                                list<string> *returnDataInfo,
                                                UINT32   LogicalSector,
                                                UINT32   TestSize,
                                                UINT32   BlockSize)
{
    //printf("%s\n", __func__);

    ErrorMsg* pErrorMsg = NULL;
    ErrorMsg* tempErrorMsg = NULL;
    timeval StartTime;
    timeval EndTime;
    timeval spendTime;
    double  Speed = 0;
    UINT32  intTempTestSize;
    intTempTestSize = TestSize;

    CHAR tempResultString[256];
    memset(tempResultString, 0, sizeof(tempResultString));
    //UINT64 TestSize = inout_pPerformanceInformation->TestSize;
    //UINT64 LogicalSector = inout_pPerformanceInformation->StartLogicalSector;


    //printf("LogicalSector=%d, TestSize=%d BlockSize=%d\n", LogicalSector, TestSize, BlockSize);

    gettimeofday(&StartTime, NULL);

    while(intTempTestSize > 0)
    {
        pErrorMsg = m_USBDiver->PREAD(m_pFileHandle,
                                      m_pDataBuffer,
                                      LogicalSector,
                                      BlockSize);


        if(pErrorMsg != NULL)
        {
            m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T) m_pDataBuffer);
            if(pErrorMsg != NULL)
            {
                tempErrorMsg = closeDeviceFile(m_pFileHandle);
                if(tempErrorMsg != NULL)
                {
                    return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_EXECUTE, NULL, tempErrorMsg);
                }
                return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, pErrorMsg);
            }
            return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_EXECUTE | ERROR_USB_DAPI_READ_DATA_FROM_STORAGE_FAIL),
                                  NULL,
                                  pErrorMsg);
        }

        LogicalSector = LogicalSector + BlockSize/USB_BLOCK_SIZE;
        intTempTestSize = intTempTestSize - BlockSize;

        updateProgress(BlockSize);
/*

        for(UINT32 i = 0 ; i < BlockSize / sizeof(UINT32) ; i++)
        {
            printf("%x ",*(m_pDataBuffer + i));
        }
        */

    }

    gettimeofday(&EndTime, NULL);

    spendTime.tv_sec = EndTime.tv_sec - StartTime.tv_sec;
    spendTime.tv_usec = EndTime.tv_usec - StartTime.tv_usec;

    if(spendTime.tv_usec < 0)
    {
        --spendTime.tv_sec;
        spendTime.tv_usec += 1000000;
    }

    double dDataSize = (double)((double)TestSize/(MEGA_BYTES_SIZE));
    double dCostTime = (double)(spendTime.tv_sec)+ ((double)(spendTime.tv_usec)/1000000);

    if(dCostTime == 0)
    {
        m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T) m_pDataBuffer);
        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_EXECUTE | ERROR_USB_DAPI_DIVIDED_ZERO_FAIL), NULL, NULL);
    }
    else
    {
        Speed = (double)dDataSize/dCostTime;
    }
    sprintf(tempResultString, "Spend time: %lf s\n", dCostTime);
    if(tempResultString[strlen(tempResultString) - 1] == '\n')
        tempResultString[strlen(tempResultString) - 1] = '\0';
    returnDataInfo->push_back(tempResultString);
    memset(tempResultString, 0, sizeof(tempResultString));
    sprintf(tempResultString,"Speed: %5.2f M/s\n", Speed);
    if(tempResultString[strlen(tempResultString) - 1] == '\n')
        tempResultString[strlen(tempResultString) - 1] = '\0';
    returnDataInfo->push_back(tempResultString);


    return NO_ERROR_MSG;
}




ErrorMsg*   USBDAPI::readPerformanceTestCleanup(UINT32 LogicalID)
{
    //printf("%s\n", __func__);

    ErrorMsg*       tempErrorMsg = NULL;
    //printf("readPerformanceTestClean\n");

    //UINT32  Storage_LogicalID = -1;
    //CHAR    wellKnownName[(USB_DEVICE_NAME_LENGTH + 1)];
    // display debug if requested

    ErrorMsg* pErrorMsg = NULL;
    pErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)m_pDataBuffer);
    if(pErrorMsg != NULL)
    {
        tempErrorMsg = closeDeviceFile(m_pFileHandle);
        if(tempErrorMsg != NULL)
        {
            return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_CLEAN, NULL, tempErrorMsg);
        }
        return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_CLEAN, NULL, pErrorMsg);
    }

    pErrorMsg = closeDeviceFile(m_pFileHandle);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_CLEAN, NULL, pErrorMsg);
    }

    return returnErrorMsg(NO_ERROR, NULL, NULL);
}





ErrorMsg*   USBDAPI::writePerformanceTest(UINT32   LogicalID,
                                          list<string> *returnDataInfo,
                                          UINT32 StartSector,
                                          UINT32 TestSize,
                                          UINT32   BlockSize)

{
    ErrorMsg* pErrorMsg = NULL;
    //printf("%s\n", __func__);
    //printf("LogicalID=%d, StartSector=%d, TestSize=%d BlockSize=%d\n", LogicalID, StartSector, TestSize, BlockSize);
    //UINT64 intStartSector = StartSector *USB_BLOCK_SIZE;
    UINT64 intTestSize = TestSize * MEGA_BYTES_SIZE;



    pErrorMsg = writePerformanceTestSetup(LogicalID, StartSector, intTestSize);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST, NULL, pErrorMsg);
    }
    pErrorMsg = writePerformanceTestExecute(LogicalID, returnDataInfo, StartSector, intTestSize);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST, NULL, pErrorMsg);
    }
    pErrorMsg = writePerformanceTestCleanup(LogicalID);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST, NULL, pErrorMsg);
    }

    return NO_ERROR_MSG;
}





ErrorMsg*   USBDAPI::writePerformanceTestSetup(UINT32   LogicalID,
                                               UINT32   LogicalSector,
                                               UINT32   TestSize,
                                               UINT32   BlockSize)
{
    //printf("writePerformanceTestSetup\n");
    //printf("%s\n", __func__);

    //initProgress(TestSize);

    UINT64 u64DeviceCapacity = 0;

    ErrorMsg*       pErrorMsg = NULL;
    ErrorMsg*       tempErrorMsg = NULL;
    list<string>    strTempList;
    strTempList.clear();

    initProgress(TestSize);

    // Check Test Size valid
    pErrorMsg = m_USBDiver->getDataByID(LogicalID, USB_DATA_CAPACITY_INFO, &strTempList);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_SETUP , NULL, pErrorMsg);
    }
    //printf("strTempList=%s\n", strTempList.front().c_str());
    u64DeviceCapacity = atoll(strTempList.front().c_str()) * MEGA_BYTES_SIZE;
    //printf("u64DeviceCapacity=%ld\n", u64DeviceCapacity);
    if(u64DeviceCapacity==0 || (u64DeviceCapacity - LogicalSector*USB_BLOCK_SIZE) < TestSize )
    {
        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_SETUP|ERROR_OUT_OF_RANGE), NULL, NULL);
    }
    strTempList.clear();

    // Get Device Node
    pErrorMsg = m_USBDiver->getDataByID(LogicalID, USB_DATA_NODE_INFO, &strTempList);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_SETUP , NULL, pErrorMsg);
    }
    //Open Device node
    //printf("Device Node=%s\n", strTempList.front().c_str());

    pErrorMsg = openDeviceFile(strTempList.front().c_str());
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_SETUP, NULL, pErrorMsg);
    }
    strTempList.clear();

    pErrorMsg = m_pMemoryDAPI->getMemoryBuffer(BlockSize, (SYSTEM_ADDRESS_T*)&m_pDataBuffer);
    if(pErrorMsg != NULL)
    {
        tempErrorMsg = closeDeviceFile(m_pFileHandle);
        if(tempErrorMsg != NULL)
        {
            return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_SETUP, NULL, tempErrorMsg);
        }
        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_SETUP | ERROR_USB_DAPI_ALLOCATE_BACKUP_BUFFER_FAIL) , NULL, pErrorMsg);
    }
    memset(m_pDataBuffer, 0, BlockSize);

    return NO_ERROR_MSG;
}





ErrorMsg*   USBDAPI::writePerformanceTestExecute(UINT32   LogicalID,
                                                 list<string> *returnDataInfo,
                                                 UINT32   LogicalSector,
                                                 UINT32   TestSize,
                                                 UINT32   BlockSize)
{
    //printf("%s\n", __func__);

    ErrorMsg* pErrorMsg = NULL;
    ErrorMsg* tempErrorMsg = NULL;
    timeval StartTime;
    timeval EndTime;
    timeval spendTime;
    double  Speed = 0;
    UINT32  intTempTestSize;
    intTempTestSize = TestSize;

    CHAR tempResultString[256];
    memset(tempResultString, 0, sizeof(tempResultString));
    //UINT64 TestSize = inout_pPerformanceInformation->TestSize;
    //UINT64 LogicalSector = inout_pPerformanceInformation->StartLogicalSector;


    gettimeofday(&StartTime, NULL);

    while(intTempTestSize > 0)
    {

        pErrorMsg = m_USBDiver->PWRITE(m_pFileHandle,
                                       m_pDataBuffer,
                                       LogicalSector,
                                       BlockSize);


        if(pErrorMsg != NULL)
        {
            if(pErrorMsg != NULL)
            {
                tempErrorMsg = closeDeviceFile(m_pFileHandle);
                if(tempErrorMsg != NULL)
                {
                    return returnErrorMsg(ERROR_USB_DAPI_READ_PERFORMANCE_TEST_EXECUTE, NULL, tempErrorMsg);
                }
                return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_EXECUTE | ERROR_MEMORY_DAPI_RELEASE_MEMORY_BUFFER), NULL, pErrorMsg);
            }
            m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T) m_pDataBuffer);
            return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_EXECUTE | ERROR_USB_DAPI_READ_DATA_FROM_STORAGE_FAIL),
                                  NULL,
                                  pErrorMsg);
        }

        LogicalSector = LogicalSector + BlockSize/USB_BLOCK_SIZE;
        intTempTestSize = intTempTestSize - BlockSize;

        updateProgress(BlockSize);

    }

    gettimeofday(&EndTime, NULL);

    spendTime.tv_sec = EndTime.tv_sec - StartTime.tv_sec;
    spendTime.tv_usec = EndTime.tv_usec - StartTime.tv_usec;

    if(spendTime.tv_usec < 0)
    {
        --spendTime.tv_sec;
        spendTime.tv_usec += 1000000;
    }

    double dDataSize = (double)((double)TestSize/(MEGA_BYTES_SIZE));
    double dCostTime = (double)(spendTime.tv_sec)+ ((double)(spendTime.tv_usec)/1000000);

    if(dCostTime == 0)
    {
        m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T) m_pDataBuffer);
        return returnErrorMsg(ERRORCODE(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_EXECUTE | ERROR_USB_DAPI_DIVIDED_ZERO_FAIL), NULL, NULL);
    }
    else
    {
        Speed = (double)dDataSize/dCostTime;
    }
    sprintf(tempResultString, "Spend time: %lf s\n", dCostTime);
    if(tempResultString[strlen(tempResultString) - 1] == '\n')
        tempResultString[strlen(tempResultString) - 1] = '\0';
    returnDataInfo->push_back(tempResultString);
    memset(tempResultString, 0, sizeof(tempResultString));
    sprintf(tempResultString,"Speed:     %5.2f M/s\n", Speed);
    if(tempResultString[strlen(tempResultString) - 1] == '\n')
        tempResultString[strlen(tempResultString) - 1] = '\0';
    returnDataInfo->push_back(tempResultString);


    return NO_ERROR_MSG;
}





ErrorMsg*   USBDAPI::writePerformanceTestCleanup(UINT32   LogicalID)
{
    //printf("%s\n", __func__);

    ErrorMsg*       tempErrorMsg = NULL;
    //printf("readPerformanceTestClean\n");

    //UINT32  Storage_LogicalID = -1;
    //CHAR    wellKnownName[(USB_DEVICE_NAME_LENGTH + 1)];
    // display debug if requested

    ErrorMsg* pErrorMsg = NULL;
    pErrorMsg = m_pMemoryDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)m_pDataBuffer);
    if(pErrorMsg != NULL)
    {
        tempErrorMsg = closeDeviceFile(m_pFileHandle);
        if(tempErrorMsg != NULL)
        {
            return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_CLEAN, NULL, tempErrorMsg);
        }
        return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_CLEAN, NULL, pErrorMsg);
    }

    pErrorMsg = closeDeviceFile(m_pFileHandle);
    if(pErrorMsg != NULL)
    {
        return returnErrorMsg(ERROR_USB_DAPI_WRITE_PERFORMANCE_TEST_CLEAN, NULL, pErrorMsg);
    }

    return returnErrorMsg(NO_ERROR, NULL, NULL);
}




ErrorMsg*
USBDAPI::openDeviceFile(STRING strDeviceNode)
{

    //if((m_pFileHandle = fopen(strDeviceNode, "rw+")) == NULL)
    if((m_pFileHandle = open(strDeviceNode, O_RDWR | O_SYNC)) == -1)
    {
        //ADD Error Message
        return returnErrorMsg((ERRORCODE)(ERROR_OPEN_FILE|ERROR_USB_DAPI_OPEN_DEVICE),NULL,NULL);
    }
    //setvbuf(m_pFileHandle, NULL, _IONBF, 0);
    return returnErrorMsg(NO_ERROR, NULL, NULL);
}




ErrorMsg*
USBDAPI::closeDeviceFile(UINT32 intFileNode)
{
    UINT32 error = 0;
    error = close(intFileNode);
    if(error != 0)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_CLOSE_FILE|ERROR_USB_DAPI_CLOSE_DEVICE), NULL, NULL);
    }
    return returnErrorMsg(NO_ERROR, NULL, NULL);
}


/*
ErrorMsg* USBDAPI::SetFileDescription
    (
    INT32 fileDescription
    )
{
    m_errorCode =  NO_ERROR_MSG;;

    m_fileDescription = fileDescription;

    return returnErrorMsg(NO_ERROR, NULL, NULL);
}
*/
