/// @file       NVMeIOCTL.cpp
/// @brief      LSI NVMe controller ioctl class derived from IOCTL class
/// @details    This is an base class which define SCSI operation API
/// @version    1.0.0
/// @date       2014.02.20
/// @author     JerryLee
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.

#include "NVMeIOCTL.hpp"

NVMeIOCTL::NVMeIOCTL(STRING in_fileName) : IOCTL(in_fileName)
{
    // TODO Auto-generated constructor stub
    myIOCTLArgument = NULL;
}

NVMeIOCTL::~NVMeIOCTL()
{
    // TODO Auto-generated destructor stub
}

/// @fn         NVMeIOCTL::allocateIOCTLArgument
/// @brief      allocate every IOCTL command's argument
/// @param      IOCTL OP code
/// @exception  Nothing
/// @return     error code
ERRORCODE NVMeIOCTL::allocateIOCTLArgument(UINT8 IOCTLOperation)
{
    switch(IOCTLOperation)
    {
        case CONTROLLER_INFORMATION:
        case NAMESPACE_INFORMATION:
        case FIRMWARE_LOG:
        case SMART_LOG:
        case ERROR_LOG:
        case FIRMWARE_ACTIVATE:
        case FIRMWARE_DOWNLOAD:
        case WRITE_ZERO:
        case FORMAT:
            if((myIOCTLArgument = (nvme_passthru_cmd *) malloc(sizeof(nvme_passthru_cmd))) != NULL)
                memset(myIOCTLArgument, 0, sizeof(nvme_passthru_cmd));

            break;
        case NAMESPACE_ID:
        default:    // do not need argument
            return NO_ERROR;
    }

    if(myIOCTLArgument == NULL)
    {
        this->closeFileHandle();
        return ERROR_SAS_IOCTL_MEM_ALLOC_FAILED;
    }

    return NO_ERROR;
}

/// @fn         NVMeIOCTL::setIOCTLOperationArgument
/// @brief      set argument by IOCTL OP code
/// @param      IOCTL OP code
/// @exception  Nothing
/// @return     error code
ERRORCODE NVMeIOCTL::setIOCTLOperationArgument(UINT8 IOCTLOperation, void* data)
{
    switch(IOCTLOperation)
    {
        case CONTROLLER_INFORMATION:
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->opcode = nvme_admin_identify;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->nsid = 0;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->addr = (__u64)(uintptr_t)(struct nvme_id_ctrl *)data;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->data_len = 0x1000;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->cdw10 = 1;

            break;
        case NAMESPACE_INFORMATION:
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->opcode = nvme_admin_identify;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->nsid = 1;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->addr = (__u64)(uintptr_t)(struct nvme_id_ns *)data;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->data_len = 0x1000;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->cdw10 = 0;

            break;
        case FIRMWARE_LOG:
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->opcode = nvme_admin_get_log_page;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->nsid = 0xffffffff;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->addr = (__u64)(uintptr_t)(struct nvme_firmware_log_page *)data;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->data_len = sizeof(struct nvme_firmware_log_page);
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->cdw10 = 3 | ((sizeof(struct nvme_firmware_log_page) >> 2) - 1) << 16;

            break;
        case SMART_LOG:
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->opcode = nvme_admin_get_log_page;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->nsid = 0xffffffff;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->addr = (__u64)(uintptr_t)(struct nvme_smart_log *)data;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->data_len = sizeof(struct nvme_smart_log);
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->cdw10 = 2 | ((sizeof(struct nvme_smart_log) >> 2) - 1) << 16;

            break;
        case ERROR_LOG:
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->opcode = nvme_admin_get_log_page;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->nsid = 0xffffffff;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->addr = (__u64)(uintptr_t)(struct nvme_error_log_page *)data;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->data_len = sizeof(struct nvme_error_log_page);
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->cdw10 = 1 | ((sizeof(struct nvme_error_log_page) >> 2) - 1) << 16;

            break;
        case FIRMWARE_ACTIVATE:
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->opcode = nvme_admin_activate_fw;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->cdw10 = (static_cast<nvme_fw_active *>(data)->action << 3) |
                                                                        static_cast<nvme_fw_active *>(data)->slot;

            break;
        case FIRMWARE_DOWNLOAD:
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->opcode = nvme_admin_download_fw;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->addr = (__u64)(uintptr_t)static_cast<nvme_fw_download_info *>(data)->fw_buf;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->data_len = static_cast<nvme_fw_download_info *>(data)->xfer;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->cdw10 = (static_cast<nvme_fw_download_info *>(data)->xfer >> 2) - 1;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->cdw11 = static_cast<nvme_fw_download_info *>(data)->offset >> 2;

            break;
        case WRITE_ZERO:
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->opcode = nvme_cmd_write_zeroes;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->nsid = static_cast<nvme_write_zero_parms *>(data)->nsid;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->cdw10 = static_cast<nvme_write_zero_parms *>(data)->startBlock & 0xffffffff;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->cdw11 = static_cast<nvme_write_zero_parms *>(data)->startBlock >> 32;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->cdw12 = static_cast<nvme_write_zero_parms *>(data)->blockCount | (NVME_RW_LR << 16);
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->cdw14 = 0;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->cdw15 = 0;

            break;
        case FORMAT:
        {
            UINT32 cdw10 = static_cast<nvme_format_cfg *>(data)->lbaf | static_cast<nvme_format_cfg *>(data)->ms << 4 |
                           static_cast<nvme_format_cfg *>(data)->pi << 5 | static_cast<nvme_format_cfg *>(data)->pil << 8 |
                           static_cast<nvme_format_cfg *>(data)->ses << 9;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->opcode = nvme_admin_format_nvm;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->nsid = static_cast<nvme_format_cfg *>(data)->nsid;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->cdw10 = cdw10;
            static_cast<nvme_passthru_cmd *>(myIOCTLArgument)->timeout_ms = 120000;

            break;
        }
        case NAMESPACE_ID:
        default:
            break;
    }

    return NO_ERROR;
}

/// @fn         NVMeIOCTL::executeIOCTL
/// @brief      sent IOCTL to specific NVMe IOC
/// @param      IOCTL OP code
/// @exception  Nothing
/// @return     error code
ERRORCODE NVMeIOCTL::executeIOCTL(UINT8 in_IOCTLOperation)
{
    struct stat nvme_stat;
    INT32 status;
    int err;

    switch(in_IOCTLOperation)
    {
        case CONTROLLER_INFORMATION:
        case NAMESPACE_INFORMATION:
        case FIRMWARE_LOG:
        case SMART_LOG:
        case ERROR_LOG:
        case FIRMWARE_ACTIVATE:
        case FIRMWARE_DOWNLOAD:
        case FORMAT:
            return nvme_submit_admin_passthru(static_cast<nvme_passthru_cmd *>(myIOCTLArgument));

        case WRITE_ZERO:
            return nvme_submit_io_passthru(static_cast<nvme_passthru_cmd *>(myIOCTLArgument));

        case EXECUTE_NVME_RESET:
            err = fstat(myFileHandle, &nvme_stat);

            if (err < 0)
                return ERROR_OPEN_FILE;

            if (!S_ISCHR(nvme_stat.st_mode))
                return ERROR_NVME_INVALID_FILE_MODE;

            status = ioctl(myFileHandle, NVME_IOCTL_RESET);
            if(status > 0)
                return convertNVMEStatustoErrorcode(status);
            else if(status < 0)
                return ERROR_NVME_DRIVER_IOCTL_FAIL;

            break;
        case EXECUTE_NVME_SUBSYSTEM_RESET:
            err = fstat(myFileHandle, &nvme_stat);

            if (err < 0)
                return ERROR_OPEN_FILE;
            if (!S_ISCHR(nvme_stat.st_mode))
                return ERROR_NVME_INVALID_FILE_MODE;

            status = ioctl(myFileHandle, NVME_IOCTL_RESET);
            if(status > 0)
                return convertNVMEStatustoErrorcode(status);
            else if(status < 0)
                return ERROR_NVME_DRIVER_IOCTL_FAIL;

            break;
        default:
            return ERROR_IOCTL_UNSUPPORT_ARGUMENT;
    }

    return NO_ERROR;
}
/// @fn         NVMeIOCTL::executeIOCTL
/// @brief      sent IOCTL to specific NVMe IOC
/// @param      IOCTL OP code
/// @param      returned IOCTL argument include several information we need
/// @exception  Nothing
/// @return     error code
ERRORCODE NVMeIOCTL::executeIOCTL(UINT8 IOCTLOperation, void* IOCTLArgument)
{
    switch(IOCTLOperation)
    {
        case NAMESPACE_ID:
            *(static_cast<int *>(IOCTLArgument)) = ioctl(myFileHandle, NVME_IOCTL_ID);

            break;
        default:
            return ERROR_IOCTL_UNSUPPORT_ARGUMENT;
    }

    return NO_ERROR;
}

ERRORCODE NVMeIOCTL:: nvme_submit_passthru(int ioctl_cmd, struct nvme_passthru_cmd *cmd)
{
    INT8 status;

    status = ioctl(myFileHandle, ioctl_cmd, cmd);
    if (status < 0)
        return ERROR_NVME_DRIVER_IOCTL_FAIL;
    else if (status > 0)
        return convertNVMEStatustoErrorcode(status);

    return NO_ERROR;
}

ERRORCODE NVMeIOCTL::nvme_submit_admin_passthru(struct nvme_passthru_cmd *cmd)
{
    INT8 status;

    status = ioctl(myFileHandle, NVME_IOCTL_ADMIN_CMD, cmd);
    if (status < 0)
        return ERROR_NVME_DRIVER_IOCTL_FAIL;
    else if (status > 0)
        return convertNVMEStatustoErrorcode(status);

    return NO_ERROR;
}

ERRORCODE NVMeIOCTL::nvme_submit_io_passthru(struct nvme_passthru_cmd *cmd)
{
    INT8 status;

    status = ioctl(myFileHandle, NVME_IOCTL_IO_CMD, cmd);
    if (status < 0)
        return ERROR_NVME_DRIVER_IOCTL_FAIL;
    else if (status > 0)
        return convertNVMEStatustoErrorcode(status);

    return NO_ERROR;
}

ERRORCODE NVMeIOCTL::convertNVMEStatustoErrorcode(UINT32 status)
{
    switch (status & 0x3ff)
    {
        case NVME_SC_SUCCESS:           return NO_ERROR;
        case NVME_SC_INVALID_OPCODE:    return ERROR_NVME_IOCTL_INVALID_OPCODE;
        case NVME_SC_INVALID_FIELD:     return ERROR_NVME_IOCTL_INVALID_FIELD;
        case NVME_SC_CMDID_CONFLICT:    return ERROR_NVME_IOCTL_CMDID_CONFLICT;
        case NVME_SC_DATA_XFER_ERROR:   return ERROR_NVME_IOCTL_DATA_XFER_ERROR;
        case NVME_SC_POWER_LOSS:        return ERROR_NVME_IOCTL_POWER_LOSS;
        case NVME_SC_INTERNAL:          return ERROR_NVME_IOCTL_INTERNAL;
        case NVME_SC_ABORT_REQ:         return ERROR_NVME_IOCTL_ABORT_REQ;
        case NVME_SC_ABORT_QUEUE:       return ERROR_NVME_IOCTL_ABORT_QUEUE;
        case NVME_SC_FUSED_FAIL:        return ERROR_NVME_IOCTL_FUSED_FAIL;
        case NVME_SC_FUSED_MISSING:     return ERROR_NVME_IOCTL_FUSED_MISSING;
        case NVME_SC_INVALID_NS:        return ERROR_NVME_IOCTL_INVALID_NS;
        case NVME_SC_CMD_SEQ_ERROR:     return ERROR_NVME_IOCTL_CMD_SEQ_ERROR;
        case NVME_SC_LBA_RANGE:         return ERROR_NVME_IOCTL_LBA_RANGE;
        case NVME_SC_CAP_EXCEEDED:      return ERROR_NVME_IOCTL_CAP_EXCEEDED;
        case NVME_SC_NS_NOT_READY:      return ERROR_NVME_IOCTL_NS_NOT_READY;
        case NVME_SC_CQ_INVALID:        return ERROR_NVME_IOCTL_CQ_INVALID;
        case NVME_SC_QID_INVALID:       return ERROR_NVME_IOCTL_QID_INVALID;
        case NVME_SC_QUEUE_SIZE:        return ERROR_NVME_IOCTL_QUEUE_SIZE;
        case NVME_SC_ABORT_LIMIT:       return ERROR_NVME_IOCTL_ABORT_LIMIT;
        case NVME_SC_ABORT_MISSING:     return ERROR_NVME_IOCTL_ABORT_MISSING;
        case NVME_SC_ASYNC_LIMIT:       return ERROR_NVME_IOCTL_ASYNC_LIMIT;
        case NVME_SC_FIRMWARE_SLOT:     return ERROR_NVME_IOCTL_FIRMWARE_SLOT;
        case NVME_SC_FIRMWARE_IMAGE:    return ERROR_NVME_IOCTL_FIRMWARE_IMAGE;
        case NVME_SC_INVALID_VECTOR:    return ERROR_NVME_IOCTL_INVALID_VECTOR;
        case NVME_SC_INVALID_LOG_PAGE:  return ERROR_NVME_IOCTL_INVALID_LOG_PAGE;
        case NVME_SC_INVALID_FORMAT:    return ERROR_NVME_IOCTL_INVALID_FORMAT;
        case NVME_SC_BAD_ATTRIBUTES:    return ERROR_NVME_IOCTL_BAD_ATTRIBUTES;
        case NVME_SC_WRITE_FAULT:       return ERROR_NVME_IOCTL_WRITE_FAULT;
        case NVME_SC_READ_ERROR:        return ERROR_NVME_IOCTL_READ_ERROR;
        case NVME_SC_GUARD_CHECK:       return ERROR_NVME_IOCTL_GUARD_CHECK;
        case NVME_SC_APPTAG_CHECK:      return ERROR_NVME_IOCTL_APPTAG_CHECK;
        case NVME_SC_REFTAG_CHECK:      return ERROR_NVME_IOCTL_REFTAG_CHECK;
        case NVME_SC_COMPARE_FAILED:    return ERROR_NVME_IOCTL_COMPARE_FAILED;
        case NVME_SC_ACCESS_DENIED:     return ERROR_NVME_IOCTL_ACCESS_DENIED;
        default:                        return ERROR_IOCTL;
    }
}
