#ifndef PCIDDAPI_H
#define PCIDDAPI_H

#include "BaseClassDDAPI.hpp"
#include "PCIDevice.hpp"
//#include "MemoryDDAPI.hpp"

#include "PCICommonFunction.hpp"
#include "PCICommonClass.hpp"


#include <string.h>
#include <list>
#include <vector>
#include <iostream>
using namespace std;

#include <algorithm>
#include <sys/io.h>


#define MAX_LINE            1024

#define PCI_MAX_DEVICE_AMOUNT   10000

#define PCI_UNUSED_PARAMETER    0x0

#define PCIE_SWITCH_EEPROM_STATUS_CTRL      0x260
#define PCIE_SWITCH_SERIAL_EEPROM_BUFFER    0x264


#define PCI_VENDOR_ID_INTEL     0x8086
#define PCI_VENDOR_ID_PLX       0x10B5


#define PLX_EEPROM_CMD_WRITE            2
#define PLX_EEPROM_CMD_READ             3
#define PLX_EEPROM_CMD_READ_STATUS      5
#define PLX_EEPROM_CMD_WRITE_ENABLE     6

#define EEPROM_START_ADDRESS            0x04
#define EEPROM_VALIDATION_SIGNATURE     0x5A
//#define EEPROM_SIZE_IN_BYTE             0x3FFFF //256K                     
#define EEPROM_SIZE_IN_BYTE             0x7FFF //256K                     

#define PCI_EXPECTED_LINK_INFO_INI  "PciExpectedLnkSta.ini"

#define SORT_PCI_DEVICES        true
#define NOT_SORT_PCI_DEVICES    false

typedef struct _PCI_NAME_MAPPING
{
    unsigned short  u16VendorID;
    unsigned short  u16DeviceID;
    string          strVendorName;
    string          strDeviceName;
}PCI_NAME_MAPPING;


typedef struct _PCI_CLASS_MAPPING
{
    UINT8   u8BaseClass;
    UINT8   u8SubClass;
    UINT8   u8ProgrammingInterface;
    string  strBassClass;
    string  strSubClass;
    string  strProgrammingInterface;
}PCI_CLASS_MAPPING;


typedef struct _PCI_ERROR
{
    string  strErrorName;
    UINT32  u32Data;
}PCI_ERROR;



typedef enum _PCI_TEST_ITEM
{
    PCI_SPEED_TEST = 0,
    PCI_WIDTH_TEST,
    PCI_RETRAIN_TEST,
}PCI_TEST_ITEM;


typedef enum _PCI_ERROR_TYPE
{
    PCI_COMPATIBLE_ERROR = 0,
    PCIE_ERROR,
    AER_ERROR,

    //pci-compatible error 
    DETECTED_PARITY_ERROR,
    SIGNALED_SYSTEM_ERROR,
    RECEIVED_MASTER_ABORT,
    RECEIVED_TARGET_ABORT,
    SIGNALED_TARGET_ABORT,
    MASTER_DATA_PARITY_ERROR,
    //--secondary side of a bridge 
    DETECTED_PARITY_ERROR_S,
    RECEIVED_SYSTEM_ERROR,
    RECEIVED_MASTER_ABORT_S,
    RECEIVED_TARGET_ABORT_S,
    SIGNALED_TARGET_ABORT_S,
    MASTER_DATA_PARITY_ERROR_S,

    //PCIE error 
    UNSUPPORTED_REQUEST_DETECTED,
    FATAL_ERROR_DETECTED,
    NON_FATAL_ERROR_DETECTED,
    CORRECTABLE_ERROR_DETECTED,

    //AER 
    UNCORRECTABLE_ERROR_STATUS,
    CORRECTABLE_ERROR_STATUS,
}PCI_ERROR_TYPE;


typedef struct _EEPROM_INFO
{
    string  strEepromName;
    UINT16  u16Domain;
    UINT16  u16Bus;
    UINT16  u16Device;
    UINT8   u8Function;
}EEPROM_INFO;


typedef struct _LINK_INFO
{
    UINT16  u16Domain;
    UINT16  u16Bus;
    UINT16  u16Device;
    UINT8   u8Function;
    float   fNegotiateSpeed;
    UINT16  u16NegotiateWidth;
}LINK_INFO;



typedef struct _PFA
{
    UINT16  u16Domain;
    UINT16  u16Bus;
    UINT16  u16Device;
    UINT8   u8Function;
}PFA;


class RetrainInfo
{
    public:
        PCIInformantion PciInfo;
        bool            LinkStatus;  //indicate the link is up or down
        bool            retrainResult;

        //the information of the downstream device in the retrain link 
        UINT16          u16DomainDown;
        UINT16          u16BusDown;
        UINT16          u16DeviceDown;
        UINT8           u8FunctionDown;
        UINT16          u16VendorIDDown;
        UINT16          u16DeviceIDDown;
        string          szDeviceNameDown;
};

typedef struct _TEST_RESULT
{
    RetrainInfo  retrainLinkInfo;
    bool         bTestPass;
}TEST_RESULT;

typedef struct _SUPPORTED_HBA
{
    UINT16  u16VendorID;
    UINT16  u16DeviceID;
    string  hbaName;
    string  description;
}SUPPORTED_HBA;

typedef struct _SLOT_INFO
{
    //UINT8           u8SlotNumber;
    UINT16          u16Domain;
    UINT16          u16Bus;
    UINT16          u16Device;
    UINT8           u8Function;
    bool            bLinkStatus;
    PCIInformantion downStreamDevice;
    string          slotType;
    string          hbaName;
    string          description;
}SLOT_INFO;



class PCIDDAPI : public BaseClassDDAPI 
{
    public:
        PCIDDAPI(STRING);
        ~PCIDDAPI();

        //void DisplayDeviceList();
        //void DisplayPCIeSwitchEepromList();

        void CreatePCIInfoList();


        unsigned char ConfigReadByte(unsigned int in_u32LogicalID,
                                     unsigned int in_u32Offset);

        //unsigned char ConfigReadByte(PFA,
        //                             UINT32   in_u32Offset);

        void ConfigReadByte(PFA       in_bdf,
                            UINT32    in_u32StartOffset,
                            UINT32    in_u32EndOffset,
                            UINT8*    out_u8Buffer);

        void    ConfigReadBytePlx(PFA       in_bdf,
                                  UINT32    in_u32StartOffset,
                                  UINT32    in_u32EndOffset,
                                  UINT8*    out_u8Buffer);

        unsigned short ConfigReadWord(PFA,
                                      UINT32   in_u32Offset);
        
        unsigned int ConfigReadDword(PFA,
                                     UINT32   in_u32Offset);
        
        void ConfigWriteDword(PFA,
                              UINT32    in_u32Offset,
                              UINT32    in_u32WriteData);

        void ConfigWriteWord(PFA,
                             UINT32     in_u32Offset,
                             UINT16     in_u16WriteData);
        
        void ConfigWriteByte(PFA,
                             UINT32     in_u32Offset,
                             UINT8      in_u8WriteData);

        void ConfigWriteByte(PFA        in_bdf,
                             UINT32     in_u32StartAddress,
                             UINT8*     in_u8pBuffer,
                             UINT32     in_u32BufferSize);

        void ConfigWriteBytePlx(PFA        in_bdf,
                                UINT32     in_u32StartAddress,
                                UINT8*     in_u8pBuffer,
                                UINT32     in_u32BufferSize);

        
        void    GetDeviceByID(unsigned int  in_u32LogicalID,
                              PCIDevice     *pciDeviceList);

        void    GetDeviceByBDF(PFA,
                               PCIDevice*);


        /*
        ERRORCODE  GetIDByBDF(unsigned short in_u16Domain,
                              unsigned short in_u16Bus,
                              unsigned short in_u16Device,
                              unsigned char  in_u8Function,
                              unsigned int   *out_u32LogicalID);
        */

        UINT32 GetDataByID(unsigned int  in_u32LogicalID,
                           DATA_TYPE     in_eDataType);

        UINT32 GetDataByBDF(PFA,
                            DATA_TYPE);

        float   GetFloatDataByID(PFA,
                                 DATA_TYPE);

        //float GetFloatDataByID(unsigned int  in_u32LogicalID,
        //                       DATA_TYPE     in_eDataType);

        string  GetDeviceNameByID(unsigned int  in_u32LogicalID);

        ERRORCODE   checkPresenceOfDevice(PFA);

        string  getLinkStatus(PFA);
        bool    getLinkStatus(PCIDevice);
        string  getPortTypeString(PFA);
        string  getClassTypeString(PFA);


        ERRORCODE PciRetrainTest(PFA);


        ErrorMsg*   getBarByBDF(UINT16    in_u16Domain,
                                UINT16    in_u16Bus,
                                UINT16    in_u16Device,
                                UINT8     in_u8Function,
                                UINT32    in_u32BarIndex,
                                UINT32    *out_u32BAR);

        ErrorMsg*   getBarByBDF(PFA       in_bdf,
                                UINT32    in_u32BarIndex,
                                UINT32    *out_u32BAR,
                                UINT32    *out_u32BarSize);

        ErrorMsg*   getPciInfoByClass(PCIClassCode                in_Class,
                                      vector<PCIInformantion>   *out_pPciInfo);
        
        void getDeviceList(vector<PCIInformantion>*   out_pPciList,
                           bool                       in_bSort);

        void savePciInfoToPciStructure(PCIDevice           in_classPciDevice,
                                       PCIInformantion     *pciInfo);


        ERRORCODE getRetrainSupportDevices(vector<RetrainInfo>*);

        ERRORCODE getErrorStatistics(PFA,
                                     PCI_ERROR_TYPE,
                                     list<PCI_ERROR>*);

        ERRORCODE clearErrors(PFA);


        ErrorMsg* readEeprom(PFA,
                             UINT32     in_u32Address,
                             UINT32     in_u32BufferSize,
                             UINT32*    out_u32pBuffer);

        ErrorMsg* writeEeprom(PFA,
                              UINT32     in_u32Address,
                              UINT32     in_u32BufferSize,
                              UINT32*    in_u32pBuffer);

        ErrorMsg* testEeprom(PFA);
        ErrorMsg* restoreEeprom(UINT32*     in_u32DataToBeRestored,
                                UINT32      in_u32DWordCount);

        
        void scanEeproms(vector<EEPROM_INFO> *out_pEepromList);

        ErrorMsg*   getEeprom(string       in_strEepromName,
                              EEPROM_INFO  *out_pEeprom);

        ErrorMsg*   getEepromSize(PFA,
                                  UINT32     *out_u32pEepromSize);

        ErrorMsg*   testRetrain(PFA);

        ErrorMsg*   testRetrainRun(PCIDevice);

        void getDevicesName(vector<PCIInformantion>*   inout_pPciList);

        ERRORCODE convertStringToClassCode(string    in_ClassString,
                                           UINT32*   out_u32ClassCode);

        bool    isPCIe(PFA);

        ErrorMsg*   scanDevicesFromIO(vector<PCIInformantion>*);
        
        ErrorMsg*   readEepromDword(UINT32     in_u32Address,
                                    UINT32*    out_u32pData);
        
        ErrorMsg*    writeEepromDword(UINT32     in_u32Address,
                                      UINT32     in_u32WriteData);

        void    setEepromBase(PFA);
        bool    isNtbDevice(UINT16  in_u16VendorID,
                            UINT16  in_u16DeviceID);
        bool    isNtbDevice(RetrainInfo in_retrainInfo);

        UINT32  getDeviceAmount();

        void    createPFA(PCIInformantion*    in_pPciInfo,
                          PFA*                out_pPFA);
        
        ERRORCODE   scanSlot(vector<SLOT_INFO>*    out_pSlotInfo);

        ErrorMsg*   showEepromImageVersion(string   in_strEepromName);


        void    showPciDefinedTypes();

        void JustTest();

    private:
        //libpci used, will closed by pci_cleanup()
        struct pci_access       *m_libPacc;
        struct pci_dev          *m_libDevice;
        struct pci_cap          *m_libCap;
        struct device           *m_first_dev;
        struct bridge            m_host_bridge = { NULL, NULL, NULL, NULL, 0, 0, 0, 0, NULL};

        //PCILibrary             *m_pPciLibrary;
        PCICommonFunction  *m_pPciLibrary;
        //PCILibrary              m_pciLibrary;
        PCICommonFunction   m_pciLibrary;
        vector<PCI_NAME_MAPPING>  m_pciNameList;
        unsigned int            m_device_amount;

        //MemoryDDAPI             *m_memoryDriver;
        PCIDevice               *m_pciDeviceList[PCI_MAX_DEVICE_AMOUNT];
        PCIDevice               *m_pciDevice;
        PCIInformantion         m_pciInfo;
        //UINT64                  m_u64PciBar;
        PHYSICAL_ADDRESS_T      m_u64PciBar;
        vector<PFA>             m_expHbaList;
        UINT32                  m_u32HbaAmount;

        ErrorMsg               *m_pErrorMsg;
        ERRORCODE               m_eErrorCode;
        //UINT8                   m_u8NtbPortNumber;

        //unsigned int parseID();
        //const char* getVendorName(UINT16 in_u16RequestID);
        //const char* getDeviceName(UINT16 in_u16RequestID);
       
        /*
        ERRORCODE getPciDeviceName(UINT16  in_u16VendorID,
                                   UINT16  in_u16DeviceID,
                                   string  *out_strVendorName,
                                   string  *out_strDeviceName);
        */

        //void createMemoryDriver();
        void releaseMemoryDriver();

        void createPciObject(PFA);
        void createPciObjects();

        void releasePciObject();
        void releasePciObjects();

        ERRORCODE   getPciDeviceNameMappingTable();
        ERRORCODE   getPciClassMappingTable(list<PCI_CLASS_MAPPING> *out_pClassList);

        UINT32  getLinkWidth(PCIDevice    in_classPciDevice,
                             DATA_TYPE    in_eDataType);

        PCI_ERROR getPciCompatibleErrorStatistics(PCI_ERROR_TYPE  in_pciError);
        
        PCI_ERROR getPCIeErrorStatistics(PCI_ERROR_TYPE     in_pciError,
                                         UINT32             in_u32PCIeCapOffset);

        PCI_ERROR getAerErrorStatistics(PCI_ERROR_TYPE  in_pciError,
                                        UINT32          in_u32ErrorOffset);

        ErrorMsg* sendEepromCommand(UINT32  in_u32Command);
        ErrorMsg* waitEepromToBeIdle();

        ErrorMsg*   readEepromBlock(UINT32     in_u32StartAddress,
                                    UINT32     in_u32DwordCount,
                                    UINT32*    out_u32pBuffer);

        //ErrorMsg*   readEepromWord(UINT32     in_u32Address,
        //                           UINT32*    out_u32pData);


        ErrorMsg*   writeEepromBlock(UINT32     in_u32StartAddress,
                                     UINT32     in_u32DWordCount,
                                     UINT32*    in_u32pWriteData);
        


        ErrorMsg*    testEepromRun(UINT32  in_u32Pattern,
                                   UINT32  in_u32DWordCount);

        void grow_tree();

        void        getDownstreamDevice(SLOT_INFO           in_slotInfo,
                                        PCIInformantion*    out_pPciInfo);

        ERRORCODE   getDownstreamDevice(RetrainInfo *TempRetrainInfo);

        void        initialPciInfoStructure(PCIInformantion*   out_pPciInfo);

        void releasePciStructs();

        int id_white_p(int c);
        int id_hex(char *p, int cnt);

        static int compare_them(const void *A, const void *B);
        void sort_them();

        //void defineName();
        void defineName(PCIInformantion*  inout_pciInfo);
        //bool getNameFromIds();
        bool getNameFromIds(PCIInformantion*  inout_pciInfo);
        bool isAvailableForRetrainTest(PCIDevice);

        void getRetrainRegs(PCIDevice  in_classPciDevice,
                            UINT32*    out_u32LnkCtlReg,
                            UINT32*    out_u32LnkStaReg);

        ERRORCODE   convertStringToBDF(string in_strParameter,
                                       PFA*   out_pBDF);

        ERRORCODE   createSlotInfo(vector<SLOT_INFO>*       out_pSlotInfoList,
                                   UINT32*                  out_u32SlotAmount,
                                   vector<SUPPORTED_HBA>*   out_pHbaList);
        

        void   getHbaInfo(vector<SLOT_INFO>*     out_pSlotInfo,
                          vector<SUPPORTED_HBA>  in_supportedHbaList);
};








#endif 
