#include "RTCDDAPI.hpp"
#include <unistd.h>
#include <sys/io.h>
#include <fcntl.h>
#include <linux/rtc.h>
#include <sys/ioctl.h>
#include <errno.h>

RTCDDAPI :: RTCDDAPI(STRING className) : BaseClassDDAPI(className)
{
    // Initializations
    m_rtcDevHandle = -1;
    m_strDay[0] = "SUN";
    m_strDay[1] = "MON";
    m_strDay[2] = "TUE";
    m_strDay[3] = "WED";
    m_strDay[4] = "THU";
    m_strDay[5] = "FRI";
    m_strDay[6] = "SAT";
    
    // 8254 timer
    if (ioperm(GPT_8254_SYSTEM_COUNTER_IOPORT, 4, IO_PERMON))
    {
        addErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_CONSTRUCTOR | ERROR_RTC_DDAPI_PORT_PERMISSION), NULL);
        return;
    }
}

RTCDDAPI :: ~RTCDDAPI()
{
    // 8254 timer
    ioperm(GPT_8254_SYSTEM_COUNTER_IOPORT, 4, IO_PERMOFF);

}

// about date and time
ErrorMsg * RTCDDAPI :: showDateTime(void)
{
    ErrorMsg* pErrorMsg;
    UINT32  year, month, date, hour, minute, second;
    INT32   tmpCentury;
    INT32   tmpYear;
    INT32   tmpMonth;
    FLOAT   tmpDay;
    INT32   retVal;
    struct  rtc_time rtc_tm;
    
    if ((m_rtcDevHandle = open(RTC_DEV_NAME, O_RDONLY)) < 0)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_SHOW_DATE_TIME | ERROR_RTC_DDAPI_RTC_DEVICE_OPEN_FAIL), NULL, NULL);
    }

    // Turn on update interrupts (one per second)
    if((retVal = ioctl(m_rtcDevHandle, RTC_UIE_ON, 0)) == -1)
    {
        if (errno != ENOTTY)
        {
            close(m_rtcDevHandle);

            sprintf(m_tmpString, "RTC IOCTL Error: RTC_UIE_ON  %s\n", strerror(errno));
            return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_SHOW_DATE_TIME | ERROR_RTC_DDAPI_RTC_IOCTL_ACCESS_FAIL), m_tmpString, NULL);
        }
    }

	// Read the RTC time/date
    if(ioctl(m_rtcDevHandle, RTC_RD_TIME, &rtc_tm))
    {
        close(m_rtcDevHandle);

        sprintf(m_tmpString, "RTC IOCTL Error: RTC_RD_TIME  %s\n", strerror(errno));
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_SHOW_DATE_TIME | ERROR_RTC_DDAPI_RTC_IOCTL_ACCESS_FAIL), m_tmpString, NULL);
    }

    year = rtc_tm.tm_year + 1900;
    month = rtc_tm.tm_mon + 1;
    date = rtc_tm.tm_mday;
    hour = rtc_tm.tm_hour;
    minute = rtc_tm.tm_min;
    second = rtc_tm.tm_sec;

    // Calculate Day (dependent on the year, month and date)
    tmpYear = year;
    if (month == 1)
    {
        tmpMonth = 11;
        tmpYear  = year - 1;
    }
    else if (month == 2)
    {
        tmpMonth = 12;
        tmpYear  = year - 1;
    }
    else
    {
        tmpMonth = month - 2;
    }
    tmpCentury = tmpYear / 100;
    tmpYear    = tmpYear % 100;
    tmpDay = (date + (INT32)(2.6 * tmpMonth - 0.2) + 5 * (tmpYear % 4) + 3 * tmpYear + 5 * (tmpCentury % 4)) % 7;

    
    printf("%04d/%02d/%02d  %s  %02d:%02d:%02d\n\n", 
           year, month, date, m_strDay[(INT32)tmpDay], hour, minute, second);

    // Turn off update interrupts
    if(ioctl(m_rtcDevHandle, RTC_UIE_OFF, 0))
    {
        close(m_rtcDevHandle);

        sprintf(m_tmpString, "RTC IOCTL Error: RTC_UIE_OFF  %s\n", strerror(errno));
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_SHOW_DATE_TIME | ERROR_RTC_DDAPI_RTC_IOCTL_ACCESS_FAIL), m_tmpString, NULL);
    }

    if (pErrorMsg != NO_ERROR_MSG)
    {
        close(m_rtcDevHandle);

        return returnErrorMsg((ERRORCODE)ERROR_RTC_DDAPI_SHOW_DATE_TIME, NULL, pErrorMsg);
    }
    
    close(m_rtcDevHandle);

    return NO_ERROR_MSG;
}
 
ErrorMsg * RTCDDAPI :: setDate(INT32 year, INT32 month, INT32 date)
{
    INT32   retVal;
    struct  rtc_time rtc_tm;

    if ((m_rtcDevHandle = open(RTC_DEV_NAME, O_RDONLY)) < 0)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_SET_DATE | ERROR_RTC_DDAPI_RTC_DEVICE_OPEN_FAIL), NULL, NULL);
    }

    // Turn on update interrupts (one per second)
    if((retVal = ioctl(m_rtcDevHandle, RTC_UIE_ON, 0)) == -1)
    {
        if (errno != ENOTTY)
        {
            close(m_rtcDevHandle);

            sprintf(m_tmpString, "RTC IOCTL Error: RTC_UIE_ON  %s\n", strerror(errno));
            return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_SET_DATE | ERROR_RTC_DDAPI_RTC_IOCTL_ACCESS_FAIL), m_tmpString, NULL);
        }
    }

	// Read the RTC time/date
    if(ioctl(m_rtcDevHandle, RTC_RD_TIME, &rtc_tm))
    {
        close(m_rtcDevHandle);

        sprintf(m_tmpString, "RTC IOCTL Error: RTC_RD_TIME  %s\n", strerror(errno));
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_SET_DATE | ERROR_RTC_DDAPI_RTC_IOCTL_ACCESS_FAIL), m_tmpString, NULL);
    }

    rtc_tm.tm_year = year - 1900;
    rtc_tm.tm_mon = month - 1;
    rtc_tm.tm_mday = date;

	// Set the RTC date
    if(ioctl(m_rtcDevHandle, RTC_SET_TIME, &rtc_tm))
    {
        close(m_rtcDevHandle);

        sprintf(m_tmpString, "RTC IOCTL Error: RTC_SET_TIME  %s\n", strerror(errno));
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_SET_DATE | ERROR_RTC_DDAPI_RTC_IOCTL_ACCESS_FAIL), m_tmpString, NULL);
    }
    
    // Turn off update interrupts
    if(ioctl(m_rtcDevHandle, RTC_UIE_OFF, 0))
    {
        close(m_rtcDevHandle);

        sprintf(m_tmpString, "RTC IOCTL Error: RTC_UIE_OFF  %s\n", strerror(errno));
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_SET_DATE | ERROR_RTC_DDAPI_RTC_IOCTL_ACCESS_FAIL), m_tmpString, NULL);
    }
    
    close(m_rtcDevHandle);
    
    return NO_ERROR_MSG;
}

ErrorMsg * RTCDDAPI :: setTime(INT32 hour, INT32 minute, INT32 second)
{
    INT32   retVal;
    struct  rtc_time rtc_tm;

    if ((m_rtcDevHandle = open(RTC_DEV_NAME, O_RDONLY)) < 0)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_SET_DATE | ERROR_RTC_DDAPI_RTC_DEVICE_OPEN_FAIL), NULL, NULL);
    }

    // Turn on update interrupts (one per second)
    if((retVal = ioctl(m_rtcDevHandle, RTC_UIE_ON, 0)) == -1)
    {
        close(m_rtcDevHandle);

        if (errno != ENOTTY)
        {
            sprintf(m_tmpString, "RTC IOCTL Error: RTC_UIE_ON  %s\n", strerror(errno));
            return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_SET_DATE | ERROR_RTC_DDAPI_RTC_IOCTL_ACCESS_FAIL), m_tmpString, NULL);
        }
    }

	// Read the RTC time/date
    if(ioctl(m_rtcDevHandle, RTC_RD_TIME, &rtc_tm))
    {
        close(m_rtcDevHandle);

        sprintf(m_tmpString, "RTC IOCTL Error: RTC_RD_TIME  %s\n", strerror(errno));
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_SET_DATE | ERROR_RTC_DDAPI_RTC_IOCTL_ACCESS_FAIL), m_tmpString, NULL);
    }

    rtc_tm.tm_hour = hour;
    rtc_tm.tm_min = minute;
    rtc_tm.tm_sec = second;
    
	// Set the RTC time
    if(ioctl(m_rtcDevHandle, RTC_SET_TIME, &rtc_tm))
    {
        close(m_rtcDevHandle);

        sprintf(m_tmpString, "RTC IOCTL Error: RTC_SET_TIME  %s\n", strerror(errno));
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_SET_DATE | ERROR_RTC_DDAPI_RTC_IOCTL_ACCESS_FAIL), m_tmpString, NULL);
    }
    
    // Turn off update interrupts
    if(ioctl(m_rtcDevHandle, RTC_UIE_OFF, 0))
    {
        close(m_rtcDevHandle);

        sprintf(m_tmpString, "RTC IOCTL Error: RTC_UIE_OFF  %s\n", strerror(errno));
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_SET_DATE | ERROR_RTC_DDAPI_RTC_IOCTL_ACCESS_FAIL), m_tmpString, NULL);
    }
    
    close(m_rtcDevHandle);
    
    return NO_ERROR_MSG;
}

// RTC 
ErrorMsg * RTCDDAPI :: setRTCTimerRate(FLOAT timerRate)
{
    FLOAT       tempFloatValue;
    UINT32      tempIntValue = 0;
    
    //Interrupt driven mode
    if ((timerRate > 500000) || (timerRate < 122.070))
    {
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_SET_RTC_TIMER_RATE | ERROR_BAD_PARAMETER_ONE), "rate isn't support", NULL);
    }

    //e.g. rate = 31.25 ms = 31250 us = 32Hz (1 / 0.03125(s) = 32)
    tempFloatValue = (1 / (timerRate / 1000000));   //(Uint: Hz)

    m_rtcTimerRate = (UINT32)tempFloatValue;

    if ((tempFloatValue - m_rtcTimerRate) >= 0.5)
        m_rtcTimerRate = m_rtcTimerRate + 1;

    //Check that the input was really a power of 2.
    while (m_rtcTimerRate > (UINT32)(1 << tempIntValue))
        tempIntValue++;
    if (m_rtcTimerRate != (UINT32)(1 << tempIntValue))
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_SET_RTC_TIMER_RATE | ERROR_BAD_PARAMETER_ONE), "rate isn't support", NULL);
    
    return NO_ERROR_MSG;
}

ErrorMsg * RTCDDAPI :: startRTCTimer(void)
{
    if (m_rtcDevHandle < 0)
    {
        if ((m_rtcDevHandle = open(RTC_DEV_NAME, O_RDWR)) < 0)
        {
            return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_START_RTC_TIMER | ERROR_RTC_DDAPI_RTC_DEVICE_OPEN_FAIL), NULL, NULL);
        }
    }

    //select rate
    if(ioctl(m_rtcDevHandle, RTC_IRQP_SET, m_rtcTimerRate))
    {
        sprintf(m_tmpString, "RTC IOCTL Error: RTC_IRQP_SET  %s\n", strerror(errno));
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_START_RTC_TIMER | ERROR_RTC_DDAPI_RTC_IOCTL_ACCESS_FAIL), m_tmpString, NULL);
    }

    //Start Periodic Interrupt
    if(ioctl(m_rtcDevHandle, RTC_PIE_ON, 0))
    {
        sprintf(m_tmpString, "RTC IOCTL Error: RTC_PIE_ON  %s\n", strerror(errno));
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_START_RTC_TIMER | ERROR_RTC_DDAPI_RTC_IOCTL_ACCESS_FAIL), m_tmpString, NULL);
    }

    return NO_ERROR_MSG;
}

void RTCDDAPI :: stopRTCTimer(void)
{
    //Interrupt driven mode
    if (m_rtcDevHandle >= 0)
    {
        ioctl(m_rtcDevHandle, RTC_PIE_OFF, 0);
    
        close(m_rtcDevHandle);
        m_rtcDevHandle = -1;
    }
}

ErrorMsg * RTCDDAPI :: readRTCTimerStatus(BOOL *timerStatus)
{
    INT32       target;
    UINT32      data;
    
    //Interrupt driven mode
    if (m_rtcDevHandle < 0)
    {
        if ((m_rtcDevHandle = open(RTC_DEV_NAME, O_RDWR)) < 0)
        {
            return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_READ_RTC_TIMER_STATUS | ERROR_RTC_DDAPI_RTC_DEVICE_OPEN_FAIL), NULL, NULL);
        }
    }
    
    target = read(m_rtcDevHandle, &data, sizeof(UINT32));
    if (target == -1)
    {
        stopRTCTimer();
        sprintf(m_tmpString, "read RTC timer status fail  %s\n", strerror(errno));
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_READ_RTC_TIMER_STATUS | ERROR_RTC_DDAPI_RTC_DEVICE_READ_FAIL), m_tmpString, NULL);
    }

    *timerStatus = TRUE;

    return NO_ERROR_MSG;

}

// 8254 Timer
void RTCDDAPI :: startGPTTimer(UINT32 timerCount, GPT_TIMER_MODE selectMode)
{
    UINT8 target;
    m_selectMode = selectMode;

    setGPTTimerControlWord(m_selectMode);

    //Write LSB
    target = (UINT8)(timerCount & 0x000000FF);
    outb(target, GPT_8254_SYSTEM_COUNTER_IOPORT);

    //Write MSB
    target = (UINT8)((timerCount & 0x0000FF00) >> 8);
    outb(target, GPT_8254_SYSTEM_COUNTER_IOPORT);

}

void RTCDDAPI :: stopGPTTimer(void)
{
    setGPTTimerControlWord(GPT_MODE0);
    
    //Write LSB
    outb(0, GPT_8254_SYSTEM_COUNTER_IOPORT);
    
    //Write MSB
    outb(0, GPT_8254_SYSTEM_COUNTER_IOPORT);
}

void RTCDDAPI :: readGPTTimerStatus(BOOL *timerStatus)
{
    UINT8 target;
    
    //set readback command
    target = 3 << 6;
    target += 1 << 1;
    outb(target, GPT_8254_CONTROL_COUNTER_IOPORT);
    
    target = inb(GPT_8254_SYSTEM_COUNTER_IOPORT);
    
    if (target & 0x80)
    {
        *timerStatus = TRUE;
    }
    else
    {
        *timerStatus = FALSE;
    }
}


void RTCDDAPI :: setGPTTimerControlWord(GPT_TIMER_MODE selectMode)
{
    UINT8 target;

    //Counter Select
    target = 0;
            
    //Read/Write LSB then MSB
    target += 3 << 4;
    
    //Counter Mode Selection 
    switch(selectMode)
    {
        case GPT_MODE0:
            break;
        case GPT_MODE1:
            target += 1 << 1;
            break;
        case GPT_MODE2:
            target += 2 << 1;
            break;
        case GPT_MODE3:
            target += 3 << 1;
            break;
        case GPT_MODE4:
            target += 4 << 1;
            break;
        case GPT_MODE5:
            target += 5 << 1;
            break;
        default:
            break;
            
    }
    
    //Binary Countdown Select
    //do nothing
    
    outb(target, GPT_8254_CONTROL_COUNTER_IOPORT);
}

void RTCDDAPI :: setGPTReadbackCommand(void)
{
    UINT8 target;

    //Counter Select = Read Back Command
    target = 3 << 6;
    
    //do nothing
    
    //Latch Status of Selected Counters.
    //do nothing
    
    //Latch Count of Selected Counters.
    //do nothing
    

    //Counter Select
    target += 1 << 1;
           
    outb(target, GPT_8254_CONTROL_COUNTER_IOPORT);
}

void RTCDDAPI :: convertGPTTimeToCount(FLOAT usTime, UINT32* timerCount)
{   
    FLOAT   tempFloatValue;
    UINT32  tempIntValue;

    usTime *= 1000;
    tempFloatValue = usTime / GPT_RATE;
    
    tempIntValue = (UINT32)tempFloatValue;    
    
    if ((tempFloatValue - (UINT32)tempFloatValue) >= 0.5)
        tempIntValue = (UINT32)tempFloatValue + 1;
        
    *timerCount = tempIntValue;

}

void RTCDDAPI :: readGPTTimer(UINT32* currentValue)
{
    UINT8 target;
    
    // set counter latch
    target = 0;
    outb(target, GPT_8254_CONTROL_COUNTER_IOPORT);
    
    //Read LSB
    target = inb(GPT_8254_SYSTEM_COUNTER_IOPORT);
    *currentValue = (UINT32)target;
    
    //Read MSB
    target = inb(GPT_8254_SYSTEM_COUNTER_IOPORT);
    *currentValue += (target << 8);
}

ErrorMsg * RTCDDAPI :: rtcWDTTest(void)
{
	return returnErrorMsg((ERRORCODE)(ERROR_RTC_DDAPI_RTC_WDT_TEST | ERROR_FUNCTION_NOT_IMPLEMENT));
}

