#ifndef AHCI_ARCHITECTURE_HPP
#define AHCI_ARCHITECTURE_HPP

#include <fcntl.h>
#include <sys/mman.h>
#include "common.hpp"

typedef volatile struct tagHBAPort
{
        UINT32 clb;          // 0x00, command list base address, 1K-byte aligned
        UINT32 clbu;          // 0x04, command list base address upper 32 bits
        UINT32 fb;          // 0x08, FIS base address, 256-byte aligned
        UINT32 fbu;          // 0x0C, FIS base address upper 32 bits
        UINT32 is;          // 0x10, interrupt status
        UINT32 ie;          // 0x14, interrupt enable
        UINT32 cmd;          // 0x18, command and status
        UINT32 rsv0;          // 0x1C, Reserved
        UINT32 tfd;          // 0x20, task file data
        UINT32 sig;          // 0x24, signature
        UINT32 ssts;          // 0x28, SATA status (SCR0:SStatus)
        UINT32 sctl;          // 0x2C, SATA control (SCR2:SControl)
        UINT32 serr;          // 0x30, SATA error (SCR1:SError)
        UINT32 sact;          // 0x34, SATA active (SCR3:SActive)
        UINT32 ci;          // 0x38, command issue
        UINT32 sntf;          // 0x3C, SATA notification (SCR4:SNotification)
        UINT32 fbs;          // 0x40, FIS-based switch control
        UINT32 rsv1[11];          // 0x44 ~ 0x6F, Reserved
        UINT32 vendor[4];          // 0x70 ~ 0x7F, vendor specific
} HBAPort;

typedef volatile struct tagHBAMEM
{
        // 0x00 - 0x2B, Generic Host Control
        UINT32 cap;          // 0x00, Host capability
        UINT32 ghc;          // 0x04, Global host control
        UINT32 is;          // 0x08, Interrupt status
        UINT32 pi;          // 0x0C, Port implemented
        UINT32 vs;          // 0x10, Version
        UINT32 ccc_ctl;          // 0x14, Command completion coalescing control
        UINT32 ccc_pts;          // 0x18, Command completion coalescing ports
        UINT32 em_loc;          // 0x1C, Enclosure management location
        UINT32 em_ctl;          // 0x20, Enclosure management control
        UINT32 cap2;          // 0x24, Host capabilities extended
        UINT32 bohc;          // 0x28, BIOS/OS handoff control and status

        // 0x2C - 0x9F, Reserved
        UINT8 rsv[0xA0 - 0x2C];

        // 0xA0 - 0xFF, Vendor specific registers
        UINT8 vendor[0x100 - 0xA0];

        // 0x100 - 0x10FF, Port control registers
        HBAPort ports[5];          // 1 ~ 32
} HBAMEM;

/*
 int main(int argc, char* argv[])
 {

 int fd = 0;
 volatile int *io = 0;

 if((fd = open("/dev/mem", O_RDWR | O_SYNC)) < 0)
 {
 printf("cant open /dev/mem\n");
 return 0;
 }

 HBA_MEM *AHCI_MEM = (HBA_MEM *)mmap(NULL, sizeof(HBA_MEM), PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0xf0842000);

 int i = 0;

 printf("cap = 0x%x\n", AHCI_MEM->cap);
 printf("ghc = 0x%x\n", AHCI_MEM->ghc);
 printf("is = 0x%x\n", AHCI_MEM->is);
 printf("pi = 0x%x\n", AHCI_MEM->pi);
 printf("vs = 0x%x\n", AHCI_MEM->vs);
 printf("ccc_ctl = 0x%x\n", AHCI_MEM->ccc_ctl);
 printf("ccc_pot = 0x%x\n", AHCI_MEM->ccc_pts);
 printf("em_loc = 0x%x\n", AHCI_MEM->em_loc);
 printf("em_ctl = 0x%x\n", AHCI_MEM->em_ctl);
 printf("cap2 = 0x%x\n", AHCI_MEM->cap2);
 printf("bohc = 0x%x\n", AHCI_MEM->bohc);


 for(i=0; i<6; i++)
 {
 printf("--------------------port: %i ---------------------\n", i);
 printf("\ncmd base = 0x%x\n", (AHCI_MEM->ports[i]).clb);
 printf("clbu = 0x%x\n", (AHCI_MEM->ports[i]).clbu);
 printf("fb = 0x%x\n", (AHCI_MEM->ports[i]).fb);
 printf("fbu = 0x%x\n", (AHCI_MEM->ports[i]).fbu);
 printf("is = 0x%x\n", (AHCI_MEM->ports[i]).is);
 printf("ie = 0x%x\n", (AHCI_MEM->ports[i]).ie);
 printf("cmd = 0x%x\n", (AHCI_MEM->ports[i]).cmd);
 printf("sig = 0x%x\n", (AHCI_MEM->ports[i]).sig);
 printf("ssts = 0x%x\n", (AHCI_MEM->ports[i]).ssts);

 printf("sctl = 0x%x\n", (AHCI_MEM->ports[i]).sctl);// 0x2C, SATA control (SCR2:SControl)
 printf("serr = 0x%x\n", (AHCI_MEM->ports[i]).serr);// 0x30, SATA error (SCR1:SError)
 printf("sact = 0x%x\n", (AHCI_MEM->ports[i]).sact);//sact;  // 0x34, SATA active (SCR3:SActive)
 printf("serr = 0x%x\n", (AHCI_MEM->ports[i]).serr);//serr
 printf("ci = 0x%x\n", (AHCI_MEM->ports[i]).ci);// 0x38, command issue
 printf("sntf = 0x%x\n", (AHCI_MEM->ports[i]).sntf);//sntf;  // 0x3C, SATA notification (SCR4:SNotification)
 printf("fbs = 0x%x\n", (AHCI_MEM->ports[i]).fbs);//fbs;  // 0x40, FIS-based switch control
 }

 //    clb;  // 0x00, command list base address, 1K-byte aligned
 //    clbu;  // 0x04, command list base address upper 32 bits
 //    fb;  // 0x08, FIS base address, 256-byte aligned
 //    fbu;  // 0x0C, FIS base address upper 32 bits
 //    is;  // 0x10, interrupt status
 //    ie;  // 0x14, interrupt enable
 //    cmd;  // 0x18, command and status
 //    rsv0;  // 0x1C, Reserved
 //    tfd;  // 0x20, task file data
 //    sig;  // 0x24, signature
 //    ssts;  // 0x28, SATA status (SCR0:SStatus)
 //    sctl;  // 0x2C, SATA control (SCR2:SControl)
 //    serr;  // 0x30, SATA error (SCR1:SError)
 //    sact;  // 0x34, SATA active (SCR3:SActive)
 //    ci;  // 0x38, command issue
 //    sntf;  // 0x3C, SATA notification (SCR4:SNotification)
 //    fbs;  // 0x40, FIS-based switch control
 //    rsv1[11]; // 0x44 ~ 0x6F, Reserved
 //    vendor[4]; // 0x70 ~ 0x7F, vendor specific

 // printf("ports = %\n", AHCI_MEM->ports[1]);
 // 0x2C - 0x9F, Reserved
 // BYTE rsv[0xA0-0x2C];

 // 0xA0 - 0xFF, Vendor specific registers
 // BYTE vendor[0x100-0xA0];

 // 0x100 - 0x10FF, Port control registers
 //     HBA_PORT ports[1]; // 1 ~ 32

 return 0;
 }
 */
#endif
