#ifndef STORAGEDDAPI_OPERATION_HPP
#define STORAGEDDAPI_OPERATION_HPP

#include <iostream>
#include <sstream>
#include <string>
#include <stdio.h>
#include <stdlib.h>

#include "BaseClassDDAPI.hpp"
#include "StorageInfo.hpp"

#include "SATADDAPI.hpp"
#include "SASDDAPI.hpp"
#include "NVMeDDAPI.hpp"
#include "USBDDAPI.hpp"

#include "SCSIFactory.hpp"

using namespace std;

/// @emun       IOCTL operations
///             List all supported IOCTL operations
typedef enum
{
     eStorageGenerateList = 0,
     eStorageListDrive,
     eStorageReadPerformanceTest,
     eStorageWritePerformanceTest,
     eStorageReadWriteTest,

     eStorageUnknownIOCTLOperation,
} StorageIOCTLOperation;

/// @emun       data elements
///             List all supported Storage data elements
typedef enum
{
     eStorageDriveDataAll,
     eStorageDriveDeviceNodeName,
     eStorageDriveSGDeviceNodeName,
     eStorageDriveVendorName,
     eStorageDriveSerialNumber,
     eStorageDriveModelName,
     eStorageDriveFirmwareVersion,
     eStorageDriveLogicalBlockSize,
     eStorageDrivePhysicalBlockSize,
     eStorageDriveSASAddress,
     eStorageDriveCapacity,
     eStorageDrivePortNumber,
     eStorageDriveLinkStatus,
     eStorageDriveLinkSpeed,
     eStorageDriveTemperature,
     eStorageDriveRotationRate,
     eStorageDriveFormFactor,
     eStorageDriveProtocolType,
     eStorageDriveUSBVersion,

     eStorageDriveUnknownDataElement,
} StorageDataElement;

/// @class      StorageDDAPIOperation StorageDDAPIOperation.hpp "StorageDDAPIOperation.hpp"
/// @brief      Provide low layer diagnostic and utility APIs for Storage module
/// @param      *m_pPCIDDAPI is a pointer to access PCI DDAPI
class StorageDDAPIOperation: public BaseClassDDAPI
{
public:
     StorageDDAPIOperation(STRING className);
     virtual ~StorageDDAPIOperation(void);
     
     ErrorMsg *IOCTL(UINT32 in_u32LogicalID, StorageIOCTLOperation in_eOperation);

     ErrorMsg *generateStorageList(void);
     ErrorMsg *listDrive(void);

     ErrorMsg *getLogicalIdByName(string in_strWellKnownName, UINT32 *out_u32LogicalID);
     ErrorMsg *getNameByID(UINT32 in_u32LogicalID, string *out_strWellKnownName);
     ErrorMsg *getDataByID(UINT32 in_u32LogicalID, StorageDataElement in_eStorageDataElement, string *out_strpReturnData);
     ErrorMsg *getDataByName(string in_strWellKnownName, StorageDataElement in_eStorageDataElement, string *out_strpReturnData);
     ErrorMsg *getControllerTypeByID(UINT32 in_u32LogicalID, UINT32 *out_u32pControllerType);
     ErrorMsg *getSMARTInfoByID(UINT32 in_u32LogicalID, string *out_strpReturnData);

protected:

private:
     list <StorageDriveStruct> m_storageDriveList;
     SATADDAPI                *m_pSATADDAPI;
     SASDDAPI                 *m_pSASDDAPI;
     NVMeDDAPI                *m_pNVMeDDAPI;
     USBDDAPI                 *m_pUSBDDAPI;
     SCSIFactory              *m_pSCSIFactory;

     string convertStringToUpperCase(string in_strInputString);
};
#endif

